#include "BuildDataOrganizer.h"

void BuildDataOrganizer::Go(DataOrganizer* organizer, Inputs* input)
{
	if (input->SimulationStringParams["Model"] == "SpatialTemperatureHydro")
	{
		BuildSpatialDataOrganizer(organizer, input);
	}
	else if (input->SimulationStringParams["Model"] == "StatisticalHydro")
	{
		BuildStatisticalDataOrganizer(organizer, input);
	}
	else if (input->SimulationStringParams["Model"] == "LCScalingTempSpatial")
	{
		BuildLCScalingDataOrganizer(organizer, input);
	}
}

//cte 20210917 Can we reconfigure BuildSpatialDataOrganizer to more simply maintain multiple folders within a data drawer
void BuildDataOrganizer::BuildSpatialDataOrganizer(DataOrganizer* organizer, Inputs* input)
{
	//NumberOfLocations is number of catchments with vertical water balance
	//NumberOfLocations = 1 for StatisticalHydro Model or = (nrows * ncols) for SpatialTemperatureHydro Model, set in Inputs.cpp
	//Increment through all catchments, starting counter i = 0, then incrementing i = i + 1 before entering loop
	for (int i = 0; i < input->LocationParams["NumberOfLocations"]; ++i)
	{
		//initialize index counter for DataFolders
		int index = 0;
		//Get dataDrawer vector from DataFolder pointer
		std::vector<DataFolder*> dataDrawer;
		//Loop through all DataFolders in first, i.e., [0], DataDrawer; count of DataFolders is accessed using DataDrawer[0].size()
		while (index < input->DataDrawers[0].size())
		{
			//Assign folderInfo the DataFolder index number
			std::map<std::string, double> folderInfo = input->DataDrawers[0][index];

			//If HydroPlusCong.xml key "Type" == BulkArea
			if (input->StringDataDrawers[0][index]["Type"] == "BulkArea")
			{
				DataFolder* folder = new DataFolder;

				//Folder ParamDict variable set to folderInfo, or input->DataDrawers[0][index]
				folder->ParamDict = folderInfo;

				//Folder ParamStringDict variable set to input->StringDataDrawers[0][index]
				folder->ParamStringDict = input->StringDataDrawers[0][index];

				//Folder aveSMD variable set to the initial average soil moisture deficit, aveSMD (m), which regulates TOPMODEL theory
				folder->VarDict["aveSMD"] = input->AveSMD;

				//Folder position variable set to i counter; i is counter in NumberOfLocations, 1 for StatisticalHydro; max = nrows * ncols for SpatialTemperatureHydro
				folder->VarDict["position"] = i;

				//Call AddFolderAreaSpatialCoverData function to get BulkArea parameters, sending pointers to folder, input, and i
				AddFolderAreaSpatialCoverData(folder, input, i);

				dataDrawer.push_back(folder);
			}
			//add 1 to index
			++index;
		}

		//Store dataDrawer to DataDrawers vector
		organizer->DataDrawers.push_back(dataDrawer);
	}
}

//Called from model = StatisticalHydro
void BuildDataOrganizer::BuildStatisticalDataOrganizer(DataOrganizer* organizer, Inputs* input)
{
	int index = 0;
	for (int j = 0; j < input->DataDrawers.size(); ++j)
	{
		std::vector<DataFolder*> dataDrawer;
		for (int i = 0; i < input->DataDrawers[j].size(); ++i)
		{
			std::map<std::string, double> folderInfo = input->DataDrawers[j][i];

			if (input->StringDataDrawers[j][i]["Type"] == "BulkArea" && folderInfo["Area"] != 0.0)
			{
				DataFolder* folder = new DataFolder;
				folder->ParamDict = folderInfo;
				folder->ParamStringDict = input->StringDataDrawers[j][i];
				// cte 20210408 Why did Reza need to add this, rather than use BuildDataOrganizer::BuildStatisticalDataOrganizer and TopographyData::getAveSoilMoistureDeficit?
				// Reza: I will take care of this later in SemiAveSMDCalc.cpp
				//folder->VarDict["aveSMD"] = input->AveSMD;
				AddFolderAreaStatisticalCoverData(folder, input);
				dataDrawer.push_back(folder);
			}

			if (input->StringDataDrawers[j][i]["Type"] != "BulkArea" && folderInfo["Area"] != 0.0)
			{
				DataFolder* folder = new DataFolder;
				folder->ParamDict = folderInfo;
				folder->ParamStringDict = input->StringDataDrawers[j][i];
				folder->VarDict["aveSMD"] = input->AveSMD;
				folder->VarDict["position"] = i;

				//cte 20210520 Created call to AddFolderAreaStatisticalCoverData to parameterize fractional areas of BulkArea and enable GI utilization of HydroPlus water routines
				//Call AddFolderAreaStatisticalCoverData function to get BulkArea parameters, sending pointers to folder and input
				AddFolderAreaStatisticalCoverData(folder, input);
				dataDrawer.push_back(folder);
			}

			++index;
		}
		organizer->DataDrawers.push_back(dataDrawer);
	}
}

// Called from model == SpatialTemperatureHydro
void BuildDataOrganizer::AddFolderAreaSpatialCoverData(DataFolder *folder, Inputs *input, int locationIndex)
{
	//TimeStep_sec from HydroPlusConfig.xml SimulationNumericalParams inputs converted to folder parameter in order to pass throughout ProcAndCalcs routines; default time step is hour or 3600 seconds
	folder->ParamDict["TimeStep_sec"] = input->SimulationNumericalParams["TimeStep_sec"];

	if (input->NLCDLandCoverData[locationIndex] == input->LocationParams["NODATA"])
	{
		folder->ParamStringDict["Type"] = "NODATA";
		return;
	}
	folder->ParamDict["ImperviousCoverUnderTreeCanopy_frac"] = input->TreeOnImpervious[locationIndex];
	folder->ParamDict["ImperviousCover_frac"] = input->ImperCoverdata[locationIndex] / 100.0; // folder->ParamDict["SurfaceWaterIA_frac"] is intentionally not added to this spatial input term, rcc 10/29/2020
	folder->ParamDict["TotalImperviousCover_frac"] = input->TreeOnImpervious[locationIndex] + folder->ParamDict["ImperviousCover_frac"];

	folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] = input->TreeOnPervious[locationIndex];
	folder->ParamDict["ShortVegetationCover_frac"] = input->ShortVegCover[locationIndex];
	folder->ParamDict["SoilCover_frac"] = input->SoilCover[locationIndex];
	folder->ParamDict["TotalPerviousCover_frac"] = input->TreeOnPervious[locationIndex] + input->ShortVegCover[locationIndex] + input->SoilCover[locationIndex];

	//DCIA for SpatialTemperatureHydro is set in Inputs.cpp, not read from HydroPlusConfig.xml
	//If there is no pervious area, all impervious area drains to outlet and DCIA_frac must equal 1, despite HydroPlusConfig.xml inputs
	double DCIA_frac = 0;
	if (folder->ParamDict["TotalPerviousCover_frac"] <= 0) {
		//DCIA_frac = 1 forces all impervious area to drain to outlet given no pervious area
		DCIA_frac = 1;
	}
	else {
		//DCIA_frac = HydroPlusConfig.xml input, and allows for some impervious area to drain to pervious area
		DCIA_frac = input->ConnectedImpervious[locationIndex];
	}
	folder->ParamDict["ImpAreaDirectlyConnected2Outlet_frac"] = DCIA_frac * folder->ParamDict["TotalImperviousCover_frac"];
	folder->ParamDict["ImpAreaNotDirectlyConnected2Outlet_frac"] = (1 - DCIA_frac) * folder->ParamDict["TotalImperviousCover_frac"];

	folder->ParamDict["NLCDLandCoverData"]= input->NLCDLandCoverData[locationIndex];

	LandCoverPCal::CalLandCover(input, folder, locationIndex);
}

//Called from model = StatisticalHydro
void BuildDataOrganizer::AddFolderAreaStatisticalCoverData(DataFolder* folder, Inputs* input)
{
	//TimeStep_sec from HydroPlusConfig.xml SimulationNumericalParams inputs converted to folder parameter in order to pass throughout ProcAndCalcs routines; default time step is hour or 3600 seconds
	folder->ParamDict["TimeStep_sec"] = input->SimulationNumericalParams["TimeStep_sec"];

	folder->ParamDict["TotalImperviousCover_frac"] = folder->ParamDict["SurfaceWaterIA_frac"] + folder->ParamDict["ImperviousCoverUnderTreeCanopy_frac"] + folder->ParamDict["ImperviousCover_frac"];
	folder->ParamDict["TotalPerviousCover_frac"] = folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] + folder->ParamDict["ShortVegetationCover_frac"] + folder->ParamDict["SoilCover_frac"];
	folder->ParamDict["TotalTreeCover"] = folder->ParamDict["ImperviousCoverUnderTreeCanopy_frac"] + folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"];

	//If there is no pervious area, all impervious area drains to outlet and DCIA_frac must equal 1, despite HydroPlusConfig.xml inputs
	double DCIA_frac = 0;
	if (folder->ParamDict["TotalPerviousCover_frac"] <= 0) {
		//DCIA_frac = 1 forces all impervious area to drain to outlet given no pervious area
		DCIA_frac = 1;
	}
	else {
		//DCIA_frac = HydroPlusConfig.xml input, and allows for some impervious area to drain to pervious area
		DCIA_frac = folder->ParamDict["DCIA_frac"];
	}
	folder->ParamDict["ImpAreaDirectlyConnected2Outlet_frac"] = DCIA_frac * folder->ParamDict["TotalImperviousCover_frac"];
	folder->ParamDict["ImpAreaNotDirectlyConnected2Outlet_frac"] = (1.0 - DCIA_frac) * folder->ParamDict["TotalImperviousCover_frac"];

	//Abort program if (TotalImperviousCover_frac + TotalPerviousCover_frac) > 1, impossible land cover scenario
	if ((folder->ParamDict["TotalImperviousCover_frac"] + folder->ParamDict["TotalPerviousCover_frac"]) > 1 || (folder->ParamDict["TotalImperviousCover_frac"] + folder->ParamDict["TotalPerviousCover_frac"]) < 1) {
		//Explain why program aborted
		cout << endl << endl << "Program aborted due to " << folder->ParamStringDict["Type"] << " total land cover fraction != 1." << endl;
		//Determine gap in desired land cover fraction of 1.0
		double lc_gap = (folder->ParamDict["TotalImperviousCover_frac"] + folder->ParamDict["TotalPerviousCover_frac"]) - 1;
		//Determine if the land cover has to be lowered or raised
		if (lc_gap > 0) {
			cout << endl << "	Lower a " << folder->ParamStringDict["Type"] << " land cover value below by:	" << lc_gap << endl << endl;
		}
		else {
			cout << endl << "	Raise a " << folder->ParamStringDict["Type"] << " land cover value below by:	" << abs(lc_gap) << endl << endl;
		}
		//List all land cover types and their fractions
		cout << "	SurfaceWaterIA_frac:			" << folder->ParamDict["SurfaceWaterIA_frac"] << endl;
		cout << "	ImperviousCoverUnderTreeCanopy_frac:	" << folder->ParamDict["ImperviousCoverUnderTreeCanopy_frac"] << endl;
		cout << "	ImperviousCover_frac:			" << folder->ParamDict["ImperviousCover_frac"] << endl;
		cout << "	PerviousCoverUnderTreeCanopy_frac:	" << folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] << endl;
		cout << "	ShortVegetationCover_frac:		" << folder->ParamDict["ShortVegetationCover_frac"] << endl;
		cout << "	SoilCover_frac:				" << folder->ParamDict["SoilCover_frac"] << endl;
		//List total land cover fraction
		cout << "	Total Land cover fraction:		" << (folder->ParamDict["TotalImperviousCover_frac"] + folder->ParamDict["TotalPerviousCover_frac"]) << endl;
		//Call abort function
		abort();
	}
}

//Called from model == LCScalingTempSpatial
void BuildDataOrganizer::AddFolderAreaAlternativeCoverData(DataFolder *folder, Inputs *input, int locationIndex)
{
	folder->ParamDict["TotalImperviousCover_frac"] = folder->ParamDict["SurfaceWaterIA_frac"] + folder->ParamDict["ImperviousCoverUnderTreeCanopy_frac"] + folder->ParamDict["ImperviousCover_frac"];
	folder->ParamDict["TotalPerviousCover_frac"] = folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] + folder->ParamDict["ShortVegetationCover_frac"] + folder->ParamDict["SoilCover_frac"];
	folder->ParamDict["TotalTreeCover"] = folder->ParamDict["ImperviousCoverUnderTreeCanopy_frac"] + folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"];
	folder->ParamDict["ImpAreaDirectlyConnected2Outlet_frac"] = folder->ParamDict["DCIA_frac"] * folder->ParamDict["TotalImperviousCover_frac"];
	folder->ParamDict["ImpAreaNotDirectlyConnected2Outlet_frac"] = (1 - folder->ParamDict["DCIA_frac"]) * folder->ParamDict["TotalImperviousCover_frac"];
	folder->ParamDict["TreeP"] = folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] + folder->ParamDict["ImperviousCoverUnderTreeCanopy_frac"];
	folder->ParamDict["SoilP"]= folder->ParamDict["SoilCover_frac"];
	folder->ParamDict["ShortVegP"]= folder->ParamDict["ShortVegetationCover_frac"];
	folder->ParamDict["ImperP"]= folder->ParamDict["TotalImperviousCover_frac"];
	input->NLCDLandCoverData.push_back(folder->ParamDict["NLCDLandCoverData"]);
	folder->ParamDict["WaterP"] = 0;

}

void BuildDataOrganizer::SetGIMissingParams(DataOrganizer *organizer, Inputs *input)
{ 
	if (input->SimulationStringParams["Model"] == "CoolRiver") return;
	if (input->SimulationStringParams["Model"] == "ECDynamic") return;
	for(int i = 0; i < organizer->DataDrawers[0].size(); ++i)
	{ 
		DataFolder *giFolder = organizer->DataDrawers[0][i];	
		if (giFolder->ParamStringDict["Type"] == "BulkArea")
			continue;
		for (const auto &myPair : input->RefParams) {
			if (giFolder->ParamDict.find(myPair.first) == giFolder->ParamDict.end()) {
				giFolder->ParamDict[myPair.first] = myPair.second;
			}
		}
	}
}


//Called from model == LCScalingTempSpatial
void BuildDataOrganizer::BuildLCScalingDataOrganizer(DataOrganizer *organizer, Inputs *input)
{
	/*	HydroPlus model = LCScaling only runs two scenarios, the reference station and the alternative.
		The HydroPlusConfig.xml used with LCScaling must give parameters for these two scenarios with the two <DataFolders>

		LCScaling uses LCScaling\BatchRunsParamAdjustor\LCScalingForTempSpatial.cs to define the gap filling when TC + IC < 1 and generate the file 100Runs_Parameters.txt,
		100Runs_Parameters.txt has TC, IC, and gap (SVeg and Soil) that sums to 1,
		LCScaling\100_20_Runs_Code\20Runs_Jun2014_Sutherland_TT_2012\20Runs_Jun2014_Sutherland_TT\ChangeParameter.cpp changes the DCIA

		LCScaling in a 121 or other multiple run sceneario overwrites the HydroPlusConfig.xml, and uses ConfigIC#TC#.txt to keep a record of each HydroPlusConfig.xml
		LCScaling generates and overwrites the BlockGroup1.txt and BlockGroup2.txt files with each run, and converts those files into IC#TC#.txt for permanent output.
	*/
	
		//Likely folder for reference cell
		std::vector<DataFolder*> dataDrawer;
	    std::map<std::string, double> folderInfo = input->DataDrawers[0][0];
	    DataFolder *folder = new DataFolder;
		folder->ParamDict = folderInfo;
		folder->ParamStringDict = input->StringDataDrawers[0][0];
		folder->VarDict["aveSMD"] = input->AveSMD;
		folder->VarDict["position"] = 0;
		AddFolderAreaAlternativeCoverData(folder, input, 0);
		dataDrawer.push_back(folder);
		organizer->DataDrawers.push_back(dataDrawer);

		//Likely folder for non-reference cell
		std::vector<DataFolder*> dataDrawer1;
		DataFolder *folder1 = new DataFolder;
		folder1->ParamDict = input->DataDrawers[0][1];;
		folder1->ParamStringDict = input->StringDataDrawers[0][1];
		folder1->VarDict["aveSMD"] = input->AveSMD;
		folder1->VarDict["position"] = 1;
		AddFolderAreaAlternativeCoverData(folder1, input, 1);
		dataDrawer1.push_back(folder1);
		organizer->DataDrawers.push_back(dataDrawer1);
}


