#include "HeatFluxCal.h"

//Energy balance determines surface heat flux and evaporative cooling or heating of air, as described by Yang et al. (2013)
//NR = H_total_W_p_m2 + LE_total_W_p_m2 + DeltaQ_W_p_m2, where NR is net radiation of shortwave and longwave terms, H_total_W_p_m2 is sensible heat flux, LE_total_W_p_m2 is latent heat flux, and DeltaQ_W_p_m2 is ground storage heat flux

//Albedo is assigned by cell land cover type 21 to 24 based on the assumption impervious cover increases from ~20 to 80%.
//				A strategic alternative is albedo is assigned by actual cell land cover fraction exposed to incoming shortwave radiation.
//				Land cover abledo values are in Table 2-3 in Stull, 2000. Meteorology for Scientists and Engineers, 2nd Edition, Brooks Cole.
//				Road albedo = 5 to 15; Building abledo = 9; Concrete albedo = 15 to 37
//				Tree albedo changes type (max albedo = 15 coniferous vs 25 deciduous, min albedo = 5 coniferous vs 10 deciduous) 
//					Tree albedo deciduous should change wiht leaf on dates
//				Green grass albedo = 26; Agricultural cover emerges as gap area (TC+IC<100%) for NCLD LC types, and albedo ranges 5 to 32
//				Soil albedo changes with wetness (abledo = 7 wet vs 17 dry) and percent clay (100% clay is albedo = 16 wet vs 23 dry) 
//				Snow albedo might be simulated (old snow albedo = 35 to 70; new snow albedo = 75 to 90)

//Emissivity is assigned as a constant for cell land cover type 21 to 24 and then by land cover type tree, short veg, soil, and water
//				A strategic alternative is emissivity is assigned by actual cell impervious cover fraction exposed for outgoing longwave radiation
//				Land cover emissivity values are in Table 2-2 in Stull, 2000. Meteorology for Scientists and Engineers, 2nd Edition, Brooks Cole.
//				Asphalt  = 0.95; Concrete = 0.71-0.9; Bricks (red) = 0.92; Soils = 0.9-0.98; Desert = 0.84-0.91; Grass lawn = 0.97; Grass = 0.9-0.95; 
//				Tree Coniferous = 0.97; Tree Deciduous = 0.95; Snow (old to new) = 0.82-0.99; Shrubs = 0.9; Alfalfa = 0.95; Leaf 10um = 0.97-0.98
//				

//Ground heat storage, DeltaQ_W_p_m2, depends on a1, a2, and a3 parameters from inputs, and on NR (net radiation)
//Algorithm Based on Eq 3 in Grimmond and Oke (1999), which is objective hysteresis model (OHM), calculating the storage heat flux as a function of net all-wave radiation and the surface properties of the site. 
//Grimmond, C. and T.R. Oke (1999), Heat storage in urban areas: Local-scale observations and evaluation of a simple model, Journal of Applied Meteorology, 38(7), 922940; https://doi.org/10.1175/1520-0450(1999)038<0922:hsiual>2.0.co;2  
//Influence of DeltaQ_W_p_m2 to a1, a2, and a3 and NR
//	Parameter a1 (unitless) determines the magnitude of influence of NR on DeltaQ_W_p_m2.
//	Parameter a2 (hr) determines the phase of influence of d(NR) / dt on DeltaQ_W_p_m2.When a2 = 0, the DeltaQ_W_p_m2 curve matches, is in phase with, the NR curve, as a2 gets more positive, the DeltaQ_W_p_m2 curve precedes the NR curve, peaking sooner.
//	Parameter a3 (W/m^2) determines the size of DeltaQ_W_p_m2 when NR becomes negative.A large negative a3 creates a negative DeltaQ_W_p_m2 before NR is negative.
/*
	Land Cover			a1		a2		a3
	Mixed forest			0.11	0.11 - 12.3
	Short grass				0.32	0.54 - 27.4
	Bare soil				0.38	0.56 - 27.3
	Bare soilwet			0.33	0.07 - 34.9
	Bare soildry			0.35	0.43 - 36.5
	Soil 					0.36	0.27 - 42.4
	Watershallow turbid	0.5		0.21 - 39.1
	Rooftop Vancouver		0.17	0.1 - 17
	Rooftop Uppsala			0.44	0.57 - 28.9
	Rooftop Kyoto			0.82	0.34 - 55.7
	Concrete Doll			0.81	0.48 - 79.9
	Concrete Asaeada		0.85	0.32 - 28.5
	Asphalt Narita			0.36	0.23 - 19.3
	Asphalt Asaeada			0.64	0.32 - 43.6
	Asphalt Anandakumar		0.82	0.68 - 20.1
	UrbanCanyon North-South	0.32	0.01 - 27.7
	UrbanCanyon Easth-West	0.71	0.04 - 39.7
*/

HeatFluxCal::HeatFluxCal(Inputs* input)
{

	resist = new ResistanceCal(input);

	//Soila1, Soila2, Soila3 are terms to compute ground heat storage based on net radiation and Eq 3 in Grimmond and Oke (1999) objective hysteresis model (OHM)
	Soila1 = input->TemperatureCalculationParams["Soila1"];
	Soila2 = input->TemperatureCalculationParams["Soila2"];
	Soila3 = input->TemperatureCalculationParams["Soila3"];

	//SoilMSD (m) is soil maximum storage deficit used in TOPMODEL based soil vertical water flux calculations
	SoilMSD = input->RefParams["MSD_m"];
	Soilheight = input->TemperatureCalculationParams["Soilheight"];
	Soilalbedo = input->TemperatureCalculationParams["Soilalbedo"];
	Soilemissivity = input->TemperatureCalculationParams["Soilemissivity"];

	// urban parameters
	//UrbanCanyon is flag (1 or 0) from HydroPlusConfig.xml for Uban21 to Urban24, where 1 = simulate urban canyon geometry for radiation balance, 0 = neglect urban canyon geometry
	UrbanCanyon = input->TemperatureCalculationParams["UrbanCanyon"];

	//Urban21a1, Urban21a2, Urban21a3 are terms to compute ground heat storage based on net radiation and Eq 3 in Grimmond and Oke (1999) objective hysteresis model (OHM)
	Urban21a1 = input->TemperatureCalculationParams["Urban21a1"];
	Urban21a2 = input->TemperatureCalculationParams["Urban21a2"];
	Urban21a3 = input->TemperatureCalculationParams["Urban21a3"];

	//Urban21height (m) is average height of surface feature for roughness in turbulence exchange calculations
	Urban21height = input->TemperatureCalculationParams["Urban21height"];
	//Urban21roadwidth (m) is average width of surface feature for urban canyon calculations
	Urban21roadwidth = input->TemperatureCalculationParams["Urban21roadwidth"];
	//Urban21albedo (fraction) is albedo of surface feature for reflecting shortwave radiation
	Urban21albedo = input->TemperatureCalculationParams["Urban21albedo"];
	//Urban21emissivity (fraction) is emissivity of surface feature for emitting longwave radiation
	Urban21emissivity = input->TemperatureCalculationParams["Urban21emissivity"];
	//Urban21buildingP (fraction) is area of surface feature with buildings vs flat impervious cover
	Urban21buildingP = input->TemperatureCalculationParams["Urban21buildingP"];
	//Urban21depression (m) is depth of storage for ponded water in surface feature; should be the same as ImperviousDepressionStorage_mm given in DataFolder
	Urban21depression = input->RefParams["ImperviousDepressionStorage_mm"] * 1000;

	//Urban22a1, Urban22a2, Urban22a3 are terms to compute ground heat storage based on net radiation and Eq 3 in Grimmond and Oke (1999) objective hysteresis model (OHM)
	Urban22a1 = input->TemperatureCalculationParams["Urban22a1"];
	Urban22a2 = input->TemperatureCalculationParams["Urban22a2"];
	Urban22a3 = input->TemperatureCalculationParams["Urban22a3"];

	//Urban22height (m) is average height of surface feature for roughness in turbulence exchange calculations
	Urban22height = input->TemperatureCalculationParams["Urban22height"];
	//Urban22roadwidth (m) is average width of surface feature for urban canyon calculations
	Urban22roadwidth = input->TemperatureCalculationParams["Urban22roadwidth"];
	//Urban22albedo (fraction) is albedo of surface feature for reflecting shortwave radiation
	Urban22albedo = input->TemperatureCalculationParams["Urban22albedo"];
	//Urban22emissivity (fraction) is emissivity of surface feature for emitting longwave radiation
	Urban22emissivity = input->TemperatureCalculationParams["Urban22emissivity"];
	//Urban22buildingP (fraction) is area of surface feature with buildings vs flat impervious cover
	Urban22buildingP = input->TemperatureCalculationParams["Urban22buildingP"];
	//Urban22depression (m) is depth of storage for ponded water in surface feature; should be the same as ImperviousDepressionStorage_mm given in DataFolder
	Urban22depression = input->RefParams["ImperviousDepressionStorage_mm"] * 1000;


	//Urban23a1, Urban23a2, Urban23a3 are terms to compute ground heat storage based on net radiation and Eq 3 in Grimmond and Oke (1999) objective hysteresis model (OHM)
	Urban23a1 = input->TemperatureCalculationParams["Urban23a1"];
	Urban23a2 = input->TemperatureCalculationParams["Urban23a2"];
	Urban23a3 = input->TemperatureCalculationParams["Urban23a3"];

	//Urban23height (m) is average height of surface feature for roughness in turbulence exchange calculations
	Urban23height = input->TemperatureCalculationParams["Urban23height"];
	//Urban23roadwidth (m) is average width of surface feature for urban canyon calculations
	Urban23roadwidth = input->TemperatureCalculationParams["Urban23roadwidth"];
	//Urban23albedo (fraction) is albedo of surface feature for reflecting shortwave radiation
	Urban23albedo = input->TemperatureCalculationParams["Urban23albedo"];
	//Urban23emissivity (fraction) is emissivity of surface feature for emitting longwave radiation
	Urban23emissivity = input->TemperatureCalculationParams["Urban23emissivity"];
	//Urban23buildingP (fraction) is area of surface feature with buildings vs flat impervious cover
	Urban23buildingP = input->TemperatureCalculationParams["Urban23buildingP"];
	//Urban23depression (m) is depth of storage for ponded water in surface feature; should be the same as ImperviousDepressionStorage_mm given in DataFolder
	Urban23depression = input->RefParams["ImperviousDepressionStorage_mm"] * 1000;

	//Urban24a1, Urban24a2, Urban24a3 are terms to compute ground heat storage based on net radiation and Eq 3 in Grimmond and Oke (1999) objective hysteresis model (OHM)
	Urban24a1 = input->TemperatureCalculationParams["Urban24a1"];
	Urban24a2 = input->TemperatureCalculationParams["Urban24a2"];
	Urban24a3 = input->TemperatureCalculationParams["Urban24a3"];

	//Urban24height (m) is average height of surface feature for roughness in turbulence exchange calculations and urban canyon calculations
	Urban24height = input->TemperatureCalculationParams["Urban24height"];
	//Urban24roadwidth (m) is average width of surface feature for urban canyon calculations
	Urban24roadwidth = input->TemperatureCalculationParams["Urban24roadwidth"];
	//Urban24albedo (fraction) is albedo of surface for reflecting shortwave radiation
	Urban24albedo = input->TemperatureCalculationParams["Urban24albedo"];
	//Urban24emissivity (fraction) is emissivity of surface for emitting longwave radiation
	Urban24emissivity = input->TemperatureCalculationParams["Urban24emissivity"];
	//Urban24buildingP (fraction) is area of surface feature with buildings vs flat impervious cover
	Urban24buildingP = input->TemperatureCalculationParams["Urban24buildingP"];
	//Urban24depression (m) is depth of storage for ponded water in surface feature; should be the same as ImperviousDepressionStorage_mm given in DataFolder
	Urban24depression = input->RefParams["ImperviousDepressionStorage_mm"] * 1000;

	//tree parameters
	//Treea1, Treea2, Treea3 are terms to compute ground heat storage based on net radiation and Eq 3 in Grimmond and Oke (1999) objective hysteresis model (OHM)
	Treea1 = input->TemperatureCalculationParams["Treea1"];
	Treea2 = input->TemperatureCalculationParams["Treea2"];
	Treea3 = input->TemperatureCalculationParams["Treea3"];

	//Treeheight (m) is average height of feature for roughness in turbulence exchange calculations
	Treeheight = input->TemperatureCalculationParams["Treeheight"];
	//Treealbedo (fraction) is albedo of feature for reflecting shortwave radiation
	Treealbedo = input->TemperatureCalculationParams["Treealbedo"];
	//Treeemissivity (fraction) is emissivity of feature  for emitting longwave radiation
	Treeemissivity = input->TemperatureCalculationParams["Treeemissivity"];
	//TreeSL (m) is converted from mm depth
	TreeSL = input->RefParams["TreeLeafStorage_mm"] * 1/1000;

	// short vegetation parameters
	//Shorta1, Shorta2, Shorta3 are terms to compute ground heat storage based on net radiation and Eq 3 in Grimmond and Oke (1999) objective hysteresis model (OHM)
	Shorta1 = input->TemperatureCalculationParams["ShortVegetationa1"];
	Shorta2 = input->TemperatureCalculationParams["ShortVegetationa2"];
	Shorta3 = input->TemperatureCalculationParams["ShortVegetationa3"];

	//Shortheight (m) is average height of feature for roughness in turbulence exchange calculations
	Shortheight = input->TemperatureCalculationParams["ShortVegetationheight"];
	//Shortalbedo (fraction) is albedo of feature for reflecting shortwave radiation
	Shortalbedo = input->TemperatureCalculationParams["ShortVegetationalbedo"];
	//Shortemissivity (fraction) is emissivity of feature for emitting longwave radiation
	Shortemissivity = input->TemperatureCalculationParams["ShortVegetationemissivity"];
	//cte 20210715 ShortVegetationMaxInterception was 50% of TreeMaxInterception, which should likely be set equal and instead change LAI to include RefParam BAI and LAI for tree and short
	//ShortSL (m) is converted from mm depth and is 50% of TreeSL
	ShortSL = input->RefParams["TreeLeafStorage_mm"] * 1 / 1000 * 0.5;

	// water parameters
	//Watera1, Watera2, Watera3 are terms to compute ground heat storage based on net radiation and Eq 3 in Grimmond and Oke (1999) objective hysteresis model (OHM)
	Watera1 = input->TemperatureCalculationParams["Watera1"];
	Watera2 = input->TemperatureCalculationParams["Watera2"];
	Watera3 = input->TemperatureCalculationParams["Watera3"];

	//Waterheight  (m) is average height of feature for roughness in turbulence exchange calculations
	Waterheight = input->TemperatureCalculationParams["Waterheight"];
	//Wateralbedo  (fraction) is albedo of feature for reflecting shortwave radiation
	Wateralbedo = input->TemperatureCalculationParams["Wateralbedo"];
	//Wateremissivity (fraction) is emissivity of surface feature for emitting longwave radiation
	Wateremissivity = input->TemperatureCalculationParams["Wateremissivity"];

}

void HeatFluxCal::assignUrbanParam(int land)
{

	if (land == 21)
	{
		Urbanalbedo = Urban21albedo;
		Urbanheight = Urban21height;
		Urbanroadwidth = Urban21roadwidth;
		Urbanemissivity = Urban21emissivity;
		UrbanDepression = Urban21depression;
		Urbana1 = Urban21a1;
		Urbana2 = Urban21a2;
		Urbana3 = Urban21a3;
		UrbanbuildingP = Urban21buildingP;
	}
	if (land == 22)
	{
		Urbanalbedo = Urban22albedo;
		Urbanheight = Urban22height;
		Urbanroadwidth = Urban22roadwidth;
		Urbanemissivity = Urban22emissivity;
		UrbanDepression = Urban22depression;
		Urbana1 = Urban22a1;
		Urbana2 = Urban22a2;
		Urbana3 = Urban22a3;
		UrbanbuildingP = Urban22buildingP;

	}
	if (land == 23)
	{
		Urbanalbedo = Urban23albedo;
		Urbanheight = Urban23height;
		Urbanroadwidth = Urban23roadwidth;
		Urbanemissivity = Urban23emissivity;
		UrbanDepression = Urban23depression;
		Urbana1 = Urban23a1;
		Urbana2 = Urban23a2;;
		Urbana3 = Urban23a3;
		UrbanbuildingP = Urban23buildingP;
	}
	if (land == 24)
	{
		Urbanalbedo = Urban24albedo;
		Urbanheight = Urban24height;
		Urbanroadwidth = Urban24roadwidth;
		Urbanemissivity = Urban24emissivity;
		UrbanDepression = Urban24depression;
		Urbana1 = Urban24a1;
		Urbana2 = Urban24a2;;
		Urbana3 = Urban24a3;
		UrbanbuildingP = Urban24buildingP;
	}

	//Parameterize impervious surfaces outside of NLCD 21 to 24
	if (land < 21 || land > 24)
	{
		Urbanalbedo = Urban21albedo;
		Urbanheight = Urban21height;
		Urbanroadwidth = Urban21roadwidth;
		Urbanemissivity = Urban21emissivity;
		UrbanDepression = Urban21depression;
		Urbana1 = Urban21a1;
		Urbana2 = Urban21a2;
		Urbana3 = Urban21a3;
		UrbanbuildingP = Urban21buildingP;
	}

	Urbanhwr = Urbanheight / Urbanroadwidth;
	//RVF = road view factor, Eq 9 from Masson (2000), A Physically based scheme for the urban energy budget in atmospheric models, Boundary Layer Meteorology, 94: 357-397.
	UrbanRVF = sqrt(Urbanhwr*Urbanhwr + 1) - Urbanhwr;
	//WVF = wall view factor, Eq 10 from Masson (2000), A Physically based scheme for the urban energy budget in atmospheric models, Boundary Layer Meteorology, 94: 357-397.
	UrbanWVF = 0.5*(Urbanhwr + 1 - sqrt(Urbanhwr + 1)) / Urbanhwr;
}

void HeatFluxCal::CalaveHeight(double treeP, double shortVegP, double soilP, double waterP, double imperP, int land)
{
	//Heights are based on equivalent roughness height, from Table 4.1 of R. Stully Meteorology book, ISBN 0-534-37214-7; 
	//Impervious cover (IC) height assumes landscape of 100% IC has obstructions. To simulate IC as a sea, create new Urbanheight = 0.001 m.

	//aveHeight is only a function of treeP and imperP if they cover 100% or more of the area
	if (treeP + imperP > 1) {
		//create extra version of variables treeP and imperP to temporarily modify them for single aveHeight calculation
		double treeP_extra = treeP;
		double imperP_extra = imperP;
		imperP_extra = 1 - treeP_extra;
		aveHeight = treeP_extra * Treeheight + imperP_extra * Urbanheight;
	} else {
		aveHeight = treeP * Treeheight + shortVegP * Shortheight + soilP * Soilheight + waterP * Waterheight + imperP * Urbanheight;
	}
	height = Urbanheight;
	if (land == 11 || land == 12)
	{
		height = 0.1;
	}
	if (land > 25)
	{
		if (treeP > 0)
			height = Treeheight;
		else
			height = Shortheight;
	}
	if (aveHeight < 0.001) 
		aveHeight = 0.001;
}


// just need to calculate once for each time step
void HeatFluxCal::VariableForAllfolders(int ts, Inputs *input)
{
	//LAI treated as time varying during transition dates, defined in Config XML as LeafOnDay and LeafOffDay
	//TreeLAI is LAI + BAI from inputs, drawing from Inputs.cpp RefParams["TreeLAI"] + RefParams["TreeBAI"] - rcc 10/28/2020
	TreeLAI= input->TreeLAI[ts];
	ShortLAI = input->ShortVegLAI[ts];
	downLW = input->DownLongWaveRadiation[ts];
	upLW = input->UpLongWaveRadiation[ts];
	DiffSWR = input->DiffuseShortWaveRadiation[ts];
	//Tair_localCell_K=inp->RW->getTair(ts)+273.15;
	//Tdew_K=inp->RW->getTdew(ts)+273.15;
	windsp = input->WindSp_mps[ts];
	//dirSW->calZenithAzimuth(ts);
}

void HeatFluxCal::assignRefTaTd(Inputs *input, int ts)
{
	Tair_localCell_K = (input->Tair[ts] - 32)*5.0 / 9.0 + 273.15;
	Tdew_K = (input->Tdew[ts] - 32)*5.0 / 9.0 + 273.15;

	//Error handling: Tair_localCell_K >= Tdew_K is the presumed condition, but weather data sometimes has rounding errors that violate this condition
	if (Tdew_K >= Tair_localCell_K)
	{
		Tdew_K = Tair_localCell_K;
		std::fstream fs;
		fs.open("InstanceTD=TA.txt", std::fstream::in | std::fstream::out | std::fstream::app);
		fs << "Timestep: " << ts << ",Tdew_K: " << Tdew_K << ",Tair_localCell_K: " << Tair_localCell_K << ",wateronImp: " << wateronImp << ",wateronSoil: " << wateronSoil << ",wateronShort: " << wateronShort << ",wateronTree: " << wateronTree << "\n";
		fs.close();
	} 
}

void HeatFluxCal::calDirSWRad(int folderID, Inputs *input, int timeStep, double &dirSWRadiation)
{
	//Eq 25 in Yang et al. (2013), adjust direct radiation based on topographic slope and aspect and solar angle for that timestep 
	dirSWRadiation = input->DirectShortWaveRadiation[timeStep] * ((sin(input->Slope[folderID]))*cos(input->Aspect[folderID])*sin(input->Zenith[timeStep])*input->CosAzimuth[timeStep] +
		sin(input->Slope[folderID])*sin(input->Aspect[folderID])*sin(input->Zenith[timeStep])*input->SinAzimuth[timeStep] +
		cos(input->Slope[folderID])*cos(input->Zenith[timeStep]));

	if (dirSWRadiation<0)
		dirSWRadiation = 0;
}

//only calculate once for each folder at each timestep, specific for each folder, but not involved in the iterations
void HeatFluxCal::StateCal(DataFolder *folder, int ts, Inputs *input, int folderID)// check curTimestepIndex curTimestepIndexboundaries localcal ??
{
	//Component S/Smax of Eq 22 in Yang et al. (2013)
	//cte 20210715 should this consider BarkAreaIndex, and use terms from DataFolder RefParams BAI and LAI for tree
	//wateronTree is function of TreeSL (tree leaf storage depth) and TreeLAI
	wateronTree = folder->VarDict["treeStoredW"] / TreeSL / TreeLAI;
	//cte 20210715 should this consider BarkAreaIndex, and use terms from DataFolder RefParams BAI and LAI for tree
	//wateronShort is function of ShortSL (short veg leaf storage depth) and ShortLAI
	wateronShort = folder->VarDict["shortVegStoredW"] / ShortSL / ShortLAI;
	//Component D/Dmax of Eq 20 in Yang et al. (2013)
	wateronSoil = 1 - folder->VarVecDict["Storage_unUsed_RootZone_TI_m"][0] / SoilMSD; // available water in the soil for evaporate
	//Component S/Smax of Eq 19 in Yang et al. (2013)
	wateronImp = folder->VarDict["impDepStor_m"] / UrbanDepression;

	folder->VarDict["prevImp000"] = wateronImp;

	int land = input->NLCDLandCoverData[folderID];

	// calculate rural canopy flux resistance
	resist->calr_rural(windsp, TreeLAI, ShortLAI, aveHeight);  

	// calculate flux resistance that depends on water content
	resist->calr_onwater(wateronSoil, TreeLAI, ShortLAI);  

	calDirSWRad(folderID, input, ts, DirSWR);
	totalSWR = DirSWR + DiffSWR;

	double treeP = folder->ParamDict["TreeP"];
	double soilP = folder->ParamDict["SoilP"];
	double shortVegP = folder->ParamDict["ShortVegP"];
	double imperP = folder->ParamDict["ImperP"];
	double landc = folder->ParamDict["LandC"];
	double waterP = folder->ParamDict["WaterP"];
	if (folder->ParamDict["ImperP"] > 0.0001)
	{

		//Urban canyon longwave radiation is adjusted for using method of Masson (2000), as explained by Yang et al. (2013) 
		//Algorithm based on Town Energy Balance model by: Masson (2000), A Physically based scheme for the urban energy budget in atmospheric models, Boundary Layer Meteorology, 94: 357-397.
		//Algorithm based on Radiation Balance by: Offerle, Grimmond, and Oke (2003), Parameterization of Net All-Wave Radiation for Urban Areas, Journal of Applied Meteorology, 42: 1154-1173
		//Similar to Urban Canopy Model by Kusaka et al. (2001), A simple single-layer urban canopy model for atmospheric models: Comparison with multi-layer and slab models, Boundary Layer Meteorol., 101(3), 329358.
		//Similar to Urban Canopy Model by Lee, S.H., and S.U. Park (2008), A vegetated urban canopy model for meteorological and environmental modelling, Boundary Layer Meteorol., 126(1), 73102.

		//Solar radiation is not adjusted for urban canyon, but this could be implemented using Town Energy Balance model Eqs in section 2.7 of Masson (2000) 

		//Surface temperature can be derived from Eq 27 of Yang et al. (2013) using observed downLW and upLW, which is based on Eq 13 of Offerle et al (2003)
		//Tsurface = [upLW - (1-UrbanEmissivity)*downLW] / (UrbanEmissivity * Stefan-Boltzmann_Constant), from Eq 27 of Yang et al. (2013)

		if ((land > 20) && (land < 25) && (UrbanCanyon == 1))
		{
			resist->calr_urban(windsp, TreeLAI, ShortLAI, aveHeight, Urbanhwr, height);
			//RVF = road view factor, Eq 9 of Masson (2000), Town Energy Balance model
			double RVF = UrbanRVF;
			//WVF = wall view factor, Eq 10 of Masson (2000), Town Energy Balance model
			double WVF = UrbanWVF;
			//longwave downwelling radiation
			double L = downLW;
			//urban emissivity
			double s = Urbanemissivity;  

			//siT = sigma*Tsurface^4, where sigma is Stefan-Boltzmann constant, used below in Eq 11 and 12 by Masson (2000)
			//The term sigma*Tsurface^4 is derived from Eq 26 and 28 of Yang et al. (2013) given we have no Tsurface value
			//Eq 26 of Yang et al. (2013) is from Eq 8 of Offerle et al. (2003)
			//Eq 28 of Yang et al. (2013) is from Eq 16 of Offerle et al. (2003); Offerle et al. (2003) explains the assumption of Tsurface = Tair at night
			//Eq 26 rearranged is sigma*Tair^4 = downLW / Urbanemissivity 
			//Eq 28 rearranged is sigma*Tsurface^4 = downLW / Urbanemissivity * (c * totalSWR * (1-Urbanalbedo)) / Urbanemissivity
			//c = 0.08 parameter given by Yang et al. (2013), but also given w/ Eq 15 of Offerle et al. (2003), and attributed to approximated from Holtslag and van Ulden (1983) https://doi.org/10.1175/1520-0450(1983)022<0517:ASSFDE>2.0.CO;2
			double siT = downLW / Urbanemissivity + (0.08 * totalSWR * (1 - Urbanalbedo)) / Urbanemissivity;   
			//Initially, this equation was coded uisng upLW rather than downLW / Urbanemissivity
			//double siT = upLW + 0.08 / Urbanemissivity * totalSWR * (1 - Urbanalbedo);   

			//longwave radiation absorbed by snow-free road, from Eq 11 of Masson (2000), A Physically based scheme for the urban energy budget in atmospheric models, Boundary Layer Meteorology, 94: 357-397.
			double Lr = s * RVF*L - s * siT + s * s*(1 - RVF)*siT +
				s * (1 - s)*(1 - RVF)*WVF*L + s * s*(1 - s)*(1 - RVF)*(1 - 2 * WVF)*siT +
				s * (1 - s)*(1 - RVF)*WVF*s*siT;
			//longwave radiation on wall from from Eq 12 of Masson (2000), A Physically based scheme for the urban energy budget in atmospheric models, Boundary Layer Meteorology, 94: 357-397.
			double Lw = s * WVF*L - s * siT + s * WVF*s*siT
				+ s * s*(1 - 2 * WVF)*siT + s * (1 - s)*RVF*WVF*L +
				s * (1 - s)*WVF*(1 - 2 * WVF)*L + s * s*(1 - s)*(1 - 2 * WVF)*(1 - 2 * WVF)*siT +
				s * s*(1 - s)*WVF*(1 - RVF)*siT + s * (1 - s)*WVF*(1 - 2 * WVF)*s*siT;

			//Defined in Eq 24 in Yang, Endreny, Nowak (2013)
			ImpNR_W_p_m2 = totalSWR * (1 - Urbanalbedo) + (1 - UrbanbuildingP)*(Lr + 2 * Urbanhwr*Lw) + UrbanbuildingP * (L - s * siT);
		}
		else
			//Defined in Eq 24 in Yang, Endreny, Nowak (2013)
			ImpNR_W_p_m2 = totalSWR * (1 - Urbanalbedo) + Urbanemissivity * downLW - Urbanemissivity * upLW;

		//Defined in Eq 29 in Yang, Endreny, Nowak (2013) 
		//Parameters a1, a2, a3 originally from concrete in Table 4 of Grimmond, C., and T. R. Oke (1999a), Heat storage in urban areas, J. Appl. Meteorol., 38(7), 922940.
		ImpDeltaQ_W_p_m2 = Urbana1 * ImpNR_W_p_m2 + Urbana2 * (ImpNR_W_p_m2 - folder->VarDict["ImpNR_W_p_m2"]) + Urbana3;
		ImpC = pow(wateronImp, 2.0 / 3.0);

	}
	else
	{
		ImpNR_W_p_m2 = 0; ImpDeltaQ_W_p_m2 = 0; ImpC = 0;
	}

	//Tree part
	if (treeP > 0.0001)
	{
		//Defined in Eq 24 in Yang, Endreny, Nowak (2013) 
		TreeNR_W_p_m2 = totalSWR * (1 - Treealbedo) + Treeemissivity * downLW - Treeemissivity * upLW;
		//Defined in Eq 29 in Yang, Endreny, Nowak (2013) 
		//Parameters a1, a2, a3 originally modified from mixed forest in Table 4 of Grimmond, C., and T. R. Oke (1999a), Heat storage in urban areas, J. Appl. Meteorol., 38(7), 922940.
		TreeDeltaQ_W_p_m2 = Treea1 * TreeNR_W_p_m2 + Treea2 * (TreeNR_W_p_m2 - folder->VarDict["TreeNR_W_p_m2"]) + Treea3;
		//Defined in Eq 22 in Yang, Endreny, Nowak (2013)
		TreeCE = pow(wateronTree, (2.0 / 3.0)) / resist->rtree;
		//Defined in Eq 23 in Yang, Endreny, Nowak (2013) 
		TreeCT = (1 - pow(wateronTree, 2.0 / 3.0)) / (resist->rtree + resist->rctree);
		TreeC = TreeCT + TreeCE;
	}
	else
	{
		TreeNR_W_p_m2 = 0; TreeDeltaQ_W_p_m2 = 0; TreeCE = 0; TreeCT = 0; TreeC = 0;
	}

	//Short vegetation part
	if (shortVegP > 0.0001)
	{
		//Defined in Eq 24 in Yang, Endreny, Nowak (2013) 
		ShortNR_W_p_m2 = totalSWR * (1 - Shortalbedo) + Shortemissivity * downLW - Shortemissivity * upLW;
		//Defined in Eq 29 in Yang, Endreny, Nowak (2013) 
		//Parameters a1, a2, a3 originally from short grass in Table 4 of Grimmond, C., and T. R. Oke (1999a), Heat storage in urban areas, J. Appl. Meteorol., 38(7), 922940.
		ShortDeltaQ_W_p_m2 = Shorta1 * ShortNR_W_p_m2 + Shorta2 * (ShortNR_W_p_m2 - folder->VarDict["ShortVegNR_W_p_m2"]) + Shorta3;
		//Defined in Eq 22 in Yang, Endreny, Nowak (2013) 
		ShortCE = pow(wateronShort, 2.0 / 3.0) / resist->rShortVeg;
		//Defined in Eq 23 in Yang, Endreny, Nowak (2013) 
		ShortCT = (1 - pow(wateronShort, 2.0 / 3.0)) / (resist->rShortVeg + resist->rcshort);
		ShortC = ShortCE + ShortCT;
	}
	else
	{
		ShortNR_W_p_m2 = 0; ShortDeltaQ_W_p_m2 = 0; ShortCE = 0; ShortCT = 0; ShortC = 0;
	}

	// soil part
	if (soilP > 0.0001)
	{
		//Defined in Eq 24 in Yang, Endreny, Nowak (2013) 
		SoilNR_W_p_m2 = 0.92*totalSWR*(1 - Soilalbedo) + 0.95*downLW - 0.95*upLW - 0.05*downLW;
		//Defined in Eq 29 in Yang, Endreny, Nowak (2013) 
		//Parameters a1, a2, a3 originally from bare soil in Table 4 of Grimmond, C., and T. R. Oke (1999a), Heat storage in urban areas, J. Appl. Meteorol., 38(7), 922940.
		SoilDeltaQ_W_p_m2 = Soila1 * SoilNR_W_p_m2 + Soila2 * (SoilNR_W_p_m2 - folder->VarDict["SoilNR_W_p_m2"]) + Soila3;
		//Defined in Eq 20 in Yang, Endreny, Nowak (2013) 
		SoilC = wateronSoil / (resist->rground + resist->rg);
	}  
	else
	{
		SoilNR_W_p_m2 = 0; SoilDeltaQ_W_p_m2 = 0; SoilC = 0;
	}

	// water part
	if (waterP > 0.0001)
	{
		WaterNR_W_p_m2 = 0.92*totalSWR*(1 - Wateralbedo) + 0.95*downLW - 0.95*upLW - 0.05*downLW;
		WaterDeltaQ_W_p_m2 = Watera1 * WaterNR_W_p_m2 + Watera2 * (WaterNR_W_p_m2 - folder->VarDict["WaterNR_W_p_m2"]) + Watera3;
		WaterC = 1.0 / resist->rground;
	}
	else
	{
		WaterNR_W_p_m2 = 0; WaterDeltaQ_W_p_m2 = 0; WaterC = 0;
	}
}

//Calculate Tair_mesoScale_K and AbsHumidity_mesoScale_kg_p_m3, mesoscale air temperature and absolute humidity, 
//	Algorithm based on first estimating local air temperature and absolute humidity as function of land cover types
void HeatFluxCal::MesoCal(DataFolder *folder, int ts)
{
	double treeP = folder->ParamDict["TreeP"];
	double soilP = folder->ParamDict["SoilP"];
	double shortVegP = folder->ParamDict["ShortVegP"];
	double imperP = folder->ParamDict["ImperP"];
	double waterP = folder->ParamDict["WaterP"];

	//Absolute humidity (AH or AbsHumidity_kg_p_m3; kg/m^3) is function of vapor pressure, from Eq 18 of Yang, Endreny, and Nowak (2013), attributed to Lawrence (2005) units T = Kelvin, AH = kg/m^3
	//Absolute humidity (rho_v) and vapor pressure also defined in Eq 5.2 and 5.5 of Stull, 2000. Meteorology for Scientists and Engineers, 2nd Edition, Brooks Cole.
	//		Absolute humidity Eq 5.5 Stull (2000): rho_v (kg/m^3) = e / (Rv * T), where e = vapor pressure Pa, Rv = gas constant for water vapor = 461 J/(K*kg), T is temperature K
	//		Vapor pressure    Eq 5.2 Stull (2000): e (Pa) = 611 * exp[ [17.27*(T-273.16)] / (T-35.86)], where T in K, e0=611 Pa; this is Tetens' formula adjusted from kPa to Pa
	//		Coefficient 1.324 in Eq 18 is derived as C = e0 / Rv = 611/461. 
	//AbsHumiditySat_kg_p_m3, saturated absolute humidity (kg/m^3), Eq 18 from Yang et al. (2013)
	AbsHumiditySat_kg_p_m3 = 1.324 / Tair_localCell_K * exp(17.27*(Tair_localCell_K - 273.15) / (Tair_localCell_K - 35.85));    // saturated vapor pressure converted to absolute humidity Kg/m^3) at Tair_localCell_K (K)														   
	//AbsHumidityAct_kg_p_m3, actual absolute humidity (kg/m^3), Eq 18 from Yang et al. (2013)
	AbsHumidityAct_kg_p_m3 = 1.324 / Tdew_K * exp(17.27*(Tdew_K - 273.15) / (Tdew_K - 35.85));  // actual absolute humidity, units Kg/m^3 at Tdew_K (K), from Stull
	//Gradient of saturated absolute humidity, given above Eq 18 in Yang et al (2013); attributed by Yang et al. (2013) to Lawrence, 2005.
	AbsHumidityGradient = -AbsHumiditySat_kg_p_m3 * (-4169.56 + Tair_localCell_K)*(-0.308 + Tair_localCell_K) / Tair_localCell_K / (-35.85 + Tair_localCell_K) / (-35.85 + Tair_localCell_K);

	//if there is impervious cover threshold
	if (imperP > 0.001)
	{
		//Combine Eq 9 and 19 from Yang, Endreny, Nowak (2013), note rimpH/rimpLE = 1 & ImpC = (S/Smax)^(2/3), expand, combine H_total_W_p_m2 terms on LHS, and isolate H_total_W_p_m2 
		//1. ImpH_W_p_m2 = ImpNR_W_p_m2 - ImpDeltaQ_W_p_m2 - ImpLE_W_p_m2 = ImpNR_W_p_m2 - ImpDeltaQ_W_p_m2 - [(lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / resist->rImp + lamda * AbsHumidityGradient*ImpH_W_p_m2 / rou / Cp)*ImpC]
		//2. ImpH_W_p_m2 = ImpNR_W_p_m2 - ImpDeltaQ_W_p_m2 - ImpC * lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / resist->rImp - ImpC * lamda * AbsHumidityGradient*ImpH_W_p_m2 / rou / Cp
		//3. ImpH_W_p_m2 + ImpC * lamda * AbsHumidityGradient*ImpH_W_p_m2 / rou / Cp = ImpNR_W_p_m2 - ImpDeltaQ_W_p_m2 - ImpC * lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / resist->rImp 
		//4. ImpH_W_p_m2 * (ImpC * lamda * AbsHumidityGradient / rou / Cp + 1) = ImpNR_W_p_m2 - ImpDeltaQ_W_p_m2 - ImpC * lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / resist->rImp 
		//5. ImpH_W_p_m2 = (ImpNR_W_p_m2 - ImpDeltaQ_W_p_m2 - ImpC * lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / resist->rImp) / (ImpC * lamda * AbsHumidityGradient / rou / Cp + 1)

		//Impervious cover sensible heat (W/m2)
		ImpH_W_p_m2 = (ImpNR_W_p_m2 - ImpDeltaQ_W_p_m2 - ImpC * lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / resist->rimp) / (ImpC*lamda*AbsHumidityGradient / Cp / rou + 1);
		//Impervious cover latent heat (W/m2)
		ImpLE_W_p_m2 = (lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / resist->rimp + lamda * AbsHumidityGradient*ImpH_W_p_m2 / rou / Cp)*ImpC;
	}
	else
	{
		//Set values to zero if there is no cover
		ImpH_W_p_m2 = 0.0; ImpLE_W_p_m2 = 0.0;
	}

	//if there is tree cover threshold
	if (treeP > 0.0001)
	{
		//Combine Eq 9 and 22 and 23 from Yang, Endreny, Nowak (2013), note rTreeH/rTreeLE = 1 & TreeCE = (S/Smax)^(2/3)/rTree TreeCT = [1-(S/Smax)^(2/3)], expand, combine H_total_W_p_m2 terms on LHS, and isolate H_total_W_p_m2 
		//1. TreeH_W_p_m2 = TreeNR_W_p_m2 - TreeDeltaQ_W_p_m2 - TreeLE_W_p_m2 = TreeNR_W_p_m2 - TreeDeltaQ_W_p_m2 - [(lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / resist->rTree + lamda * AbsHumidityGradient*TreeH_W_p_m2 / rou / Cp)*TreeCE] - [(lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / (resist->rTree + resist->rcTree) + lamda * AbsHumidityGradient*TreeH_W_p_m2 / rou / Cp)*TreeCT]
		//2. TreeH_W_p_m2 = TreeNR_W_p_m2 - TreeDeltaQ_W_p_m2 - TreeCE * lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / resist->rTree - TreeCE* lamda * AbsHumidityGradient*TreeH_W_p_m2 / rou / Cp - TreeCT * lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / (resist->rTree + resist->rcTree) - TreeCT* lamda * AbsHumidityGradient*TreeH_W_p_m2 / rou / Cp
		//3. TreeH_W_p_m2 + TreeCE* lamda * AbsHumidityGradient*TreeH_W_p_m2 / rou / Cp + TreeCT* lamda * AbsHumidityGradient*TreeH_W_p_m2 / rou / Cp = TreeNR_W_p_m2 - TreeDeltaQ_W_p_m2 - TreeCE * lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / resist->rTree - TreeCT * lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / (resist->rTree + resist->rcTree)
		//4. TreeH_W_p_m2 * (TreeCE* lamda * AbsHumidityGradient / rou / Cp + TreeCT* lamda * AbsHumidityGradient / rou / Cp + 1) = TreeNR_W_p_m2 - TreeDeltaQ_W_p_m2 - TreeCE * lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / resist->rTree - TreeCT * lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / (resist->rTree + resist->rcTree)
		//5. TreeH_W_p_m2 = (TreeNR_W_p_m2 - TreeDeltaQ_W_p_m2 - TreeCE * lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / resist->rTree - TreeCT * lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / (resist->rTree + resist->rcTree)) / (TreeCE* lamda * AbsHumidityGradient / rou / Cp + TreeCT* lamda * AbsHumidityGradient / rou / Cp + 1)
		//N.B. This form works with resistance is removed from TreeCT of Eq 22, TreeH_W_p_m2 = (TreeNR_W_p_m2 - TreeDeltaQ_W_p_m2 - TreeCE * lamda * (AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / resist->rTree - TreeCT * lamda * (AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / (resist->rTree + resist->rcTree)) / (TreeCE * lamda * AbsHumidityGradient / rou / Cp + TreeCT * lamda * AbsHumidityGradient / rou / Cp + 1);

		//Tree sensible heat (W/m2)
		TreeH_W_p_m2 = (TreeNR_W_p_m2 - TreeDeltaQ_W_p_m2 - TreeC * lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3)) / (TreeC*resist->rtree*lamda*AbsHumidityGradient / Cp / rou + 1);
		//Tree latent energy from evaporation (W/m2)
		TreeLEE_W_p_m2 = (lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) + lamda * AbsHumidityGradient*TreeH_W_p_m2*resist->rtree / rou / Cp)*TreeCE;
		//Tree latent energy from transpiration (W/m2)
		TreeLET_W_p_m2 = (lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) + lamda * AbsHumidityGradient*TreeH_W_p_m2*resist->rtree / rou / Cp)*TreeCT;
		//Tree latent energy total (W/m2)
		TreeLE_W_p_m2 = TreeLEE_W_p_m2 + TreeLET_W_p_m2;
	}
	else
	{
		//Set values to zero if there is no cover
		TreeH_W_p_m2 = 0.0; TreeLEE_W_p_m2 = 0.0; TreeLET_W_p_m2 = 0.0; TreeLE_W_p_m2 = 0.0;
	}

	//if there is short veg cover threshold
	if (shortVegP > 0.0001)
	{
		//Combine Eq 9 and 22 from Yang, Endreny, Nowak (2013), note rShortH/rShortLE = 1 & ShortCE = (S/Smax)^(2/3)/rShortVeg ShortCT = [1-(S/Smax)^(2/3)], expand, combine H_total_W_p_m2 terms on LHS, and isolate H_total_W_p_m2 
		//1. ShortH_W_p_m2 = ShortNR_W_p_m2 - ShortDeltaQ_W_p_m2 - ShortLE_W_p_m2 = ShortNR_W_p_m2 - ShortDeltaQ_W_p_m2 - [(lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / resist->rShortVeg + lamda * AbsHumidityGradient*ShortH_W_p_m2 / rou / Cp)*ShortCE] - [(lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / (resist->rShortVeg + resist->rcShortVeg) + lamda * AbsHumidityGradient*ShortH_W_p_m2 / rou / Cp)*ShortCT]
		//2. ShortH_W_p_m2 = ShortNR_W_p_m2 - ShortDeltaQ_W_p_m2 - ShortCE * lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / resist->rShortVeg - ShortCE* lamda * AbsHumidityGradient*ShortH_W_p_m2 / rou / Cp - ShortCT * lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / (resist->rShortVeg + resist->rcShortVeg) - ShortCT* lamda * AbsHumidityGradient*ShortH_W_p_m2 / rou / Cp
		//3. ShortH_W_p_m2 + ShortCE* lamda * AbsHumidityGradient*ShortH_W_p_m2 / rou / Cp + ShortCT* lamda * AbsHumidityGradient*ShortH_W_p_m2 / rou / Cp = ShortNR_W_p_m2 - ShortDeltaQ_W_p_m2 - ShortCE * lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / resist->rShortVeg - ShortCT * lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / (resist->rShortVeg + resist->rcShortVeg)
		//4. ShortH_W_p_m2 * (ShortCE* lamda * AbsHumidityGradient / rou / Cp + ShortCT* lamda * AbsHumidityGradient / rou / Cp + 1) = ShortNR_W_p_m2 - ShortDeltaQ_W_p_m2 - ShortCE * lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / resist->rShortVeg - ShortCT * lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / (resist->rShortVeg + resist->rcShortVeg)
		//5. ShortH_W_p_m2 = (ShortNR_W_p_m2 - ShortDeltaQ_W_p_m2 - ShortCE * lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / resist->rShortVeg - ShortCT * lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / (resist->rShortVeg + resist->rcShortVeg)) / (ShortCE* lamda * AbsHumidityGradient / rou / Cp + ShortCT* lamda * AbsHumidityGradient / rou / Cp + 1)
		//N.B. This form works with resistance is removed from TreeCT of Eq 22, ShortH_W_p_m2 = (ShortNR_W_p_m2 - ShortDeltaQ_W_p_m2 - ShortCE * lamda * (AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / resist->rShortVeg - ShortCT * lamda * (AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / (resist->rShortVeg + resist->rcShortVeg)) / (ShortCE * lamda * AbsHumidityGradient / rou / Cp + ShortCT * lamda * AbsHumidityGradient / rou / Cp + 1); 

		//Short veg sensible heat (W/m2)
		ShortH_W_p_m2 = (ShortNR_W_p_m2 - ShortDeltaQ_W_p_m2 - ShortC * lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3)) / (ShortC*resist->rShortVeg*lamda*AbsHumidityGradient / Cp / rou + 1);
		//Short veg latent energy from evaporation (W/m2)
		ShortLEE_W_p_m2 = (lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) + lamda * AbsHumidityGradient*ShortH_W_p_m2*resist->rShortVeg / rou / Cp)*ShortCE;
		//Short veg latent energy from transpiration (W/m2)
		ShortLET_W_p_m2 = (lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) + lamda * AbsHumidityGradient*ShortH_W_p_m2*resist->rShortVeg / rou / Cp)*ShortCT;
		//Short veg latent energy total (W/m2)
		ShortLE_W_p_m2 = ShortLEE_W_p_m2 + ShortLET_W_p_m2;
	}
	else
	{
		//Set values to zero if there is no cover
		ShortH_W_p_m2 = 0.0; ShortLEE_W_p_m2 = 0.0; ShortLET_W_p_m2 = 0.0; ShortLE_W_p_m2 = 0.0;
	}

	//if there is soil cover threshold
	if (soilP > 0.0001)
	{
		//Combine Eq 9 and 20 from Yang, Endreny, Nowak (2013), note rSoilH/rSoilLE = 1 & SoilC = (1-D/Dmax) = wateronSoil, expand, combine H_total_W_p_m2 terms on LHS, and isolate H_total_W_p_m2 
		//1. SoilH_W_p_m2 = SoilNR_W_p_m2 - SoilDeltaQ_W_p_m2 - SoilLE_W_p_m2 = SoilNR_W_p_m2 - SoilDeltaQ_W_p_m2 - [(lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / (resist->rGround + resist->rG) + lamda * AbsHumidityGradient*SoilH_W_p_m2 / rou / Cp)*SoilC]
		//2. SoilH_W_p_m2 = SoilNR_W_p_m2 - SoilDeltaQ_W_p_m2 - SoilC * lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / (resist->rGround + resist->rG) - SoilC* lamda * AbsHumidityGradient*SoilH_W_p_m2 / rou / Cp
		//3. SoilH_W_p_m2 + SoilC* lamda * AbsHumidityGradient*SoilH_W_p_m2 / rou / Cp = SoilNR_W_p_m2 - SoilDeltaQ_W_p_m2 - SoilC * lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / (resist->rGround + resist->rG) 
		//4. SoilH_W_p_m2 * (SoilC* lamda * AbsHumidityGradient / rou / Cp + 1) = SoilNR_W_p_m2 - SoilDeltaQ_W_p_m2 - SoilC * lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / (resist->rGround + resist->rG)
		//5. SoilH_W_p_m2 = (SoilNR_W_p_m2 - SoilDeltaQ_W_p_m2 - SoilC * lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / (resist->rGround + resist->rG)) / (SoilC * lamda * AbsHumidityGradient / rou / Cp + 1)
		//N.B. This form works with resistance is removed from SoilC, SoilH_W_p_m2 = (SoilNR_W_p_m2 - SoilDeltaQ_W_p_m2 - SoilC * lamda * (AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / (resist->rGround + resist->rG)) / (SoilC * lamda * AbsHumidityGradient / rou / Cp + 1);
		
		//Soil sensible heat (W/m2)
		SoilH_W_p_m2 = (SoilNR_W_p_m2 - SoilDeltaQ_W_p_m2 - SoilC * lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3)) / (SoilC*lamda*AbsHumidityGradient*resist->rground / Cp / rou + 1);
		//Soil latent heat (W/m2)
		SoilLE_W_p_m2 = (lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) + lamda * AbsHumidityGradient*SoilH_W_p_m2*resist->rground / rou / Cp)*SoilC;
	}
	else
	{
		//Set values to zero if there is no cover
		SoilH_W_p_m2 = 0.0; SoilLE_W_p_m2 = 0.0;
	}

	if (waterP > 0.0001)
	{
		//Combine Eq 9 and 20 (modified for water) from Yang, Endreny, Nowak (2013), note rWaterH/rWaterLE = 1 & WaterC = 1, expand, combine H_total_W_p_m2 terms on LHS, and isolate H_total_W_p_m2 
		//1. WaterH_W_p_m2 = WaterNR_W_p_m2 - WaterDeltaQ_W_p_m2 - WaterLE_W_p_m2 = WaterNR_W_p_m2 - WaterDeltaQ_W_p_m2 - [(lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / resist->rGround  + lamda * AbsHumidityGradient*WaterH_W_p_m2 / rou / Cp)*WaterC]
		//2. WaterH_W_p_m2 = WaterNR_W_p_m2 - WaterDeltaQ_W_p_m2 - WaterC * lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / resist->rGround - WaterC* lamda * AbsHumidityGradient*WaterH_W_p_m2 / rou / Cp
		//3. WaterH_W_p_m2 + WaterC* lamda * AbsHumidityGradient*WaterH_W_p_m2 / rou / Cp = WaterNR_W_p_m2 - WaterDeltaQ_W_p_m2 - WaterC * lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / resist->rGround 
		//4. WaterH_W_p_m2 * (WaterC* lamda * AbsHumidityGradient / rou / Cp + 1) = WaterNR_W_p_m2 - WaterDeltaQ_W_p_m2 - WaterC * lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / resist->rGround 
		//5. WaterH_W_p_m2 = (WaterNR_W_p_m2 - WaterDeltaQ_W_p_m2 - WaterC * lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) / resist->rGround) / (WaterC * lamda * AbsHumidityGradient / rou / Cp + 1)

		//Water sensible heat (W/m2)
		WaterH_W_p_m2 = (WaterNR_W_p_m2 - WaterDeltaQ_W_p_m2 - WaterC * lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3)) / (WaterC*lamda*AbsHumidityGradient*resist->rground / Cp / rou + 1);
		//Water latent heat (W/m2)
		WaterLE_W_p_m2 = (lamda*(AbsHumiditySat_kg_p_m3 - AbsHumidityAct_kg_p_m3) + lamda * AbsHumidityGradient*WaterH_W_p_m2*resist->rground / rou / Cp)*WaterC;
	}
	else
	{
		//Set values to zero if there is no cover
		WaterH_W_p_m2 = 0.0; WaterLE_W_p_m2 = 0.0;
	}

	//Total sensible heat flux (W/m^2), sum of fractional components from each cover type sensible heat flux
	H_total_W_p_m2 = imperP * ImpH_W_p_m2 + treeP * TreeH_W_p_m2 + shortVegP * ShortH_W_p_m2 + soilP * SoilH_W_p_m2 + waterP * WaterH_W_p_m2;
	//Total latent heat flux (W/m^2), sum of fractional components from each cover type latent heat flux
	LE_total_W_p_m2 = imperP * ImpLE_W_p_m2 + treeP * TreeLE_W_p_m2 + shortVegP * ShortLE_W_p_m2 + soilP * SoilLE_W_p_m2 + waterP * WaterLE_W_p_m2;

	//Total net radiation (W/m^2), sum of fractional components from each cover type net radiation
	NR_total_W_p_m2 = imperP * ImpNR_W_p_m2 + treeP * TreeNR_W_p_m2 + shortVegP * ShortNR_W_p_m2 + soilP * SoilNR_W_p_m2 + waterP * WaterNR_W_p_m2;
	//Total ground heat flux (W/m^2), sum of fractional components from each cover type ground heat flux
	DeltaQ_W_p_m2 = imperP * ImpDeltaQ_W_p_m2 + treeP * TreeDeltaQ_W_p_m2 + shortVegP * ShortDeltaQ_W_p_m2 + soilP * SoilDeltaQ_W_p_m2 + waterP * WaterDeltaQ_W_p_m2;

	//Tair_mesoScale_K (K) is mesoscale air temperature, Eq 1 and Figure 1 from Yang et al. (2013), where rou is air density, Cp is specific heat constant pressure
	Tair_mesoScale_K = Tair_localCell_K - H_total_W_p_m2 * resist->rcanopy / rou / Cp;

	//AbsHumidity_mesoScale_kg_p_m3 (kg/m^3) is mesoscale actual absolute humidity, Eq 2 and Figure 1 from Yang et al. (2013), where AbsHumidityAct_kg_p_m3 is local actual absolute humidity
	AbsHumidity_mesoScale_kg_p_m3 = AbsHumidityAct_kg_p_m3 - LE_total_W_p_m2 * resist->rcanopy / lamda;
}

//Function to estimate Tair_mesoScale_K, mesoscale air temperature and absolute humidity
void HeatFluxCal::IterationCalTb(DataFolder *folder, double ts)
{
	//Initial guess of Tair_mesoScale_K, mesoscale air temperature is set to local air temperature at start of routine
	// set the initial condition that Tair_mesoScale_K=Tair_localCell_K, then the F=1
	Tair_mesoScale_K = Tair_localCell_K; 
	//Initial prior value of Tair_mesoScale_K, needed to enter loop to search for actual current value of Tair_mesoScale_K
	Tb_i = 0;
	//counter for loop to search for actual current value of Tair_mesoScale_K
	int n = 0;

	//Iterative loop to solve for 10 unknown variables in the Cool Air energy balance, as described by Yang et al. (2013)
	//From Yang et al. (2013) We have 10 equations, (1), (2), (9), (10), (11), (12), (13), (19), (20), and (21)
	//	We have 10 unknown variables, the values of H_total_W_p_m2, LE_total_W_p_m2, H1, H2, H3, LE1, LE2, LE3, Tair_localCell_K, Tdew_K.
	//	Set criteria for closing the loop, based on difference between prior and current estimate of Tair_mesoScale_K
	while (abs(Tb_i - Tair_mesoScale_K) > 0.1 && n < 100)
	{
		//define prior mesoscale air temperature, Tb_i
		Tb_i = Tair_mesoScale_K;
		//iteratively call function to calculate the aerodynamic resistance factors based on estimated Tair_mesoScale_K, at 200 m elevation
		resist->cal_factors(Tb_i, Tair_localCell_K, windsp, aveHeight, 200);

		//iteratively call function to derive Tair_mesoScale_K from local cell energy balance using resistance factors
		MesoCal(folder, ts);
		//advance counter
		n++;
	}

}

/*
bool HeatFluxCal::IterationCalTbTemp(DataFolder *folder, double ts)
{
	Tair_mesoScale_K = Tair_localCell_K; 
	Tb_i = 0;
	int n = 0;
	while (abs(Tb_i - Tair_mesoScale_K) > 0.1 && n < 100)
	{
		Tb_i = Tair_mesoScale_K;
		resist->cal_factors(Tb_i, Tair_localCell_K, windsp, aveHeight, 200);
		MesoCal(folder, ts);
		n++;
	}
	if (abs(Tair_localCell_K - Tair_mesoScale_K) > 30)
	{
		//Tair_mesoScale_K = Tair_localCell_K;
		return true;
	}
	return false;
}
*/

void HeatFluxCal::savefolderValue(DataFolder *folder, DataFolder* reffolder, int timestep, int index)
{

	folder->VarDict["ImperviousH_W_p_m2"] = ImpH_W_p_m2;
	folder->VarDict["ImperviousLE_W_p_m2"] = ImpLE_W_p_m2;

	folder->VarDict["TreeH_W_p_m2"] = TreeH_W_p_m2;
	//std::cout << folder->VarDict["TreeH_W_p_m2"] << std::endl;
	folder->VarDict["TreeLE_W_p_m2"] = TreeLE_W_p_m2;
	folder->VarDict["TreeLEE_W_p_m2"] = TreeLEE_W_p_m2;
	folder->VarDict["TreeLET_W_p_m2"] = TreeLET_W_p_m2;

	folder->VarDict["ShortVegH_W_p_m2"] = ShortH_W_p_m2;
	folder->VarDict["ShortVegLE_W_p_m2"] = ShortLE_W_p_m2;
	folder->VarDict["ShortVegLEE_W_p_m2"] = ShortLEE_W_p_m2;
	folder->VarDict["ShortVegLET_W_p_m2"] = ShortLET_W_p_m2;


	folder->VarDict["SoilNR_W_p_m2"] = SoilNR_W_p_m2;
	folder->VarDict["SoilH_W_p_m2"] = SoilH_W_p_m2;
	folder->VarDict["SoilLE_W_p_m2"] = SoilLE_W_p_m2;

	folder->VarDict["WaterNR_W_p_m2"] = WaterNR_W_p_m2;
	folder->VarDict["WaterH_W_p_m2"] = WaterH_W_p_m2;
	folder->VarDict["WaterLE_W_p_m2"] = WaterLE_W_p_m2;

	folder->VarDict["H_total_W_p_m2"] = H_total_W_p_m2;
	folder->VarDict["LE_total_W_p_m2"] = LE_total_W_p_m2;
	folder->VarDict["NR_total_W_p_m2"] = NR_total_W_p_m2;
	folder->VarDict["DeltaQ_W_p_m2"] = DeltaQ_W_p_m2;
	folder->VarDict["Tair_mesoScale_K"] = Tair_mesoScale_K;
	folder->VarDict["Tair_K"] = Tair_localCell_K;
	folder->VarDict["Tdew_K"] = Tdew_K;
	//Absolute humidity (AH or AbsHumidity_kg_p_m3; kg/m^3) is function of vapor pressure, from Eq 18 of Yang, Endreny, and Nowak (2013), attributed to Lawrence (2005) units T = Kelvin, AH = kg/m^3
	//Absolute humidity (rho_v) and vapor pressure also defined in Eq 5.2 and 5.5 of Stull, 2000. Meteorology for Scientists and Engineers, 2nd Edition, Brooks Cole.
	//		Absolute humidity Eq 5.5 Stull (2000): rho_v (kg/m^3) = e / (Rv * T), where e = vapor pressure Pa, Rv = gas constant for water vapor = 461 J/(K*kg), T is temperature K
	//		Vapor pressure    Eq 5.2 Stull (2000): e (Pa) = 611 * exp[ [17.27*(T-273.16)] / (T-35.86)], where T in K, e0=611 Pa; this is Tetens' formula adjusted from kPa to Pa
	//		Coefficient 1.324 in Eq 18 is derived as C = e0 / Rv = 611/461. 
	//AbsHumidity_kg_p_m3, actual absolute humidity (kg/m^3), Eq 18 from Yang et al. (2013)
	folder->VarDict["AbsHumidity_kg_p_m3"] = 1.324 / Tdew_K * exp(17.27*(Tdew_K - 273.15) / (Tdew_K - 35.85));
	folder->VarDict["AbsHumidity_mesoScale_kg_p_m3"] = AbsHumidity_mesoScale_kg_p_m3;

	folder->VarDict["DeltaQTree0"] = TreeDeltaQ_W_p_m2;
	folder->VarDict["DeltaQTree1"] = Treea1 * TreeNR_W_p_m2;
	folder->VarDict["DeltaQTree2"] = Treea2 * (TreeNR_W_p_m2 - folder->VarDict["TreeNR_W_p_m2"]);
	folder->VarDict["DeltaQTree3"] = Treea3;
	folder->VarDict["TreeNR_W_p_m2"] = TreeNR_W_p_m2;

	folder->VarDict["DeltaQShortVeg0"] = ShortDeltaQ_W_p_m2;
	folder->VarDict["DeltaQShortVeg1"] = Shorta1 * ShortNR_W_p_m2;
	folder->VarDict["DeltaQShortVeg2"] = Shorta2 * (ShortNR_W_p_m2 - folder->VarDict["ShortVegNR_W_p_m2"]);
	folder->VarDict["DeltaQShortVeg3"] = Shorta3;
	folder->VarDict["ShortVegNR_W_p_m2"] = ShortNR_W_p_m2;

	folder->VarDict["DeltaQImpervious0"] = ImpDeltaQ_W_p_m2;
	folder->VarDict["DeltaQImpervious1"] = Urbana1 * ImpNR_W_p_m2;
	folder->VarDict["DeltaQImpervious2"] = Urbana2 * (ImpNR_W_p_m2 - folder->VarDict["ImpNR_W_p_m2"]);
	folder->VarDict["DeltaQImpervious3"] = Urbana3;
	folder->VarDict["ImpNR_W_p_m2"] = ImpNR_W_p_m2;

	folder->VarDict["ImpDeltaQ_W_p_m2"] = ImpDeltaQ_W_p_m2;
	folder->VarDict["TreeDeltaQ_W_p_m2"] = TreeDeltaQ_W_p_m2;
	folder->VarDict["ShortDeltaQ_W_p_m2"] = ShortDeltaQ_W_p_m2;
	folder->VarDict["SoilDeltaQ_W_p_m2"] = SoilDeltaQ_W_p_m2;
	folder->VarDict["WaterDeltaQ_W_p_m2"] = WaterDeltaQ_W_p_m2;
	
	// cte can use for debugging, code by sc 2019
	/*if (index == 249)
	{
		std::fstream fs;
		fs.open("wateronimp249.txt", std::fstream::in | std::fstream::out | std::fstream::app);
		fs << wateronImp << std::endl;
		fs.close();
	}*/

}

// calculate local canopy temperature and humidity
void HeatFluxCal::LocalTCal(DataFolder *folder, DataFolder* reffolder, int ts, int n_iters) {

	double Tb_obs = reffolder->VarDict["Tair_mesoScale_K"];   // the calculated mesoscale air temperature
	double eb_obs = reffolder->VarDict["AbsHumidity_mesoScale_kg_p_m3"];   // the calculated mesoscale air humidity

	double Ta_est = reffolder->VarDict["Tair_K"];   // the start value of the estimated air temperature of a folder
	double Td_est = reffolder->VarDict["Tdew_K"];     // the start value of the estimated dew point temperature of a folder
												  //changed below code by Vamsi to get the number of iterations from config file
												  //int n_iters = inp->getPM()->get_iterationCount();      // total iteration number for searching the solution
	double alamda = 0.1;   // the lamda value in LM method
	double w1 = 10000;    // the weight for eb 
	double W[2][2];     // the weight matrix
	double J[2][2];
	double H[2][2];
	double H_lm[2][2];
	double e = 0.0;    // error value
	double d1 = 0.0;
	double d2 = 0.0;
	W[0][0] = 1;
	W[0][1] = 0;
	W[1][0] = 0;
	W[1][1] = w1;  //give values to the weight matrix member

	int updateJ = 1;

	double dTa = 0.01;     // for numerical diff
	double dTd = 0.01;     // for numerical diff

	for (int it = 0; it < n_iters; it++)
	{
		if (updateJ == 1)
		{
			Tair_localCell_K = Ta_est;
			Tdew_K = Td_est;
			IterationCalTb(folder, ts);
			double Tb0 = Tair_mesoScale_K; double eb0 = AbsHumidity_mesoScale_kg_p_m3;

			Tair_localCell_K = Ta_est + dTa;
			IterationCalTb(folder, ts);
			double Tb1 = Tair_mesoScale_K; double eb1 = AbsHumidity_mesoScale_kg_p_m3;

			Tair_localCell_K = Ta_est; Tdew_K = Td_est + dTd;
			IterationCalTb(folder, ts);
			double Tb2 = Tair_mesoScale_K; double eb2 = AbsHumidity_mesoScale_kg_p_m3;

			J[0][0] = (Tb1 - Tb0) / dTa;
			J[0][1] = (Tb2 - Tb0) / dTd;
			J[1][0] = (eb1 - eb0) / dTa;
			J[1][1] = (eb2 - eb0) / dTd;

			d1 = Tb_obs - Tb0;
			d2 = (eb_obs - eb0)*w1;

			H[0][0] = J[0][0] * J[0][0] + J[1][0] * J[1][0] * w1;
			H[0][1] = J[0][0] * J[0][1] + J[1][0] * J[1][1] * w1;
			H[1][0] = J[0][1] * J[0][0] + J[1][1] * J[1][0] * w1;
			H[1][1] = J[0][1] * J[0][1] + J[1][1] * J[1][1] * w1;

			if (it == 0)
				e = d1 * d1 + d2 * d2;

		}

		H_lm[0][0] = H[0][0] + alamda;
		H_lm[0][1] = H[0][1];
		H_lm[1][0] = H[1][0];
		H_lm[1][1] = H[1][1] + alamda;

		double bb = 1 / (H_lm[0][0] * H_lm[1][1] - H_lm[0][1] * H_lm[1][0]);
		double dp1 = bb * (H_lm[1][1] * (J[0][0] * d1 + J[1][0] * d2) - H_lm[0][1] * (J[0][1] * d1 + J[1][1] * d2));
		double dp2 = bb * (-H_lm[1][0] * (J[0][0] * d1 + J[1][0] * d2) + H_lm[0][0] * (J[0][1] * d1 + J[1][1] * d2));
		double Ta_lm = Ta_est + dp1;
		double Td_lm = Td_est + dp2;

		Tair_localCell_K = Ta_lm;
		Tdew_K = Td_lm;

		IterationCalTb(folder, ts);

		double Tb_est_lm = Tair_mesoScale_K;
		double eb_est_lm = AbsHumidity_mesoScale_kg_p_m3;
		double d_lm1 = Tb_obs - Tb_est_lm;
		double d_lm2 = (eb_obs - eb_est_lm)*w1;


		double  e_lm = d_lm1 * d_lm1 + d_lm2 * d_lm2;

		if (e_lm < e)
		{
			alamda = alamda / 10;
			Ta_est = Ta_lm;
			Td_est = Td_lm;
			e = e_lm;
			updateJ = 1;
		}
		else
		{
			updateJ = 0;
			alamda = alamda * 10;
		}

	}

}
