#include "ResistanceCal.h"

ResistanceCal::ResistanceCal(Inputs* input)
{
	
	//satPoint (m3/m3) is soil volumetric saturation from DataFolder
	satPoint = input->RefParams["SoilSaturationPoint_m3pm3"];
	//wiltPoint (m3/m3) is soil volumetric wilting point from DataFolder
	wiltPoint = input->RefParams["SoilWiltingPoint_m3pm3"];
	//fieldPoint (m3/m3) is soil volumetric field capacity from DataFolder
	fieldPoint = input->RefParams["SoilFieldCapacity_m3pm3"];
	//MSD (m) is soil maximum storage deficit used in TOPMODEL based soil vertical water flux calculations
	MSD = input->RefParams["MSD_m"];
	
	//TreeMinR (s/m) is canopy minimum resistance for tree
	TreeMinR = input->TemperatureCalculationParams["TreeMinimumResistance"];
	//ShortMinR (s/m) is canopy minimum resistance for short vegetation
	ShortMinR = input->TemperatureCalculationParams["ShortVegetationMinimumResistance"];
	//Zref (m) is reference height for wind speed measurements
	Zref = input->TemperatureCalculationParams["WeatherWindHeight"];
	//Treew (m) is tree leaf width used in resistance calculations
	Treew = input->TemperatureCalculationParams["TreeLeafWidth"];
	//Shortw (m) is short vegetation leaf width used in resistance calculations
	Shortw = input->TemperatureCalculationParams["ShortVegetationLeafWidth"];
	
	//Latent heat of vaporization for water (J/kg)
	lamda = 2257000.0;
	//Specific heat of air at constant pressure (J/kg/K) at 310 K
	Cp = 1006.5;
	//Density of air (kg/m3)
	rou = 1.205;
	//von Karman constant
	k = 0.40;
	//gravitational acceleration (m/s2)
	g = 9.8;
	//attenuation coefficient
	ak = 2.25;  
	a = 0.00662;
	//roughness height of ground (m)
	rD = 0.00137;
}

void ResistanceCal::cal_factors(double Tair_mesoScale_K, double Tair, double Uref, double aveHeight, double height)
{
	double Zd = 0.67*aveHeight;
	double Zom = 0.123*aveHeight;
	double kk = 0.40;
	
	// Set a minium wind speed (m/s) to ensure reasonable resistance values in calculations below
	if (Uref < 1)
		Uref = 1;

	//Eq 33 from Yang et al. (2013), sets wind measurement height = 10m, used by original PASATH in Syracuse despite wind meas. height = 2m. 
	rcanopy = log((10 - 4.0) / 0.5)*log((10 - 4.0) / 0.5) / kk / kk / Uref;

	/*if (rcanopy>5000)
		rcanopy = 5000;*/
}

void ResistanceCal::calr_onwater(double SoilWaterContent, double treeLAI, double shortLAI)    // calculate the resistances that depend on soil water content
{
	rg = exp(8.206 - 4.255*(SoilWaterContent*satPoint - wiltPoint) / (satPoint - wiltPoint)); // flux resistance from deeper soil to soil surface
	
	if (SoilWaterContent*satPoint > wiltPoint)
	{
		rctree = TreeMinR*(fieldPoint - wiltPoint) / (SoilWaterContent*satPoint - wiltPoint) / treeLAI; // flux resistance from tree root to tree canopy
		rcshort = ShortMinR*(fieldPoint - wiltPoint) / (SoilWaterContent*satPoint - wiltPoint) / shortLAI;  // flux resistance from short vegetation root to short vegetation canopy
	}
	else
	{
		// soil water content cannot be smaller than the wilting piont. When the soil water content is near the 
		// wilting point, the resistance will be very large. Here set as 5000s/m
		rctree = 5000;
		rcshort = 5000;
	}
}

// U wind speed, ts: time step; aveHeight is the average height of the canopy and height is the building height or tree height or short vegetation height
void ResistanceCal::calr_urban(double Uref, double treeLAI, double shortLAI, double aveHeight, double hwR, double height)  //calculate urban canopy resistances
{

	double Zd = 0.67*aveHeight;
	double Zom = 0.123*aveHeight;
	double U = Uref * (log(height / rD) / log(Zref / rD));  // calculate wind speed at canopy average height 

	//PASATH used 0.25 m/s as minimum wind speed
	if (U < 0.25)
		U = 0.25;
	//hwR is the Urbanhwr or height (h) to width (w) variable, which ranges from hwR=0.1 for LC=21 to hwR=1.2 for LC=24. 
	//Based on Table 2 of Yang et al. (2013), LC=21 had hwR=0.01 and h=0.1 and w=10, but the PASATH HeatCal.cpp code for LC=21 used h=1 and hwR=0. 
	//hwR should be computed from inputs of h and w, making LC=21 hwR=0.1, which results in appropriate diminishment of wind for Developed Open Space. Windspeed changes by ~4% with changing hwR=0.1 from hwR=0.01 or hwR=0. 
	//Ucan = 96.2% of U when hwR=0.1, = 82.4% of U when hwR=0.5 (shallow urban canyon), = 73.4% of U when hwR=0.8, = 62.9% of U when hwR=1.2, and = 42.7% of U when hwR=2.2 (deep urban canyon)
	double Ucan = U*exp(-0.386*hwR); // calculate the windspeed within the canopy
	//Equations used in PASATH for published runs by Yang et al. (2013)
	rimp = rou*Cp / (11.48 + 4.2*Ucan);   // flux resistance from impervious surface to folder canopy level
	rtree = 30 * (1 + 0.55*treeLAI) / sqrt(Ucan) / (2.5*(1 - exp(-0.4*treeLAI)));	// flux resistance from tree canopy to folder canopy level
	rShortVeg = 30 * (1 + 0.55*shortLAI) / sqrt(Ucan) / (2.5*(1 - exp(-0.4*shortLAI))); // flux resistance from short vegetation canopy to folder canopy level.		
}

void ResistanceCal::calr_rural(double Uref, double treeLAI, double shortLAI, double aveHeight)  //calculate rural canopy resistances
{

	double Zd = 0.67*aveHeight;
	double Zom = 0.123*aveHeight;
	double U = Uref * (log(aveHeight / rD) / log(Zref / rD)); // calculate the wind speed at average height	

	//PASATH used 0.25 m/s as minimum wind speed
	if (U < 0.25)
		U = 0.25;
	double Ke = k*k*U*(aveHeight - Zd) / log((aveHeight - Zd) / Zom);
	rground = aveHeight*exp(ak) / ak / Ke*(exp(-ak*rD / aveHeight) - exp(-ak*(rD + Zom) / aveHeight));  //flux resistance from soil to canopy level
	rimp = rground;         // flux resistance from impervious to canopy level
	rtree = ak / 2 / a / treeLAI*sqrt(Treew / U) / (1 - exp(-ak / 2));   // flux resistance from tree canopy to folder canopy level
	rShortVeg = ak / 2 / a / shortLAI*sqrt(Shortw / U) / (1 - exp(-ak / 2));  // flux resistance from short vegetation canopy to folder canopy level
}
