#include "GI_Outlets.h"

//These GiLayerTypes are used in a generic function for the water balance
enum GILayerTypes {
	SURF,                    // surface layer
	SOIL,                    // soil layer
	STOR,                    // storage layer
	PAVE,                    // pavement layer
	DRAIN					 // underdrain system
};

//-----------------------------------------------------------------------------
//  Globally shared variables 
//-----------------------------------------------------------------------------

double     Vevap;         // evaporation
double     Vpevap;        // pervious area evaporation
double     Vinfil;        // non-GI infiltration
double     VGiInfil;     // infiltration from GI units

static double     SurfaceInflow;  // precip. + runon to GI unit (m/dt)
static double     SurfaceInfil;   // infil. rate from surface layer (m/dt)
static double     SurfaceEvap;    // evap. rate from surface layer (m/dt)
static double     SurfaceOutflow; // outflow from surface layer (m/dt)
static double     SurfaceVolume;  // volume in surface storage (m)

static double     EvapRate;       // evaporation rate (m/dt)
static double     SoilEvap;       // evap. from soil layer (m/dt)
static double     PaveEvap;       // evap. from pavement layer (m/dt)
static double     StorageEvap;    // evap.rate from storage layer (m/dt)
static double     SoilVolume;     // volume in soil/pavement storage (m)
static double     StorageVolume;  // volume in storage layer (m)
static double	  PavePerc;		  // percolation from pavement layer (m/dt)
static double     PaveVolume;     // volume stored in pavement layer  (m)

static double     SoilPerc;       // percolation from soil layer (m/dt)
static double     StorageExfil;   // exfil. rate from storage layer (m/dt)
static double     StorageDrain;   // underdrain flow rate layer (m/dt)
static double     StorageInflow;  // inflow rate to storage layer (m/dt)

static double Xold[MAX_LAYERS];  // previous moisture level in GI layers
int surfaceCanOverflow;			 // 1 if immediate outflow of excess water
double alpha;					 // sqrt(slope)/roughness term in Manning Eq
double alpha_Drain;

int iaIndex;					 // The bin number for the GI 

//initialize NAC, number of topographic index (TI) values in statistical model TI histogram
int NAC;
//cte 20210715 Flag flag_GI_TI_IndexSize_single for testing GI simulation topographic index size; = 0 run with IndexSize same as BulkArea; = 1 run with IndexSize of 1
//NB Set in LocalSoilMoistureCalc.cpp line 55 and GI_Outlets.cpp line 52
int flag_GI_TI_IndexSize_single = 0;


//Sequence of steps in GI subroutines: GI_WaterBalancePrepareUpdate > GI_WaterBalanceUpdate > GI_FluxRatesAllTypesInitialize > GIFluxRates (GI_FluxRatesSwale) which updates terms in GI_WaterBalanceUpdate
void GI_Outlets::GI_WaterBalancePrepareUpdate(Inputs* input, DataFolder* folder, int timestep) {

	// Purpose: evaluates performance of a specific GI unit over current time step.
	// Output:
	// qRunoff   = sum of surface runoff from all GIs (m/dt)
	// qDrain    = sum of drain flows from all GIs (m/dt)

	double giEvap,			 // evaporation rate from GI unit (m/dt)
		giInfil,             // infiltration rate from GI unit (m/dt)
		giDrain;             // drain flow rate from GI unit (m/dt)
	double giAllDevicesArea_m2;

	giAllDevicesArea_m2 = folder->ParamDict["Area"] * folder->ParamDict["giNumber"];

	//initialize NAC, number of topographic index (TI) values in statistical model TI histogram
	NAC = input->Index.size();
	//cte 20210715 redefine for testing of new GI routine
	if (flag_GI_TI_IndexSize_single == 1) {
		NAC = 1;
	}

	//dryTime parameter (seconds) created as an initial value of 1 day, but not widely used beyond rain barrel; intended for updating some devices or rates
	folder->VarDict["dryTime"] = 1 * 3600; 

	// initialize evap and infil losses
	giEvap = 0.0;
	giInfil = 0.0;

	// find surface runoff from the GI unit (m/dt)
	GI_WaterBalanceUpdate(folder, &giEvap, &giInfil, &giDrain, timestep);

	// convert drain flow to cms
	giDrain *= giAllDevicesArea_m2;

	// update moisture losses (m3)
	Vevap += giEvap * giAllDevicesArea_m2;
	VGiInfil += giInfil * giAllDevicesArea_m2;

	// determines if a GI process allows infiltration or not
	if (folder->ParamDict["storThickness_m"] == 0.0 ||
		folder->ParamDict["seepageRate_mph"] > 0.0) {

		Vpevap += giEvap * giAllDevicesArea_m2;
	}

	//dryTime parameter created but not yet used; intended for updating some devices or rates
	if (folder->VarDict["rainfall"] > MIN_RUNOFF) {
		//if rainfall occured, then dryTime reset to 0
		folder->VarDict["dryTime"] = 0.0;
	}
	else {
		//else drytime increased by unit of seconds
		folder->VarDict["dryTime"] += folder->ParamDict["TimeStep_sec"];
	}

	// update GI water balance and save results
	GI_WaterBalanceFinal(folder, input);


}
//Sequence of steps in GI subroutines: GI_SubRoutinesBegin > GI_WaterBalancePrepareUpdate > GI_WaterBalanceUpdate > GI_FluxRatesAllTypesInitialize > GIFluxRates (GI_FluxRatesSwale) which updates terms in GI_WaterBalanceUpdate
void GI_Outlets::GI_WaterBalanceUpdate(DataFolder* folder, double* giEvap,
	double* giInfil, double* giDrain, int timestep) {

	//  Purpose: computes outflows from the surface of a GI unit.
	//  Output:  
	//  GIEvap  = evaporation rate for GI unit (m/dt)
	//	GIInfil = infiltration rate for GI unit (m/dt)
	//  GIDrain = drain flow for GI unit (m/dt)
	//  returns surface runoff rate from the GI unit (m/dt)

	int    i;
	double x[MAX_LAYERS];        // layer moisture levels
	double xOld[MAX_LAYERS];     // work vector
	double xPrev[MAX_LAYERS];    // work vector
	double xMin[MAX_LAYERS];     // lower limit on moisture levels
	double xMax[MAX_LAYERS];     // upper limit on moisture levels
	double fOld[MAX_LAYERS];     // previously computed flux rates
	double f[MAX_LAYERS];        // newly computed flux rates
	// net inflow - outflow from previous time step for each GI layer (m/dt)
	double   oldFluxRates[MAX_LAYERS];

	// convergence tolerance on moisture levels (ft, moisture fraction , ft)
	double xTol[MAX_LAYERS] = { STOPTOL, STOPTOL, STOPTOL, STOPTOL };
	//Li: for keeping ParamDict["storThickness_m"]
	double sthick = 0;


	// rain barrels have 100% void space and impermeable bottom
	if (folder->ParamStringDict["Type"] == "RainBarrel")
	{
		//storVoidFrac = 1 when entire volume is available, absent any solids (e.g., vegetation, substrate)
		folder->ParamDict["storVoidFrac"] = 1.0;
		//seepageRate_mph (m) is set to 0 for RainBarrel
		folder->ParamDict["seepageRate_mph"] = 0.0;
	}
	// set storage layer parameters of a green roof 
	if (folder->ParamStringDict["Type"] == "GreenRoof")
	{
		//storThickness_m (m) is set to drainMatThickness_m for GreenRoof
		folder->ParamDict["storThickness_m"] = folder->ParamDict["drainMatThickness_m"];
		//storVoidFrac = 1 when entire volume is available, absent any solids (e.g., vegetation, substrate)
		folder->ParamDict["storVoidFrac"] = folder->ParamDict["drainVoidFrac"];
		//storClogFactor is set to 0 for GreenRoof
		folder->ParamDict["storClogFactor"] = 0.0;
		//seepageRate_mph (m) is set to 0 for GreenRoof
		folder->ParamDict["seepageRate_mph"] = 0.0;
	}

	//Initial time step, set initial values for GI device
	if (timestep == 0.0) {
		folder->VarDict["surfaceDepth"] = 0.0;
		folder->VarDict["soilMoisture"] = 0.0;
		folder->VarDict["paveDepth"] = 0.0;
		folder->VarDict["storageDepth"] = 0.0;
		folder->VarDict["total_Evap"]=0.0;

		//Set prior time step flux rates (m) to 0 at initial time step, for all layers of GI device
		for (int i = 0; i < MAX_LAYERS; i++)
		{
			//oldFluxRates is vector of flux from prior time step
			oldFluxRates[i] = 0.0;
		}
	}

	//GI layers as vectors are updated as x[] vectors
	//GI layers as x[] vectors are defined for each of the GI layers, surface, soil, pavement, and storage
	x[SURF] = folder->VarDict["surfaceDepth"];
	x[SOIL] = folder->VarDict["soilMoisture"];
	x[PAVE] = folder->VarDict["paveDepth"];
	x[STOR] = folder->VarDict["storageDepth"];
	//SurfaceInflow (m) is water transfered from upSlope BulkArea pervious and impervious area, and from precipitation on GI surfaces
	SurfaceInflow = folder->VarDict["giInflow"];

	//Initialize layer flux rates and moisture limits at each time step
	SurfaceInfil = 0.0;
	SurfaceEvap = 0.0;
	PaveEvap = 0.0;
	PavePerc = 0.0;
	SoilEvap = 0.0;
	SoilPerc = 0.0;
	StorageInflow = 0.0;
	StorageExfil = 0.0;
	StorageEvap = 0.0;
	StorageDrain = 0.0;

	//For each layer in GI, where layers include SURF, SOIL, PAVE, and STOR
	for (i = 0; i < MAX_LAYERS; i++)
	{
		//set current flux rate depth (m) to 0
		f[i] = 0.0;
		//set the prior flux rate depth (m) to oldFluxRates (m)
		fOld[i] = oldFluxRates[i];

		//set the minimum depth value (m) to 0
		xMin[i] = 0.0;
		//set the maximum depth value (m) to BIG, which is defined in GI_Outlets.h
		xMax[i] = BIG;
		//set the prior state depth (m) to the current depth
		Xold[i] = x[i];
	}

	//sthick (m) is the soil thickness, defined as quotient of maximum storage depth (m) and effective porosity (m3/m3)
	sthick= folder->ParamDict["MSD_m"] / folder->ParamDict["SoilPorosity_m3pm3"];

	//SurfaceInfil (m) set to 0 for PermeablePavement
	if (folder->ParamStringDict["Type"] == "PermeablePavement") 
		SurfaceInfil = 0.0;

	//SurfaceInfil (m) set to Infiltration_viaInfilEx_m for all other GI types if hydraulic conductivity > 0
	else if (folder->ParamDict["K0_mph"] > 0.0) {
		SurfaceInfil = folder->VarDict["Infiltration_viaInfilEx_m"];
	}

	//SurfaceInfil (m) set to NativeInfil for all other GI types if hydraulic conductivity = 0
	else SurfaceInfil = folder->VarDict["NativeInfil"];

	//xMin[SOIL] and xMax[SOIL] of GI layer is set when sthick (m) soil thickness > 0
      
	if (sthick > 0.0) {
		//xMin[SOIL] set to SoilWiltingPoint_m3pm3
		xMin[SOIL] = folder->ParamDict["SoilWiltingPoint_m3pm3"];
		//xMax[SOIL] set to SoilPorosity_m3pm3
		xMax[SOIL] = folder->ParamDict["SoilPorosity_m3pm3"];
	}

	//xMax[PAVE] is set to paveThickness_m if pavement thickness > 0
	if (folder->ParamDict["paveThickness_m"] > 0.0)
	{
		//Water depth (m) within layer; actual water depth without solids is found by multiplying by void ratio
		xMax[PAVE] = folder->ParamDict["paveThickness_m"];
	}

	//xMax[STOR] is set to storThickness_m if storThickness_m > 0
	if (folder->ParamDict["storThickness_m"] > 0.0)
	{
		//Water depth (m) within layer; actual water depth without solids is found by multiplying by void ratio
		xMax[STOR] = folder->ParamDict["storThickness_m"];
	}

	//xMax[STOR] is set to drainMatThickness_m if GI is GreenRoof
	if (folder->ParamStringDict["Type"] == "GreenRoof")
	{
		//Water depth (m) within layer; actual water depth without solids is found by multiplying by void ratio
		xMax[STOR] = folder->ParamDict["drainMatThickness_m"];
	}

	//Determine flux rate functions based on GI type, then update layer depths and flux rates over the time step
	GI_FluxRatesAllTypesInitialize(folder, MAX_LAYERS, x, xOld, xPrev, xMin, xMax, xTol, fOld, f, timestep);

	//Set GI layer depths (m) based on x[] vector values for each layer, SURF, SOIL, PAVE, and STOR 
	folder->VarDict["surfaceDepth"] = x[SURF];
	folder->VarDict["soilMoisture"] = x[SOIL];
	folder->VarDict["paveDepth"] = x[PAVE];
	folder->VarDict["storageDepth"] = x[STOR];

	//Update oldFluxRates depths (m) to current flux rates, now that end of time step
	for (i = 0; i < MAX_LAYERS; i++) oldFluxRates[i] = f[i];

	//Pointers to giEvap, giInfil, giDrain are set to rates of GI unit evaporation (m), infiltration (m) & drain flow (m) 
	*giEvap = SurfaceEvap + PaveEvap + SoilEvap + StorageEvap;
	if (*giEvap < 0.0)* giEvap = 0.0;
	*giInfil = StorageExfil;
	*giDrain = StorageDrain;
}


void GI_Outlets::GI_FluxRatesAllTypesInitialize(DataFolder* folder, int n, double* x, double* xOld, double* xPrev,
	double* xMin, double* xMax, double* xTol, double* qOld, double* q, double dt) {

	//  Purpose: solves system of equations dx/dt = q(x) for x at end of time step dt using a modified Puls method.
	int i;
	// initialize state variable values
	for (i = 0; i < n; i++)
	{
		xOld[i] = x[i];
		xPrev[i] = x[i];
	}

	// compute flux rates for current state levels
	if (folder->ParamStringDict["Type"] == "BioRetention") {
		GI_FluxRatesBioRetentionRaingarden(folder, x, q);
	}
	if (folder->ParamStringDict["Type"] == "RainGarden") {
		//Raingarden is a special case of bioretention where storThickness_m = 0 
		GI_FluxRatesBioRetentionRaingarden(folder, x, q);

	}
	if (folder->ParamStringDict["Type"] == "GreenRoof") {
		GI_FluxRatesGreenRoof(folder, x, q);
	}
	if (folder->ParamStringDict["Type"] == "InfilTrench") {
		GI_FluxRatesInfiltrationTrench(folder, x, q);
	}
	if (folder->ParamStringDict["Type"] == "PermeablePavement") {
		GI_FluxRatesPermeablePavement(folder, x, q);
	}
	if (folder->ParamStringDict["Type"] == "RainBarrel") {
		GI_FluxRatesRainBarrel(folder, x, q);
	}
	//Li:
	if (folder->ParamStringDict["Type"] == "RoofDiscon") {
		GI_FluxRatesRoofDisconnect(folder, x, q);
	}
	if (folder->ParamStringDict["Type"] == "Swale") {
		GI_FluxRatesSwale(folder, x, q);
	}

	// update state levels based on current flux rates, 
	// i represents layers [0 = surface, 1 = layer below; see Figure 5.8 from dissertation of Reza Abdi]
	for (i = 0; i < n; i++)
	{
		//update water balance, find min and max, reset negative value to 0; q[i] is equal to f[] in modPuls function
		//for most GI, q[0] = f[SURF] = (SurfaceInflow - SurfaceEvap - SurfaceInfil - SurfaceOutflow)
		//for swale GI q[0] = f[SURF] = dVdt/surfArea and dVdT = surfInflow - SurfaceEvap - SoilPerc - SurfaceOutflow
		x[i] = xOld[i] + q[i];
		x[i] = MIN(x[i], xMax[i]);
		x[i] = MAX(x[i], xMin[i]);
		if (x[i] < 0.0) { x[i] = 0.0; }
			   
	}
	folder->VarDict["waterToPerDep"] = x[0];

}

//Bioretention layers, top to bottom: surface, soil, storage with drain option, native soil
//RainGarden layers, top to bottom: surface, soil with no drain option, native soil
void GI_Outlets::GI_FluxRatesBioRetentionRaingarden(DataFolder* folder, double x[], double f[]) {

	//  Purpose: computes flux rates from the layers of a bio-retention cell GI.

	// Moisture level variables
	double surfaceDepth;
	double soilTheta;
	double storageDepth;

	// Intermediate variables
	double availVolume;
	double maxRate;

	// GI layer properties
	// Hydro based equation
	double soilThickness = folder->ParamDict["MSD_m"] / folder->ParamDict["SoilPorosity_m3pm3"];
	double SoilPorosity_m3pm3 = folder->ParamDict["SoilPorosity_m3pm3"];
	double soilFieldCap = folder->ParamDict["SoilFieldCapacity_m3pm3"];
	double SoilWiltingPoint_m3pm3 = folder->ParamDict["SoilWiltingPoint_m3pm3"];
	double storageThickness = folder->ParamDict["storThickness_m"];
	double storageVoidFrac = folder->ParamDict["storVoidFrac"];

	// retrieve moisture levels from input vector
	surfaceDepth = x[SURF];
	soilTheta = x[SOIL];
	storageDepth = x[STOR];
	double total_Evap;
	
	// The SWMM evaporation function is kept commented out, in case an alternative to i-Tree Hydro is needed
	// Convert moisture levels to volumes
	//SurfaceVolume = surfaceDepth * (1.0 - folder->ParamDict["surfaceVoidRatio"]);
	//SoilVolume = soilTheta * soilThickness;
	//StorageVolume = storageDepth * storageVoidFrac;
	// Get ET rates using SWMM functions; this could be used if Hydro ET is not wanted
	//availVolume = SoilVolume - SoilWiltingPoint_m3pm3 * soilThickness;
	//GI_RateOfEvaporation(folder, SurfaceVolume, 0.0, availVolume, StorageVolume, 1.0);

	// Call i-Tree Hydro evaporation rates
	SurfaceEvap = folder->VarDict["impEvap_m"] + folder->VarDict["perDepEvap"];
	SoilEvap = folder->VarVecDict["EvapoTranspiration_RootZone_TI_m"][folder->ParamDict["binNumber"]];
	// No storage evap based on the calculations of the Hydro
	StorageEvap = 0.0;   
	if (soilTheta >= SoilPorosity_m3pm3) StorageEvap = 0.0;

	total_Evap = SurfaceEvap + PaveEvap + SoilEvap + StorageEvap;
	folder->VarDict["total_Evap"] = total_Evap;

	// soil layer perc rate
	// A new Drainage_RootZone_TI_m created for GI is defined by percolation function  
	double Drainage_RootZone_TI_m = GI_RateOfSoilPercolation(folder, soilTheta);
	SoilPerc = Drainage_RootZone_TI_m;

	//Loop through NAC, all topographic index values and assign same SoilPerc (m) to each Drainage_RootZone_TI_m[ia], which is flow into unsatZone
	for (int ia = 0; ia < NAC; ia++) {
		//Drainage_RootZone_TI_m (m) becomes SoilPerc (m)
		folder->VarVecDict["Drainage_RootZone_TI_m"][ia] = SoilPerc;
	}

	// exfiltration rate out of storage layer
	StorageExfil = GI_RateOfStorageExfiltration(folder);

	// underdrain flow rate
	StorageDrain = 0.0;
	if (folder->ParamDict["drainOrificeQ_coef"] > 0.0)
	{
		//StorageDrain only operates in storage layer when StorageDepth > 0 and DrainCoefficient > 0; it should not operate in soil layer
		StorageDrain = GI_RateOfStorageDrain(folder, storageDepth, soilTheta, 0.0,
			surfaceDepth);
	}

	//Theory of BulkArea (BA) groundwater entering GI cell is in development. 
	//Theory should be used to update aveSMD_GI based on condition of water table depth, Zi (m), relative to GI layers
	//  Layers are numbered #1 to 4 and correspond with upSat_# names: 1) surface, 2) soil, 3) storage, 4) pavement
	//  upSat_# represents depth (m) of water from groundwater table within each zone
	//	BA groundwater presumed to be at equilibrium with GI device at start of simulation, and at end of every timestep
	//		Equilibrium would have groundwater depth Zi >= Drainage pipe crest depth, regardless of GI localSMD
	//	BA groundwater level in GI without storage drain may rise above soil
	//	BA groundwater level in GI with storage drain <= storage drain crest height (if drain coefficient > 0) 
	//	BA groundwater fluxes from GI cell must be constrained by the transmissivity rate
	//		Groundwater presumably removed by return flow or open drain is kept separate from GI water balance q1 or q3
	//		Groundwater presumably removed by return flow or open drain is estimated using BA baseflow rates, to enforce magnitude
	//	GI water leakage to native soils, StorageExfil, is constrained to soils above BA groundwater
	//		Fraction of soil above BA groundwater should consider drainage invert elevation
	//	upSat terms should not represent rates of water movement, given constraint of transmissivity and baseflow rates
	//See Abdi PhD dissertation Chpt 5 Eqns 1 to 22 explain original ideas for monitoring Zi and impacts on GI water balance
	//	N.B. term q3_wt is the StorageDrain flow containing groundwater, and should be removed from AveSMD

	//upSat_2 (m) is depth of water from groundwater table within layer 2, the soil layer
	folder->VarDict["upSat_2"] = 0.0;
	//upSat_3 (m) is depth of water from groundwater table within layer 3, the storage layer
	folder->VarDict["upSat_3"] = 0.0;
	//satZoneQ (m) is available water in sat zone of the GI which should go to BA
	folder->VarDict["satZoneQ"] = 0.0;

	//Depth to groundwater table, Zi, is assessed relative to layer thickness to determine adjustment to StorageExfil
	//StorageExfilDepth_frac is fraction to adjust storage exfiltration (m) for all storage and native soil above groundwater table
	double StorageExfilDepth_frac = 0;
	//SoilLayerDepthOfWater_m is depth of water in soil layer 
	double SoilLayerDepthOfWater_m = 0;

	//initialize groundwater upwelling terms
	folder->VarDict["upSat_2"] = 0;
	folder->VarDict["upSat_3"] = 0;
	folder->VarDict["upSat_4"] = 0;

	//Zi (m) is above the surface, within layer 1
	if (folder->VarDict["Zi"] < 0.0)
	{
		//Groundwater depth (m) set to surface
		folder->VarDict["Zi"] = 0.0;
		//StorageExfilDepth_frac set to 0, meaning no depth of infiltrated water can exfiltrate to groundwater given groundwater has upwelled to saturate layer
		StorageExfilDepth_frac = 0;
		//soilThickness > 0 means soil layer exists to be saturated by groundwater
		if (soilThickness > 0)
		{
			//upSat_2 saturates soilThickness; cte this eqn seems to always have Zi as 0
			folder->VarDict["upSat_2"] = MAX(((soilThickness - folder->VarDict["Zi"]) * folder->ParamDict["SoilPorosity_m3pm3"]), 0.0);
		}

		//drainOrificeQ_coef > 0 means storage drain is open and draining groundwater
		if (storageThickness > 0 && folder->ParamDict["drainOrificeQ_coef"] > 0.0)
		{
			//upSat_2 (m) depth of groundwater in layer 2 is redefined to 0 given groundwater table cannot rise above open drain
			folder->VarDict["upSat_2"] = 0;

			//Groundwater depth (m) set to drain invert crest
			folder->VarDict["Zi"] = (soilThickness + storageThickness - folder->ParamDict["drainOffset"]);

			//StorageExfilDepth_frac set to 1, meaning entire depth of infiltrated water can exfiltrate to groundwater
			StorageExfilDepth_frac = 1;

		}
		//drainOrificeQ_coef = 0 means storage drain is closed and retaining groundwater
		if (storageThickness > 0 && folder->ParamDict["drainOrificeQ_coef"] == 0.0)
		{
			//upSat_3 (m) depth of groundwater in layer 3 is equal to storageThickness given Zi = 0 and storage drain is closed
			folder->VarDict["upSat_3"] = MAX((storageThickness * folder->ParamDict["storVoidFrac"]), 0.0);
		}

		//aveSMD (m), soil moisture deficit, decreases with inflow of StorageExfil (m), which should raise Zi.
		//cte 20210715 AveSMD_GI updating based on StorageExfil may be unneeded, given LocalSoilMoistureCalc.cpp has BulkArea Drainage_UnsatZone_m = input->RepoDict["GIstorageExfil_m3"] / GI_Area, 
		// where GIstorageExfil_m3 is based on SoilPerc or StorageExfil
		//cte showing code from LocalSoilMoistureCalc.cpp folder->VarDict["Drainage_UnsatZone_m"] += input->RepoDict["GIstorageExfil_m3"] / (folder->ParamDict["Area"] * folder->ParamDict["TotalPerviousCover_frac"]);
		//cte showing code from SemiAveSMDCalc.cpp folder->VarDict["aveSMD"] = folder->VarDict["aveSMD"] - folder->VarDict["Drainage_UnsatZone_m"] + folder->VarDict["Runoff_Subsurface_m"];

		//cte 20210715 Try commenting out to see how it affects water balance, given SoilPerc or  StorageExfil transfer as GIstorageExfil_m3 to BulkArea Drainage_UnsatZone_m
		folder->VarDict["AveSMD_GI"] = folder->VarDict["AveSMD_GI"] - StorageExfil * StorageExfilDepth_frac;

		//cte 20210715 Try commenting out to see how it affects water balance, given SoilPerc or  StorageExfil transfer as GIstorageExfil_m3 to BulkArea Drainage_UnsatZone_m
		//Drainage_UnsatZone_m (m) is defined to contain StorageExfil from storage layer and additional drainage from lower native soils
		folder->VarDict["Drainage_UnsatZone_m"] = (StorageExfil * StorageExfilDepth_frac);
	}

	//Zi (m) is within the soil layer. No drain permitted in soil layer.
	else if ((folder->VarDict["Zi"] < soilThickness) && (folder->VarDict["Zi"] >= 0.0)) {

		//SoilLayerGravityWater_m is available gravity drainage in soil layer
		SoilLayerDepthOfWater_m = soilTheta / SoilPorosity_m3pm3;

		//StorageExfilDepth_frac represents fraction of (soil layer gravity water actually draining to groundwater) to (soil layer gravity water)
		StorageExfilDepth_frac = (SoilLayerDepthOfWater_m - (soilThickness - folder->VarDict["Zi"])) / SoilLayerDepthOfWater_m;

		//Prohibit negative StorageExfilDepth_frac 
		if (StorageExfilDepth_frac < 0) {
			StorageExfilDepth_frac = 0;
		}

		folder->VarDict["upSat_2"] = MAX(((soilThickness - folder->VarDict["Zi"]) * folder->ParamDict["SoilPorosity_m3pm3"]), 0.0);

		//storage layer with open drain below soil layer, remove groundwater table
		if (storageThickness > 0 && folder->ParamDict["drainOrificeQ_coef"] > 0.0)
		{
			//StorageExfilDepth_frac represents fraction of storageDepth that is draining
			StorageExfilDepth_frac = ((SoilLayerDepthOfWater_m + storageThickness) - folder->ParamDict["drainOffset"]) /
				(SoilLayerDepthOfWater_m + storageThickness);

			//Prohibit negative StorageExfilDepth_frac 
			if (StorageExfilDepth_frac < 0) {
				StorageExfilDepth_frac = 0;
			}

			//set Zi to drain invert crest
			folder->VarDict["Zi"] = (soilThickness + storageThickness - folder->ParamDict["drainOffset"]);

		}
		if (storageThickness > 0 && folder->ParamDict["drainOrificeQ_coef"] == 0.0)
		{
			//StorageExfilDepth_frac represents fraction of (soil layer gravity water actually draining to groundwater) to (soil layer gravity water)
			StorageExfilDepth_frac = ((SoilLayerDepthOfWater_m + storageThickness) - (soilThickness + storageThickness - folder->VarDict["Zi"])) /
				(SoilLayerDepthOfWater_m + storageThickness);

			if (StorageExfilDepth_frac < 0) {
				StorageExfilDepth_frac = 0;
			}
			//Groundwater inflow (m) to storage layer = limited to storage drain invert crest
			folder->VarDict["upSat_3"] = MAX((storageThickness * folder->ParamDict["storVoidFrac"]), 0.0);
		}

		//cte 20210715 Try commenting out to see how it affects water balance, given SoilPerc or  StorageExfil transfer as GIstorageExfil_m3 to BulkArea Drainage_UnsatZone_m
		//storage layer is present, drainage is from storage exfiltration, not soil percolation
		//aveSMD (m), soil moisture deficit, decreases with inflow of StorageExfil (m) 
		folder->VarDict["AveSMD_GI"] = folder->VarDict["AveSMD_GI"] - (StorageExfil * StorageExfilDepth_frac);
		
		//cte 20210715 Try commenting out to see how it affects water balance, given SoilPerc or  StorageExfil transfer as GIstorageExfil_m3 to BulkArea Drainage_UnsatZone_m
		//Drainage_UnsatZone_m (m) is defined to contain StorageExfil from storage layer above groundwater table
		folder->VarDict["Drainage_UnsatZone_m"] = StorageExfil * StorageExfilDepth_frac;

		//storage layer is absent, drainage is from soil percolation, not storage exfiltration
		if (storageThickness <= 0)
		{
			//cte 20210715 Try commenting out to see how it affects water balance, given SoilPerc or  StorageExfil transfer as GIstorageExfil_m3 to BulkArea Drainage_UnsatZone_m
			//aveSMD (m), soil moisture deficit, decreases with inflow of StorageExfil (m) 
			folder->VarDict["AveSMD_GI"] = folder->VarDict["AveSMD_GI"] - (SoilPerc * (folder->VarDict["Zi"] / (soilThickness)));

			//cte 20210715 Try commenting out to see how it affects water balance, given SoilPerc or  StorageExfil transfer as GIstorageExfil_m3 to BulkArea Drainage_UnsatZone_m
			//Drainage_UnsatZone_m (m) is defined to contain StorageExfil from storage layer above groundwater table
			folder->VarDict["Drainage_UnsatZone_m"] = SoilPerc * StorageExfilDepth_frac;
		}
	}

	//Zi (m) is within the storage layer (or at base of soil layer for case when storage layer = 0 depth)
	if ((folder->VarDict["Zi"] > soilThickness) && (folder->VarDict["Zi"] <= (storageThickness + soilThickness - folder->ParamDict["drainOffset"]))) {

		if (storageThickness > 0 && folder->ParamDict["drainOrificeQ_coef"] > 0.0)
		{
			//StorageExfilDepth_frac represents fraction of storageDepth that is draining, above groundwater table at drain crest
			StorageExfilDepth_frac = (storageThickness - folder->ParamDict["drainOffset"]) / storageThickness;

			//set Zi to drain invert crest
			folder->VarDict["Zi"] = (soilThickness + storageThickness - folder->ParamDict["drainOffset"]);

		}
		if (storageThickness > 0 && folder->ParamDict["drainOrificeQ_coef"] == 0.0)
		{
			//StorageExfilDepth_frac represents fraction of (storage layer thickness actually draining to groundwater) to (entire storage layer thickness)
			StorageExfilDepth_frac = (storageThickness - (soilThickness + storageThickness - folder->VarDict["Zi"])) / storageThickness;
			
			//Groundwater inflow (m) to storage layer = (height of water table in layer) * (layer porosity)
			folder->VarDict["upSat_3"] = MAX(((soilThickness + storageThickness - folder->VarDict["Zi"]) * folder->ParamDict["storVoidFrac"]), 0.0);
		}
		if (storageThickness == 0)
		{
			//StorageExfilDepth_frac represents = 1, as all exfiltration is above groundwater
			StorageExfilDepth_frac = 1;
		}

		//cte 20210715 Try commenting out to see how it affects water balance, given SoilPerc or  StorageExfil transfer as GIstorageExfil_m3 to BulkArea Drainage_UnsatZone_m
		//aveSMD (m), soil moisture deficit, decreases with inflow of StorageExfil (m) 
		folder->VarDict["AveSMD_GI"] = folder->VarDict["AveSMD_GI"] - (StorageExfil * StorageExfilDepth_frac);
		
		//cte 20210715 Try commenting out to see how it affects water balance, given SoilPerc or  StorageExfil transfer as GIstorageExfil_m3 to BulkArea Drainage_UnsatZone_m
		//Drainage_UnsatZone_m (m) is defined to contain StorageExfil from storage layer above groundwater table
		folder->VarDict["Drainage_UnsatZone_m"] = StorageExfil * StorageExfilDepth_frac;
	}

	//Zi (m) is below the storage layer, or below soil layer for case when storage layer = 0. 
	if (folder->VarDict["Zi"] >= soilThickness + storageThickness) {

		//storageThickness can be defined to extend beyond storageThickness and include native soils; this is not currently used
		if (storageThickness > 0)
		{
			//StorageExfilDepth_frac represents fraction of (storage layer + native soil layer thickness actually draining to groundwater) to (entire storage layer thickness)
			StorageExfilDepth_frac = (storageThickness + (folder->VarDict["Zi"] - (soilThickness + storageThickness))) / storageThickness;

			//cte 20210715 Try commenting out to see how it affects water balance, given SoilPerc or  StorageExfil transfer as GIstorageExfil_m3 to BulkArea Drainage_UnsatZone_m
			//aveSMD (m), soil moisture deficit, decreases with inflow of StorageExfil (m), which should raise Zi.
			folder->VarDict["AveSMD_GI"] = folder->VarDict["AveSMD_GI"] - StorageExfil * StorageExfilDepth_frac;

			//cte 20210715 Try commenting out to see how it affects water balance, given SoilPerc or  StorageExfil transfer as GIstorageExfil_m3 to BulkArea Drainage_UnsatZone_m
			//Drainage_UnsatZone_m (m) is defined to contain StorageExfil from storage layer and additional drainage from lower native soils
			folder->VarDict["Drainage_UnsatZone_m"] = (StorageExfil * StorageExfilDepth_frac);
		}
		//storageThickness can be defined to extend beyond soilThickness and include native soils; this is not currently used
		if (storageThickness <= 0)
		{
			//StorageExfilDepth_frac represents fraction of (soil layer + native soil layer thickness actually draining to groundwater) to (entire soil layer thickness)
			StorageExfilDepth_frac = (soilThickness + (folder->VarDict["Zi"] - soilThickness)) / soilThickness;

			//cte 20210715 Try commenting out to see how it affects water balance, given SoilPerc or  StorageExfil transfer as GIstorageExfil_m3 to BulkArea Drainage_UnsatZone_m
			//aveSMD (m), soil moisture deficit, decreases with inflow of SoilPerc (m), which should raise Zi.
			folder->VarDict["AveSMD_GI"] = folder->VarDict["AveSMD_GI"] - (SoilPerc * (folder->VarDict["Zi"] / (soilThickness)));

			//cte 20210715 Try commenting out to see how it affects water balance, given SoilPerc or  StorageExfil transfer as GIstorageExfil_m3 to BulkArea Drainage_UnsatZone_m
			//Drainage_UnsatZone_m (m) is defined to contain StorageExfil from storage layer and additional drainage from lower native soils
			folder->VarDict["Drainage_UnsatZone_m"] = (SoilPerc * StorageExfilDepth_frac);
		}
	}

	//cte 20210715 cout << "GI_Outlets.cpp AveSMD_GI: " << folder->VarDict["AveSMD_GI"] << endl;


	// limit perc rate by available water
	availVolume = (soilTheta - soilFieldCap) * soilThickness;
	maxRate = MAX(availVolume, 0.0) - SoilEvap;
	SoilPerc = MIN(SoilPerc, maxRate);
	SoilPerc = MAX(SoilPerc, 0.0);

	// special case of no storage layer present
	if (storageThickness == 0.0)
	{
		StorageEvap = 0.0;
		maxRate = MIN(SoilPerc, StorageExfil);
		SoilPerc = maxRate;
		//StorageExfil should be 0 if storageThickness = 0; in this case, exfiltration to native soils uses SoilPerc
		StorageExfil = 0;

		// limit surface infil. by unused soil volume
		maxRate = (SoilPorosity_m3pm3 - soilTheta) * soilThickness +
			SoilPerc + SoilEvap;
		SurfaceInfil = MIN(SurfaceInfil, maxRate);

	}

	// storage & soil layers are full
	else if (soilTheta >= SoilPorosity_m3pm3 && storageDepth >= storageThickness)
	{
		// limiting rate is smaller of soil perc and storage outflow
		maxRate = StorageExfil + StorageDrain;
		if (SoilPerc < maxRate)
		{
			maxRate = SoilPerc;
			if (maxRate > StorageExfil) StorageDrain = maxRate - StorageExfil;
			else
			{
				StorageExfil = maxRate;
				StorageDrain = 0.0;
			}
		}
		else SoilPerc = maxRate;

		// apply limiting rate to surface infil.
		SurfaceInfil = MIN(SurfaceInfil, maxRate);
	}

	// either layer not full
	else if (storageThickness > 0.0)
	{
		// limit storage exfiltration by available storage volume
		maxRate = SoilPerc - StorageEvap + storageDepth * storageVoidFrac;

		//storageExfil (m) is subsurface flow from GI to native soil (GI=bioretention and rainGarden)
		//storageExfil (m) at end of GI waterbalance should be passed to BulkArea when GI is connected to BulkArea groundwater table
		StorageExfil = MIN(StorageExfil, maxRate);
		StorageExfil = MAX(StorageExfil, 0.0);

		// limit underdrain flow by volume above drain offset
		if (StorageDrain > 0.0)
		{
			maxRate = -StorageExfil - StorageEvap;
			if (storageDepth >= storageThickness) maxRate += SoilPerc;
			if (folder->ParamDict["drainOffset"] <= storageDepth)
			{
				maxRate += (storageDepth - (folder->ParamDict["drainOffset"] *
					storageVoidFrac));

			}
			maxRate = MAX(maxRate, 0.0);
			StorageDrain = MIN(StorageDrain, maxRate);
		}

		// limit soil perc by unused storage volume
		maxRate = StorageExfil + StorageDrain + StorageEvap + (storageThickness - storageDepth) * storageVoidFrac;
		SoilPerc = MIN(SoilPerc, maxRate);

		// limit surface infil. by unused soil volume
		maxRate = (SoilPorosity_m3pm3 - soilTheta) * soilThickness +
			SoilPerc + SoilEvap;
		SurfaceInfil = MIN(SurfaceInfil, maxRate);
	}

	//InfilToSoil is not used in this .cpp file or solution, and could likely be removed (20201214)
	folder->VarDict["infilToSoil"] = StorageExfil;

	// Surface outflow in this section was occuring at the end of the time step for each device (bioretention, raingarden, permeable pavement, infiltration trench)
	// Commented out to allow for surface outflow to be called at the beginning of the time step, to minimize ponding and impacts on infiltration
	// Leave these commented lines until additional model testing confirms they can be deleted - August 2019
	//SurfaceOutflow = GI_RateOfSurfaceFlow(folder, surfaceDepth);

	// compute overall layer flux rates
	// the value within that time step, represents Runoff_PervInfilEx_m when qo > f1
	f[SURF] = (SurfaceInflow - SurfaceEvap - SurfaceInfil - SurfaceOutflow) /
		folder->ParamDict["surfaceVoidRatio"];

	//this is the change in theta2 not d2
	f[SOIL] = (SurfaceInfil - SoilEvap - SoilPerc) / soilThickness;
	

	if (storageThickness == 0.0)
	{
		f[STOR] = 0.0;
	}
	else {
		f[STOR] = (SoilPerc - StorageEvap - StorageExfil - StorageDrain + folder->VarDict["upSat_3"]) /
			folder->ParamDict["storVoidFrac"];
	}
}

//GreenRoof layers, top to bottom: surface, soil, drainage mat with drain option
void GI_Outlets::GI_FluxRatesGreenRoof(DataFolder* folder, double x[], double f[]) {

	//
	//  Purpose: computes flux rates from the layers of a green roof.
	//  Input:   x = vector of storage levels
	//  Output:  f = vector of flux rates
	//

	 // Moisture level variables
	double surfaceDepth;
	double soilTheta;
	double storageDepth;

	// Intermediate variables
	double availVolume;
	double maxRate;


	// Green roof properties
	double soilThickness = folder->ParamDict["MSD_m"] / folder->ParamDict["SoilPorosity_m3pm3"];
	double SoilPorosity_m3pm3 = folder->ParamDict["SoilPorosity_m3pm3"];
	double soilFieldCap = folder->ParamDict["SoilFieldCapacity_m3pm3"];
	double SoilWiltingPoint_m3pm3 = folder->ParamDict["SoilWiltingPoint_m3pm3"];
	double storageThickness = folder->ParamDict["storThickness_m"];
	double storageVoidFrac = folder->ParamDict["storVoidFrac"];
	//Added by Li:
	double total_Evap;

	// retrieve moisture levels from input vector
	surfaceDepth = x[SURF];
	soilTheta = x[SOIL];
	storageDepth = x[STOR];

	// The SWMM evaporation function is kept commented out, in case an alternative to i-Tree Hydro is needed
	//SurfaceVolume = surfaceDepth * folder->ParamDict["surfaceVoidRatio"];
	//SoilVolume = soilTheta * soilThickness;
	//StorageVolume = storageDepth * storageVoidFrac;
	// get ET rates
	//availVolume = SoilVolume - SoilWiltingPoint_m3pm3 * soilThickness;
	//GI_RateOfEvaporation(folder, SurfaceVolume, 0.0, availVolume, StorageVolume, 1.0);

	// Call i-Tree Hydro evaporation rates
	SurfaceEvap = folder->VarDict["impEvap_m"] + folder->VarDict["perDepEvap"];
	SoilEvap = folder->VarVecDict["EvapoTranspiration_RootZone_TI_m"][0];
	StorageEvap = 0.0;   // No storage evap based on the calculations of the Hydro
	if (soilTheta >= SoilPorosity_m3pm3) StorageEvap = 0.0;
	//Added by Li:
	total_Evap = SurfaceEvap + SoilEvap + StorageEvap;
	folder->VarDict["total_Evap"] = total_Evap;

	// soil layer perc rate
	SoilPerc = GI_RateOfSoilPercolation(folder, soilTheta);

	// limit perc rate by available water
	availVolume = (soilTheta - soilFieldCap) * soilThickness;
	maxRate = MAX(availVolume, 0.0) - SoilEvap;
	SoilPerc = MIN(SoilPerc, maxRate);
	SoilPerc = MAX(SoilPerc, 0.0);

	//storageExfil (m) is subsurface flow from GI to native soil (GI=greenRoof)
	//storageExfil (m) at end of GI waterbalance should be passed to BulkArea when GI is connected to BulkArea groundwater table
	//Green roof storageExfil assumed to be 0, no connectivity with groundwater table
	StorageExfil = 0.0;

	// storage (drain mat) outflow rate
	StorageDrain = GI_RateOfDrainageMat(folder, storageDepth);

	// unit is full
	if (soilTheta >= SoilPorosity_m3pm3 && storageDepth >= storageThickness)
	{
		// outflow from both layers equals limiting rate
		maxRate = MIN(SoilPerc, StorageDrain);
		SoilPerc = maxRate;
		StorageDrain = maxRate;

		// adjust inflow rate to soil layer
		SurfaceInfil = MIN(SurfaceInfil, maxRate);
	}

	// unit not full
	else
	{
		// limit drainmat outflow by available storage volume
		maxRate = storageDepth * storageVoidFrac - StorageEvap;
		if (storageDepth >= storageThickness) maxRate += SoilPerc;
		maxRate = MAX(maxRate, 0.0);
		StorageDrain = MIN(StorageDrain, maxRate);

		// limit soil perc inflow by unused storage volume
		maxRate = (storageThickness - storageDepth) * storageVoidFrac +
			StorageDrain + StorageEvap;
		SoilPerc = MIN(SoilPerc, maxRate);

		// adjust surface infil. so soil porosity not exceeded
		maxRate = (SoilPorosity_m3pm3 - soilTheta) * soilThickness +
			SoilPerc + SoilEvap;
		SurfaceInfil = MIN(SurfaceInfil, maxRate);
	}
	

	// compute overall layer flux rates
	f[SURF] = (SurfaceInflow - SurfaceEvap - SurfaceInfil - SurfaceOutflow) /
		folder->ParamDict["surfaceVoidRatio"];
	//f[SOIL] = (SurfaceInfil - SoilEvap - SoilPerc) /
		//folder->ParamDict["SoilPorosity_m3pm3"];
	//Li:
	f[SOIL] = (SurfaceInfil - SoilEvap - SoilPerc) / soilThickness;

	f[STOR] = (SoilPerc - StorageEvap - StorageDrain) /
		folder->ParamDict["storVoidFrac"];

}

//Infiltration Trench layers, top to bottom: surface, storage with drain option, native soil
void GI_Outlets::GI_FluxRatesInfiltrationTrench(DataFolder* folder, double x[], double f[]) {
	//
	//  Purpose: computes flux rates from the layers of an infiltration trench GI.
	//  Input:   x = vector of storage levels
	//  Output:  f = vector of flux rates
	//
	 // Moisture level variables
	double surfaceDepth;
	double storageDepth;

	// Intermediate variables
	double availVolume;
	double maxRate;

	// Storage layer properties
	double storageThickness = folder->ParamDict["storThickness_m"];
	double storageVoidFrac = folder->ParamDict["storVoidFrac"];
	double soilThickness = 0.0; // should be 0
	//Added by Li:
	double total_Evap;


	// retrieve moisture levels from input vector
	surfaceDepth = x[SURF];
	storageDepth = x[STOR];

	// convert moisture levels to volumes
	//SurfaceVolume = surfaceDepth * folder->ParamDict["surfaceVoidRatio"];
	//SoilVolume = 0.0;
	//StorageVolume = storageDepth * storageVoidFrac;
	// get ET rates
	//availVolume = (storageThickness - storageDepth) * storageVoidFrac;
	//GI_RateOfEvaporation(folder, SurfaceVolume, 0.0, 0.0, StorageVolume, 1.0);

	// Call i-Tree Hydro evaporation rates
	SurfaceEvap = folder->VarDict["impEvap_m"] + folder->VarDict["perDepEvap"];
	StorageEvap = SoilEvap = folder->VarVecDict["EvapoTranspiration_RootZone_TI_m"][folder->ParamDict["binNumber"]];
	// no storage evap if surface ponded
	if (surfaceDepth > 0.0) StorageEvap = 0.0;
	//Added by Li:
	total_Evap = SurfaceEvap + SoilEvap + StorageEvap;
	folder->VarDict["total_Evap"] = total_Evap;

	// nominal storage inflow
	StorageInflow = SurfaceInflow + SurfaceVolume;

	// exfiltration rate out of storage layer
	StorageExfil = GI_RateOfStorageExfiltration(folder);
	//StorageExfil = folder->VarDict["Drainage_UnsatZone_m"];	// f3 in the manual

	// underdrain flow rate
	StorageDrain = 0.0;
	if (folder->ParamDict["drainOrificeQ_coef"] > 0.0)
	{
		StorageDrain = GI_RateOfStorageDrain(folder, storageDepth, 0.0, 0.0, surfaceDepth);
	}

	//Theory of BulkArea (BA) groundwater entering GI cell is in development. 
	//Theory should be used to update aveSMD_GI based on condition of water table depth, Zi (m), relative to GI layers
	//  Layers are numbered #1 to 4 and correspond with upSat_# names: 1) surface, 2) soil, 3) storage, 4) pavement
	//  upSat_# represents depth (m) of water from groundwater table within each zone
	//	BA groundwater presumed to be at equilibrium with GI device at start of simulation, and at end of every timestep
	//		Equilibrium would have groundwater depth Zi >= Drainage pipe crest depth, regardless of GI localSMD
	//	BA groundwater level in GI without storage drain may rise above soil
	//	BA groundwater level in GI with storage drain <= storage drain crest height (if drain coefficient > 0) 
	//	BA groundwater fluxes from GI cell must be constrained by the transmissivity rate
	//		Groundwater presumably removed by return flow or open drain is kept separate from GI water balance q1 or q3
	//		Groundwater presumably removed by return flow or open drain is estimated using BA baseflow rates, to enforce magnitude
	//	GI water leakage to native soils, StorageExfil, is constrained to soils above BA groundwater
	//		Fraction of soil above BA groundwater should consider drainage invert elevation
	//	upSat terms should not represent rates of water movement, given constraint of transmissivity and baseflow rates
	//See Abdi PhD dissertation Chpt 5 Eqns 1 to 22 explain original ideas for monitoring Zi and impacts on GI water balance
	//	N.B. term q3_wt is the StorageDrain flow containing groundwater, and should be removed from AveSMD

	//StorageExfilDepth_frac is fraction to adjust storage exfiltration (m) for all storage and native soil above groundwater table
	double StorageExfilDepth_frac = 0;
	//SoilLayerDepthOfWater_m is depth of water in soil layer 
	double SoilLayerDepthOfWater_m = 0;

	// parameters showing the differences between SWMM & Hydro layers in RZ & USZ
	folder->VarDict["upSat_3"] = 0.0;
	folder->VarDict["satZoneQ"] = 0.0;	// The available water in sat zone of the GI which should go to BA

	//cte 20210408 This needs to be carefully checked and perhaps corrected
	// Depth to water table, Zi, is assessed relative to layer thickness to determine adjustment to StorageExfil
	// Zi is located below the storage layer
	if (folder->VarDict["Zi"] < 0.0) 
	{ 
		folder->VarDict["Zi"] = 0.0; 
		if (storageThickness > 0 && folder->ParamDict["drainOrificeQ_coef"] > 0.0)
		{
			//No groundwater entering storage or soil or pave
			folder->VarDict["upSat_3"] = 0;
			folder->VarDict["upSat_2"] = 0;
			folder->VarDict["upSat_4"] = 0;

			//set Zi to drain invert crest
			folder->VarDict["Zi"] = (soilThickness + storageThickness - folder->ParamDict["drainOffset"]);
		}
		if (storageThickness > 0 && folder->ParamDict["drainOrificeQ_coef"] == 0.0)
		{
			//upSat_3 saturates storageThickness
			folder->VarDict["upSat_3"] = MAX((storageThickness * folder->ParamDict["storVoidFrac"]), 0.0);
			//upSat_2 saturates soilThickness
			folder->VarDict["upSat_2"] = MAX(((soilThickness - folder->VarDict["Zi"]) * folder->ParamDict["SoilPorosity_m3pm3"]), 0.0);

		}
		if (storageThickness == 0)
		{
			//upSat_2 saturates soilThickness
			folder->VarDict["upSat_2"] = MAX(((soilThickness - folder->VarDict["Zi"]) * folder->ParamDict["SoilPorosity_m3pm3"]), 0.0);

		}

		//aveSMD (m), soil moisture deficit, decreases with inflow of StorageExfil (m), which should raise Zi.
		folder->VarDict["AveSMD_GI"] = folder->VarDict["AveSMD_GI"] - StorageExfil * StorageExfilDepth_frac;

		//Drainage_UnsatZone_m (m) is defined to contain StorageExfil from storage layer and additional drainage from lower native soils
		folder->VarDict["Drainage_UnsatZone_m"] = (StorageExfil * StorageExfilDepth_frac);
	}

	//Zi (m) is below the storage layer
	if (folder->VarDict["Zi"] >= storageThickness) {

		//storageThickness can be defined to extend beyond storageThickness and include native soils; this is not currently used
		if (storageThickness > 0) {
			//StorageExfilDepth_frac represents fraction of (storage layer + native soil layer thickness actually draining to groundwater) to (entire storage layer thickness)
			StorageExfilDepth_frac = (storageThickness + (folder->VarDict["Zi"] - (soilThickness + storageThickness))) / storageThickness;
		}
		//storageThickness can be defined to extend beyond soilThickness and include native soils; this is not currently used
		else {
			//StorageExfilDepth_frac represents fraction of (soil layer + native soil layer thickness actually draining to groundwater) to (entire soil layer thickness)
			StorageExfilDepth_frac = (soilThickness + (folder->VarDict["Zi"] - soilThickness)) / soilThickness;
		}

		//StorageExfilDepth_frac set to 1 to keep GI only contributer to BA water, deactivating above calculations
		StorageExfilDepth_frac = 1;

		//aveSMD (m), soil moisture deficit, decreases with inflow of StorageExfil (m), which should raise Zi.
		folder->VarDict["AveSMD_GI"] = folder->VarDict["AveSMD_GI"] - StorageExfil * StorageExfilDepth_frac;

		//Drainage_UnsatZone_m (m) is defined to contain StorageExfil from storage layer and additional drainage from lower native soils
		folder->VarDict["Drainage_UnsatZone_m"] = (StorageExfil * StorageExfilDepth_frac);
	}

	//Zi (m) is within the storage layer
	else if ((folder->VarDict["Zi"] < storageThickness) && (folder->VarDict["Zi"] >= 0.0))
	{
		if (storageThickness > 0 && folder->ParamDict["drainOrificeQ_coef"] > 0.0)
		{
			//StorageExfilDepth_frac represents fraction of storageDepth that is draining, above groundwater table at drain crest
			StorageExfilDepth_frac = (storageThickness - folder->ParamDict["drainOffset"]) / storageThickness;

			//Groundwater inflow (m) to storage layer = limited to storage drain invert crest
			folder->VarDict["upSat_3"] = 0;
			folder->VarDict["upSat_2"] = 0;
			folder->VarDict["upSat_4"] = 0;
			//set Zi to drain invert crest
			folder->VarDict["Zi"] = (soilThickness + storageThickness - folder->ParamDict["drainOffset"]);

		}
		if (storageThickness > 0 && folder->ParamDict["drainOrificeQ_coef"] == 0.0)
		{
			//StorageExfilDepth_frac represents fraction of (storage layer thickness actually draining to groundwater) to (entire storage layer thickness)
			StorageExfilDepth_frac = (storageThickness - (soilThickness + storageThickness - folder->VarDict["Zi"])) / storageThickness;

			//Groundwater inflow (m) to storage layer = (height of water table in layer) * (layer porosity)
			folder->VarDict["upSat_3"] = MAX(((soilThickness + storageThickness - folder->VarDict["Zi"]) *
				folder->ParamDict["storVoidFrac"]), 0.0);
		}
		if (storageThickness == 0)
		{
			//StorageExfilDepth_frac represents = 1, as all exfiltration is above groundwater
			StorageExfilDepth_frac = 1;
			//Groundwater inflow (m) to storage layer = 0 
			folder->VarDict["upSat_3"] = 0;
		}

		//aveSMD (m), soil moisture deficit, decreases with inflow of StorageExfil (m) 
		folder->VarDict["AveSMD_GI"] = folder->VarDict["AveSMD_GI"] - (StorageExfil * StorageExfilDepth_frac);

		//Drainage_UnsatZone_m (m) is defined to contain StorageExfil from storage layer above groundwater table
		folder->VarDict["Drainage_UnsatZone_m"] = StorageExfil * StorageExfilDepth_frac;
	}

	// limit storage exfiltration by available storage volume
	maxRate = StorageInflow - StorageEvap + storageDepth * storageVoidFrac;

	//storageExfil (m) is subsurface flow from GI to native soil (GI=infiltrationTrench)
	//storageExfil (m) at end of GI waterbalance should be passed to BulkArea when GI is connected to BulkArea groundwater table
	StorageExfil = MIN(StorageExfil, maxRate);
	StorageExfil = MAX(StorageExfil, 0.0);

	// limit underdrain flow by volume above drain offset
	if (StorageDrain > 0.0)
	{
		maxRate = -StorageExfil - StorageEvap;
		if (storageDepth >= storageThickness) maxRate += StorageInflow;
		if (folder->ParamDict["drainOffset"] <= storageDepth)
		{
			maxRate += (storageDepth - folder->ParamDict["drainOffset"]) *
				storageVoidFrac;
		}
		maxRate = MAX(maxRate, 0.0);
		StorageDrain = MIN(StorageDrain, maxRate);
	}

	// limit storage inflow to not exceed storage layer capacity
	maxRate = (storageThickness - storageDepth) * storageVoidFrac +	StorageExfil + StorageEvap + StorageDrain;
	StorageInflow = MIN(StorageInflow, maxRate);

	// equate surface infil to storage inflow
	SurfaceInfil = StorageInflow;

	// find net fluxes for each layer
	f[SURF] = SurfaceInflow - SurfaceEvap - StorageInflow - SurfaceOutflow /
		folder->ParamDict["surfaceVoidRatio"];

	f[STOR] = (StorageInflow - StorageEvap - StorageExfil - StorageDrain + folder->VarDict["upSat_3"]) /
		folder->ParamDict["storVoidFrac"];

	f[SOIL] = 0.0;
}

//PermeablePavement layers, top to bottom: surface, pavement, soil, storage with drain option, native soil
void GI_Outlets::GI_FluxRatesPermeablePavement(DataFolder* folder, double x[], double f[]) {
	//
	//  Purpose: computes flux rates for the layers of a porous pavement GI.
	//  Input:   x = vector of storage levels
	//  Output:  f = vector of flux rates
	//

	// Moisture level variables
	double surfaceDepth;
	double paveDepth;
	double soilTheta;
	double storageDepth;

	// Intermediate variables
	double pervFrac = (1.0 - folder->ParamDict["pavementImpervFrac"]);
	double storageInflow;    // inflow rate to storage layer (ft/s)
	double availVolume;
	double maxRate;

	// GI layer properties
	double paveVoidFrac = folder->ParamDict["pavementVoidFrac"] * pervFrac;
	double paveThickness = folder->ParamDict["paveThickness_m"];
	double soilThickness = folder->ParamDict["MSD_m"] / folder->ParamDict["SoilPorosity_m3pm3"];
	double SoilPorosity_m3pm3 = folder->ParamDict["SoilPorosity_m3pm3"];
	double soilFieldCap = folder->ParamDict["SoilFieldCapacity_m3pm3"];
	double SoilWiltingPoint_m3pm3 = folder->ParamDict["SoilWiltingPoint_m3pm3"];
	double storageThickness = folder->ParamDict["storThickness_m"];
	double storageVoidFrac = folder->ParamDict["storVoidFrac"];
	//Added by Li:
	double total_Evap;


	// retrieve moisture levels from input vector
	surfaceDepth = x[SURF];
	paveDepth = x[PAVE];
	soilTheta = x[SOIL];
	storageDepth = x[STOR];

	// convert moisture levels to volumes
	SurfaceVolume = surfaceDepth * folder->ParamDict["surfaceVoidRatio"];
	PaveVolume = paveDepth * paveVoidFrac;
	SoilVolume = soilTheta * soilThickness;
	StorageVolume = storageDepth * storageVoidFrac;

	// The SWMM evaporation function is kept commented out, in case an alternative to i-Tree Hydro is needed
	// get ET rates
	//availVolume = SoilVolume - SoilWiltingPoint_m3pm3 * soilThickness;
	//GI_RateOfEvaporation(folder, SurfaceVolume, PaveVolume, availVolume, StorageVolume, pervFrac);

	// Call i-Tree Hydro evaporation rates
	SurfaceEvap = folder->VarDict["impEvap_m"] + folder->VarDict["perDepEvap"];
	PaveEvap = folder->VarVecDict["EvapoTranspiration_RootZone_TI_m"][0];
	StorageEvap = 0.0;

	// no storage evap if soil or pavement layer saturated
	if (paveDepth >= paveThickness ||
		(soilThickness > 0.0 && soilTheta >= SoilPorosity_m3pm3)
		) StorageEvap = 0.0;
	//Added by Li:
	total_Evap = SurfaceEvap + PaveEvap + SoilEvap + StorageEvap;
	folder->VarDict["total_Evap"] = total_Evap;

	// find nominal rate of surface infiltration into pavement layer
	SurfaceInfil = SurfaceInflow + (SurfaceVolume);

	// find perc rate out of pavement layer due to clogging (inactive)
	PavePerc = GI_CloggingPermeablePavement(folder);
	
	// surface infiltration can't exceed pavement permeability              //(5.1.013)
	SurfaceInfil = MIN(SurfaceInfil, PavePerc);

	// limit pavement perc by available water
	maxRate = PaveVolume + SurfaceInfil - PaveEvap;
	maxRate = MAX(maxRate, 0.0);
	PavePerc = MIN(PavePerc, maxRate);

	// find soil layer perc rate
	if (soilThickness > 0.0)
	{
		SoilPerc = GI_RateOfSoilPercolation(folder, soilTheta);
		availVolume = (soilTheta - soilFieldCap) * soilThickness;
		maxRate = MAX(availVolume, 0.0) - SoilEvap;
		SoilPerc = MIN(SoilPerc, maxRate);
		SoilPerc = MAX(SoilPerc, 0.0);
	}
	else SoilPerc = PavePerc;

	// exfiltration rate out of storage layer
	StorageExfil = GI_RateOfStorageExfiltration(folder);


	// underdrain flow rate
	StorageDrain = 0.0;
	if (folder->ParamDict["drainOrificeQ_coef"] > 0.0)
	{
		StorageDrain = GI_RateOfStorageDrain(folder, storageDepth, soilTheta, paveDepth,
			surfaceDepth);
	}

	//Theory of BulkArea (BA) groundwater entering GI cell is in development. 
	//Theory should be used to update aveSMD_GI based on condition of water table depth, Zi (m), relative to GI layers
	//  Layers are numbered #1 to 4 and correspond with upSat_# names: 1) surface, 2) soil, 3) storage, 4) pavement
	//  upSat_# represents depth (m) of water from groundwater table within each zone
	//	BA groundwater presumed to be at equilibrium with GI device at start of simulation, and at end of every timestep
	//		Equilibrium would have groundwater depth Zi >= Drainage pipe crest depth, regardless of GI localSMD
	//	BA groundwater level in GI without storage drain may rise above soil
	//	BA groundwater level in GI with storage drain <= storage drain crest height (if drain coefficient > 0) 
	//	BA groundwater fluxes from GI cell must be constrained by the transmissivity rate
	//		Groundwater presumably removed by return flow or open drain is kept separate from GI water balance q1 or q3
	//		Groundwater presumably removed by return flow or open drain is estimated using BA baseflow rates, to enforce magnitude
	//	GI water leakage to native soils, StorageExfil, is constrained to soils above BA groundwater
	//		Fraction of soil above BA groundwater should consider drainage invert elevation
	//	upSat terms should not represent rates of water movement, given constraint of transmissivity and baseflow rates
	//See Abdi PhD dissertation Chpt 5 Eqns 1 to 22 explain original ideas for monitoring Zi and impacts on GI water balance
	//	N.B. term q3_wt is the StorageDrain flow containing groundwater, and should be removed from AveSMD

	//StorageExfilDepth_frac is fraction to adjust storage exfiltration (m) for all storage and native soil above groundwater table
	double StorageExfilDepth_frac = 0;
	//SoilLayerDepthOfWater_m is depth of water in soil layer 
	double SoilLayerDepthOfWater_m = 0;

	folder->VarDict["upSat_2"] = 0.0;	// Two parameters showing the differences between SWMM & Hydro layers in RZ & USZ
	folder->VarDict["upSat_3"] = 0.0;
	folder->VarDict["upSat_4"] = 0.0;
	folder->VarDict["satZoneQ"] = 0.0;	// The available water in sat zone of the GI which should go to BA

	//cte 20210408 This needs to be carefully checked and perhaps corrected
	// Depth to water table, Zi, is assessed relative to layer thickness to determine adjustment to StorageExfil
	//Zi (m) is above the surface layer, zone 1
	if (folder->VarDict["Zi"] < 0.0)
	{
		folder->VarDict["Zi"] = 0.0;
		if (storageThickness > 0 && folder->ParamDict["drainOrificeQ_coef"] > 0.0)
		{
			//No groundwater entering storage or soil or pave
			folder->VarDict["upSat_3"] = 0;
			folder->VarDict["upSat_2"] = 0;
			folder->VarDict["upSat_4"] = 0;

			//set Zi to drain invert crest
			folder->VarDict["Zi"] = (soilThickness + storageThickness - folder->ParamDict["drainOffset"]);
		}
		if (storageThickness > 0 && folder->ParamDict["drainOrificeQ_coef"] == 0.0)
		{
			//upSat_3 saturates storageThickness
			folder->VarDict["upSat_3"] = MAX((storageThickness * folder->ParamDict["storVoidFrac"]), 0.0);
			//upSat_2 saturates soilThickness
			folder->VarDict["upSat_2"] = MAX(((soilThickness - folder->VarDict["Zi"]) * folder->ParamDict["SoilPorosity_m3pm3"]), 0.0);
			//upSat_4 saturates paveThickness
			folder->VarDict["upSat_4"] = MAX(((paveThickness - folder->VarDict["Zi"]) * folder->ParamDict["pavementVoidFrac"]), 0.0);
		}
		if (storageThickness == 0)
		{
			//upSat_2 saturates soilThickness
			folder->VarDict["upSat_2"] = MAX(((soilThickness - folder->VarDict["Zi"]) * folder->ParamDict["SoilPorosity_m3pm3"]), 0.0);
			//upSat_4 saturates paveThickness
			folder->VarDict["upSat_4"] = MAX(((paveThickness - folder->VarDict["Zi"]) * folder->ParamDict["pavementVoidFrac"]), 0.0);

		}
	}

	//Zi (m) is below the storage layer, and below any soil layer or pavement layer for case when storage layer = 0. 
	if (folder->VarDict["Zi"] >= soilThickness + storageThickness + paveThickness)
	{
		//StorageExfilDepth_frac set to 1 to keep GI only contributer to BA water, deactivating above calculations
		StorageExfilDepth_frac = 1;

		//aveSMD (m), soil moisture deficit, decreases with inflow of StorageExfil (m), which should raise Zi.
		folder->VarDict["AveSMD_GI"] = folder->VarDict["AveSMD_GI"] - StorageExfil * StorageExfilDepth_frac;

		//Drainage_UnsatZone_m (m) is defined to contain StorageExfil from storage layer and additional drainage from lower native soils
		folder->VarDict["Drainage_UnsatZone_m"] = (StorageExfil * StorageExfilDepth_frac);
	}

	//Zi (m) is within the storage layer or at base of soil layer for case when storage layer = 0
	if ((folder->VarDict["Zi"] > (soilThickness + paveThickness)) && (folder->VarDict["Zi"] <= (storageThickness + soilThickness + paveThickness - folder->ParamDict["drainOffset"]))) {

		if (storageThickness > 0 && folder->ParamDict["drainOrificeQ_coef"] > 0.0)
		{
			//StorageExfilDepth_frac represents fraction of storageDepth that is draining, above groundwater table at drain crest
			StorageExfilDepth_frac = (storageThickness - folder->ParamDict["drainOffset"]) / storageThickness;

			//Groundwater inflow (m) to storage layer = limited to storage drain invert crest
			folder->VarDict["upSat_3"] = 0;
			folder->VarDict["upSat_2"] = 0;
			folder->VarDict["upSat_4"] = 0;
			//set Zi to drain invert crest
			folder->VarDict["Zi"] = (soilThickness + storageThickness - folder->ParamDict["drainOffset"]);

		}
		if (storageThickness > 0 && folder->ParamDict["drainOrificeQ_coef"] == 0.0)
		{
			//StorageExfilDepth_frac represents fraction of (storage layer thickness actually draining to groundwater) to (entire storage layer thickness)
			StorageExfilDepth_frac = (storageThickness - (paveThickness + soilThickness + storageThickness - folder->VarDict["Zi"])) / storageThickness;

			if (StorageExfilDepth_frac < 0) 
			{
				StorageExfilDepth_frac = 0;
			}

			//Groundwater inflow (m) to storage layer = (height of water table in layer) * (layer porosity)
			folder->VarDict["upSat_3"] = MAX(((paveThickness + soilThickness + storageThickness - folder->VarDict["Zi"]) *
				folder->ParamDict["storVoidFrac"]), 0.0);
		}
		if (storageThickness == 0)
		{
			//StorageExfilDepth_frac represents = 1, as all exfiltration is above groundwater
			StorageExfilDepth_frac = 1;
			//Groundwater inflow (m) to storage layer = 0 
			folder->VarDict["upSat_3"] = 0;
		}

		//aveSMD (m), soil moisture deficit, decreases with inflow of StorageExfil (m) 
		folder->VarDict["AveSMD_GI"] = folder->VarDict["AveSMD_GI"] - (StorageExfil * StorageExfilDepth_frac);

		//Drainage_UnsatZone_m (m) is defined to contain StorageExfil from storage layer above groundwater table
		folder->VarDict["Drainage_UnsatZone_m"] = StorageExfil * StorageExfilDepth_frac;
	}

	// Zi is within soil layer 
	else if ((folder->VarDict["Zi"] > paveThickness) && (folder->VarDict["Zi"] <= (soilThickness + paveThickness))) {
		//StorageExfil = folder->VarDict["Drainage_UnsatZone_m"];
		//SoilPerc = folder->VarVecDict["Drainage_RootZone_TI_m"][folder->ParamDict["binNumber"]];
		folder->VarDict["AveSMD_GI"] = folder->VarDict["AveSMD_GI"] + StorageDrain - StorageExfil;
		folder->VarDict["upSat_3"] = MAX((storageThickness * folder->ParamDict["storVoidFrac"]), 0.0);
		folder->VarDict["upSat_2"] = MAX(((soilThickness + paveThickness - folder->VarDict["Zi"]) * folder->ParamDict["SoilPorosity_m3pm3"]), 0.0);
		// updating the Drainage_UnsatZone_m
		folder->VarDict["Drainage_UnsatZone_m"] = StorageExfil;
	}

	//Zi (m) is within the soil layer. No drain permitted in soil layer.
	else if ((folder->VarDict["Zi"] > 0) && (folder->VarDict["Zi"] <= paveThickness))
	{

		//SoilLayerGravityWater_m is available gravity drainage in soil layer
		SoilLayerDepthOfWater_m = soilTheta / SoilPorosity_m3pm3;

		//StorageExfilDepth_frac represents fraction of (soil layer gravity water actually draining to groundwater) to (soil layer gravity water)
		StorageExfilDepth_frac = (SoilLayerDepthOfWater_m - (soilThickness - folder->VarDict["Zi"])) / SoilLayerDepthOfWater_m;

		//Prohibit negative StorageExfilDepth_frac 
		if (StorageExfilDepth_frac < 0) {
			StorageExfilDepth_frac = 0;
		}

		folder->VarDict["upSat_2"] = MAX(((paveThickness + soilThickness - folder->VarDict["Zi"]) * folder->ParamDict["SoilPorosity_m3pm3"]), 0.0);
		folder->VarDict["upSat_4"] = MAX(((paveThickness - folder->VarDict["Zi"]) * folder->ParamDict["pavementVoidFrac"]), 0.0);

		//storage layer with open drain below soil layer, remove groundwater table
		if (storageThickness > 0 && folder->ParamDict["drainOrificeQ_coef"] > 0.0)
		{
			//StorageExfilDepth_frac represents fraction of storageDepth that is draining
			StorageExfilDepth_frac = ((SoilLayerDepthOfWater_m + storageThickness) - folder->ParamDict["drainOffset"]) /
				(SoilLayerDepthOfWater_m + storageThickness);

			//Prohibit negative StorageExfilDepth_frac 
			if (StorageExfilDepth_frac < 0) {
				StorageExfilDepth_frac = 0;
			}

			//Groundwater inflow (m) to storage layer = limited to storage drain invert crest
			folder->VarDict["upSat_3"] = 0;
			folder->VarDict["upSat_2"] = 0;
			folder->VarDict["upSat_4"] = 0;

			//set Zi to drain invert crest
			folder->VarDict["Zi"] = (soilThickness + storageThickness - folder->ParamDict["drainOffset"]);

		}
		if (storageThickness > 0 && folder->ParamDict["drainOrificeQ_coef"] == 0.0)
		{
			//StorageExfilDepth_frac represents fraction of (soil layer gravity water actually draining to groundwater) to (soil layer gravity water)
			StorageExfilDepth_frac = ((SoilLayerDepthOfWater_m + storageThickness) - (soilThickness + storageThickness - folder->VarDict["Zi"])) /
				(SoilLayerDepthOfWater_m + storageThickness);

			if (StorageExfilDepth_frac < 0) {
				StorageExfilDepth_frac = 0;
			}
			//Groundwater inflow (m) to storage layer = limited to storage drain invert crest
			folder->VarDict["upSat_3"] = MAX((storageThickness * folder->ParamDict["storVoidFrac"]), 0.0);

		}

		//aveSMD (m), soil moisture deficit, decreases with inflow of StorageExfil (m) 
		folder->VarDict["AveSMD_GI"] = folder->VarDict["AveSMD_GI"] - (StorageExfil * StorageExfilDepth_frac);

		//Drainage_UnsatZone_m (m) is defined to contain StorageExfil from storage layer above groundwater table
		folder->VarDict["Drainage_UnsatZone_m"] = StorageExfil * StorageExfilDepth_frac;
	}

	// check for adjacent saturated layers

	// no soil layer, pavement & storage layers are full
	if (soilThickness == 0.0 &&
		storageDepth >= storageThickness &&
		paveDepth >= paveThickness)
	{
		// pavement outflow can't exceed storage outflow
		maxRate = StorageEvap + StorageDrain + StorageExfil;
		if (PavePerc > maxRate) PavePerc = maxRate;

		// storage outflow can't exceed pavement outflow
		else
		{
			// use up available exfiltration capacity first
			//storageExfil (m) is subsurface flow from GI to native soil (GI=permeablePavement)
			//storageExfil (m) at end of GI waterbalance should be passed to BulkArea when GI is connected to BulkArea groundwater table
			StorageExfil = MIN(StorageExfil, PavePerc);
			StorageDrain = PavePerc - StorageExfil;
		}

		// set soil perc to pavement perc
		SoilPerc = PavePerc;

		// limit surface infil. by pavement perc
		SurfaceInfil = MIN(SurfaceInfil, PavePerc);
	}

	// pavement, soil & storage layers are full
	else if (soilThickness > 0 &&
		storageDepth >= storageThickness &&
		soilTheta >= SoilPorosity_m3pm3 &&
		paveDepth >= paveThickness)
	{
		// find which layer has limiting flux rate
		maxRate = StorageExfil + StorageDrain;
		if (SoilPerc < maxRate) maxRate = SoilPerc;
		else maxRate = MIN(maxRate, PavePerc);

		// use up available storage exfiltration capacity first
		if (maxRate > StorageExfil) StorageDrain = maxRate - StorageExfil;
		else
		{
			StorageExfil = maxRate;
			StorageDrain = 0.0;
		}
		SoilPerc = maxRate;
		PavePerc = maxRate;

		// limit surface infil. by pavement perc
		SurfaceInfil = MIN(SurfaceInfil, PavePerc);
	}

	// storage & soil layers are full
	else if (soilThickness > 0.0 &&
		storageDepth >= storageThickness &&
		soilTheta >= SoilPorosity_m3pm3)
	{
		// soil perc can't exceed storage outflow
		maxRate = StorageDrain + StorageExfil;
		if (SoilPerc > maxRate) SoilPerc = maxRate;

		// storage outflow can't exceed soil perc
		else
		{
			// use up available exfiltration capacity first
			//storageExfil (m) is subsurface flow from GI to native soil (GI=permeablePavement)
			//storageExfil (m) at end of GI waterbalance should be passed to BulkArea when GI is connected to BulkArea groundwater table
			StorageExfil = MIN(StorageExfil, SoilPerc);
			StorageDrain = SoilPerc - StorageExfil;
		}

		// limit surface infil. by available pavement volume
		availVolume = (paveThickness - paveDepth) * paveVoidFrac;
		maxRate = availVolume + PavePerc + PaveEvap;
		SurfaceInfil = MIN(SurfaceInfil, maxRate);
	}

	// soil and pavement layers are full
	else if (soilThickness > 0.0 &&
		paveDepth >= paveThickness &&
		soilTheta >= SoilPorosity_m3pm3)
	{
		PavePerc = MIN(PavePerc, SoilPerc);
		SoilPerc = PavePerc;
		SurfaceInfil = MIN(SurfaceInfil, PavePerc);
	}

	// no adjoining layers are full
	else
	{
		// limit storage exfiltration by available storage volume
		//    (if no soil layer, SoilPerc is same as PavePerc)
		maxRate = SoilPerc - StorageEvap + StorageVolume;
		maxRate = MAX(0.0, maxRate);

		//storageExfil (m) is subsurface flow from GI to native soil (GI=permeablePavement)
		//storageExfil (m) at end of GI waterbalance should be passed to BulkArea when GI is connected to BulkArea groundwater table
		StorageExfil = MIN(StorageExfil, maxRate);

		// limit underdrain flow by volume above drain offset
		if (StorageDrain > 0.0)
		{
			maxRate = -StorageExfil - StorageEvap;
			if (storageDepth >= storageThickness) maxRate += SoilPerc;
			if (folder->ParamDict["drainOffset"] <= storageDepth)
			{
				maxRate += (storageDepth - folder->ParamDict["drainOffset"]) *
					storageVoidFrac;
			}
			maxRate = MAX(maxRate, 0.0);
			StorageDrain = MIN(StorageDrain, maxRate);
		}

		// limit soil & pavement outflow by unused storage volume
		availVolume = (storageThickness - storageDepth) * storageVoidFrac;
		maxRate = availVolume + StorageEvap + StorageDrain + StorageExfil;
		maxRate = MAX(maxRate, 0.0);
		if (soilThickness > 0.0)
		{
			SoilPerc = MIN(SoilPerc, maxRate);
			maxRate = (SoilPorosity_m3pm3 - soilTheta) * soilThickness +
				SoilPerc;
		}
		PavePerc = MIN(PavePerc, maxRate);

		// limit surface infil. by available pavement volume
		availVolume = (paveThickness - paveDepth) * paveVoidFrac;
		maxRate = availVolume + PavePerc + PaveEvap;
		SurfaceInfil = MIN(SurfaceInfil, maxRate);
	}

	// PermeablePavement uses pavePerc to later redefine folder->VarDict["Infiltration_m"], total infiltration
	folder->VarDict["pavePerc"] = PavePerc;


	// surface outflow
	//SurfaceOutflow = GI_RateOfSurfaceFlow(folder, surfaceDepth);

	// compute overall layer flux rates
	f[SURF] = SurfaceInflow - SurfaceEvap - SurfaceInfil - SurfaceOutflow;
	f[PAVE] = (SurfaceInfil - PaveEvap - PavePerc + folder->VarDict["upSat_4"]) /
		paveVoidFrac;

	// This next conditional is equivalent to that used by SWMM, if ( theLidProc->soil.thickness > 0.0), so use "if MSD_m > 0" 
	if (folder->ParamDict["MSD_m"] > 0.0)
	{
		f[SOIL] = (PavePerc - SoilEvap - SoilPerc + folder->VarDict["upSat_2"]) /
			folder->ParamDict["SoilPorosity_m3pm3"];
		storageInflow = SoilPerc;
	}
	else
	{
		f[SOIL] = 0.0;
		storageInflow = PavePerc;
		SoilPerc = 0.0;
	}
	f[STOR] = (storageInflow - StorageEvap - StorageExfil - StorageDrain + folder->VarDict["upSat_3"]) /
		storageVoidFrac;


}

//Roof Disconnect layers, top to bottom: impermeable surface
void GI_Outlets::GI_FluxRatesRoofDisconnect(DataFolder* folder, double x[], double f[]) {
	//
	//  Purpose: computes flux rates for roof disconnection.
	//  Input:   x = vector of storage levels
	//  Output:  f = vector of flux rates
	//
	double surfaceDepth = x[SURF];
	//Added by Li:
	double total_Evap;

	//GI_RateOfEvaporation(folder, surfaceDepth, 0.0, 0.0, 0.0, 1.0);
	SurfaceEvap = folder->VarDict["impEvap_m"] + folder->VarDict["perDepEvap"];
	
	//total_Evap (m) is set to SurfaceEvap when in RoofDisconnect GI type
	total_Evap = SurfaceEvap;
	folder->VarDict["total_Evap"] = total_Evap;

	//SurfaceVolume (m) is set to surfaceDepth for RoofDisconnect GI type
	SurfaceVolume = surfaceDepth;

	//SurfaceInfil (m) is set to 0 for RoofDisconnect GI type
	SurfaceInfil = 0.0;

	//StorageDrain depth (m) is minimum of SurfaceOutflow, available water, or maximum rate of flow through drain, regulated with drainCoef
	StorageDrain = MIN(folder->ParamDict["drainCoef"] / folder->VarDict["waterOnImperviousArea"], SurfaceOutflow);
	
	//Reduce SurfaceOutflow (m) by flow out StorageDrain (m)
	SurfaceOutflow -= StorageDrain;

	f[SURF] = (SurfaceInflow - SurfaceEvap - StorageDrain - SurfaceOutflow);
	
}

//Rainbarrel layers, top to bottom: impermeable storage
void GI_Outlets::GI_FluxRatesRainBarrel(DataFolder* folder, double x[], double f[]) {
	//
	//  Purpose: computes flux rates for a rain barrel GI.
	//  Input:   x = vector of storage levels
	//  Output:  f = vector of flux rates
	//
	double storageDepth = x[STOR];
	double head;
	double maxValue;
	//Added by Li:
	double total_Evap;

	// assign values to layer volumes
	SurfaceVolume = 0.0;
	SoilVolume = 0.0;
	StorageVolume = storageDepth;

	// initialize flows
	SurfaceInfil = 0.0;
	SurfaceOutflow = 0.0;
	StorageDrain = 0.0;

	// compute outflow if time since last rain exceeds drain delay
	//    (dryTime is updated in lid.evalLidUnit at each time step)
	if (folder->ParamDict["drainDelay"] == 0.0 ||
		folder->VarDict["dryTime"] >= (folder->ParamDict["drainDelay"] * 60.0))
	{
		head = storageDepth - folder->ParamDict["drainOffset"];
		if (head > 0.0)
		{
			StorageDrain = GI_RateOfStorageDrain(folder, storageDepth, 0.0, 0.0, 0.0);
			maxValue = (head);
			StorageDrain = MIN(StorageDrain, maxValue);
		}
	}

	//Added by Li:
	total_Evap = 0;
	folder->VarDict["total_Evap"] = total_Evap;

	// limit inflow to available storage
	StorageInflow = SurfaceInflow;
	maxValue = (folder->ParamDict["storThickness_m"] - storageDepth) +
		StorageDrain;
	StorageInflow = MIN(StorageInflow, maxValue);
	SurfaceInfil = StorageInflow;

	// assign values to layer flux rates
	f[SURF] = SurfaceInflow - StorageInflow;
	f[STOR] = StorageInflow - StorageDrain;
	f[SOIL] = 0.0;

}

//Swale layers, top to bottom: surface, soil with no drain option, native soil
void GI_Outlets::GI_FluxRatesSwale(DataFolder* folder, double x[], double f[]) {
	//  Purpose: computes flux rates from a vegetative swale GI.
	//  Input:   x = vector of storage levels
	//  Output:  f = vector of flux rates
	//

	double depth;            // depth of surface water in swale (m)
	double topWidth;         // top width of full swale (m)
	double botWidth;         // bottom width of swale (m)
	double length;           // length of swale (m)
	double surfInflow;       // inflow rate to swale (m/dt)
	double surfWidth;        // top width at current water depth (m)
	double surfArea;         // surface area of current water depth (m2)
	double flowArea_m2;         // x-section flow area (m2)
	double giOneDeviceArea_m2;          // surface area of full swale (m2)
	double hydRadius_m;        // hydraulic radius for current depth (fmt)
	double slope;            // slope of swale side wall (run/rise)
	double volume;           // swale volume at current water depth (m3)
	double dVdT;             // change in volume w.r.t. time (m/dt)
	double dStore;           // depression storage depth (m)
	double xDepth;           // depth above depression storage (m)
	double soilTheta;		 // soil volumetric moisture (m^3/m^3)
	double soilThickness = folder->ParamDict["MSD_m"] / folder->ParamDict["SoilPorosity_m3pm3"]; // soil thickness (m) = (maximum storage depth for water) / (effective porosity)
	double soilFieldCap = folder->ParamDict["SoilFieldCapacity_m3pm3"]; // soil field capacity (m^3/m^3)
	double total_Evap; // total evaporation to maintain water balance (m)
	double maxRate; // term used to find rate limiting step in water balance algorithm
	double storageThickness = folder->ParamDict["storThickness_m"]; // storage layer below the soil, optional part of device
	double SoilPorosity_m3pm3 = folder->ParamDict["SoilPorosity_m3pm3"]; // porosity or less, if parts of porosity not available


	// retrieve state variable from work vector
	depth = x[SURF];
	// added the soil moisture in Swale; not simulated by SWMM
	soilTheta = x[SOIL];

	depth = MIN(depth, folder->ParamDict["surfaceBermH_m"]);

	// depression storage depth
	dStore = 0.0;

	// get swale's bottom width
	slope = folder->ParamDict["surfaceSideSlope"];
	topWidth = folder->ParamDict["giWidthAtOutlet_m"];
	// set to 0.5 m minimum to avoid numerical problems
	topWidth = MAX(topWidth, 0.5);
	//Eq from Chin (2013) figure 5.2
	botWidth = topWidth - 2.0 * slope * folder->ParamDict["surfaceBermH_m"];
	// set to 0.25 m minimum to avoid numerical problems
	if (botWidth < 0.25)
	{
		botWidth = 0.25;
		slope = 0.5 * (topWidth - 0.25) / folder->ParamDict["surfaceBermH_m"];
	}

	// swale's length
	giOneDeviceArea_m2 = folder->ParamDict["Area"];
	length = giOneDeviceArea_m2 / topWidth;

	// top width, surface area and flow area of current ponded depth
	surfWidth = botWidth + 2.0 * slope * depth;
	surfArea = length * surfWidth;
	//Eq 5.3 from Chin (2013) A = b*y + m*y^2, where b is bottom width, y is water depth, m is side slope
	flowArea_m2 = (depth * (botWidth + slope * depth)) *
		folder->ParamDict["surfaceVoidRatio"];

	// volume is actual water volume within swale, wet volume and effective depth
	volume = length * flowArea_m2;

	// surface inflow into swale (m/dt) * giOneDeviceArea_m2 (m^2) = q0 (m^3/dt)
	surfInflow = SurfaceInflow * giOneDeviceArea_m2;

	//Creating SurfaceEvap term (m) for water budget of device
	SurfaceEvap = folder->VarDict["impEvap_m"] + folder->VarDict["perDepEvap"];

	//SurfaceEvap is minimum of SurfaceEvap demand or available water depth (volume/giOneDeviceArea_m2) to get depth per time
	//convert from m^3 to m^3/sec for comparison with other volume per time variables, dV/dt
	SurfaceEvap = MIN(SurfaceEvap, (volume / giOneDeviceArea_m2) / folder->ParamDict["TimeStep_sec"]);

	//Creating soil evaporation for water balance
	SoilEvap = folder->VarVecDict["EvapoTranspiration_RootZone_TI_m"][0];
	//Summing total evaporation for water balance
	total_Evap = SurfaceEvap + PaveEvap + SoilEvap + StorageEvap;
	folder->VarDict["total_Evap"] = total_Evap;

	// Soil layer perc rate
	SoilPerc = GI_RateOfSoilPercolation(folder, soilTheta);

	//Loop through NAC, all topographic index values and assign same SoilPerc (m) to each Drainage_RootZone_TI_m[ia], which is flow into unsatZone
	for (int ia = 0; ia < NAC; ia++) {
		//Drainage_RootZone_TI_m (m) becomes SoilPerc (m)
		folder->VarVecDict["Drainage_RootZone_TI_m"][ia] = SoilPerc;
	}

	//storageExfil (m) is subsurface flow from GI to native soil (GI=swale)
	//storageExfil (m) at end of GI waterbalance should be passed to BulkArea when GI is connected to BulkArea groundwater table
	StorageExfil = GI_RateOfStorageExfiltration(folder);

	// Special case of no storage layer below soil, typical for swale, rainGarden and similar devices
	if (storageThickness == 0.0)
	{
		StorageEvap = 0.0;
		//maxRate is minimum of SoilPerc or StorageExfil, as rate limiting step
		maxRate = MIN(SoilPerc, StorageExfil);
		SoilPerc = maxRate;
		StorageExfil = maxRate;

		// limit surface infil. by unused soil volume
		maxRate = (SoilPorosity_m3pm3 - soilTheta) * soilThickness +
			SoilPerc + SoilEvap;
		SurfaceInfil = MIN(SurfaceInfil, maxRate);
	}

	//Theory of BulkArea (BA) groundwater entering GI cell is in development. 
	//Theory should be used to update aveSMD_GI based on condition of water table depth, Zi (m), relative to GI layers
	//  Layers are numbered #1 to 4 and correspond with upSat_# names: 1) surface, 2) soil, 3) storage, 4) pavement
	//  upSat_# represents depth (m) of water from groundwater table within each zone
	//	BA groundwater presumed to be at equilibrium with GI device at start of simulation, and at end of every timestep
	//		Equilibrium would have groundwater depth Zi >= Drainage pipe crest depth, regardless of GI localSMD
	//	BA groundwater level in GI without storage drain may rise above soil
	//	BA groundwater level in GI with storage drain <= storage drain crest height (if drain coefficient > 0) 
	//	BA groundwater fluxes from GI cell must be constrained by the transmissivity rate
	//		Groundwater presumably removed by return flow or open drain is kept separate from GI water balance q1 or q3
	//		Groundwater presumably removed by return flow or open drain is estimated using BA baseflow rates, to enforce magnitude
	//	GI water leakage to native soils, StorageExfil, is constrained to soils above BA groundwater
	//		Fraction of soil above BA groundwater should consider drainage invert elevation
	//	upSat terms should not represent rates of water movement, given constraint of transmissivity and baseflow rates
	//See Abdi PhD dissertation Chpt 5 Eqns 1 to 22 explain original ideas for monitoring Zi and impacts on GI water balance
	//	N.B. term q3_wt is the StorageDrain flow containing groundwater, and should be removed from AveSMD

	//Setting to 0 the parameter for upwelling saturation from rising watertable  
	folder->VarDict["upSat_2"] = 0.0;

	//cte 20210408 This needs to be carefully checked and perhaps corrected
	// Depth to water table, Zi, is assessed relative to layer thickness to determine adjustment to StorageExfil
	// Zi is located below the soil layer
	if (folder->VarDict["Zi"] < 0.0) 
	{ 
		folder->VarDict["Zi"] = 0.0; 
	}

	if (folder->VarDict["Zi"] >= soilThickness) 
	{
		//SoilPerc = folder->VarVecDict["Drainage_RootZone_TI_m"][folder->ParamDict["binNumber"]];
		// This is an incremental Average SMD which will be added to the total AveSMD of the bulk area
		folder->VarDict["AveSMD_GI"] = folder->VarDict["AveSMD_GI"] - (SoilPerc *
			(folder->VarDict["Zi"] / (soilThickness)));
		// updating the Drainage_UnsatZone_m
		folder->VarDict["Drainage_UnsatZone_m"] = (SoilPerc * (folder->VarDict["Zi"] / (soilThickness)));
	}

	// Zi is within soil layer 
	else if ((folder->VarDict["Zi"] < soilThickness) && (folder->VarDict["Zi"] >= 0.0)) 
	{
		folder->VarDict["AveSMD_GI"] = folder->VarDict["AveSMD_GI"] - SoilPerc;
		folder->VarDict["upSat_2"] = MAX(((soilThickness - folder->VarDict["Zi"]) * folder->ParamDict["SoilPorosity_m3pm3"]), 0.0);
		// updating the Drainage_UnsatZone_m
		folder->VarDict["Drainage_UnsatZone_m"] = SoilPerc;
	}

	// Intermediate variables
	double availVolume;
	double maxRate_Swale;
	double wettedPerimeter_m = 0;
	// no surface outflow if depth below depression storage
	xDepth = depth - dStore;
	if (xDepth <= ZERO) SurfaceOutflow = 0.0;

	// otherwise compute a surface outflow
	else
	{
		// modify flow area to remove depression storage,
		flowArea_m2 -= (dStore * (botWidth + slope * dStore)) *
			folder->ParamDict["surfaceVoidRatio"];
		if (flowArea_m2 < ZERO) SurfaceOutflow = 0.0;
		else
		{
			// compute hydraulic radius
			//cte 20210527 SWMM used botWidth = botWidth + 2.0 * dStore * slope, apparently computing top width, and then used it to compute wetted perimeter
			//cte 20210527 botWidth = botWidth + 2.0 * dStore * slope; 
			//Eq 5.4 from Chin (2013) P = b + 2*y(1+m^2)^0.5 where b is bottom width, y is water depth, m is side slope
			wettedPerimeter_m = botWidth + 2.0 * xDepth * sqrt(1.0 + slope * slope);
			hydRadius_m = flowArea_m2 / wettedPerimeter_m;

			// use Manning Eqn. to find outflow rate units of cms or m^3/second
			SurfaceOutflow = alpha * flowArea_m2 * pow(hydRadius_m, (2.0 / 3.0));

			//convert from m^3/s to m^3/dt for comparison with other volume per time variables, dV/dt
			SurfaceOutflow *= folder->ParamDict["TimeStep_sec"];
			SurfaceOutflow /= giOneDeviceArea_m2;

		}
	}
	
	// net water balance (m)
	//HydroPlus correctly computes water balance with area-adjusted depths, 
	f[SURF] = (SurfaceInflow - SurfaceEvap - SurfaceInfil - SurfaceOutflow);

	// when full, any net positive inflow becomes spillage
	// SurfaceBermH_m term is used in place of the SWMM term surface.thickness; they are equivalent
	if (depth >= folder->ParamDict["surfaceBermH_m"] && f[SURF] > 0.0)
	{
		SurfaceOutflow += f[SURF];
		f[SURF] = 0.0;
	}

	// express soil moisture as per soil thickness, as volumetric or fractional moisture (m^3/m^3)
	f[SOIL] = (SurfaceInfil - SoilEvap - SoilPerc) / soilThickness;
	f[STOR] = 0.0;

	// assign values to layer volumes
	SurfaceVolume = volume / giOneDeviceArea_m2;
	SoilVolume = 0.0;
	StorageVolume = 0.0;

	//SWMM lidproc.c code erroneously computed water balance with volume, and then adjusted to depth with surface area 
	/*
	//SWMM error below. Do not use for HydroPlus
	//dVdT = surfInflow - SurfaceEvap - StorageExfil - SurfaceOutflow
	//if (depth == theLidProc->surface.thickness && dVdT > 0.0)
	//{
	//	SurfaceOutflow += dVdT;
	//	dVdT = 0.0;
	//}
	//SurfaceEvap /= lidArea;
	//StorageExfil /= lidArea;
	//SurfaceOutflow /= lidArea;
	//f[SURF] = dVdT / surfArea
	//f[SOIL] = 0.0;
	//f[STOR] = 0.0;
	//SurfaceVolume = volume / lidArea;
	//SoilVolume = 0.0;
	//StorageVolume = 0.0;
	*/


}


//========================
//============================================================================
//============================================================================
void GI_Outlets::GI_RateOfEvaporation(DataFolder* folder, double surfaceVol, double paveVol, double soilVol,
	double storageVol, double pervFrac) {

	//  Purpose: computes surface, pavement, soil, and storage evaporation rates.
	//  Input:   
	// surfaceVol = volume/area of ponded water on surface layer (m)
	// paveVol    = volume/area of water in pavement pores (m)
	// soilVol    = volume/area of water in soil (or pavement) pores (m)
	// storageVol = volume/area of water in storage layer (m)
	// pervFrac   = fraction of surface layer that is pervious

	double availEvap;

	// surface evaporation flux
	availEvap = EvapRate; // m/dt
	SurfaceEvap = MIN(availEvap, surfaceVol);
	SurfaceEvap = MAX(0.0, SurfaceEvap);
	availEvap = MAX(0.0, (availEvap - SurfaceEvap));
	availEvap *= pervFrac;

	// no subsurface evap if water is infiltrating
	if (SurfaceInfil > 0.0)
	{
		PaveEvap = 0.0;
		SoilEvap = 0.0;
		StorageEvap = 0.0;
	}
	else
	{
		// pavement evaporation flux
		PaveEvap = MIN(availEvap, paveVol);
		availEvap = MAX(0.0, (availEvap - PaveEvap));

		// soil evaporation flux
		SoilEvap = MIN(availEvap, soilVol);
		availEvap = MAX(0.0, (availEvap - SoilEvap));

		// storage evaporation flux
		StorageEvap = MIN(availEvap, storageVol);
	}

}

//=============================================================================
double GI_Outlets::GI_RateOfSoilPercolation(DataFolder* folder, double theta) {

	//  Purpose: computes percolation rate of water through a GI's soil layer.
	//  Input:   theta = moisture content (fraction)
	//  Output:  returns percolation rate within soil layer

	double soilWaterGravitational_m3_p_m3;            // moisture deficit
	double perc;
	// no percolation if soil moisture <= field 
	if (theta <= folder->ParamDict["SoilFieldCapacity_m3pm3"]) return 0.0;

	// perc rate = unsaturated hydraulic conductivity
	soilWaterGravitational_m3_p_m3 = folder->ParamDict["SoilPorosity_m3pm3"] - theta;

	//Equation comes from SWMM code for LID processes
	//K0 hydraulic conductivity (m / timestep) is adjusted to per time step based on input as per hour and TimeStep_sec
	perc = folder->ParamDict["K0_mph"] * folder->ParamDict["TimeStep_sec"] / 3600 * exp(-soilWaterGravitational_m3_p_m3 * folder->ParamDict["soilCondSlope"]);

	//Equation comes from Abdi and Endreny to limit percolation to length of water available for gravitational drainage
	//	theta * SoilFieldCapacity_m3pm3 gives available fractional water, MSD_m * SoilPorosity_m3pm3 gives length of soil
	if (perc > (theta - folder->ParamDict["SoilFieldCapacity_m3pm3"]) *
		(folder->ParamDict["MSD_m"] / folder->ParamDict["SoilPorosity_m3pm3"])) {
		perc = (theta - folder->ParamDict["SoilFieldCapacity_m3pm3"]) * ((folder->ParamDict["MSD_m"] / folder->ParamDict["SoilPorosity_m3pm3"]));
	}
	else {
		//K0 hydraulic conductivity (m / timestep) is adjusted to per time step based on input as per hour and TimeStep_sec
		perc = folder->ParamDict["K0_mph"] * folder->ParamDict["TimeStep_sec"] / 3600 * exp(-soilWaterGravitational_m3_p_m3 * folder->ParamDict["soilCondSlope"]);
	}


	return perc;
}

//============================================================================
double GI_Outlets::GI_RateOfStorageExfiltration(DataFolder* folder) {

	//  Purpose: computes exfiltration rate from storage zone into
	//           native soil beneath a GI.
	//  Input:   depth = depth of water storage zone (m)
	//  Output:  returns exfiltration rate (mph) called infil

	double exfil = 0.0;
	double clogFactor = 0.0;

	if (folder->ParamDict["seepageRate_mph"] == 0.0) return 0.0;
	if (folder->VarDict["MaxNativeExfil"] == 0.0) return 0.0;

	// reduction due to clogging
	clogFactor = folder->ParamDict["storClogFactor"];
	if (clogFactor > 0.0)
	{
		clogFactor = folder->VarDict["GI_Inflow_m"] / clogFactor;
		clogFactor = MIN(clogFactor, 1.0);
	}

	// infiltration rate = storage storage seepageRate_mph reduced by any clogging; seepageRate_mph adjusted by TimeStep_sec to actual time step
	exfil = folder->ParamDict["seepageRate_mph"] * folder->ParamDict["TimeStep_sec"] / 3600 * (1.0 - clogFactor);

	// limit infiltration rate by any groundwater-imposed limit
	return MIN(exfil, folder->VarDict["MaxNativeExfil"]);
}

//=============================================================================
double GI_Outlets::GI_RateOfStorageDrain(DataFolder* folder, double storageDepth,
	double soilTheta, double paveDepth, double surfaceDepth) {

	//  Purpose: computes underdrain flow rate in a GI's storage layer.
	//  Input:   storageDepth = depth of water in storage layer (m)
	//           soilTheta    = moisture content of soil layer
	//           paveDepth    = effective depth of water in pavement layer (m)
	//           surfaceDepth = depth of ponded water on surface layer (m)
	//  Output:  returns flow in underdrain (m/s)

	// Abdi: Note storage depth can sharply decreases due to storage drainage >> percolation in prior time step.
	// Abdi: Adjust drainOrificeQ_coef variable to solve the problem of the rapid variations in drainage. 

	double head = storageDepth;
	double outflow = 0.0;
	double paveThickness = folder->ParamDict["paveThickness_m"];
	double soilThickness = folder->ParamDict["MSD_m"] / folder->ParamDict["SoilPorosity_m3pm3"];
	double SoilPorosity_m3pm3 = folder->ParamDict["SoilPorosity_m3pm3"];
	double soilFieldCap = folder->ParamDict["SoilFieldCapacity_m3pm3"];
	double storageThickness = folder->ParamDict["storThickness_m"];

	// Assess thickness of layers to determine which layers are saturated storage layer is full
	if (storageDepth >= storageThickness && storageThickness > 0)
	{
		//head (m) enters routine as storageDepth, actual level of water within porous media
		head = head;
		
		// a soil layer exists
		if (soilThickness > 0.0)
		{
			// increase head by fraction of soil layer saturated
			if (soilTheta > soilFieldCap)
			{
				head += (soilTheta - soilFieldCap) /
					(SoilPorosity_m3pm3 - soilFieldCap) * soilThickness;

				// soil layer is saturated, increase head by water
				//     depth in layer above it
				if (soilTheta >= SoilPorosity_m3pm3)
				{
					if (paveThickness > 0.0) head += paveDepth;
					else head += surfaceDepth;
				}
			}
		}

		// no soil layer so increase head by water level in pavement layer and possibly surface layer
		if (paveThickness > 0.0)
		{
			head += paveDepth;
			if (paveDepth >= paveThickness) head += surfaceDepth;
		}
	}

	folder->VarDict["head"] = head;

	// make head relative to drain offset
	// offset is the distance from the bottom of the storage layer to the bottom of the pipe (units meters)
	head -= folder->ParamDict["drainOffset"];

	// compute drain outflow from underdrain flow equation 
	if (head > ZERO)
	{
		head /= (0.3048); // to conver to ft

		// This equation is in the SWMM BG units (ft), and will be converted to SI (meters) per time step 
		// See how EPA SWMM interface allwos user input choice; Drain Advisor from SWMM interface
		//  compute drain outflow in user ft/timestep
		outflow = folder->ParamDict["drainOrificeQ_coef"] *
			pow(head, folder->ParamDict["drainOrificeQ_exponent"]);

		// Convert to meters/timestep
		outflow *= (0.3048); 

		//	constraining drainage outflow to minimum of head (m) or outflow
		outflow = MIN(outflow, head*0.3048);

	}
	
	//Set negative flows to 0
	if (outflow <= 0) outflow = 0;

	folder->VarDict["newDrainFlow"] = outflow;

	return outflow;
}

//=============================================================================
// Computing outflow rate using some undefined form of Manning Equation
double GI_Outlets::GI_RateOfSurfaceFlow(DataFolder* folder, double depth) {

	//  Purpose: computes outflow rate from a GI's surface layer.
	//  Input:   depth = depth of ponded water on surface layer (m)
	//  Output:  returns outflow from surface layer (m)

	//  Note: this function should not be applied to swales or rain barrels.

	double surfaceDepthAboveBerm_m;
	double outflow;
	
	// no outflow if ponded depth below storage depth of berm crest
	surfaceDepthAboveBerm_m = depth - folder->ParamDict["surfaceBermH_m"];

	if (surfaceDepthAboveBerm_m < 0.0) return 0.0;

	// Compute outflow from overland flow Manning equation from SWMM code (theory needed)
	// Based on Eq 3.3 of SWMM Ref Manual Vol I - Hydrology (2016), 
	// Derivation:	Volumetric flow = Q = 1/n * Width * Depth^(5/3) * Slope^(1/2)
	//				Q (m^3/s) is function of Manning velocity and channel cross section area
	//				XS Area = Axs = Width * Depth
	//				Hydraulic Radius = R = Depth 
	//				R^(2/3) * Axs^(3/3) = Width * Depth^(5/3)
	//				alpha = 1/n * Slope^(1/2)
	//				surfaceDepthAboveBerm_m = Depth = effective runoff depth
	//				Q (m^3/s) divided by basin area is runoff depth across basin
	outflow = alpha * pow(surfaceDepthAboveBerm_m, 5.0 / 3.0) *
		folder->ParamDict["giWidthAtOutlet_m"] / folder->ParamDict["Area"];
	
	//convert from m/s to m/dt for comparison with other depth per time variables, dD/dt
	outflow = MIN(outflow * folder->ParamDict["TimeStep_sec"], surfaceDepthAboveBerm_m);
		
	return outflow;
}

//=============================================================================
// Based on emergency spillway to reset depth to berm height and excess water is sent to overflow 
double GI_Outlets::GI_RateOfEmergencySpillway(DataFolder* folder, double surfaceDepth) {

	//  Purpose: finds surface overflow rate from a GI unit.
	//  Input:   surfaceDepth = depth of water stored in surface layer (m)
	//  Output:  returns the overflow rate (m)
	//surfaceDepthAboveEmergencyBerm_m (m) is depth of water overtopping the emergency berm height
	double surfaceDepthAboveEmergencyBerm_m = 0;
	
	//surfaceDepthAboveEmergencyBerm_m (m) is depth of water overtopping the emergency berm height
	surfaceDepthAboveEmergencyBerm_m = surfaceDepth - folder->ParamDict["surfaceEmergencyH_m"];

	//if surface water depth (m) is below emergency spillway berm, leave routine with surfaceDepthAboveEmergencyBerm_m = 0 if surfaceDepthAboveEmergencyBerm_m is negative or 0
	if (surfaceDepthAboveEmergencyBerm_m <= 0.0) {
		return 0.0;
	}

	//if surface water depth (m) overtops emergency spillway berm (surfaceDepthAboveEmergencyBerm_m > 0), then remain in routine and reset surfaceDepth
	if (surfaceDepthAboveEmergencyBerm_m > 0) {
		//surfaceDepth (m) is reset to height of surfaceEmergencyH_m now that excess was drained off to surfaceOutflow
		surfaceDepth = folder->ParamDict["surfaceEmergencyH_m"];

		//surfaceDepthAboveEmergencyBerm_m (m) is returned as depth adjusted for HydroPlusConfig.xml input of surfaceVoidRatio
		//		surfaceVoidRatio = 1 is all surface volume is voids, = 0 is all surface volume is solids
		return surfaceDepthAboveEmergencyBerm_m * folder->ParamDict["surfaceVoidRatio"];

	}
}
//=============================================================================
// Internal function to save GI results to output writer
void GI_Outlets::GI_WaterBalanceFinal(DataFolder* folder, Inputs* input) {

	//  Purpose: calls GI water balance and saves to save fluxes and storages to the GI report file.

	//totalEvap (m) is evaporation rate from GI
	double totalEvap;
	//totalVolume (m) is stored in GI, with term volume coming from SWMM and should likely be reset to depth
	double totalVolume;
	//isDry is flag set to TRUE if current state of GI is dry
	int isDry = FALSE;
	//elapsedHrs is counter, cte 20210408 we may want to use this for regulating when we transfer water
	double elapsedHrs;

	//Assign null value to pavePerc if GI device is not PermeablePavement
	if (folder->ParamStringDict["Type"] != "PermeablePavement") {
		folder->VarDict["pavePerc"] = 0;
	}

	//totalEvap (m) is equal to sum of all GI evaporation depths, from surface, pavement, soil, and storage
	totalEvap = SurfaceEvap + PaveEvap + SoilEvap + StorageEvap;

	//Set depths (m) less than 0 to 0
	if (totalEvap < 0.0) { totalEvap = 0; }

	//totalVolume (m) is sum of all GI storage depths, from surface, pavement, soil, and storage (terms below are static variables, globally defined)
	totalVolume = SurfaceVolume + PaveVolume + SoilVolume + StorageVolume;

	//GI_WaterBalanceStoreTerms will update mass balance totals writing to WaterBalance_[GI].csv
	GI_WaterBalanceStoreTerms(folder, SurfaceInflow, SurfaceInfil, SoilPerc, StorageExfil,
			SurfaceOutflow, StorageDrain, folder->VarDict["paveDepth"], 
			folder->VarDict["surfaceDepth"],folder->VarDict["soilMoisture"],
			folder->VarDict["storageDepth"], folder->VarDict["pavePerc"], folder->VarDict["total_Evap"]);
	
	//Transfer StorageExfil water from GI to Catchment; SurfaceOutflow and StorageDrain transfer in GI_WaterBalanceStoreTerms
	//GIstorageExfil (m3) is subsurface flow from GI to native soil, with option to or not to connect with BulkArea
	//GIstorageExfil (m3) passed in UnsatZoneDrainageCalc.cpp to BulkArea when GI is connected to BulkArea groundwater table
	if (folder->ParamDict["storThickness_m"] == 0.0) {

		//Transfer of GI pervious area drainage water to Bulk Area pervious saturated zone; converting to volume to uphold conservation of mass in transfer
		//GIstorageExfil_m3 (m^3) is volume of SoilPerc (m) at pervious area depth that will be put into Bulk Area saturated zone in LocalSoilMoistureCalc.cpp
		input->RepoDict["GIstorageExfil_m3"] = SoilPerc * folder->ParamDict["Area"] * folder->ParamDict["TotalPerviousCover_frac"] * folder->ParamDict["giNumber"];
	}
	else {

		//Transfer of GI pervious area drainage water to Bulk Area pervious saturated zone; converting to volume to uphold conservation of mass in transfer
		//GIstorageExfil_m3 (m^3) is volume of StorageExfil (m) at pervious area depth that will be put into Bulk Area saturated zone in LocalSoilMoistureCalc.cpp
		input->RepoDict["GIstorageExfil_m3"] = StorageExfil * folder->ParamDict["Area"] * folder->ParamDict["TotalPerviousCover_frac"] * folder->ParamDict["giNumber"];
	}

	//cte 20210520 Moved this from GI_WaterBalanceStoreTerms and put into GI_WaterBalanceFinal, where it is done using input->RepoDict rather than folder->VarDict
	//Transfer of GI total area surface water to Bulk Area outlet; converting to volume to uphold conservation of mass in transfer
	//GIsurfaceOutflow_m3 (m^3) is volume of SurfaceOutflow (m) at total area depth that will be put into Bulk Area outlet in FlowSumCalc.cpp
	input->RepoDict["GIsurfaceOutflow_m3"] = SurfaceOutflow * folder->ParamDict["Area"] * folder->ParamDict["giNumber"];

	//Transfer of GI pervious area drainage water to Bulk Area outlet; converting to volume to uphold conservation of mass in transfer
	//GIstorageDrain_m3 (m^3) is volume of StorageDrain (m) at pervious area depth that will be put into Bulk Area outlet in FlowSumCalc.cpp
	input->RepoDict["GIstorageDrain_m3"] = StorageDrain * folder->ParamDict["Area"] * folder->ParamDict["TotalPerviousCover_frac"] * folder->ParamDict["giNumber"];

	// check if dry-weather conditions hold
	if (SurfaceInflow < MINFLOW &&
		SurfaceOutflow < MINFLOW &&
		StorageDrain < MINFLOW &&
		StorageExfil < MINFLOW &&
		totalEvap < MINFLOW
		) isDry = TRUE;
}

//=============================================================================
// Providing vectors for output writer, generic to all GI but Permeable Pavement (which has unique variable pavePerc_out)
void GI_Outlets::GI_WaterBalanceStoreTerms(DataFolder* folder, double inflow, double infil, double soilPerc, double exfil,
	double surfFlow, double drainFlow, double paveDepth, double surfaceDepth, double soilMoisture,
	double storageDepth, double pavePerc, double total_Evap) {

	//  Purpose: updates components of the water mass balance for a GI unit over the current time step.

	folder->VarDict["GI_Inflow_m"] = inflow;
	folder->VarDict["GI_Infiltration_m"] = infil;
	folder->VarDict["GI_SoilPercolation_m"] = soilPerc;
	folder->VarDict["GI_StorageExfiltration_m"] = exfil;
	folder->VarDict["GI_SurfaceOutflow_m"] = surfFlow;
	folder->VarDict["GI_StorageDrainFlow_m"] = drainFlow;
	folder->VarDict["GI_PavementStorageDepth_m"] = paveDepth;
	folder->VarDict["GI_SurfaceDepth_m"] = surfaceDepth;
	folder->VarDict["GI_SoilMoisture_m3_p_m3"] = soilMoisture;
	folder->VarDict["GI_StorageDepth_m"] = storageDepth;
	folder->VarDict["GI_PavementPercolation_m"] = pavePerc;
	folder->VarDict["GI_GI_EvaporationAllLayers_m"] = total_Evap;

	//Optionally record water transfered water from GI to Catchment; using a volume to uphold conservation of mass in transfer
	//Commented out to save memory until these are needed
	//folder->VarVecDict["GIstorageExfil_m3"].emplace_back(folder->VarDict["GIstorageExfil_m3"]);
	//folder->VarVecDict["GIsurfaceOutflow_m3"].emplace_back(folder->VarDict["GIsurfaceOutflow_m3"]);
	//folder->VarVecDict["GIstorageDrain_m3"].emplace_back(folder->VarDict["GIstorageDrain_m3"]);

	folder->VarVecDict["GI_Inflow_m"].emplace_back(folder->VarDict["GI_Inflow_m"]);
	folder->VarVecDict["GI_Infiltration_m"].emplace_back(folder->VarDict["GI_Infiltration_m"]);
	folder->VarVecDict["GI_SoilPercolation_m"].emplace_back(folder->VarDict["GI_SoilPercolation_m"]);
	folder->VarVecDict["GI_StorageExfiltration_m"].emplace_back(folder->VarDict["GI_StorageExfiltration_m"]);
	folder->VarVecDict["GI_SurfaceOutflow_m"].emplace_back(folder->VarDict["GI_SurfaceOutflow_m"]);
	folder->VarVecDict["GI_StorageDrainFlow_m"].emplace_back(folder->VarDict["GI_StorageDrainFlow_m"]);
	folder->VarVecDict["GI_PavementStorageDepth_m"].emplace_back(folder->VarDict["GI_PavementStorageDepth_m"]);
	folder->VarVecDict["GI_SurfaceDepth_m"].emplace_back(folder->VarDict["GI_SurfaceDepth_m"]);
	folder->VarVecDict["GI_SoilMoisture_m3_p_m3"].emplace_back(folder->VarDict["GI_SoilMoisture_m3_p_m3"]);
	folder->VarVecDict["GI_StorageDepth_m"].emplace_back(folder->VarDict["GI_StorageDepth_m"]);
	folder->VarVecDict["GI_PavementPercolation_m"].emplace_back(folder->VarDict["GI_PavementPercolation_m"]);
	folder->VarVecDict["GI_GI_EvaporationAllLayers_m"].emplace_back(folder->VarDict["GI_GI_EvaporationAllLayers_m"]);
	folder->VarVecDict["GI_WaterTableDepth_m"].emplace_back(folder->VarDict["Zi"]);
	folder->VarVecDict["upSat_2"].emplace_back(folder->VarDict["upSat_2"]);
	folder->VarVecDict["upSat_3"].emplace_back(folder->VarDict["upSat_3"]);
	folder->VarVecDict["upSat_4"].emplace_back(folder->VarDict["upSat_4"]);
	
}

//=============================================================================
//Called from TestSimulation at beginning of time step, after bulkArea complete, to initiate the GI vectors and variables 
void GI_Outlets::GI_WaterBalanceInitial(DataFolder* folder) {

	//  Purpose: initializes the water balance components of a GI unit.

	double initVol = 0.0;
	double soilMoisture = 0.0;
	double storageDepth = 0.0;
	//Li: for keeping ParamDict["soilThickness_m"]
	double sthick = 0.0;
	//folder->ParamDict["soilThickness_m"] = folder->ParamDict["MSD_m"] / folder->ParamDict["SoilPorosity_m3pm3"];

	//cte 20210408 Li, why did you create sthick rather than use folder->ParamDict["soilThickness_m"] ?
	sthick = folder->ParamDict["MSD_m"] / folder->ParamDict["SoilPorosity_m3pm3"];
	//initiallySat_frac (range 0 to 1) used to modify SoilPorosity_m3pm3 (frac) and soil thickness and get depth of water in soil zones
	//	Initialized by SD0_percent (0 = saturated, 100 = dry), initial soil moisture deficit parameter from HydroPlusConfig.xml
	folder->ParamDict["initiallySat_frac"] = (1.0 - (folder->ParamDict["SD0_percent"] / 100.0));

	/*if (folder->ParamDict["soilThickness_m"] > 0.0)
	{
		soilMoisture = folder->ParamDict["soilThickness_m"] +
			folder->ParamDict["initiallySat_frac"] * (folder->ParamDict["SoilPorosity_m3pm3"] -
				folder->ParamDict["SoilWiltingPoint_m3pm3"]);
		initVol += soilMoisture * folder->ParamDict["soilThickness_m"];
	}*/
	//Li:
	if (sthick > 0.0)
	{
		soilMoisture = sthick +
			folder->ParamDict["initiallySat_frac"] * (folder->ParamDict["SoilPorosity_m3pm3"] -
				folder->ParamDict["SoilWiltingPoint_m3pm3"]);
		initVol += soilMoisture * sthick;
	}
	if (folder->ParamDict["storThickness_m"] > 0.0)
	{
		storageDepth = folder->ParamDict["initiallySat_frac"] *
			folder->ParamDict["storThickness_m"];
		initVol += storageDepth * folder->ParamDict["storVoidFrac"];
	}
	if (folder->ParamDict["drainMatThickness_m"] > 0.0)
	{
		storageDepth = folder->ParamDict["initiallySat_frac"] *
			folder->ParamDict["drainMatThickness_m"];
		initVol += storageDepth * folder->ParamDict["drainMatThickness_m"];
	}

	folder->VarDict["VolTreated"] = 0.0;
	folder->VarDict["GI_Inflow_m"] = 0.0;
	folder->VarDict["waterBalanceEvap"] = 0.0;
	folder->VarDict["GI_Infiltration_m"] = 0.0;
	folder->VarDict["GI_SurfaceOutflow_m"] = 0.0;
	folder->VarDict["GI_StorageDrainFlow_m"] = 0.0;
	folder->VarDict["waterBalanceInitVol"] = initVol;
	folder->VarDict["waterBalanceFinalVol"] = initVol;

	folder->VarDict["waterToPerDep_GI"] = 0.0;
	folder->VarDict["waterOnImperviousArea_GI"] = 0.0;

	//iaIndex = folder->ParamDict["iaIndex"];
	
}

//Called from TestSimulation after PerviousDepressionStorage:calculate, before entering functors["calculateInfiltration"], to ensure BulkArea runon into GI is transferred
//GI_SurfaceInflowSurfaceOutflow transfers BulkArea runoff into GI; runon into GI generated by BulkArea Runoff_PervInfilEx_m, Runoff_SatExcess_m, Runoff_ImperviousToOutlet_m
void GI_Outlets::GI_SurfaceInflowSurfaceOutflow(Inputs* input, DataFolder* folder, int timestep) {
	//
	//  Purpose: computes runoff and drain flows from the GIs for the available data folders.
	//  Output:  updates following global quantities after GI applied:
	//           Vevap, Vpevap, VGiInfil, VlidIn, VlidOut, VlidDrain.
	//

	//qImperv (m) is runoff from GI impervious areas
	double qImperv = 0.0;
	//qPerv (m) is runoff from GI pervious areas
	double qPerv = 0.0;
	//qRunoff (m) is all GI surface runoff
	double qRunoff = 0.0;
	//qDrain (m) is GI subsurface drain flow
	double qDrain = 0.0;
	//SurfaceOutflow(m) is surface flow leaving GI, denoted as q1 in SWMM; static double shareable across all classes
	SurfaceOutflow = 0.0;


	//Exit this GI routine when product of giNumber and Area = 0
	if (folder->ParamDict["giNumber"] * folder->ParamDict["Area"] == 0.0) {
		cout << "GI_Outlets:GI_SurfaceInflowSurfaceOutflow GI calculations ignored due to HydroPlusConfig.xml keys giNumber * Area = 0." << endl;
		return;
	}
	//Transfer water into GI, from BulkArea (accumulated during last timestep) and from precipitation landing on GI surfaces
	//giInflow (m) transfers to GI the BulkArea runoff and precipitation landing on GI surface, giInflow becomes q1 or SurfaceInflow 
	//	giInflow RHS terms were previously adjusted to depth appropriate for GI area, maintaining continuity
	//	waterToPerDep_pre (m) is precipitation on GI pervious area + BulkArea pervious (infExQ satExQ) and impervious (DCIAQ) runoff
	//	waterOnImperviousArea (m) is precipitation on GI impervious area
	//	giInflow is q0 term used in SWMM
	//cte 20210715 Rename waterToPerDep_pre into two terms, PrecipAndRunonOnPervious_m and Runoff_BA_to_GI_m, and rename waterOnImpervousArea PrecipOnImpervious_m
	folder->VarDict["giInflow"] = folder->VarDict["waterToPerDep_pre"] + folder->VarDict["waterOnImperviousArea"]; // PAQ + DCIAQ + Rainfall


	//Transfer surface water runoff depth (m) out of GI to catchment outlet 
	//surfaceRoughness is the Manning n value, and must be larger than zero to compute runoff velocity (or discharge)
	if (folder->ParamDict["surfaceRoughness"] > 0.0) {
		//alpha represents 2 terms on RHS of Manning Eq, squareroot of longitudinal slope divided by surfaceRoughness
		alpha = sqrt(folder->ParamDict["surfaceSlope_frac"]) / folder->ParamDict["surfaceRoughness"];
	}
	//alpha represents 2 terms on RHS of Manning Eq, squareroot of longitudinal slope divided by surfaceRoughness
	else alpha = 0.0;

	if (folder->ParamDict["drainRoughness"] > 0.0) {
		//alpha_Drain represents 2 terms on RHS of Manning Eq, squareroot of longitudinal slope divided by surfaceRoughness
		alpha_Drain = sqrt(folder->ParamDict["surfaceSlope_frac"]) / folder->ParamDict["drainRoughness"];
	}
	//alpha_Drain represents 2 terms on RHS of Manning Eq, squareroot of longitudinal slope divided by surfaceRoughness
	else alpha_Drain = 0.0;

	//cte 20210715 seems SurfaceOutflow does not leave this routine, and calculatons are unneeded. 
	//SurfaceOutflow (m) is surface flow leaving GI, for all GI except RainBarrel and Swale (which use different functions)
	if (folder->ParamStringDict["Type"] != "RainBarrel" && folder->ParamStringDict["Type"] != "Swale") {

		//Calculate SurfaceOutflow, q1 term used in SWMM, based on the surfaceDepth (m) relative to berm height
		SurfaceOutflow = GI_RateOfSurfaceFlow(folder, folder->VarDict["surfaceDepth"]);
	}

	//Transfer emergency spillway water overflow depth (m) water out of GI to catchment outlet
	//surfaceCanOverflow = TRUE allows surface spillage from GI. 
	surfaceCanOverflow = TRUE;

	//cte 20210408 Li, what is done with this EvapRate in this routine, and is this needed
	//EvapRate (m) is evaporation from GI
	//EvapRate = folder->VarDict["totalEvap"];

	//NativeInfil (m) is infiltration from GI to native soils of BulkArea 
	folder->VarDict["NativeInfil"] = folder->VarDict["infilToSoil"];
	//Set MaxNativeExfil (m) to large value BIG defined in GI_Outlets.h; may depend on groundwater
	folder->VarDict["MaxNativeExfil"] = BIG;

	//cte 20210408 Li, does SWMM require giWidthAtOutlet_m be 0 to have overflow? This seems wrong, as water can leave outlet and emergency spillway
	//if (surfaceCanOverflow || folder->ParamDict["giWidthAtOutlet_m"] == 0.0)
	if (surfaceCanOverflow)
	{
		//SurfaceOutflow is updated to include additional water overtopping the berm height
		SurfaceOutflow += GI_RateOfEmergencySpillway(folder, folder->VarDict["surfaceDepth"]);
	}
	
	//Set water depths (m) to 0 if values are negative
	if (SurfaceOutflow < 0.0) {SurfaceOutflow = 0.0;}

	//Set water depths (m) to 0 if values are negative
	if (folder->VarDict["waterOnImperviousArea"] < 0) {folder->VarDict["waterOnImperviousArea"] = 0.0;}
	if (folder->VarDict["waterToPerDep"] < 0) {folder->VarDict["waterToPerDep"] = 0.0;}
	if (folder->VarDict["waterOnPerviousArea"] < 0) {folder->VarDict["waterOnPerviousArea"] = 0.0;}

}

double GI_Outlets::GI_RateOfDrainageMat(DataFolder* folder, double depth)
//
//  Purpose: computes flow rate through a green roof's drainage mat.
//  Input:   depth = depth of water in drainage mat (m)
//  Output:  returns flow in drainage mat (m/s)
//
{
	//... default is to pass all inflow
	double result = SoilPerc;

	//... otherwise use Manning eqn. if its parameters were supplied
	if (alpha_Drain > 0.0)
	{
		result = alpha_Drain * pow(depth, 5.0 / 3.0) *
			folder->ParamDict["giWidthAtOutlet_m"] / folder->ParamDict["Area"] *
			folder->ParamDict["drainVoidFrac"];
	}
	return result;
}

// cte: This regeneration option is currently not active; uncomment and test against SWMM code if needed 
//=============================================================================
double GI_Outlets::GI_CloggingPermeablePavement(DataFolder* folder) {

	//
	//  Purpose: computes reduced permeability of a pavement layer due to clogging.
	//  Input:   none
	//  Output:  returns the reduced permeability of the pavement layer.
	//

	double permReduction = 0.0;
	double clogFactor = folder->ParamDict["paveClogFactor"];
	double regenDays = folder->ParamDict["paveRegenDays"];

	// find permeability reduction due to clogging     
	if (clogFactor > 0.0)
	{
		//  see if permeability regeneration has occurred (regeneration is assumed to reduce the total volumetric loading that the pavement has received)
		if (regenDays > 0.0)
		{
			
			//if (OldRunoffTime / 1000.0 / SECperDAY >= theLidUnit->nextRegenDay)
			//{
			//	// ... reduce total volume treated by degree of regeneration
			//	theLidUnit->volTreated *=
			//		(1.0 - theLidProc->pavement.regenDegree);

			//	// ... update next day that regenration occurs
			//	theLidUnit->nextRegenDay += regenDays;
			//}

		}

		// find permeabiity reduction factor
		//permReduction = theLidUnit->volTreated / clogFactor;
		//permReduction = MIN(permReduction, 1.0);
	}

	// return the effective pavement permeability
	return (folder->ParamDict["pavePermeability_mph"] * (1.0 - permReduction)) / folder->VarDict["rainfall"];
}

//=============================================================================

