#ifndef GIOUTLETS_H
#define GIOUTLETS_H
//////////////////////////////////////////////////////////////////////////////////
//																		        //
// GI_Outlets.h - Green Infrastructure algorithms designed for the i-Tree Hydro //
//                       addupted from SWMM v5.1 LID controls                   //																		   
// ver 1.0                                                                      //
// Language:    C++ (Visual Studio 2019)                                        //
// Platform:    Win10           										        //
// Application: Unified Hydro, Winter, Spring, and Summer 2019                  //
// Programmer:  Reza Abdi, reabdi@syr.edu               				        //
//                                                                              //
//////////////////////////////////////////////////////////////////////////////////
/*
Origin:
==================
Program: SWMM v5.1

Source Code File(s): Lid.c & LidProc.c in SWMM v5.1 source code

Publication(s): Reza Abdi PHDThesis

Module Operations:
==================
...
/*
Public Interface:
=================


Related changes on other routines:
==============
//SimulationNumericalParams["TotalTimeSteps"] initially defined in HydroPlusConfig.xml, and redefined in Inputs::readHydroPlusConfigFile() if value in HydroPlusConfig.xml = 0 or empty, then TotalTimeSteps = daysDiff * 24.0
ImperviousDepressionStorageCalc.cpp
InfiltrationExpDecayCalc.cpp
InfiltrationPowDecayCalc.cpp
TestSimulation.cpp
UnsatZoneSaturationExcessQ.cpp
WaterOnPerviousAreaCalc.cpp
FlowSumCalc.cpp
RootZoneETCalc.cpp
SemiAveSMDCalc.cpp

Maintenance History:
====================

*/


#include "../Inputs/Inputs.h"
#include "../DataFolder.h"

//Define constants used in GI_Outlets.cpp
// 4 layers are: surface, soil, storage, and pavement
#define MAX_LAYERS 4		
#define BIG 1.E10
// Effective zero value
#define ZERO 1.E-10         
// Integration error tolerance set to 0.001 m = 1 mm
#define STOPTOL  0.001      
// Value for TRUE state
#define TRUE     1              
// Value for FALSE state
#define FALSE    0              
#define SECperDAY   86400
// Set runoff minumum from SWMM 0.001 in/hr to m/sec
#define MIN_RUNOFF  7.06e-9     
// Set flow cutoff for dry conditions from SWMM 0.001 in/hr to m/sec
#define MINFLOW  7.06e-9        

// Function for finding minimum of x and y    
#define MIN(x,y) (((x)<=(y)) ? (x) : (y))        
// Function for finding maximum of x and y    
#define MAX(x,y) (((x)>=(y)) ? (x) : (y))        



class GI_Outlets
{
public:

	// Get water balance (runoff, drainage, infil, evap) for GI units; Call GI_WaterBalanceUpdate
	static void GI_WaterBalancePrepareUpdate(Inputs* input, DataFolder* folder, int timestep);
	// Get water balance (runoff, drainage, infil, evap) for GI units; Call GI_FluxRatesAllTypesInitialize 
	static void GI_WaterBalanceUpdate(DataFolder* folder, double* giEvap, double* giInfil, double* giDrain, int timestep);
	// Call flux rate function for the specific GI device, Get water levels and flux rates over the time step
	static void GI_FluxRatesAllTypesInitialize(DataFolder* folder, int n, double* x, double* xOld, double* xPrev,
		double* xMin, double* xMax, double* xTol, double* qOld, double* q, double dt);

	// Called by GI_FluxRatesAllTypesInitialize: computes flux rates from the layers of a bioretention
	static void GI_FluxRatesBioRetentionRaingarden(DataFolder* folder, double x[], double f[]);
	// Called by GI_FluxRatesAllTypesInitialize: computes flux rates from the layers of a green roof
	static void GI_FluxRatesGreenRoof(DataFolder* folder, double x[], double f[]);
	// Called by GI_FluxRatesAllTypesInitialize: computes flux rates from the layers of a infiltration trench
	static void GI_FluxRatesInfiltrationTrench(DataFolder* folder, double x[], double f[]);
	// Called by GI_FluxRatesAllTypesInitialize: computes flux rates from the layers of a permeable pavement
	static void GI_FluxRatesPermeablePavement(DataFolder* folder, double x[], double f[]);
	// Called by GI_FluxRatesAllTypesInitialize: computes flux rates from the layers of a rain barrel
	static void GI_FluxRatesRainBarrel(DataFolder* folder, double x[], double f[]);
	// Called by GI_FluxRatesAllTypesInitialize: computes flux rates from the layers of a roof disconnect
	static void GI_FluxRatesRoofDisconnect(DataFolder* folder, double x[], double f[]);
	// Called by GI_FluxRatesAllTypesInitialize: computes flux rates from the layers of a swale
	static void GI_FluxRatesSwale(DataFolder* folder, double x[], double f[]);

	// Not used by i-Tree Hydro; was SWWM function to get GI evaporation rates.
	static void GI_RateOfEvaporation(DataFolder* folder, double surfaceVol, double paveVol, double soilVol, double storageVol, double pervFrac);
	// Get percolation rate of water through a GI's soil layer; using SWMM function; Hydro rootZoneQ in GI area = percolation
	static double GI_RateOfSoilPercolation(DataFolder* folder, double theta);
	// Get exfiltration rate from storage zone into the native soil; this water decreases the AveSMD_GI, which updates AveSMD of the BulkArea and affects baseflow
	static double GI_RateOfStorageExfiltration(DataFolder* folder);
	// Get underdrain flow rate in a GI's storage layer; this water increases the AveSMD_GI, which updates AveSMD of the BulkArea and affects baseflow
	static double GI_RateOfStorageDrain(DataFolder* folder, double storageDepth,
		double soilTheta, double paveDepth, double surfaceDepth);
	// Get outflow rate from a GI's surface layer.
	static double GI_RateOfSurfaceFlow(DataFolder* folder, double surfaceDepth);
	// The function finds/computes surface overflow rate from a GI unit.
	static double GI_RateOfEmergencySpillway(DataFolder* folder, double surfaceDepth);
	// In this function the code updates the mass balance for a GI unit and saves
	//  current flux rates to the GI report file.
	static void GI_WaterBalanceFinal(DataFolder* folder, Inputs* input);
	// The function updates components of the water mass balance for a GI unit
	// The vectors then are used in the output writer functions
	//Li:Added the argument double total_Evap
	static void GI_WaterBalanceStoreTerms(DataFolder* folder, double inflow, double infil, double soilPerc, double exfil,
		double surfFlow, double drainFlow, double paveDepth, double surfaceDepth,
		double soilMoisture, double storageDepth, double pavePerc, double total_Evap);
	// The function updates components of the water mass balance for a GI unit over the current time step.

	/*cte 20210408 Li, can we remove this 
	static void GI_WaterBalanceFinalPermeablePavement(DataFolder* folder, double inflow, double infil, double soilPerc, double exfil,
		double surfFlow, double drainFlow, double paveDepth, double surfaceDepth,
		double soilMoisture, double storageDepth, double pavePerc, double total_Evap);
		*/

	// The function computes flow rate through a green roof's drainage mat.
	static double GI_RateOfDrainageMat(DataFolder* folder, double depth);
	// The function initializes the water balance components of a GI unit
	static void GI_WaterBalanceInitial(DataFolder* folder);
	// The function computes reduced permeability of a pavement layer due to
	//  clogging.
	static double GI_CloggingPermeablePavement(DataFolder* folder);
	//GI_SurfaceInflowSurfaceOutflow transfers BulkArea runoff into GI; runon into GI generated by BulkArea Runoff_PervInfilEx_m, Runoff_SatExcess_m, Runoff_ImperviousToOutlet_m
	static void GI_SurfaceInflowSurfaceOutflow(Inputs* input, DataFolder* folder, int timestep);

};





#endif