#include "ImperviousDepressionStorageCalc.h"

void ImperviousDepressionStorageCalc::calculateImpEvapTemperature(Inputs *input, DataFolder *folder, int timeStep)
{
//Latent heat of vaporization (J/kg)
#define lamda 2257000.0 
	//ImperviousLE > 0 when water is available, determined in HeatFluxCal::StateCal wateronImp = folder->VarDict["impDepStor_m"] / UrbanDepression;
	//cte 20210609 seems folder->VarDict["ImperviousLE"] is undefined, and should be folder->VarDict["ImperviousLE_W_p_m2"]
	//folder->VarDict["impEvap_m"] = folder->VarDict["ImperviousLE"] / lamda;
	folder->VarDict["impEvap_m"] = folder->VarDict["ImperviousLE_W_p_m2"] / lamda;
}

void ImperviousDepressionStorageCalc::calculateImpEvapStatistical(Inputs *input, DataFolder *folder, int timeStep)
{
	if (folder->VarDict["impDepStor_m"] > 0.0) {
		folder->VarDict["impEvap_m"] = (folder->VarDict["impDepStor_m"] / folder->VarDict["impDepStorMax_m"]) * input->PE[timeStep];
	}
	else
		folder->VarDict["impEvap_m"] = 0.0;
}

void ImperviousDepressionStorageCalc::calculate(Inputs *input, DataFolder *folder, int timeStep, std::map<std::string, void(*)(Inputs *input, DataFolder *folder, int timeStep)> &functors)
{
	// impDepStorMax = param.dat impervious storage maximum value (in mm) * 0.001 -> to convert to meters
	//ImperviousDepressionStorage_mm, if needed, could be updated for spatial Hydro (Cool Air) to obtain a value for each category of impervious cover (e.g., 21, .., 24), or based on slope (e.g., greater slope, lower depth)
	folder->VarDict["impDepStorMax_m"] = 0.001 * folder->ParamDict["ImperviousDepressionStorage_mm"];

	//PerviousAreaOfFolder_frac is fraction of folder area in pervious cover, used to check (1-DCIA) based partitioning of flow from impervious area to pervious area
	double PerviousAreaOfFolder_frac = 0.0;
	PerviousAreaOfFolder_frac = folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] + folder->ParamDict["ShortVegetationCover_frac"] + folder->ParamDict["SoilCover_frac"];

	// Impervious Depression Storage Previous = impDepStor  (Shifts the current impervious depression storage to the temp variable)
	double impDepStorPrev_m = folder->VarDict["impDepStor_m"];
	double impPrecip_m = 0.0;
	// increment in pervious Storage
	double incrImpDepStor = 0.0; 
    impPrecip_m = folder->VarDict["waterOnImperviousArea"];
	if (folder->VarDict["snowOpen"] <= 0.0)
	{
		functors["calculateImpEvap"](input, folder, timeStep);
	}
	else {
		// Otherwise set CurrentEimp (0) - the water won't evaporate if there is snow on the ground.
		// and we must protect against divide by 0 errors
		folder->VarDict["impEvap_m"] = 0.0;
	}

	//If ImperviousDepressionStorage_mm set to zero in HydroPlusConfig.xml, then impEvap_m = zero 
	if (folder->ParamDict["ImperviousDepressionStorage_mm"] <= 0.0)
		folder->VarDict["impEvap_m"] = 0.0;


	// Modify the (current)  Impervious Depression Storage = Impervious Depression Storage + Precip_mpdt - Evap
	folder->VarDict["impDepStor_m"] = folder->VarDict["impDepStor_m"] + impPrecip_m - folder->VarDict["impEvap_m"]; // update interception storage

	         
	//No impervious area runoff generated if impDepStor_m < impDepStorMax_m
	if(folder->VarDict["impDepStor_m"] < folder->VarDict["impDepStorMax_m"])
	{
			folder->VarDict["Runoff_ImperviousToOutlet_m"] = 0.0;          // If the amount stored is less than the max, there is no Impervious runoff
			folder->VarDict["Runoff_ImperviousToPervious_m"] = 0.0;     // If the amount stored is less than the max, ther is no Impervious flow to soil
	}

	//Potential for impervious area runoff when impDepStor_m >= impDepStorMax_m, depending on depression storage level at previous timestep and evaporation
	//	Impervious area runoff is equal to all precipitation minus change in storage and evaporation
	else if(folder->VarDict["impDepStor_m"] >= folder->VarDict["impDepStorMax_m"])
	{
		//cte 20210520 consider keeping Runoff_ImperviousToPervious_m as depth for impervious area, not folder area, until StoreTimeStepData or StatisticalOutputWriter
		//Pervious area allows for division of impervious area runoff into flow going to outlet and flow going to pervious area, based on DCIA fraction
		//Runoff_ImperviousToPervious_m (m) is depth of impervious runoff going to pervious area, adjusted to catchment depth by impervious area fraction not draining to outlet
		//Continuity maintained when adjusting to folder area from cover type area (e.g., pervious or impervious). 
		//		Depth_adjusted_CoverType_m = Depth_unadjusted_CoverType_m * Area_CoverType_fraction = Depth_unadjusted_CoverType_m * Area_CoverType_m2 / Area_Folder_m2 
		folder->VarDict["Runoff_ImperviousToPervious_m"] = (impPrecip_m - (folder->VarDict["impDepStorMax_m"] - impDepStorPrev_m) - folder->VarDict["impEvap_m"]) 
			* folder->ParamDict["ImpAreaNotDirectlyConnected2Outlet_frac"];

		//cte 20210520 consider keeping Runoff_ImperviousToOutlet_m as depth for impervious area, not folder area, until FlowSumCalc
		//Runoff_ImperviousToOutlet_m (m) is depth of impervious runoff going to outlet, adjusted to catchment depth by impervious area fraction draining to outlet
		//Continuity maintained when adjusting to folder area from cover type area (e.g., pervious or impervious). 
		//		Depth_adjusted_CoverType_m = Depth_unadjusted_CoverType_m * Area_CoverType_fraction = Depth_unadjusted_CoverType_m * Area_CoverType_m2 / Area_Folder_m2 
		folder->VarDict["Runoff_ImperviousToOutlet_m"] = (impPrecip_m - (folder->VarDict["impDepStorMax_m"] - impDepStorPrev_m) - folder->VarDict["impEvap_m"])
			* folder->ParamDict["ImpAreaDirectlyConnected2Outlet_frac"];

		//The impervious area depression storage is filled at end of timestep 
		folder->VarDict["impDepStor_m"] = folder->VarDict["impDepStorMax_m"];
	}

	// Making sure that impervious runoff on the DCIA isn't < 0
	if(folder->VarDict["Runoff_ImperviousToOutlet_m"] < 0.0)
		folder->VarDict["Runoff_ImperviousToOutlet_m"] = 0.0;

	// Making sure that impervious runoff on the (1-DCIA) isn't < 0
	if(folder->VarDict["Runoff_ImperviousToPervious_m"] < 0.0)
		folder->VarDict["Runoff_ImperviousToPervious_m"] = 0.0;

	// Making sure that the amount currently stored in impervious depression storage isn't < 0
	if (folder->VarDict["impDepStor_m"] < 0.0)
		folder->VarDict["impDepStor_m"] = 0.0;


	//If GI is operational in HydroPlusConfig.xml then implement the transfer of water between BulkArea and GI
	//Determine the directly connected impervious area runoff (m) for the BulkArea, and take fraction as runon to the GI area
	if (input->RepoDict["GI_FolderOperational"] = 1 && folder->ParamStringDict["Type"] == "BulkArea") {

		//RepoDict is used for transfer between BulkArea and GI area folders
		//BulkArea area (m2) transferred to GI
		input->RepoDict["BulkArea_Area_m2"] = folder->ParamDict["Area"];

		//Fraction of BulkArea directly connected impervious cover draining to GI area, converting Bulk Area draining to outlet to draining to GI
		//BulkAreaDCIA_DrainingToGI_frac (frac) is from HydroPlusConfig.xml as Bulk Area DCIA draining to GI
		double BulkAreaImperviousDrainingToGI_frac = input->RepoDict["BulkAreaDCIA_DrainingToGI_frac"];
		
		//Get BulkArea Runoff_ImperviousToOutlet_m (m) going to GI area; adjusted above to folder bulk area depth from cover type depth with ImpAreaDirectlyConnected2Outlet_frac
		//Continuity maintained by working in folder area depth before transfering to other cover type analysis area
		input->RepoDict["BulkAreaImpRunoffDrainingToGI_m"] = folder->VarDict["Runoff_ImperviousToOutlet_m"] * BulkAreaImperviousDrainingToGI_frac;
		//Get BulkArea Runoff_ImperviousToOutlet_m (m) staying on BulkArea, adjusted above to folder bulk area depth
		folder->VarDict["Runoff_ImperviousToOutlet_m"] = folder->VarDict["Runoff_ImperviousToOutlet_m"] - input->RepoDict["BulkAreaImpRunoffDrainingToGI_m"];

	}
	//Determine the impervious area runoff to outlet (m3) for the impervious upSlopeArea within BulkArea, that drains to GI area
	//	Drainage to GI from pervious upSlopeArea is accounted for elsewhere
	if (folder->ParamStringDict["Type"] != "BulkArea") {
		
		//RepoDict is used for transfer between BulkArea and GI area folders
		//impRunoffFromContArea_m3 (m^3) is volume of runon to GI, computed as depth from BulkArea (m)
		input->RepoDict["impRunoffFromContArea_m3"] = input->RepoDict["BulkAreaImpRunoffDrainingToGI_m"] * input->RepoDict["BulkArea_Area_m2"];

		//impRunoffFromContAreaToGI_m (m) is depth of runon, adjusted from volume based on GI area (m2)
			//Area (m2) is for each GI device, giNumber is number of GI devices
		input->RepoDict["impRunoffFromContAreaToGI_m"] = (input->RepoDict["impRunoffFromContArea_m3"] ) /
			(folder->ParamDict["Area"] * folder->ParamDict["giNumber"]);

		//Forcing folder->VarDict["Runoff_ImperviousToOutlet_m"] to 0 for GI, given all impervious runoff water remains onsite in the GI as storage depth
		//Option to create new term that tracks all internal runoff that is contained as storage
		folder->VarDict["Runoff_ImperviousToOutlet_m"] = 0;
	}

	storeValues(input,folder, timeStep);
}

// With the routed runoff, we have to be careful not to double count!  The runoff routed to a folder must be split between the runoff routed
// to impervious area and the runoff routed to pervious area.  And then some of the runoff routed to impervious area will be routed to 
// pervious area since on a fraction of the impervious area (based on the DCIA) is routed directly to the channel?
// But can we just added the weighted impervious area water/precip amount to the runoff amount, or does a weighting of the runoff amount also have to occur?



void ImperviousDepressionStorageCalc::storeValues(Inputs *input, DataFolder *folder, int timeStep)
{
	//Impervious area runoff (m) adjusted for fractional areas of impervious area that are directly and not directly connected
	folder->VarDict["impRunoff"] = folder->VarDict["Runoff_ImperviousToOutlet_m"] + folder->VarDict["Runoff_ImperviousToPervious_m"];
	folder->VarDict["totalImpEvap"] += folder->VarDict["impEvap_m"];
	//Total impervious area runoff to outlet (m) adjusted for fraction of impervious that is DCIA 
	folder->VarDict["totalImpRunoffDCIA"] += folder->VarDict["Runoff_ImperviousToOutlet_m"];
	//Total impervious area runoff to soil (m) adjusted for fraction of impervious that is not DCIA 
	folder->VarDict["totalImpRunoffToSoil"] += folder->VarDict["Runoff_ImperviousToPervious_m"];
	//Total impervious runoff (m) adjusted for fractional area
	folder->VarDict["totalImpRunoff"] += folder->VarDict["impRunoff"];
	if (input->SimulationStringParams["Model"]=="StatisticalHydro")
	{
		// Updating vector variables
		folder->VarVecDict["impDepEvap"].emplace_back(folder->VarDict["impEvap_m"]);
		folder->VarVecDict["impDepStor"].emplace_back(folder->VarDict["impDepStor_m"]);
		folder->VarVecDict["impRunoff"].emplace_back(folder->VarDict["impRunoff"]);
		folder->VarVecDict["impRunoffDCIA"].emplace_back(folder->VarDict["Runoff_ImperviousToOutlet_m"]);
		folder->VarVecDict["impRunoffToSoil"].emplace_back(folder->VarDict["Runoff_ImperviousToPervious_m"]);
	}
}





