#include "InfiltrationExpDecayCalc.h"
#include <iostream>

//Partitions Water_forSoilRecharge_m (m) into Infiltration_viaInfilEx_m (m) and Runoff_PervInfilEx_m (m), noting Infiltration_viaInfilEx_m is potential and may be reduced if saturated areas exists

//Green Ampt infiltration routines are based on USGS Topmodel Fortran Code from D. Wolock (USGS) and K. Bevin, "C  TOPMODEL DEMONSTRATION PROGRAM VERSION 95.02"
	//Theory from Wang et al. (2005). Flexible Modeling Package for Topographically Based Watershed Hydrology. Journal of Hydrology, 314(1-4), 78-91. 
	//Theory from Eq. 2 to 8 in Beven (1984). Infiltration into a class of vertically non-uniform soils. Hydrological Sciences Journal 29: 425434.
	//		Before ponding, potential infiltration rate > rainfall rate
	//		Potential infiltration rate declines from maximum when cumulative infiltration = 0 toward minimum capacity, defined as K0
	//		Rainfall rate has to exceed potential infiltration rate for ponding to occur

//Cumulative infiltration determines performance of Green Ampt function.
//		Code developed as part of Wang et al. (2008) used temp = total infiltrated so far (CumInfilQ) - 0.1 * (BaseFlow + SumRET), SumRET = RootZone Evapotransipration
//		Code elaborated as part of Yang et al. (2011) used cumulative infiltration = intial rootzone deficit  - current (previous ts) root zone deficit

void SoilInfiltrationExpDecayCalc::calculate(Inputs *input, DataFolder *folder, int timeStep)
{

	//macroPore_frac is macropore fraction (0 to 1) within soils, 
	//		macroPore_frac > 0 allows a fraction of surface water to bypass infiltration and rootzone evapotranspiration, and directly enter groundwater
	double macroPore_frac = folder->ParamDict["PMacro_frac"];
	//Drainage_macroPore_m (m) is the water from Water_forSoilRecharge_m that bypasses infiltration and root zone, going directly to subsurface flow
	//	Water_forSoilRecharge_m (m) is the vertical precipitation on pervious area and lateral runon from impervious area 
	folder->VarDict["Drainage_macroPore_m"] = folder->VarDict["Water_forSoilRecharge_m"] * macroPore_frac;
	//Infiltration_viaSatEx_m (m) is Water_forSoilRecharge_m reduced by macroPore_frac and 1-InfiltExcessGovernedArea_frac, regulated only later by saturation excess and not by Green Ampt infiltration
	folder->VarDict["Infiltration_viaSatEx_m"] = folder->VarDict["Water_forSoilRecharge_m"] * (1.0 - macroPore_frac) * (1 - folder->ParamDict["InfiltExcessGovernedArea_frac"]);
	//cte 20210918 Should this be multiplied by InfiltExcessGovernedArea_frac
	folder->VarVecDict["Water_forInfiltration_m"].emplace_back(folder->VarDict["Water_forSoilRecharge_m"] * (1.0 - macroPore_frac));

	//R (m) is Water_forSoilRecharge_m reduced by macroPore_frac and InfiltExcessGovernedArea_frac, regulated by Green Ampt infiltration and later by saturation excess
	double R = folder->VarDict["Water_forSoilRecharge_m"] * (1.0 - macroPore_frac) * folder->ParamDict["InfiltExcessGovernedArea_frac"];
	//DT is simulation time interval, such as an 1 hour timestep or a 5 min timestep; determined by timestep of weather.dat inputs
	double DT = 1.0;
	//PP is Water_forSoilRecharge_m rate (m/deltaT), like a precipitation rate; needs to be at same length and timestep as hydraulic conductivity 
	double PP = R / DT;

	//consider resetting cumulative infiltration if not first timestep of simulation
	if (timeStep > 0) {
		//Outflow_RootZone_prior_m (m) represents water leaving root zone by drainage and evapotranpiration
		double Outflow_RootZone_prior_m = (folder->VarDict["Drainage_RootZone_prior_m"] + folder->VarDict["EvapoTranspiration_RootZone_prior_m"]);

		//Infiltration_viaInfilEx_cumulative_m (m) is reduced by the amount of water that left root zone
		folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] = folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] - Outflow_RootZone_prior_m;

		//if Infiltration_viaInfilEx_cumulative_m (m) < 0 then set to 0
		if (folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] < 0.0) {
			folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] = 0.0;
		}
	}

	//Infiltration decision tree with multiple if-else options
	//Option 1: Soil treated as impervious area; K0_mph is hydraulic conductivity (m/deltaT)
	if (folder->ParamDict["K0_mph"] == 0.0)
	{
		//Infiltration excess runoff (m) is runoff to infiltration that arrives at a rate greater than the infiltration rate
		//cte 20210918 Given this does not remain as depression storage, it was determined earlier
		folder->VarDict["Runoff_PervInfilEx_m"] = R;
		//cumInfil (m) is cumulative infiltration, set to 0 
		folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] = 0.0;
	}
	//Option 2: Infiltration occurring in ponded conditions; ponding is Green Ampt infiltration condition of pooled water above soil surface, adding pressure head to infiltration
	//	Ponded water becomes infiltration excess runoff; should we create option for ponded depth that ultimately infiltrates (similar to SWMM)
	else if (folder->VarDict["ponding"] > 0)
	{
		//inPonding routine uses PP = Water_forSoilRecharge_m rate (m/deltaT), and DT (timestep) to partition between ponding and infiltration
		inPonding(timeStep, PP, DT, folder, input);
	}
	//Option 3: No infiltration occurring; this option for no ponding is also used later when R > 0
	//	this may need to be modified to allow for intentional delayed infiltration of ponded water 
	else if (R == 0.0)
	{
		//noPonding routine uses R = Water_forSoilRecharge_m, and will record 0 infiltration in time series
		noPonding(R, folder, input, timeStep);
	}
	//Option 4: Infiltration starting anew, with cumulative infiltration = 0 and the need to estimate time to ponding
	else if (folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] == 0.0)
	{
		//estimateTP routine will estimate time to ponding based on Water_forSoilRecharge_m and timestep and soil properties in routine
		estimateTP(timeStep, R, DT, folder, input, timeStep);
	}
	//Option 5: Infiltration continues without ponding condition
	else {
		//beforePonding routine	uses Water_forSoilRecharge_m and timestep to determine infiltration and check if ponding will occur
		beforePonding(timeStep, R, DT, folder, input, timeStep);
	}
	//cte 20210427 ponding (flag) set to false (0) to avoid anomalous jump in the infiltration excess runoff routine infilEXQ
	folder->VarDict["ponding"] = 0;

	//Infiltration_viaInfilEx_m (m) is renamed infilQ (m), which is fraction of R (m) that infiltrates, determined above by calling Green-Ampt routines below
	folder->VarDict["Infiltration_viaInfilEx_m"] = folder->VarDict["infilQ"];

	//Compute Runoff_PervInfilEx_m (m) with infiltration excess overland runoff routine; Runoff_PervInfilEx_m (m) = R (m) - InfilQ (m)
	infilEXQ(R, folder, input, timeStep);

	//cte 20210524 Test new algorithm that passes all water to infiltration as Water_forSoilRecharge_m, but retains any ponded water up to the depth of available storage StorageAvailable_Ponding_m
	//cte 20210524 In new algorithm, this available ponding depth StorageAvailable_Ponding_m would fill with avoided runoff from infiltration excess and saturation excess 
	//cte 20210524 In new algorithm, the following statements would be used: 
	//if (folder->VarDict["StorageAvailable_Ponding_m"] > 0.0) {
		//if (folder->VarDict["Runoff_PervInfilEx_m"] >= folder->VarDict["StorageAvailable_Ponding_m"]) {
			//perDepStor (m) is increased based on what did not runoff
			//folder->VarDict["perDepStor"] = folder->VarDict["perDepStor"] + folder->VarDict["StorageAvailable_Ponding_m"];

			//Runoff_PervInfilEx_m (m) is reduced based on storage available
			//folder->VarDict["Runoff_PervInfilEx_m"] = folder->VarDict["Runoff_PervInfilEx_m"] - folder->VarDict["StorageAvailable_Ponding_m"];
			//folder->VarDict["StorageAvailable_Ponding_m"] = 0.0;
		//}

		//if (folder->VarDict["Runoff_PervInfilEx_m"] < folder->VarDict["StorageAvailable_Ponding_m"]) {
			//perDepStor (m) is increased based on what did not runoff
			//folder->VarDict["perDepStor"] = folder->VarDict["perDepStor"] + folder->VarDict["Runoff_PervInfilEx_m"];

			//Runoff_PervInfilEx_m (m) is reduced based on storage available
			//folder->VarDict["Runoff_PervInfilEx_m"] = 0;
			//folder->VarDict["StorageAvailable_Ponding_m"] = folder->VarDict["StorageAvailable_Ponding_m"] - folder->VarDict["Runoff_PervInfilEx_m"];
		//}

		//Runoff_PervInfilEx_m (m) constrained to zero or greater
		//if (folder->VarDict["Runoff_PervInfilEx_m"] < 0.0) {
			//folder->VarDict["Runoff_PervInfilEx_m"] = 0.0;
		//}
		//StorageAvailable_Ponding_m (m) constrained to zero or greater
		//if (folder->VarDict["StorageAvailable_Ponding_m"] < 0.0) {
			//folder->VarDict["StorageAvailable_Ponding_m"] = 0.0;
		//}
	//}

	//Infiltrated water will be reduced by the depth of saturation excess runoff in UnsatZoneDrainageCalc.cpp
	//satExcessInflow_m (m) is Water_forSoilRecharge_m reduced by macroPore_frac and 1-InfiltExcessGovernedArea_frac, regulated only later by saturation excess and not by Green Ampt infiltration
	double satExcessInflow_m = folder->VarDict["Infiltration_viaSatEx_m"];
	//infilExcessInflow_m (m) is Water_forSoilRecharge_m reduced by macroPore_frac and InfiltExcessGovernedArea_frac, regulated by Green Ampt infiltration and later by saturation excess
	double infilExcessInflow_m = folder->VarDict["Infiltration_viaInfilEx_m"];
	//Infiltration_m (m) is sum of two components, satExcessInflow_m and infilExcessInflow_m, weighted by InfiltExcessGovernedArea_frac
	folder->VarDict["Infiltration_m"] = satExcessInflow_m + infilExcessInflow_m;
}

//Routine from USGS Topmodel Fortran Code from Wolock (USGS)/Bevin, subroutine titled "THERE IS NO PONDING IN THIS TIME STEP"
//Option: all Water_forSoilRecharge_m infiltrates, no ponding during infiltration; npR is Water_forSoilRecharge_m for no ponding situation
void SoilInfiltrationExpDecayCalc::noPonding(double npR, DataFolder* folder, Inputs *input, int timeStep)
{
	//Set flag ponding = 0, which is false and indicates no ponding
	folder->VarDict["ponding"] = 0;
	//infilQ (m) is depth of water infiltrated; if no ponding, this equals all Water_forSoilRecharge_m
	folder->VarDict["infilQ"] = npR;
	//      TOPMODEL, DF = RINT*DT

	//Prohibit negative values for infilQ (m)
	if (folder->VarDict["infilQ"] < 0.0) {
		folder->VarDict["infilQ"] = 0.0;
	}

	//Update cumulative infiltration (m) depth
	double temp = folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] + folder->VarDict["infilQ"];
	folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] = temp;
}

//Routine from USGS Topmodel Fortran Code from Wolock (USGS)/Bevin, subroutine titled "C  INFILTRATION EXCESS CALCULATIONS"
//Option: infiltration excess runoff; ifR is Water_forSoilRecharge_m for infiltration excess runoff situation
void SoilInfiltrationExpDecayCalc::infilEXQ(double ifR, DataFolder* folder, Inputs *input, int timeStep)
{
	//Runoff_PervInfilEx_m (m) initiated to = 0 for pervious areas 
	folder->VarDict["Runoff_PervInfilEx_m"] = 0.0;
	//Runoff_PervInfilEx_m (m) is residual of Water_forSoilRecharge_m (m) - infilQ (m), the water that infiltrated to rootzone; 
	//		No ponded water is retained on site, all is routed to overland runoff
	folder->VarDict["Runoff_PervInfilEx_m"] = ifR - folder->VarDict["infilQ"];
	//	       TOPMODEL, REX = P - DF 

	//Prohibit negative values for Runoff_PervInfilEx_m (m)
	if (folder->VarDict["Runoff_PervInfilEx_m"] < 0.0) {
		folder->VarDict["Runoff_PervInfilEx_m"] = 0.0;
	}
}

//Routine from USGS Topmodel Fortran Code from Wolock (USGS)/Bevin, subroutine titled "INITIAL ESTIMATE OF TIME TO PONDING & PONDING STARTS AT BEGINNING OF TIME STEP"
//Option: before ponding during infiltration; biT is timestep number, bR is Water_forSoilRecharge_m for no ponding situation, bDT is timestep duration
void SoilInfiltrationExpDecayCalc::beforePonding(int bIT,  double bR, double bDT, DataFolder *folder, Inputs* input, int timeStep)
{
	//mM is scaling parameter used to adjust infiltration rates from Config inputs; TOPMODEL theory 
	double mM = folder->ParamDict["m"];
	//K0 hydraulic conductivity (m / timestep) is adjusted to per time step based on input as per hour and TimeStep_sec
	double K0 = folder->ParamDict["K0_mph"] * folder->ParamDict["TimeStep_sec"] / 3600;
	//      TOPMODEL, XKF = XK0; define hydraulic conductivity
	//fF = 1/mM, scaling parameter for infiltration rates, taken directly from TOPMODEL code SZF = 1./SZM
	//fF could be thetaSoilMoisture/mMin, given in text below Eq 10 of Wang et al. (2005). Flexible Modeling Package for Topographically Based Watershed Hydrology. Journal of Hydrology, 314(1-4), 78-91. 
	double fF = 1 / mM;
	//      TOPMODEL, SZF = 1./SZM
	//CCSF (m) coefficient is product of WFS_m (m), wetting front suction, and DWSMI (deficit water soil moisture initial or delta theta), the initial available storage fraction used in Green Ampt (has incorrect units of meters in name, and should be m3/m3, or volumetric moisture)
	double CSSF = folder->ParamDict["WFS_m"] * folder->ParamDict["DWSMI"];
	//      TOPMODEL, CD=HF*DTH
	//bPP is Water_forSoilRecharge_m Rate (m / timestep) 
	double bPP = bR / bDT;
	//	    TOPMODEL, RINT = P/DT

	//F1 (m) is cumulative infiltration estimate at start 
	double F1 = 0.0;
	F1 = folder->VarDict["Infiltration_viaInfilEx_cumulative_m"];
	//      TOPMODEL, F1=CUMF

	//RR (m / timestep) value of rainfall rate when it equals potential infilration rate at start of ponding, based on Green Ampt theory 
	//Theory from Eq. 2 to 8 in Beven (1984). Infiltration into a class of vertically non-uniform soils. Hydrological Sciences Journal 29: 425434.
	//		Before ponding, potential infiltration rate > rainfall rate
	//		Potential infiltration rate declines from maximum when cumulative infiltration = 0 toward minimum capacity, defined as K0
	//		Rainfall rate has to exceed potential infiltration rate for ponding to occur
	double RR = 0.0;
	//RR is modified version of Eq 6 and 7 from Beven (1984) Infiltration paper; lifted directly from Beven TOPMODEL code. 
	//RR is represented in Eq 10 of Wang et al. (2005). Flexible Modeling Package for Topographically Based Watershed Hydrology. Journal of Hydrology, 314(1-4), 78-91. 
	RR = - K0 * fF * (CSSF + F1) / (1 - exp(fF * F1));  
	//      TOPMODEL, R2=-XKF*SZF*(CD+F1)/(1-EXP(SZF*F1))

	//Ponding begins due to (Water_forSoilRecharge_m rate, bPP) > (rainfall rate when ponding begins, RR), and then infiltration excess runoff generated
	if(RR < bPP) {
	//		TOPMODEL, IF(R2.LT.RINT)THEN
		//pondTime (time units) is time ponding occurs = timestep number * timestep duration
		folder->VarDict["pondTime"] = bIT*bDT;             
		//		TOPMODEL, TP=(IT-1.)*DT
		if (folder->VarDict["pondTime"] < 0.0) {
			folder->VarDict["pondTime"] = 0.0;
		}

		//Flag ponding = true (1)
		folder->VarDict["ponding"] = 1;
		//		TOPMODEL, IROF=1
		//F (m) is cumulative infiltration
		folder->VarDict["F"] = folder->VarDict["Infiltration_viaInfilEx_cumulative_m"];
		//		TOPMODEL, F=CUMF
		//prepPonding prepare parameters for infiltration during ponding
		prepPonding(bIT,bPP,bDT, folder, input);  
		//      TOPMODEL, GO TO 8 ... SET UP DEFINITE INTEGRAL CONSTANT USING FP
	}
	//Ponding does not begin, (Water_forSoilRecharge_m rate, bPP) < (rainfall rate when ponding begins, RR)
	else {
		//estimateTP to get estimate of when ponding will occur
		estimateTP(bIT, bR, bDT, folder, input, timeStep);
		//		TOPMODEL, INITIAL ESTIMATE OF TIME TO PONDING
	}
}

//Routine from USGS Topmodel Fortran Code from Wolock (USGS)/Bevin, subroutine titled "INITIAL ESTIMATE OF TIME TO PONDING &     7 F2=CUMF+DT*RINT"
//Option: infiltrate all water if ponding has not occured, estimate time when ponding will occur based on current Water_forSoilRecharge_m rate; tpIT is timestep number, tpR is Water_forSoilRecharge_m for no ponding situation, tpDT is timestep duration
void SoilInfiltrationExpDecayCalc::estimateTP(int tpIT,  double tpR, double tpDT, DataFolder* folder, Inputs *input, int timeStep)
{
	//mM is scaling parameter used to adjust infiltration rates from Config inputs; TOPMODEL theory 
	double mM = folder->ParamDict["m"];
	//K0 hydraulic conductivity (m / timestep) is adjusted to per time step based on input as per hour and TimeStep_sec
	double K0 = folder->ParamDict["K0_mph"] * folder->ParamDict["TimeStep_sec"] / 3600;
	//fF = 1/mM, scaling parameter for infiltration rates, taken directly from TOPMODEL code SZF = 1./SZM
	//fF could be thetaSoilMoisture/mMin, given in text below Eq 10 of Wang et al. (2005). Flexible Modeling Package for Topographically Based Watershed Hydrology. Journal of Hydrology, 314(1-4), 78-91. 
	double fF = 1 / mM;
	//CCSF (m) coefficient is product of WFS_m (m), wetting front suction, and DWSMI (deficit water soil moisture initial or delta theta), the initial available storage fraction used in Green Ampt (has incorrect units of meters in name, and should be m3/m3, or volumetric moisture)
	double CSSF = folder->ParamDict["WFS_m"]*folder->ParamDict["DWSMI"];                   // CSSF = WFSuction*DWSMI

	//tpPP is Water_forSoilRecharge_m Rate (m / timestep) 
	double tpPP = tpR / tpDT;
	//FI is cumulative infiltration at start
	double F1 = 0.0;

	//RR (m / timestep) value of rainfall rate when it equals potential infilration rate at start of ponding, based on Green Ampt theory 
	//Theory from Eq. 2 to 8 in Beven (1984). Infiltration into a class of vertically non-uniform soils. Hydrological Sciences Journal 29: 425434.
	double RR = 0.0;
	//E (m) is epsilon error threhold for closure of iteration to find cumulative infiltration
	double E = 0.00001;
	//FF (m) is estimate of cumulative infiltration
	double FF = 0.0;
	//F2 (m) is cumulative infiltration + Water_forSoilRecharge_m
	double F2 = folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] + tpR;
	//		TOPMODEL, F2=CUMF+DT*RINT
	
	//Ponding will not occur during timestep, cumulative infiltration + Water_forSoilRecharge_m = 0, from TOPMODEL code as IF(F2.EQ.0.)GO TO 20
	if (F2 == 0.0) {
		//noPonding routine called, all Water_forSoilRecharge_m (tpR) is infiltrated as infilQ
		noPonding(tpR, folder, input, timeStep);
	}

	//Ponding may occur during timestep, cumulative infiltration + runoffToIfil > 0
	else 
	{
		//RR (m / timestep) value of rainfall rate when it equals potential infilration rate at start of ponding, based on Green Ampt theory 
		//Theory from Eq. 2 to 8 in Beven (1984). Infiltration into a class of vertically non-uniform soils. Hydrological Sciences Journal 29: 425434.
		//		Before ponding, potential infiltration rate > rainfall rate
		//		Potential infiltration rate declines from maximum when cumulative infiltration = 0 toward minimum capacity, defined as K0
		//		Rainfall rate has to exceed potential infiltration rate for ponding to occur
		//RR is modified version of Eq 6 and 7 from Beven (1984) Infiltration paper; lifted directly from Beven TOPMODEL code. 
		//RR is represented in Eq 10 of Wang et al. (2005). Flexible Modeling Package for Topographically Based Watershed Hydrology. Journal of Hydrology, 314(1-4), 78-91. 
		RR = -K0 * fF * (CSSF + F2) / (1 - exp(fF * F2));
		//      TOPMODEL, R2=-XKF*SZF*(CD+F2)/(1-EXP(SZF*F2))

		//Ponding does not begin due to (Water_forSoilRecharge_m rate, tpPP) < (rainfall rate when ponding begins, RR), from TOPMODEL code IF(R2.GT.RINT)GO TO 20
		if (RR > tpPP) {
			//noPonding routine called, all Water_forSoilRecharge_m (tpR) is infiltrated as infilQ
			noPonding(tpR, folder, input, timeStep);
		}

		//Ponding does begin due to (Water_forSoilRecharge_m rate, tpPP) > (rainfall rate when ponding begins, RR), from TOPMODEL code IF(R2.GT.RINT)GO TO 20
		else
		{
			//initial estimate of F (m) cumulative infiltration when ponding occurs = cumulative infiltration + (rainfall rate when ponding begins) * timestep duration, from TOPMODEL code F=CUMF+R2*DT
			folder->VarDict["F"] =  folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] + tpDT*RR;
			//      TOMODEL, F=CUMF+R2*DT
			
			//for loop to find F (m) cumulative infiltration when ponding occurs, from TOPMODEL code = DO 9 I=1,20
			for(int i=0; i<20; i++)               
			{
				//RR (m / timestep) value of rainfall rate when it equals potential infilration rate at start of ponding, based on Green Ampt theory 
				//RR is modified version of Eq 6 and 7 from Beven (1984) Infiltration paper; lifted directly from Beven TOPMODEL code. 
				RR =-K0 * fF * (CSSF+ folder->VarDict["F"]) / (1-exp(fF* folder->VarDict["F"]));
				//      TOPMODEL, R2=-XKF*SZF*(CD+F)/(1-EXP(SZF*F))
		
				//Ponding does not begin due to (Water_forSoilRecharge_m rate, tpPP) < (rainfall rate when ponding begins, RR)
				if(RR > tpPP) {
				//      TOPMODEL, IF(R2.GT.RINT)THEN
					//FI is cumulative infiltration = folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] + tpDT*RR, from TOPMODEL F1=F
					F1 = folder->VarDict["F"];
					//	  TOPMODEL, F1=F
					//F is average of two estimates of cumulative infiltration, F2 = folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] + tpR and folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] + tpDT*RR;
					folder->VarDict["F"] = 0.5 * (F2 + folder->VarDict["F"]);
					//	  TOPMODEL, F=(F2+F)*0.5
					//FF (m) is estimate of cumulative infiltration at ponding from prior timestep
					FF = F1;
				}

				//Ponding does begin due to (Water_forSoilRecharge_m rate, tpPP) < (rainfall rate when ponding begins, RR)
				else
				{
					//FI is cumulative infiltration = folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] + tpDT*RR, from TOPMODEL F1=F, from TOPMODEL F2=F
					F2 = folder->VarDict["F"];
					//	  TOPMODEL, F2=F
					//F is average of two estimates of cumulative infiltration, F1 = 0 and folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] + tpDT*RR;
					folder->VarDict["F"] =  0.5 * (F1 + folder->VarDict["F"]);
					//	  TOPMODEL, F=(F1+F)*0.5
					//FF (m) is estimate of cumulative infiltration at ponding from prior timestep
					FF = F2;
				}

				//close for loop to estimate ponding if change in estimated cumulative infiltration between timesteps < error threshold
				if(fabs(folder->VarDict["F"] - FF) < E) {
				//      TOPMODEL, IF(ABS(F-F1).LT.E)GO TO 11

					//pondTime (time units) is time ponding occurs = (timestep number * timestep duration) - interval of time for additional infiltration at given runoffToInfilRate
					folder->VarDict["pondTime"] = tpIT * tpDT + (folder->VarDict["F"] - (folder->VarDict["Infiltration_viaInfilEx_cumulative_m"])) / tpPP;
					//      TOPMODEL, TP=(IT-1)*DT+(F-CUMF)/RINT

					//no ponding occurs in this timestep if pondTime occurs after the start of the next time step
					if(folder->VarDict["pondTime"] > ((tpIT + 1) * tpDT) ) {
					//      TOPMODEL, IF(TP.GT.IT*DT)GO TO 20
						//noPonding routine called, all Water_forSoilRecharge_m (tpR) is infiltrated as infilQ
						noPonding(tpR, folder, input, timeStep);
						//End iteration
						break;
					}
			
					//ponding occurs in this timestep 
					//prepPonding prepare parameters for infiltration during ponding
					prepPonding(tpIT,tpPP,tpDT,folder,input);
					//End iteration
					break;
				}
			
			}
		
		}
	
	}
}

//Routine from USGS Topmodel Fortran Code from Wolock (USGS)/Bevin, subroutine titled "NEWTON-RAPHSON SOLUTION FOR F(T)"
//Option: find infiltration rate during ponding; ipIT is timestep number, ipPP is Water_forSoilRecharge_m for no ponding situation, ipDT is timestep duration
void SoilInfiltrationExpDecayCalc::inPonding(int ipIT, double ipPP, double ipDT, DataFolder *folder, Inputs *input)
{
	//mM is scaling parameter used to adjust infiltration rates from Config inputs; TOPMODEL theory 
	double mM = folder->ParamDict["m"];
	//K0 hydraulic conductivity (m / timestep) is adjusted to per time step based on input as per hour and TimeStep_sec
	double K0 = folder->ParamDict["K0_mph"] * folder->ParamDict["TimeStep_sec"] / 3600;
	//fF = 1/mM, scaling parameter for infiltration rates, taken directly from TOPMODEL code SZF = 1./SZM
	//fF could be thetaSoilMoisture/mMin, given in text below Eq 10 of Wang et al. (2005). Flexible Modeling Package for Topographically Based Watershed Hydrology. Journal of Hydrology, 314(1-4), 78-91. 
	double fF = 1 / mM;
	//CCSF (m) coefficient is product of WFS_m (m), wetting front suction, and DWSMI (deficit water soil moisture initial or delta theta), the initial available storage fraction used in Green Ampt (has incorrect units of meters in name, and should be m3/m3, or volumetric moisture)
	double CSSF = folder->ParamDict["WFS_m"] * folder->ParamDict["DWSMI"];			// CSSF = WFSuction*DWSMI

	//variables for Newton-Raphson algorithm
	int FAC ;
	double ADD = 0.0;
	double SUM ;
	//root function
	double FUNC = 0.0;
	//derivative of root function
	double DFUNC = 0.0;
	double FC = 0.0;
	//E (m) is epsilon error threhold for closure of iteration to find cumulative infiltration
	double E = 0.00001;


	//infilQ (m) initialized to 0
	folder->VarDict["infilQ"] = 0.0;

		//Newton-Raphson algorithm to infiltration rate during ponding
		//for loop to find infiltration, from TOPMODEL code = DO 14 I = 1, 20
		for(int i=0; i<20; i++)
		{
			//FC (m) is used in Lambda function, cumulative infiltration + Green Ampt coefficient (wetting front suction * delta theta)
			FC = folder->VarDict["F"] + CSSF;                    
			//      FC=(F+CD)
			//SUM initialized to 0 on each loop 
			SUM = 0.0;
			//      SUM=0.
			//FAC initialized to 1 on each loop 
			FAC = 1;
			//      FAC=1.

			//for loop, from TOPMODEL code = DO 13 J=1,10
			for(int j = 1; j<10; j++)
			{
				//FAC is incremented by j factor for each loop
				FAC = FAC * j ;
				//      TOPMODEL, FAC=FAC*J
				//ADD is updated with j and FAC for each loop
				ADD = pow(FC * fF , j) / (j * FAC);
				//      TOPMODEL, ADD=(FC*SZF)**J/(J*FAC)
				//SUM (or constant) is updated each loop
				SUM = SUM + ADD ;
				//      TOPMODEL, SUM=SUM+ADD
			}
		
			//FUNC is Lambda defined below Eq 8 from Beven (1984) Infiltration paper; lifted directly from Beven TOPMODEL code
			//FUNC is Lambda in Eq 13 of Wang et al. (2005). Flexible Modeling Package for Topographically Based Watershed Hydrology. Journal of Hydrology, 314(1-4), 78-91. 
			FUNC = -(log(FC) - (log(FC) + SUM) / exp(fF * CSSF) - folder->VarDict["constant"] ) / (K0 * fF)
				   - ((ipIT + 1) * ipDT - folder->VarDict["pondTime"]) ;
			//      TOPMODEL, FUNC=-(DLOG(FC)-(DLOG(FC)+SUM)/DEXP(SZF*CD)-CONST)/(XKF*SZF) - (IT*DT - TP)

			//Derivative of Lambda function for computing infiltration rate, from TOPMODEL, DFUNC=(EXP(SZF*F)-1)/(XKF*SZF*FC)
			DFUNC = ( exp(fF * folder->VarDict["F"]) - 1) / (K0 * fF * FC) ;
			//      TOPMODEL, DFUNC=(EXP(SZF*F)-1)/(XKF*SZF*FC)

			//Newton-Raphson (N-R) correction factor = (-FUNC/DFUNC) used to get InfilQ (m), incrementally updates cumulative infiltration to satisfy root
			folder->VarDict["infilQ"] = (-FUNC / DFUNC);
			//      TOPMODEL, DF=-FUNC/DFUNC

			//N-R estimated F (m), cumulative infiltration, based on finding root that creates no additional infiltration to root zone
			folder->VarDict["F"] = folder->VarDict["F"] + folder->VarDict["infilQ"];
			//      TOPMODEL, F=F+DF

			//close for loop to estimate cumulative infiltration if incremental improvement of cumulative infiltration estimate is less than threshold
			if (fabs(folder->VarDict["infilQ"]) < E) {
				//      TOPMODEL, IF(ABS(DF).LE.E)GO TO 15
					//End iteration
				break;
			}
		}

	//Infiltration excess constraint when estimated N-R estimate of cumulative infiltration (m) < (prior cumulative infiltration + Water_forSoilRecharge_m)
	if(folder->VarDict["F"] < (folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] + ipPP) )
	//      TOPMODEL, IF(F.LT.CUMF+RINT)THEN
	{
		//infilQ (m) is infiltration for timestep = maximum possible cumulative infiltration during timestep - cumulative infiltration from prior timestep
		folder->VarDict["infilQ"] = (folder->VarDict["F"] - folder->VarDict["Infiltration_viaInfilEx_cumulative_m"]);
		//		TOPMODEL, DF=F-CUMF
		//cumInfil (m) updated to new value at end of timestep
		folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] = folder->VarDict["F"];
		//		TOPMODEL, CUMF=F
		//F (m) is updated to include the water infiltrated this timestep
		folder->VarDict["F"] = folder->VarDict["F"] + folder->VarDict["infilQ"];
		//		TOPMODEL, F=F+DF

		//Prohibit negative values for infilQ (m)
		if (folder->VarDict["infilQ"] < 0.0) {
			folder->VarDict["infilQ"] = 0.00001;
		}
	}
       
}

//Routine from USGS Topmodel Fortran Code from Wolock (USGS)/Bevin, subroutine titled "SET UP DEFINITE INTEGRAL CONSTANT USING FP"
//Option: prepare parameters for infiltration during ponding; prIT is timestep number, prPP is Water_forSoilRecharge_m for no ponding situation, prDT is timestep duration
void SoilInfiltrationExpDecayCalc::prepPonding(int prIT, double prPP, double prDT, DataFolder *folder, Inputs *input)
{
	
	//mM is scaling parameter used to adjust infiltration rates from Config inputs; TOPMODEL theory 
	double mM = folder->ParamDict["m"];
	//K0 hydraulic conductivity (m / timestep) is adjusted to per time step based on input as per hour and TimeStep_sec
	double K0 = folder->ParamDict["K0_mph"] * folder->ParamDict["TimeStep_sec"] / 3600;
	//fF = 1/mM, scaling parameter for infiltration rates, taken directly from TOPMODEL code SZF = 1./SZM
	//fF could be thetaSoilMoisture/mMin, given in text below Eq 10 of Wang et al. (2005). Flexible Modeling Package for Topographically Based Watershed Hydrology. Journal of Hydrology, 314(1-4), 78-91. 
	double fF = 1 / mM;
	//CCSF (m) coeficient is product of WFS_m (m), wetting front suction, and DWSMI (deficit water soil moisture initial or delta theta), the initial available storage fraction used in Green Ampt (has incorrect units of meters in name, and should be m3/m3, or volumetric moisture)
	double CSSF = folder->ParamDict["WFS_m"] * folder->ParamDict["DWSMI"];
	//F (m) is cumulative infiltration 
	double F = folder->VarDict["F"];

	//variables for Newton-Raphson algorithm
	int FAC = 1;
	//	TOPMODEL, FAC=1
	double ADD = 0.0;
	folder->VarDict["constant"] = 0.0;
	//	TOPMODEL, CONST =0
	double FC = folder->VarDict["F"] + CSSF;
	//	TOPMODEL, FC=(F+CD)

	//for loop, from TOPMODEL code = DO 12 J=1,10
	for(int j = 1; j<=10; j++)
	{
		//FAC is incremented by j factor for each loop
		FAC = FAC * j;
		//	TOPMODEL, FAC=FAC*J

		//ADD is updated with j and FAC for each loop
		ADD = pow(FC * fF,j) / (j * FAC);
		//	TOPMODEL, ADD = (FC * SZF) * *J / (J * FAC)

		//constant (or SUM) is updated each loop
		folder->VarDict["constant"] = folder->VarDict["constant"] + ADD;
		//	TOPMODEL, CONST=CONST+ADD
	}
	
	//constant is Lambda defined below Eq 8 from Beven (1984) Infiltration paper; lifted directly from Beven TOPMODEL code
	//constant is Lambda in Eq 13 of Wang et al. (2005). Flexible Modeling Package for Topographically Based Watershed Hydrology. Journal of Hydrology, 314(1-4), 78-91. 
	folder->VarDict["constant"] = log(FC) - (log(FC) + folder->VarDict["constant"]) / exp(fF * CSSF);
	//	TOPMODEL, CONST=DLOG(FC)-(DLOG(FC)+CONST)/DEXP(SZF*CD)

	//Flag ponding = true (1)
	folder->VarDict["ponding"] = 1;
	//	TOPMODEL, IROF=1

	//F (m) cumulative infiltration is average of Water_forSoilRecharge_m at end of time step and at time of ponding
	folder->VarDict["F"] = folder->VarDict["F"] + 0.5 * prPP * ( (prIT + 1) * prDT -  folder->VarDict["pondTime"] );
	//TOPMODEL, F=F+0.5*RINT*(IT*DT-TP)

	//inPonding routine called
	inPonding(prIT, prPP, prDT, folder, input);
	

	
}