#include "InfiltrationPowDecayCalc.h"
#include <iostream>

//Partitions Water_forSoilRecharge_m (m) into Infiltration_viaInfilEx_m (m) and Runoff_PervInfilEx_m (m), noting Infiltration_viaInfilEx_m is potential and may be reduced if saturated areas exists

//Green Ampt infiltration routines are based on USGS Topmodel Fortran Code from D. Wolock (USGS) and K. Bevin, "C  TOPMODEL DEMONSTRATION PROGRAM VERSION 95.02"
	//Theory from Wang et al. (2006). Power function decay of hydraulic conductivity for a TOPMODEL-based infiltration routine Hydrological Processes, 20(18), 3825-3834. 
	//Theory from Wang et al. (2005). Flexible Modeling Package for Topographically Based Watershed Hydrology. Journal of Hydrology, 314(1-4), 78-91. 
	//Theory from Eq. 2 to 8 in Beven (1984). Infiltration into a class of vertically non-uniform soils. Hydrological Sciences Journal 29: 425434.
	//		Before ponding, potential infiltration rate > rainfall rate
	//		Potential infiltration rate declines from maximum when cumulative infiltration = 0 toward minimum capacity, defined as K0
	//		Rainfall rate has to exceed potential infiltration rate for ponding to occur

	//Theory of power function decay requires m <= S based on constraint governing decay with depth of transmissivity, Tz = T0*(1-S/m)^n  
	//S (m) is soil moisture deficit between top of soil and groundwater table, defined as Theta_sat-Theta_wp or Theta_sat-Theta_fc (based on Beven and Wood, 1983).
	//Beven and Wood (1983). Catchment Geomorphology and the Dynamics of Runoff Contributing Areas, Journal of Hydrology. 65:139-158. In Special Issue: Scale Problems in Hydrology, Editor
	//Theory explained after Eq 5 of Wang, J., Endreny, T. A., & Hassett, J. M. (2006). Power function decay of hydraulic conductivity for a TOPMODEL-based infiltration routine Hydrological Processes, 20(18), 3825-3834. 
	//Figure 1 from Wang et al. (2006) illustrates the decay of transmissivity, Tz, with depth as a function of n, varying from 0.25 to 16

//Cumulative infiltration determines performance of Green Ampt function.
//		Code developed as part of Wang et al. (2008) used temp = total infiltrated so far (CumInfilQ) - 0.1 * (BaseFlow + SumRET), SumRET = RootZone Evapotransipration
//		Code elaborated as part of Yang et al. (2011) used cumulative infiltration = intial rootzone deficit  - current (previous ts) root zone deficit

void SoilInfiltrationPowDecayCalc::calculate(Inputs *input, DataFolder *folder, int timeStep)
{
	//macroPore_frac is macropore fraction (0 to 1) within soils, 
	//		macroPore_frac > 0 allows a fraction of surface water to bypass infiltration and rootzone evapotranspiration, and directly enter groundwater
	double macroPore_frac = folder->ParamDict["PMacro_frac"];
	//Drainage_macroPore_m (m) is the water from Water_forSoilRecharge_m that bypasses infiltration and root zone, going directly to subsurface flow
	//	Water_forSoilRecharge_m (m) is the vertical precipitation on pervious area and lateral runon from impervious area 
	folder->VarDict["Drainage_macroPore_m"] = folder->VarDict["Water_forSoilRecharge_m"] * macroPore_frac;

	//Water_forInfiltration_m (m) is Water_forSoilRecharge_m reduced by macroPore_frac, now available for Infiltration_viaSatEx_m and Infiltration_viaInfilEx_m; the latter is initially called R
	folder->VarVecDict["Water_forInfiltration_m"].emplace_back(folder->VarDict["Water_forSoilRecharge_m"] * (1.0 - macroPore_frac));

	//Infiltration_viaSatEx_m (m) is Water_forSoilRecharge_m reduced by macroPore_frac and 1-InfiltExcessGovernedArea_frac, regulated only later by saturation excess and not by Green Ampt infiltration
	folder->VarDict["Infiltration_viaSatEx_m"] = folder->VarDict["Water_forSoilRecharge_m"] * (1.0 - macroPore_frac) * (1 - folder->ParamDict["InfiltExcessGovernedArea_frac"]);

	//R (m) is Water_forSoilRecharge_m reduced by macroPore_frac and InfiltExcessGovernedArea_frac, regulated by Green Ampt infiltration and later by saturation excess; synonymous with Infiltration_viaInfilEx_m
	double R = folder->VarDict["Water_forSoilRecharge_m"] * (1.0 - macroPore_frac) * folder->ParamDict["InfiltExcessGovernedArea_frac"];

	//DT is simulation time interval, such as an 1 hour timestep or a 5 min timestep; determined by timestep of weather.dat inputs
	double DT = 1.0;
	//PP is Water_forSoilRecharge_m rate (m/deltaT), like a precipitation rate; needs to be at same length and timestep as hydraulic conductivity 
	double PP = R / DT;

	//consider resetting cumulative infiltration if not first timestep of simulation
	if (timeStep > 0) {
		//Outflow_RootZone_prior_m (m) represents water leaving root zone by drainage and evapotranpiration
		double Outflow_RootZone_prior_m = (folder->VarDict["Drainage_RootZone_prior_m"] + folder->VarDict["EvapoTranspiration_RootZone_prior_m"]);

		//Infiltration_viaInfilEx_cumulative_m (m) is reduced by the amount of water that left root zone
		folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] = folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] - Outflow_RootZone_prior_m;

		//if Infiltration_viaInfilEx_cumulative_m (m) < 0 then set to 0
		if (folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] < 0.0) {
			folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] = 0.0;
		}
	}

	//Infiltration decision tree with multiple if-else options
	//Option 1: Soil treated as impervious area; K0_mph is hydraulic conductivity (m/deltaT)
	if (folder->ParamDict["K0_mph"] == 0.0)
	{
		//Infiltration excess runoff (m) is runoff to infiltration that arrives at a rate greater than the infiltration rate
		folder->VarDict["Runoff_PervInfilEx_m"] = R;
		//cumInfil (m) is cumulative infiltration, set to 0 
		folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] = 0.0;
	}
	//Option 2: Infiltration occurring in ponded conditions; ponding is Green Ampt infiltration condition of pooled water above soil surface, adding pressure head to infiltration
	//	Ponded water becomes infiltration excess runoff; should we create option for ponded depth that ultimately infiltrates (similar to SWMM)
	else if (folder->VarDict["ponding"] > 0)
	{
		//inPonding routine uses PP = Water_forSoilRecharge_m rate (m/deltaT), and DT (timestep) to partition between ponding and infiltration
		inPonding(timeStep, PP, DT, folder, input);
	}
	//Option 3: No infiltration occurring; this option for no ponding is also used later when R > 0
	//	this may need to be modified to allow for intentional delayed infiltration of ponded water 
	else if (R == 0.0)
	{
		//noPonding routine uses R = Water_forSoilRecharge_m, and will record 0 infiltration in time series
		noPonding(R, folder, input, timeStep);
	}
	//Option 4: Infiltration starting anew, with cumulative infiltration = 0 and the need to estimate time to ponding
	else if (folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] == 0.0)
	{
		//estimateTP routine will estimate time to ponding based on Water_forSoilRecharge_m and timestep and soil properties in routine
		estimateTP(timeStep, R, DT, folder, input);
	}
	//Option 5: Infiltration continues without ponding condition
	else {
		//beforePonding routine	uses Water_forSoilRecharge_m and timestep to determine infiltration and check if ponding will occur
		beforePonding(timeStep, R, DT, folder, input);
	}
	//cte 20210427 ponding (flag) set to false (0) to avoid anomalous jump in the infiltration excess runoff routine infilEXQ
	folder->VarDict["ponding"] = 0;

	//Infiltration_viaInfilEx_m (m) is renamed infilQ (m), which is fraction of R (m) that infiltrates, determined above by calling Green-Ampt routines below
	folder->VarDict["Infiltration_viaInfilEx_m"] = folder->VarDict["infilQ"];

	//Compute Runoff_PervInfilEx_m (m) with infiltration excess overland runoff routine; Runoff_PervInfilEx_m (m) = R (m) - InfilQ (m)
	infilEXQ(R, folder, input, timeStep);

	//cte 20210524 Test new algorithm that passes all water to infiltration as Water_forSoilRecharge_m, but retains any ponded water up to the depth of available storage StorageAvailable_Ponding_m
	//cte 20210524 In new algorithm, this available ponding depth StorageAvailable_Ponding_m would fill with avoided runoff from infiltration excess and saturation excess 
	//cte 20210524 In new algorithm, the following statements would be used: 
	//if (folder->VarDict["StorageAvailable_Ponding_m"] > 0.0) {
		//if (folder->VarDict["Runoff_PervInfilEx_m"] >= folder->VarDict["StorageAvailable_Ponding_m"]) {
			//perDepStor (m) is increased based on what did not runoff
			//folder->VarDict["perDepStor"] = folder->VarDict["perDepStor"] + folder->VarDict["StorageAvailable_Ponding_m"];

			//Runoff_PervInfilEx_m (m) is reduced based on storage available
			//folder->VarDict["Runoff_PervInfilEx_m"] = folder->VarDict["Runoff_PervInfilEx_m"] - folder->VarDict["StorageAvailable_Ponding_m"];
			//folder->VarDict["StorageAvailable_Ponding_m"] = 0.0;
		//}

		//if (folder->VarDict["Runoff_PervInfilEx_m"] < folder->VarDict["StorageAvailable_Ponding_m"]) {
			//perDepStor (m) is increased based on what did not runoff
			//folder->VarDict["perDepStor"] = folder->VarDict["perDepStor"] + folder->VarDict["Runoff_PervInfilEx_m"];

			//Runoff_PervInfilEx_m (m) is reduced based on storage available
			//folder->VarDict["Runoff_PervInfilEx_m"] = 0;
			//folder->VarDict["StorageAvailable_Ponding_m"] = folder->VarDict["StorageAvailable_Ponding_m"] - folder->VarDict["Runoff_PervInfilEx_m"];
		//}

		//Runoff_PervInfilEx_m (m) constrained to zero or greater
		//if (folder->VarDict["Runoff_PervInfilEx_m"] < 0.0) {
			//folder->VarDict["Runoff_PervInfilEx_m"] = 0.0;
		//}
		//StorageAvailable_Ponding_m (m) constrained to zero or greater
		//if (folder->VarDict["StorageAvailable_Ponding_m"] < 0.0) {
			//folder->VarDict["StorageAvailable_Ponding_m"] = 0.0;
		//}
	//}

	//Infiltrated water will be reduced by the depth of saturation excess runoff in UnsatZoneDrainageCalc.cpp
	//satExcessInflow_m (m) is Water_forSoilRecharge_m reduced by macroPore_frac and 1-InfiltExcessGovernedArea_frac, regulated only later by saturation excess and not by Green Ampt infiltration
	double satExcessInflow_m = folder->VarDict["Infiltration_viaSatEx_m"];
	//infilExcessInflow_m (m) is Water_forSoilRecharge_m reduced by macroPore_frac and InfiltExcessGovernedArea_frac, regulated by Green Ampt infiltration and later by saturation excess
	double infilExcessInflow_m = folder->VarDict["Infiltration_viaInfilEx_m"];
	//Infiltration_m (m) is sum of two components, satExcessInflow_m and infilExcessInflow_m, weighted by InfiltExcessGovernedArea_frac
	folder->VarDict["Infiltration_m"] = satExcessInflow_m + infilExcessInflow_m;
}

//Routine from USGS Topmodel Fortran Code from Wolock (USGS)/Bevin, subroutine titled "THERE IS NO PONDING IN THIS TIME STEP"
//Option: all Water_forSoilRecharge_m infiltrates, no ponding during infiltration; npR is Water_forSoilRecharge_m for no ponding situation
void SoilInfiltrationPowDecayCalc::noPonding(double npR, DataFolder* folder, Inputs *input, int timeStep)
{
	//Set flag ponding = 0, which is false and indicates no ponding
	folder->VarDict["ponding"] = 0;
	//infilQ (m) is depth of water infiltrated; if no ponding, this equals all Water_forSoilRecharge_m
	folder->VarDict["infilQ"] = npR;
	//      TOPMODEL, DF = RINT*DT

	//Prohibit negative values for infilQ (m)
	if (folder->VarDict["infilQ"] < 0.0) {
		folder->VarDict["infilQ"] = 0.0;
	}
	//Update cumulative infiltration (m) depth
	double temp = folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] + folder->VarDict["infilQ"];
	folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] = temp;
}

//Routine from USGS Topmodel Fortran Code from Wolock (USGS)/Bevin, subroutine titled "C  INFILTRATION EXCESS CALCULATIONS"
//Option: infiltration excess runoff; ifR is Water_forSoilRecharge_m for infiltration excess runoff situation
void SoilInfiltrationPowDecayCalc::infilEXQ(double ifR, DataFolder* folder, Inputs *input, int timeStep)
{
	//Runoff_PervInfilEx_m (m) initiated to = 0 for pervious areas 
	folder->VarDict["Runoff_PervInfilEx_m"] = 0.0;
	//Runoff_PervInfilEx_m (m) is residual of Water_forSoilRecharge_m (m) - infilQ (m), the water that infiltrated to rootzone; 
	//		No ponded water is retained on site, all is routed to overland runoff
	folder->VarDict["Runoff_PervInfilEx_m"] = ifR - folder->VarDict["infilQ"];
	//	       TOPMODEL, REX = P - DF 

	//Prohibit negative values for Runoff_PervInfilEx_m (m)
	if (folder->VarDict["Runoff_PervInfilEx_m"] < 0.0) {
		folder->VarDict["Runoff_PervInfilEx_m"] = 0.0;
	}
}

//Routine from USGS Topmodel Fortran Code from Wolock (USGS)/Bevin, subroutine titled "INITIAL ESTIMATE OF TIME TO PONDING & PONDING STARTS AT BEGINNING OF TIME STEP"
//Option: before ponding during infiltration; biT is timestep number, bR is Water_forSoilRecharge_m for no ponding situation, bDT is timestep duration
void SoilInfiltrationPowDecayCalc::beforePonding(int bIT,  double bR, double bDT, DataFolder *folder, Inputs *input)
{
	
	// Scale parameter of power function infiltration (N)
	double nN = folder->ParamDict["n"];

	//mM is scaling parameter used to adjust infiltration rates from Config inputs; TOPMODEL theory 
	double mM = folder->ParamDict["m"];
	//K0 hydraulic conductivity (m / timestep) is adjusted to per time step based on input as per hour and TimeStep_sec
	double K0 = folder->ParamDict["K0_mph"] * folder->ParamDict["TimeStep_sec"] / 3600;
	//      TOPMODEL, XKF = XK0; define hydraulic conductivity
	//fF = 1/mM, scaling parameter for infiltration rates, taken directly from TOPMODEL code SZF = 1./SZM
	//fF could be thetaSoilMoisture/mMin, given in text below Eq 10 of Wang et al. (2005). Flexible Modeling Package for Topographically Based Watershed Hydrology. Journal of Hydrology, 314(1-4), 78-91. 
	double fF = 1 / mM;
	//      TOPMODEL, SZF = 1./SZM
	//CCSF (m) coefficient is product of WFS_m (m), wetting front suction, and DWSMI (deficit water soil moisture initial or delta theta), the initial available storage fraction used in Green Ampt (has incorrect units of meters in name, and should be m3/m3, or volumetric moisture)
	double CSSF = folder->ParamDict["WFS_m"] * folder->ParamDict["DWSMI"];
	//      TOPMODEL, CD=HF*DTH
	//bPP is Water_forSoilRecharge_m Rate (m / timestep) 
	double bPP = bR / bDT;
	//	    TOPMODEL, RINT = P/DT

	//F1 (m) is cumulative infiltration estimate at start 
	double F1 = 0.0;
	F1 = folder->VarDict["Infiltration_viaInfilEx_cumulative_m"];
	//      TOPMODEL, F1=CUMF

	//RR (m / timestep) value of rainfall rate when it equals potential infilration rate at start of ponding, based on Green Ampt theory 
	//Theory from Eq. 2 to 8 in Beven (1984). Infiltration into a class of vertically non-uniform soils. Hydrological Sciences Journal 29: 425434.
	//		Before ponding, potential infiltration rate > rainfall rate
	//		Potential infiltration rate declines from maximum when cumulative infiltration = 0 toward minimum capacity, defined as K0
	//		Rainfall rate has to exceed potential infiltration rate for ponding to occur
	double RR = 0.0;

	//Theory of power function decay requires m <= S based on constraint governing decay with depth of transmissivity, Tz = T0*(1-S/m)^n  
	//S (m) is soil moisture deficit between top of soil and groundwater table, defined as Theta_sat-Theta_wp or Theta_sat-Theta_fc (based on Beven and Wood, 1983).
	//Theory explained after Eq 5 of Wang, J., Endreny, T. A., & Hassett, J. M. (2006). Power function decay of hydraulic conductivity for a TOPMODEL-based infiltration routine Hydrological Processes, 20(18), 3825-3834. 
	//Wang et al (2006) after Eq 5. explains in power decay conditions (when n<=1) the m parameter sets the upper limit for the soil moisture deficit S (m), due to Eq 7, T = T0(1-S/m)^n. If n = 2, then S > m.
	if(F1 >= mM)
	{
		//set cumulative infiltration to 1/1000 below mM
		F1 = 0.999 * mM;
		folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] = F1;
		//set water depth to 1/1000 above mM
		RR = 0.001 * mM;
		folder->VarDict["infilQ"] = RR;

		// Edited by Tom - 8-16-16: check to make sure the infiltrated water (at this point it is set equal to 0.001*mM, but that might be greater than BP
		if (folder->VarDict["infilQ"] > bPP) {
			folder->VarDict["infilQ"] = bPP;
		}
	}
	else
	{
		//linear version of power function decay
		if (nN <= 1.0) {
			//RR is represented in Eq 16 of Wang et al. (2006). Power function decay of hydraulic conductivity for a TOPMODEL-based infiltration routine Hydrological Processes, 20(18), 3825-3834. 
			RR = -K0 * fF * (CSSF + F1) / log(1 - F1 * fF);
		}
		//parabolic version of power function decay
		else {
			//RR is represented in Eq 20 of Wang et al. (2006). Power function decay of hydraulic conductivity for a TOPMODEL-based infiltration routine Hydrological Processes, 20(18), 3825-3834. 
			RR = K0 * (CSSF + F1) * (1 - F1 * fF) / F1;
		}
		//Ponding begins due to (Water_forSoilRecharge_m rate, bPP) > (rainfall rate when ponding begins, RR), and then infiltration excess runoff generated
		if(RR < bPP)
		//		TOPMODEL, IF(R2.LT.RINT)THEN
		{
			//pondTime (time units) is time ponding occurs = timestep number * timestep duration
			folder->VarDict["pondTime"] = bIT * bDT;
			//		TOPMODEL, TP=(IT-1.)*DT
			if (folder->VarDict["pondTime"] < 0.0) {
				folder->VarDict["pondTime"] = 0.0;
			}
			//Flag ponding = 1
			folder->VarDict["ponding"] = 1;
			//		TOPMODEL, IROF=1
			//F (m) is cumulative infiltration
			folder->VarDict["IP"] = folder->VarDict["Infiltration_viaInfilEx_cumulative_m"];
			//		TOPMODEL, F=CUMF
			
			//inPonding prepare parameters for infiltration during ponding
			inPonding(bIT,bPP,bDT, folder, input);
		}
		//Ponding does not begin, (Water_forSoilRecharge_m rate, bPP) < (rainfall rate when ponding begins, RR)
		else {
			//estimateTP to get estimate of when ponding will occur
			estimateTP(bIT, bR, bDT, folder, input);     // estimate ponding start time 
			//		TOPMODEL, INITIAL ESTIMATE OF TIME TO PONDING
		}
	}

}

//Routine from USGS Topmodel Fortran Code from Wolock (USGS)/Bevin, subroutine titled "INITIAL ESTIMATE OF TIME TO PONDING &     7 F2=CUMF+DT*RINT"
//Option: infiltrate all water if ponding has not occured, estimate time when ponding will occur based on current Water_forSoilRecharge_m rate; tpIT is timestep number, tpR is Water_forSoilRecharge_m for no ponding situation, tpDT is timestep duration
void SoilInfiltrationPowDecayCalc::estimateTP(int tpIT,  double tpR, double tpDT, DataFolder *folder, Inputs *input)
{
	
	folder->VarDict["pondTime"] = 0.0;

	// Scale parameter of power function infiltration (N)
	double nN = folder->ParamDict["n"];   // Scale parameter of power function (N)

	//mM is scaling parameter used to adjust infiltration rates from Config inputs; TOPMODEL theory 
	double mM = folder->ParamDict["m"];
	//K0 hydraulic conductivity (m / timestep) is adjusted to per time step based on input as per hour and TimeStep_sec
	double K0 = folder->ParamDict["K0_mph"] * folder->ParamDict["TimeStep_sec"] / 3600;
	//fF = 1/mM, scaling parameter for infiltration rates, taken directly from TOPMODEL code SZF = 1./SZM
	//fF could be thetaSoilMoisture/mMin, given in text below Eq 10 of Wang et al. (2005). Flexible Modeling Package for Topographically Based Watershed Hydrology. Journal of Hydrology, 314(1-4), 78-91. 
	double fF = 1 / mM;
	//CCSF (m) coefficient is product of WFS_m (m), wetting front suction, and DWSMI (deficit water soil moisture initial or delta theta), the initial available storage fraction used in Green Ampt (has incorrect units of meters in name, and should be m3/m3, or volumetric moisture)
	double CSSF = folder->ParamDict["WFS_m"] * folder->ParamDict["DWSMI"];                   // CSSF = WFSuction*DWSMI

	//tpPP is Water_forSoilRecharge_m Rate (m / timestep) 
	double tpPP = tpR / tpDT;
	//FI is cumulative infiltration at start
	double F1 = 0.0;

	//RR (m / timestep) value of rainfall rate when it equals potential infilration rate at start of ponding, based on Green Ampt theory 
	//Theory from Eq. 2 to 8 in Beven (1984). Infiltration into a class of vertically non-uniform soils. Hydrological Sciences Journal 29: 425434.
	double RR = 0.0;
	//E (m) is epsilon error threhold for closure of iteration to find cumulative infiltration
	double E = 0.00001;
	//FF (m) is estimate of cumulative infiltration
	double FF = 0.0;
	//F2 (m) is cumulative infiltration + Water_forSoilRecharge_m
	double F2 = folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] + tpPP;
	//		TOPMODEL, F2=CUMF+DT*RINT
	
	//Ponding will not occur during timestep, cumulative infiltration + Water_forSoilRecharge_m = 0, from TOPMODEL code as IF(F2.EQ.0.)GO TO 20
	if (F2 == 0.0) {
		//noPonding routine called, all Water_forSoilRecharge_m (tpR) is infiltrated as infilQ
		noPonding(tpR, folder, input, tpIT);
	}
	//Ponding may occur during timestep, cumulative infiltration + runoffToIfil > 0
	else
	{
		//RR (m / timestep) value of rainfall rate when it equals potential infilration rate at start of ponding, based on Green Ampt theory 
		//Theory from Eq. 2 to 8 in Beven (1984). Infiltration into a class of vertically non-uniform soils. Hydrological Sciences Journal 29: 425434.
		//		Before ponding, potential infiltration rate > rainfall rate
		//		Potential infiltration rate declines from maximum when cumulative infiltration = 0 toward minimum capacity, defined as K0
		//		Rainfall rate has to exceed potential infiltration rate for ponding to occur

		//Cumulative infiltration F (m) constrained by theory of soil moisture deficit S (m)
		//Theory of power function decay requires m <= S based on constraint governing decay with depth of transmissivity, Tz = T0*(1-S/m)^n  
		//S (m) is soil moisture deficit between top of soil and groundwater table, defined as Theta_sat-Theta_wp or Theta_sat-Theta_fc (based on Beven and Wood, 1983).
		//Theory explained after Eq 5 of Wang, J., Endreny, T. A., & Hassett, J. M. (2006). Power function decay of hydraulic conductivity for a TOPMODEL-based infiltration routine Hydrological Processes, 20(18), 3825-3834. 
		//Wang et al (2006) after Eq 5. explains in power decay conditions (when n<=1) the m parameter sets the upper limit for the soil moisture deficit S (m), due to Eq 7, T = T0(1-S/m)^n. If n = 2, then S > m.
		if(F2 >= mM)
		{
			//set water depth to 1/1000 above mM
			RR = 0.001 * mM;
			//set cumulative infiltration to 1/1000 below mM
			F2 = 0.999 * mM;
		}
		else
		{
			//linear version of power function decay
			if (nN <= 1.0) {
				//RR is represented in Eq 16 of Wang et al. (2006). Power function decay of hydraulic conductivity for a TOPMODEL-based infiltration routine Hydrological Processes, 20(18), 3825-3834. 
				RR = -K0 * fF * (CSSF + F2) / log(1 - F2 * fF);
			}
			//parabolic version of power function decay
			else {
				//RR is represented in Eq 20 of Wang et al. (2006). Power function decay of hydraulic conductivity for a TOPMODEL-based infiltration routine Hydrological Processes, 20(18), 3825-3834. 
				RR = K0 * (CSSF + F2) * (1 - F2 * fF) / F2;
			}
		}


		//Ponding does not begin due to (Water_forSoilRecharge_m rate, tpPP) < (rainfall rate when ponding begins, RR), from TOPMODEL code IF(R2.GT.RINT)GO TO 20
		if (RR > tpPP) {
			//noPonding routine called, all Water_forSoilRecharge_m (tpR) is infiltrated as infilQ
			noPonding(tpR, folder, input, tpIT);
		}

		//Ponding does begin due to (Water_forSoilRecharge_m rate, tpPP) > (rainfall rate when ponding begins, RR), from TOPMODEL code IF(R2.GT.RINT)GO TO 20
		else
		{
			//initial estimate of F (m) cumulative infiltration when ponding occurs = cumulative infiltration + (rainfall rate when ponding begins) * timestep duration, from TOPMODEL code F=CUMF+R2*DT
			folder->VarDict["F"] = folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] + tpDT * RR;
			//      TOMODEL, F=CUMF+R2*DT

			
			//Cumulative infiltration F (m) constrained by theory of soil moisture deficit S (m)
			//Theory of power function decay requires m <= S based on constraint governing decay with depth of transmissivity, Tz = T0*(1-S/m)^n  
			//S (m) is soil moisture deficit between top of soil and groundwater table, defined as Theta_sat-Theta_wp or Theta_sat-Theta_fc (based on Beven and Wood, 1983).
			//Theory explained after Eq 5 of Wang, J., Endreny, T. A., & Hassett, J. M. (2006). Power function decay of hydraulic conductivity for a TOPMODEL-based infiltration routine Hydrological Processes, 20(18), 3825-3834. 
			if (folder->VarDict["F"] >= mM) {
				//set cumulative infiltration to 1/1000 below mM
				folder->VarDict["F"] = 0.999 * mM;
			}
			//for loop to find F (m) cumulative infiltration when ponding occurs, from TOPMODEL code = DO 9 I=1,20
			for(int i=0; i<20; i++)
			{
				//linear version of power function decay
				if (nN <= 1.0) {
					//RR is represented in Eq 16 of Wang et al. (2006). Power function decay of hydraulic conductivity for a TOPMODEL-based infiltration routine Hydrological Processes, 20(18), 3825-3834. 
					RR = -K0 * fF * (CSSF + F2) / log(1 - F2 * fF);
				}
				//parabolic version of power function decay
				else {
					//RR is represented in Eq 20 of Wang et al. (2006). Power function decay of hydraulic conductivity for a TOPMODEL-based infiltration routine Hydrological Processes, 20(18), 3825-3834. 
					RR = K0 * (CSSF + F2) * (1 - F2 * fF) / F2;
				}
				//Ponding does not begin due to (Water_forSoilRecharge_m rate, tpPP) < (rainfall rate when ponding begins, RR)
				if (RR > tpPP) {
					//      TOPMODEL, IF(R2.GT.RINT)THEN
					//FI is cumulative infiltration = folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] + tpDT*RR, from TOPMODEL F1=F
					F1 = folder->VarDict["F"];
					//	  TOPMODEL, F1=F
					//F is average of two estimates of cumulative infiltration, F2 = folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] + tpR and folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] + tpDT*RR;
					folder->VarDict["F"] = (0.5 * (F2 + folder->VarDict["F"]));
					//	  TOPMODEL, F=(F2+F)*0.5
					//FF (m) is estimate of cumulative infiltration at ponding from prior timestep
					FF = F1;
				}

				//Ponding does begin due to (Water_forSoilRecharge_m rate, tpPP) < (rainfall rate when ponding begins, RR)
				else
				{
					//FI is cumulative infiltration = folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] + tpDT*RR, from TOPMODEL F1=F, from TOPMODEL F2=F
					F2 = folder->VarDict["F"];
					//	  TOPMODEL, F2=F
					//F is average of two estimates of cumulative infiltration, F1 = 0 and folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] + tpDT*RR;
					folder->VarDict["F"] = (0.5 * (F1 + folder->VarDict["F"]));
					//	  TOPMODEL, F=(F1+F)*0.5
					//FF (m) is estimate of cumulative infiltration at ponding from prior timestep
					FF = F2;
				}

				//close for loop to estimate ponding if change in estimated cumulative infiltration between timesteps < error threshold
				if(fabs(folder->VarDict["F"] - FF) < E)
				{
					//pondTime (time units) is time ponding occurs = (timestep number * timestep duration) - interval of time for additional infiltration at given runoffToInfilRate
					folder->VarDict["pondTime"] = ((tpIT - 1) * tpDT + (folder->VarDict["F"] - folder->VarDict["Infiltration_viaInfilEx_cumulative_m"])/tpPP);
					//      TOPMODEL, TP=(IT-1)*DT+(F-CUMF)/RINT

					//no ponding occurs in this timestep if pondTime occurs after the start of the next time step
					if(folder->VarDict["pondTime"] > tpIT * tpDT ) {
					//      TOPMODEL, IF(TP.GT.IT*DT)GO TO 20
						//noPonding routine called, all Water_forSoilRecharge_m (tpR) is infiltrated as infilQ
						noPonding(tpR, folder, input, tpIT);
						//End iteration
						break;
					}
					
					//ponding occurs in this timestep
					//IP (m) is cumulative infiltration at time of ponding
					folder->VarDict["IP"] = folder->VarDict["F"];

					//Flag ponding = true (1)
					folder->VarDict["ponding"] = 1;
					//	TOPMODEL, IROF=1

					//inPonding routine called
					inPonding(tpIT,tpPP,tpDT,folder, input);
					//End iteration
					break;
				}
			
			}
		
		}
	
	}
}


//Routine from USGS Topmodel Fortran Code from Wolock (USGS)/Bevin, subroutine titled "NEWTON-RAPHSON SOLUTION FOR F(T)"
//Option: find infiltration rate during ponding; ipIT is timestep number, ipPP is Water_forSoilRecharge_m for no ponding situation, ipDT is timestep duration
void SoilInfiltrationPowDecayCalc::inPonding(int ipIT, double ipPP, double ipDT, DataFolder *folder, Inputs * input)
{

	//n is coefficient for power function decay of saturated hydraulic conductivity with depth, due to decreasing macropores with depth
	//Figure 1 of Wang et al. (2006) shows how Transmissivity varies with n ranging from 0.25 to 16
	double nN = folder->ParamDict["n"];
	//m (m) is scaling parameter describing exponential decay of saturated hydraulic conductivity with depth, due to decreasing macropores with depth
	//Beven, Lamb, et al. (1995) give a physical interpretation of the decay parameter m (m) is that it controls the effective depth, z (m) of the catchment soil profile. This it does interactively in Eq T = T0 * exp (-f*z), where f = (Theta_sat - Theta_fc)/m. See Figure 18.1 for values of m ranging from 0.02 to 0.05 creating effective depths z ranging from 1 to 2 m.
	//Beven, K., Lamb, R., Quinn, P., Romanowics, R., & Freer, J. (1995). TOPMODEL. In V. P. Singh (Ed.), Computer models of watershed hydrology (pp. 627-688). Colorado: Water Resources Publications.
	double mM = folder->ParamDict["m"];
	//K0 hydraulic conductivity (m / timestep) is adjusted to per time step based on input as per hour and TimeStep_sec
	double K0 = folder->ParamDict["K0_mph"] * folder->ParamDict["TimeStep_sec"] / 3600;
	//fF = 1/mM, scaling parameter for infiltration rates, taken directly from TOPMODEL code SZF = 1./SZM
	//f defined differently by Beven, Lamb, et al. (1995) before Eq 1 explain f = (Theta_sat - Theta_wp) / m. Beven, K., Lamb, R., Quinn, P., Romanowics, R., & Freer, J. (1995). TOPMODEL. In V. P. Singh (Ed.), Computer models of watershed hydrology (pp. 627-688). Colorado: Water Resources Publications.
	//f defined as (Change in Soil Theta) / m in text below Eq 10 of Wang et al. (2005). Flexible Modeling Package for Topographically Based Watershed Hydrology. Journal of Hydrology, 314(1-4), 78-91. 
	//f = 1 / m can be derived from Beven, Lamb et al. (1995) when combining conversion of exponent (-S/m) to exponent (-fz)
	double fF = 1 / mM;
	//CCSF (m) coefficient is product of WFS_m (m), wetting front suction, and DWSMI (deficit water soil moisture initial or delta theta), the initial available storage fraction used in Green Ampt (has incorrect units of meters in name, and should be m3/m3, or volumetric moisture)
	double CSSF = folder->ParamDict["WFS_m"] * folder->ParamDict["DWSMI"];			// CSSF = WFSuction*DWSMI

	//EE (log space) is epsilon error threhold for closure of iteration to find cumulative infiltration
	double EE = 10.0;

	//F1 (m) is cumulative infiltration at time of ponding
	double F1 = folder->VarDict["IP"];
	//F2 (m) is cumulative infiltration at end of time step if all water infiltrates
	double F2 = folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] + ipDT * ipPP;

	//temporary infiltration rate estimate initialized
	double CC = 0.0;
	//infiltration rate estimate initialized
	double IR = 0.0;      

	//time between end of timestep and start of ponding (time unit)
	double tt = ipIT * ipDT - folder->VarDict["pondTime"]; 

	//linear version of power function decay
	if (nN <= 1.0) {
		//CC is represented in rearranged Eq 17 of Wang et al. (2006). Power function decay of hydraulic conductivity for a TOPMODEL-based infiltration routine Hydrological Processes, 20(18), 3825-3834. 
		CC = K0 / mM * tt;   
	}
	//parabolic version of power function decay
	else {
		//RR is represented in rearranged Eq 22 of Wang et al. (2006). Power function decay of hydraulic conductivity for a TOPMODEL-based infiltration routine Hydrological Processes, 20(18), 3825-3834. 
		CC = mM / (K0 * (mM + CSSF));    
	}
	
	//Cumulative infiltration F (m) constrained by theory of soil moisture deficit S (m)
	//Theory of power function decay requires m <= S based on constraint governing decay with depth of transmissivity, Tz = T0*(1-S/m)^n  
	//S (m) is soil moisture deficit between top of soil and groundwater table, defined as Theta_sat-Theta_wp or Theta_sat-Theta_fc (based on Beven and Wood, 1983).
	//Theory explained after Eq 5 of Wang, J., Endreny, T. A., & Hassett, J. M. (2006). Power function decay of hydraulic conductivity for a TOPMODEL-based infiltration routine Hydrological Processes, 20(18), 3825-3834. 
	//Wang et al (2006) after Eq 5. explains in power decay conditions the m parameter sets the upper limit for the soil moisture deficit S (m), due to Eq 7, T = T0(1-S/m)^n. If n = 2, then S > m. If n <=1, S>m will generate negative T profiles., if n >=2, S>m will generate non-monotonic T profiles.
	if(F2 >= mM)
		//set cumulative infiltration to 1/1000 below mM
		F2 = 0.999 * mM;
	
	folder->VarDict["F"] = F2;

	//IP (m) is cumulative infiltration at time of ponding
	//Cumulative infiltration IP (m) constrained by theory of soil moisture deficit S (m)
	//Theory of power function decay requires m <= S based on constraint governing decay with depth of transmissivity, Tz = T0*(1-S/m)^n  
	//S (m) is soil moisture deficit between top of soil and groundwater table, defined as Theta_sat-Theta_wp or Theta_sat-Theta_fc (based on Beven and Wood, 1983).
	//Theory explained after Eq 5 of Wang, J., Endreny, T. A., & Hassett, J. M. (2006). Power function decay of hydraulic conductivity for a TOPMODEL-based infiltration routine Hydrological Processes, 20(18), 3825-3834. 
	//Wang et al (2006) after Eq 5. explains in power decay conditions the m parameter sets the upper limit for the soil moisture deficit S (m), due to Eq 7, T = T0(1-S/m)^n. If n = 2, then S > m. If n <=1, S>m will generate negative T profiles., if n >=2, S>m will generate non-monotonic T profiles.
	//Figure 1 from Wang et al. (2006) illustrates the decay of transmissivity, Tz, with depth as a function of n, varying from 0.25 to 16
	if (folder->VarDict["IP"] >= mM) {
		//set cumulative infiltration to 1/1000 below mM
		folder->VarDict["IP"] = 0.999 * mM;
	}

	//If cummulative infiltration at end of time step w/ all water infiltrated F2 < cumulative infiltration at time of ponding IP
	if (F2 < folder->VarDict["IP"]) {
		//set infiltrated water to 0 
		folder->VarDict["infilQ"] = 0.0;
	}

	//Numerical routine to find infiltration rate
	for(int i=0; i<45; i++)
	{

		//Cumulative infiltration F (m) constrained by theory of soil moisture deficit S (m)
		//Theory of power function decay requires m <= S based on constraint governing decay with depth of transmissivity, Tz = T0*(1-S/m)^n  
		//S (m) is soil moisture deficit between top of soil and groundwater table, defined as Theta_sat-Theta_wp or Theta_sat-Theta_fc (based on Beven and Wood, 1983).
		//Theory explained after Eq 5 of Wang, J., Endreny, T. A., & Hassett, J. M. (2006). Power function decay of hydraulic conductivity for a TOPMODEL-based infiltration routine Hydrological Processes, 20(18), 3825-3834. 
		//Wang et al (2006) after Eq 5. explains in power decay conditions the m parameter sets the upper limit for the soil moisture deficit S (m), due to Eq 7, T = T0(1-S/m)^n. If n = 2, then S > m. If n <=1, S>m will generate negative T profiles., if n >=2, S>m will generate non-monotonic T profiles.
		//Figure 1 from Wang et al. (2006) illustrates the decay of transmissivity, Tz, with depth as a function of n, varying from 0.25 to 16
		if (folder->VarDict["F"] >= mM) {
			//set cumulative infiltration to 1/1000 below mM
			folder->VarDict["F"] = 0.999 * mM;
		}

		//linear version of power function decay
		if(nN <= 1.0)
		{
			//call simpsonIntegration in rearranged Eq 17 of Wang et al. (2006). Power function decay of hydraulic conductivity for a TOPMODEL-based infiltration routine Hydrological Processes, 20(18), 3825-3834. 
			simpsonIntegration(folder->VarDict["IP"], folder->VarDict["F"], IR, folder, input);   // Simpson integration
			//compute error between two infiltration rate estimates
			EE = IR - CC;
		}
		//parabolic version of power function decay
		else 
		{
			//compute error between two infiltration rates; RHS is represented in rearranged Eq 22 of Wang et al. (2006). Power function decay of hydraulic conductivity for a TOPMODEL-based infiltration routine Hydrological Processes, 20(18), 3825-3834. 
			EE = CC * (-CSSF * log((CSSF + folder->VarDict["F"]) / (CSSF + folder->VarDict["IP"])) + mM * log((1 - folder->VarDict["IP"] * fF) / (1 - folder->VarDict["F"] * fF))) - tt;
		}

		if(folder->VarDict["F"] == 0.999 * mM && EE < 0.0)
		{
			folder->VarDict["F"] = 0.999*mM;
			break;
		}

		if(EE > 0.0)
		{
	       F2 = folder->VarDict["F"];
		   folder->VarDict["F"] = 0.5 * (F1 + folder->VarDict["F"]);
		   if(EE < 0.00001)
			   break;
		}
		else
		{
		   F1 = folder->VarDict["F"];
		   folder->VarDict["F"] = 0.5 * (F2 + folder->VarDict["F"]);
		   if(fabs(EE) < 0.00001)
			   break;
		}

	}
	
	folder->VarDict["infilQ"] = folder->VarDict["F"] - folder->VarDict["Infiltration_viaInfilEx_cumulative_m"];	
	folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] = folder->VarDict["F"];
	
	if (folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] >= mM) {
		folder->VarDict["Infiltration_viaInfilEx_cumulative_m"] = 0.999 * mM;
	}
	if (folder->VarDict["infilQ"] < 0.0) {
		folder->VarDict["infilQ"] = 0.00001;
	}
}

//Routine introduced by Wang et al. (2006). Power function decay of hydraulic conductivity for a TOPMODEL-based infiltration routine Hydrological Processes, 20(18), 3825-3834. 
//Simpson numerical method is used to solve Wang et al. (2006) Eq 22 for I at any time t after ponding when tp and Ip are known from Eq 20.
void SoilInfiltrationPowDecayCalc::simpsonIntegration(double IP, double F, double& IR, DataFolder *folder, Inputs * input)
{

	int N =1;
	double H = F - IP;
	double T1 = 0.0;
	double T2 =0.0;
	double S1 = 0.0;
	double S2 = 0.0;
	double X =0.0;
	double P ;
	double fF = 1/folder->ParamDict["m"];
	double CSSF = (folder->ParamDict["WFS_m"]*folder->ParamDict["DWSMI"]);



	bool finish = false;
	T1 = H * (-log(1 - IP * fF) / (CSSF + IP) - log(1 - F * fF) / (CSSF + F) ) / 2.0 ;
	S1 = T1;

	do
	{
		P =0.0;
		for(int k=0; k<=N-1; k++)
		{
			X = IP + (k + 0.5) * H ;
			P +=-log(1 - X * fF)/(CSSF + X);
		}

		T2 = (T1 + H * P) / 2.0;
		S2 = (4 * T2 - T1) / 3.0;

		if(fabs(S2-S1) >= 0.00001)
		{
			T1 = T2;
			N += N;
			H = 0.5 * H;
			S1 = S2;
			finish = false;
		}
		else
		{
			IR = S2;
			finish = true;
		}
	} while(!finish);
}