#include "IndexCalculator.h"
#include <utility>
#include <iomanip>

//Computes topographic index using exponential decay or power decay approaches to transform ratio of flow accumulation area per contour width to slope
//transform function converts topographic index (TI) values in map and histogram from exponential decay ln formulation to power decay formulation
//	TI transformed by taking exponential of ln(a/tanB), raised to power (1/n), units change from ln([L]) to [L]^(1/n)

//Computes flow acumulation area using D8 (O'Callaghan and Mark, 1984), MFD (Quinn et al., 1991), or D-Infinity (Tarboton, 1997) flow direction approaches
//O'Callaghan, J. F., & Mark, D. M. (1984). The Extraction of Drainage Networks from Digital Elevation Data. Computer Vision, Graphics, and Image Processing, 28, 323-344. 
//Quinn, P., Beven, K., Chevallier, P., & Planchon, O. (1991). The Prediction of Hillslope Flow Paths for Distributed Hydrological Modelling using Digital Terrain Models. Hydrological Processes, 5, 59-79. 
//Tarboton, D. G. (1997). A New Method for the Determination of Flow Directions and Upslope Areas in Grid Digital Elevation Models. Water Resources Research, 33, 309-319. 

//Impervious area removed in IndexCalculator::calcTWI, keeping with TOPURBAN theory of Valeo and Moin (2000)
//Valeo, C., and Moin, S. M. A. (2000). Variable source area modelling in urbanizing watersheds. Journal of Hydrology, 228(1-2), 68-81. 

//Topographic index algorithm taken from TOPMODEL code developed by Keith Beven and used by USGS D.M. Wolock https://pubs.er.usgs.gov/publication/wri934124
//Wolock, D. M. (1993). Simulating the variable-source-area concept of watershed hydrology with TOPMODEL (USGS Water-Resources Investigation Report 93-4124).
//	The algorithm requires the topographic index (TI) bins (ia) are sorted from highest (wettest) to lowest TI values
//	The algorithm requires when ia = 0, the TI bin Index[0] should be the upper limit and its IndexArea[0] = 0
//	The algorithm presumes for each ia, the area of catchment between discrete TI values Index[ia] and Index[ia+1] equals IndexArea[ia]
//	The algorithm presumes for each ia, the area of catchment bounding Index[ia] equals (IndexArea[ia] + IndexArea[ia+1])/2 
//	The algorithm in C++ has TI values for ia = 0 to ia = [IndexSize - 1], yet requires use of IndexArea[IndexSize] = 0 in the above fractional area equation

//IndexCalculator constructor for PowerDecay TI indicated when passing variable N
//	Variables passed include: file directory, N power decay parameter, NAC number of topographic index bins, simulationMode, 
//cte 20210520 adding IA to function
//IndexCalculator::IndexCalculator(std::string directory, double N, int NAC, std::string simulationMode, std::string routingAlgorithm)
IndexCalculator::IndexCalculator(std::string directory, double N, int NAC, std::string simulationMode, std::string routingAlgorithm, double ImperviousArea_total_frac)
//	Variable success initialized to 1, indicating successful run; it is set to 0 if readDEMData fails
	: directory(directory), N(N), NAC(NAC), simulationMode(simulationMode), routingAlgorithm(routingAlgorithm), pi(3.14159265359), success(1)
{
	//init() function is in IndexCalculator.h file, initializes private data members of IndexCalculator class: 
	//  rows = 0; cols = 0;  NATB = 0; DX = 0.0; fArea = 1.0; totalFImpArea = 0.0;
	init();
	//clear() function clears all the vectors used in the index calculations
	clear();

	//Resizing the vectors to contain NAC number of topographic index bins
	//y is counter of occurrences of TI values in each index bin
	y.resize(NAC);
	//AC is area of catchment between two topographic index (TI) values
	AC.resize(NAC);
	//ST is TI, defined as Ln(a/tanB) or (a/tanB)^n 
	ST.resize(NAC);   
		
	//if readDEMData() = True, call indexCalcPow function
	if (readDEMData())
		//cte 20210520 adding IA to function
		//indexCalPow();
		indexCalPow(ImperviousArea_total_frac);
	//if readDEMData() = False, reset success variable to 0 to report failure
	else
		success = 0;
}


//IndexCalculator constructor for ExponentialDecay TI indicated by absence of passing variable N
//	Variables passed include: file directory, N array size, simulationMode, routingAlgorithm
//cte 20210520 adding IA to function
//IndexCalculator::IndexCalculator(std::string directory, int NAC, std::string simulationMode, std::string routingAlgorithm)
IndexCalculator::IndexCalculator(std::string directory, int NAC, std::string simulationMode, std::string routingAlgorithm, double ImperviousArea_total_frac)
//	Variable success initialized to 1, indicating successful run; it is set to 0 if readDEMData fails
	: directory(directory), NAC(NAC), simulationMode(simulationMode), routingAlgorithm(routingAlgorithm), pi(3.14159265359), success(1)
{
	//init() function is in IndexCalculator.h file, initializes private data members of IndexCalculator class: 
	//  rows = 0; cols = 0;  NATB = 0; DX = 0.0; fArea = 1.0; totalFImpArea = 0.0;
	init();
	//clear() function clears all the vectors used in the index calculations
	clear();

	//Resizing the vectors to contain NAC number of topographic index bins
	//y is counter of occurrences of TI values in each index bin
	y.resize(NAC);
	//AC is area of catchment between two topographic index (TI) values
	AC.resize(NAC);
	//ST is TI, defined as Ln(a/tanB) or (a/tanB)^n 
	ST.resize(NAC);

	//if readDEMData() = True, call indexCalcExp function
	if (readDEMData())
		//cte 20210520 adding IA to function
		//indexCalExp();
		indexCalExp(ImperviousArea_total_frac);
	//if readDEMData() = False, reset success variable to 0 to report failure
	else
		success = 0;
}

// The clear() clear all the following vectors of their contents
void IndexCalculator::clear()
{
	//clear variable vectors
	elevation.clear();
	ATB.clear();
	Area.clear();
	y.clear();
	AC.clear();
	ST.clear();
}

//checkfolder function tests if map number of rows or columns are value of 0 for current folder
//	checks the bounds of the files, including: DEM, slopeorganizer, flowDirorganizer, flowAccumorganizer, TWIorganizer
bool IndexCalculator::checkfolder(int row, int col)
{
	if (row<0 || row>(rows - 1) || col<0 || col>(cols - 1))
		return false;
	else
		return true;
}

//elevation function gets value of map at specific rows and column for current folder
double IndexCalculator::getElevation(int row, int col)
{
	return elevation[row][col];
}

//fileExists function tests if filename exists
bool IndexCalculator::fileExists(const std::string& filename)
{
	struct stat buf;
	if (stat(filename.c_str(), &buf) != -1)
	{
		return true;
	}
	return false;
}

//readDEMData function reads in the DEMDATA
bool IndexCalculator::readDEMData()
{
	//tempElv temporarily stores the read in elevation data
	double tempElv;  
	//initialize size
	int size = 0;
	//MAX is capacity of buffer vector, which holds header rows of DEM file
	const int MAX = 80;
	//initialize buffer vector, which holds header rows of DEM file
	char buffer[MAX];

	//file name is dem.dat, but will try dem.txt if dem.dat does not exist
	std::string location = directory + "dem.dat";
	bool datExists = fileExists(location);
	//if does not exist = True
	if (!datExists)
	{
		location = directory + "dem.txt";
	}

	//if neither dem.dat or dem.txt exists
	else if (!fileExists(location))
	{
		std::cout << "Cannot find DEM file" << std::endl;
		LogMessage::log("Cannot find DEM file. Should be named dem.dat or dem.txt in directory: " + directory);

		//exit function
		return 0;
	}
	//ifstream function reads infile from location
	ifstream infile(location);    

	//Tokenizer function, may check every line of data is followed by a carriage return without spaces
	if (!infile.good())
	{
		std::cout << "Cannot find DEM file" << std::endl;
		LogMessage::log("Cannot find DEM file. Should be named dem.dat or dem.txt in directory: "+directory);
		//exit function
		return 0;
	}

	//Console message with program status
	std::cout << "Reading DEM file\n" << std::endl;

	//buffer vector takes on values of infile, up to first 80 rows set by MAX
	infile >> buffer;

	//_catName used to hold map header data read from infile
	_catName = buffer;

	//Obtain rows, columns (cols), and cell size (DX) from header rows, using 1 of 2 formats of header
	//header has 6 lines of data if _catName has ncols or NCOLS, written in ESRI ASCII map format, when converted from Grid
	if (_catName == "ncols" || _catName == "NCOLS")  
	{
		std::cout << "DEM stats:" << std::endl;
		//parse infile to get 6 lines of header information from map file, rows, columns, lower left x and y coordinates, cell size, no-data value
		infile >> cols >> buffer >> rows >> buffer >> xllcorner >> buffer >> yllcorner >> buffer >> DX >> buffer >> NODATA;
		//write out rows, columns, cell size, no-data value
		std::cout << "Rows: " << rows << "   Cols: " << cols << "   cell size: " << DX << "   NODATA: " << NODATA << std::endl;
	}
	//header has 4 lines of data
	else  
	{
		std::cout << "DEM stats:" << std::endl;
		infile.clear();
		infile.seekg(0, ios::beg);
		infile.getline(buffer, MAX);
		_catName = buffer;
		//parse infile to get 4 lines of header information from map file, rows, columns, cell size, no-data value
		infile >> buffer >> cols >> buffer >> rows >> buffer >> DX >> buffer >> NODATA;
		//write out rows, columns, cell size, no-data value
		std::cout << "Rows: " << rows << "   Cols: " << cols << "   cell size: " << DX << "   NODATA: " << NODATA << std::endl;
	}
	
	//initialize rows in files that will hold data extracted from DEM file
	elevation.reserve(rows);
	slopeorganizer.reserve(rows);
	aspectorganizer.reserve(rows);
	flowDirorganizer.reserve(rows);
	flowAccumorganizer.reserve(rows);
	temporganizer.reserve(rows);
	infloworganizer.reserve(rows);
	TWIorganizer.reserve(rows);

	//initialize cols in files that will hold data extracted from DEM file
	for (int k = 0; k<rows; k++)
	{
		elevation.push_back(*new vector<double>(cols));
		slopeorganizer.push_back(*new vector<double>(cols));
		aspectorganizer.push_back(*new vector<double>(cols));
		flowDirorganizer.push_back(*new vector<double>(cols));
		flowAccumorganizer.push_back(*new vector<double>(cols));
		temporganizer.push_back(*new vector<double>(cols));
		infloworganizer.push_back(*new vector<double>(cols));
		TWIorganizer.push_back(*new vector<double>(cols));
	}

	double temp = 0.0;

	//loop through rows in DEM file along i counter
	for (int i = 0; i<rows; i++) {
		//loop through cols in DEM file along j counter
		for (int j = 0; j<cols; j++) {
			//Tokenizer function, may test if every line of data is followed by a carriage return without spaces
			if (!infile.good()) {
				infile.clear();
				//break from reading line
				break;
			}

			//read elevation data from DEM file
			infile >> tempElv;
			elevation[i][j] = tempElv;
			elevation1D.push_back(tempElv);

			//Initializing new organizers using NODATA, and real values later assigned
			slopeorganizer[i][j] = NODATA;
			aspectorganizer[i][j] = NODATA;
			flowDirorganizer[i][j] = NODATA;
			temporganizer[i][j] = NODATA;
			infloworganizer[i][j] = NODATA;
			TWIorganizer[i][j] = NODATA;

			//Testing if DEM is actually NODATA
			if (elevation[i][j] == NODATA)
				//flowAccumorganizer is set to NODATA
				flowAccumorganizer[i][j] = NODATA;
			else
				//flowAccumorganizer is set to 1 for all data other than NODATA
				flowAccumorganizer[i][j] = 1;
		}
	}
	//close DEM file
	infile.close();

	std::cout << "DEM file read." << std::endl << std::endl;

	//Return 1 to represent success
	return 1;
}

//Functions called for the Power Decay Topographic Index calculation
//cte 20210517 adding inputs
//void IndexCalculator::indexCalPow()
void IndexCalculator::indexCalPow(double ImperviousArea_total_frac)
{
	//Power function decay call to TI_Cal, to compute topographic index using different flow routing options
	//cte 20210517 adding inputs
	//TI_Cal();
	TI_Cal(ImperviousArea_total_frac);
	//If simulationMode is statistical, and has word "Stat" in string
	if (simulationMode.find("Stat") != std::string::npos) {
		//indexHist function calculates a TI histogram using natural logarithm, with bins holding magnitude of each TI value
		indexHist();
	}
	
	//transform function converts topographic index (TI) values in map and histogram from exponential decay ln formulation to power decay formulation
	//	TI transformed by taking exponential of ln(a/tanB), raised to power (1/n), units change from ln([L]) to [L]^(1/n)
	transform();

	//createDistorganizerFiles function creates maps of Topographic Index, Slope, Aspect, Flow Direction, Flow Accumulation as .dat files
	// maps called TIorganizer.dat, Slope.dat, Aspect.dat, FDorganizer.dat, FlowAccum.dat
	createDistorganizerFiles();
}

//Functions called for the Exponential Decay Topographic Index calculation
//cte 20210517 adding inputs
//void IndexCalculator::indexCalExp()
void IndexCalculator::indexCalExp(double ImperviousArea_total_frac)
{
	//Exponential function decay call to TI_Cal, to compute topographic index using different flow routing options
	//cte 20210517 adding inputs
	//TI_Cal();
	TI_Cal(ImperviousArea_total_frac);
	//If simulationMode is statistical, and has word "Stat" in string
	if (simulationMode.find("Stat") != std::string::npos) {
		//indexHist function calculates a TI histogram using natural logarithm, with bins holding magnitude of each TI value
		indexHist();
	}

	//createDistorganizerFiles function creates maps of Topographic Index, Slope, Aspect, Flow Direction, Flow Accumulation as .dat files
	// maps called TIorganizer.dat, Slope.dat, Aspect.dat, FDorganizer.dat, FlowAccum.dat
	createDistorganizerFiles();
}


//TI_Cal shared by exponential and power function decay approaches
//	Calls topographic index calculations: slope, and contributing area based on flow direction algorithm
//cte 20210517 testing use of input
//void IndexCalculator::TI_Cal()
void IndexCalculator::TI_Cal(double ImperviousArea_total_frac)
{
	cout << "Starting the Topographic Index calculation." << endl;
	LogMessage::log("Starting the Topographic Index calculation.");


	std::cout << "Computing the slope of each folder." << endl;
	LogMessage::log("Computing the slope of each folder.");
	
	//calcSlope computes DEM slope used in TI function, based on Horn (1981)
	calcSlope();

	std::cout << "Computing the flow direction of each folder." << endl;
	LogMessage::log("Computing the flow direction of each folder. Routing Algorithm: "+routingAlgorithm);
	
	//calcFlowDirection_DInifity computes the direction DEM cells will flow, using D-Infinity algorithm (Tarboton, 1997)
	if (routingAlgorithm == "DInfinity")
	{
		calcFlowDirection_DInf();
	}
	//calcFlowDirection_D8 computes the direction DEM cells will flow, using D8 algorithm (O'Callaghan and Mark, 1984)
	else if (routingAlgorithm == "D8")
	{
		calcFlowDirection_D8();
	}
	//calcFlowDirection_MFD computes the direction DEM cells will flow, using multiple flow direction algorithm (Quinn et al. 1991)
	else if (routingAlgorithm == "MFD")
	{
		calcFlowDirection_MFD();
	}

	std::cout << "Computing the specfic contributing area of each folder using " << routingAlgorithm << "." << endl;
	LogMessage::log("Computing the specfic contributing area of each folder.");

	//calcSCA_Dinf computes the contributing area to DEM cell will flow, using D-Infinity algorithm (Tarboton, 1997)
	if (routingAlgorithm == "DInfinity")
		calcSCA_Dinf();
	//calcSCA_D8 computes the contributing area to DEM cell will flow, using D8 algorithm (O'Callaghan and Mark, 1984)
	else if (routingAlgorithm == "D8")
		calcSCA_D8();
	//calcSCA_DMFD computes the contributing area to DEM cell will flow, using multiple flow direction algorithm (Quinn et al., 1991)
	else if (routingAlgorithm == "MFD")
		calcSCA_MFD();

	std::cout << "Computing the topographic index value of each folder." << endl;
	LogMessage::log("Computing the topographic index value of each folder.");

	//calcTWI function computes the topographic index using the contributing area and slope, used for both exponential decay and power decay function approaches
	//cte 20210517 add input to function
	//calcTWI();
	calcTWI(ImperviousArea_total_frac);
}

//calcSlope function will calculate DEM slope and aspect w/ 3rd order difference algorithm published by Horn (1981)
//	Nearest points weighted more than diagonal neighbors, center point has no influence; also known as the Sobel operator (Richards, 1986)
void IndexCalculator::calcSlope()
{
	//DEM folder layout: 	|6|7|0|
	//						|5|8|1|
	//						|4|3|2|

	double z;
	double organizer[8];
	double fx, fy;
	double slopefolder;
	double progress = 0;

	//Note: Whitebox flow direction vectors are arranged with clockwise indexing, x=1,y=-1 in NorthEast (top right), x=-1,y=1 in SouthWest (bottom left)
	int Dx[8] = { 1, 1, 1, 0, -1, -1, -1, 0 };
	int Dy[8] = { -1, 0, 1, 1, 1, 0, -1, -1 };

	//constant for conversion from radians to degrees
	double radToDeg = 180 / pi;
	
	//loop through all map rows 
	for (int row = 0; row<rows; row++) {
		//loop through all map columns
		for (int col = 0; col<cols; col++) {
			//z is elevation value for pixel
			z = getElevation(row, col);
			
			//if z is real data
			if (z != NODATA) {
				//loop through all 8 neighbors
				for (int k = 0; k<8; k++) {

					//checkfolder function confirms pixel is within map
					if (checkfolder(row + Dy[k], col + Dx[k]))
						//organizer updated for neighbor
						organizer[k] = getElevation(row + Dy[k], col + Dx[k]);
					else
						organizer[k] = NODATA;

					//if organizer is NODATA
					if (organizer[k] == NODATA)
						//organizer equal to elevation of center pixel
						organizer[k] = z;      
				}

				//Calculating the folder's slope based on Horn (1981), with clockwise rotation from NorthEast pixel = organizer[0]
				//New form of Horn Eqn (1981) for fx and fy with sorted addition and subtraction terms
				fx = (organizer[2] + 2 * organizer[1] + organizer[0] - organizer[4] - 2 * organizer[5]  - organizer[6]) / (8 * DX);
				fy = (organizer[6] + 2 * organizer[7] + organizer[0] - organizer[4] - 2 * organizer[3] - organizer[2]) / (8 * DX);
				//Old form of Horn Eqn (1981) for fx and fy with intermixed addition and subtraction terms
				//fx = (organizer[2] - organizer[4] + 2 * (organizer[1] - organizer[5]) + organizer[0] - organizer[6]) / (8 * DX);
				//fy = (organizer[6] - organizer[4] + 2 * (organizer[7] - organizer[3]) + organizer[0] - organizer[2]) / (8 * DX);

				//slopefolder (deg) is slope based on Horn (1981), taking atan of (rise over run) in degrees
				slopefolder = (atan(sqrt(fx * fx + fy * fy)) * radToDeg);

				//fx forced to minimum value when 0 for computation of aspect
				if (fx == 0)
					fx = 0.0000001;
				//fx forced to minimum value when 0
				if (fy == 0)
					fy = 0.0000001;

				//aspectorganizer is aspect based on Horn (1981) and WhiteBox, w/ Pi = 3.1415926 and Pi/2 = 1.5707963 to properly rotate aspect
				aspectorganizer[row][col] = 3.1415926 - atan(fy / fx) + 1.5707963 * fx / abs(fx);

				//slopefolder set to minimum value for ensuring topographic index calculations work
				if (slopefolder <= 0.001)
					slopeorganizer[row][col] = 0.001;
				else
					slopeorganizer[row][col] = slopefolder;

				//t is slope in radians
				//double t = slopefolder / radToDeg;
			}
			else
			{
				slopeorganizer[row][col] = NODATA;
			}
		}
	}
}


//calcFlowDirection_DInf function calculates upslope contributing areas using the D-Infinity algorithm (Tarboton, 1997) from Whitebox GAT
//Tarboton, D. G. (1997). A New Method for the Determination of Flow Directions and Upslope Areas in Grid Digital Elevation Models. Water Resources Research, 33, 309-319. 
void IndexCalculator::calcFlowDirection_DInf()
{
	//DX2 is distance between diagnonal folder centers
	double DX2 = DX * sqrt(2.0);
	//e0 is elevation
	double e0;
	//maxSlope is maximum slope
	double maxSlope;
	//flowDir is flow direction
	double flowDir = 0;
	//pi value
	double pi = 3.14159265358979323846;
	//constant arctan of 1
	double atanof1 = atan(1.0);
	//constants
	double af;
	double ac;
	double e1, r, s1, s2, s, e2;
	int current_row1, current_col1;
	int current_row2, current_col2;

	//DEM facet pixel coordinates for 8 neighbors, as defined in Table 1 of Tarboton (1997) describing D-Infinity
	int acVals[8] = { 0, 1, 1, 2, 2, 3, 3, 4 };
	int afVals[8] = { 1, -1, 1, -1, 1, -1, 1, -1 };

	//DEM 1 pixel coordinates for 8 neighbors, as defined in Table 1 of Tarboton (1997) describing D-Infinity
	int e1Col[8] = { 1, 0, 0, -1, -1, 0, 0, 1 };
	int e1Row[8] = { 0, -1, -1, 0, 0, 1, 1, 0 };

	//DEM 2 pixel coordinates for 8 neighbors, as defined in Table 1 of Tarboton (1997) describing D-Infinity
	int e2Col[8] = { 1, 1, -1, -1, -1, -1, 1, 1 };
	int e2Row[8] = { -1, -1, -1, -1, 1, 1, 1, 1 };

	//loop through all map rows 
	for (int row = 0; row<rows; row++) {
		//loop through all map columns
		for (int col = 0; col<cols; col++) {
			//e0 is elevation from DEM map 
			e0 = elevation[row][col];
			//if e0 is real data
			if (e0 != NODATA)
			{
				//initialize variables for algorithm
				maxSlope = -9999999999;
				flowDir = 360;
				//loop through all 8 neighbors
				for (int k = 0; k < 8; k++)
				{
					//ac and af are updated for neighbor
					ac = acVals[k];
					af = afVals[k];
					current_row1 = row + e1Row[k];
					current_col1 = col + e1Col[k];
					current_row2 = row + e2Row[k];
					current_col2 = col + e2Col[k];

					//checkfolder function confirms pixel is within map
					if (checkfolder(current_row1, current_col1))
						//e1 updated for neighbor
						e1 = elevation[current_row1][current_col1];
					else
						e1 = NODATA;

					//checkfolder function confirms pixel is within map
					if (checkfolder(current_row2, current_col2))
						//e2 updated for neighbor
						e2 = elevation[current_row2][current_col2];
					else
						e2 = NODATA;

					//if e1 and e2 are real data
					if (e1 != NODATA && e2 != NODATA)
					{
						//if e0 is higher than e1 and e2
						if (e0 > e1 && e0 > e2)
						{
							//Equations from Tarboton (1997)
							s1 = (e0 - e1) / DX;
							if (s1 == 0) {
								s1 = 0.00001;
							}
							s2 = (e1 - e2) / DX;
							r = atan(s2 / s1);
							s = sqrt(s1 * s1 + s2 * s2);
							if (s1 < 0 && s2 < 0) {
								s = -1 * s;
							}
							if (s1 < 0 && s2 == 0) {
								s = -1 * s;
							}
							if (s1 == 0 && s2 < 0) {
								s = -1 * s;
							}
							if (s1 == 0.00001 && s2 < 0) {
								s = -1 * s;
							}
							if (r < 0 || r > atanof1)
							{
								if (r < 0) {
									r = 0;
									s = s1;
								}
								else {
									r = atanof1;
									s = (e0 - e2) / DX2;
								}
							}
							if (s >= maxSlope && s != 0.00001)
							{
								maxSlope = s;
								flowDir = af * r + ac * (pi / 2);
							}
						}
						else if (e0 > e1 || e0 > e2) {
							if (e0 > e1) {
								r = 0;
								s = (e0 - e1) / DX;
							}
							else {
								r = atanof1;
								s = (e0 - e2) / DX2;
							}
							if (s >= maxSlope && s != 0.00001)
							{
								maxSlope = s;
								flowDir = af * r + ac * (pi / 2);
							}
						}
					}
				}
				if (maxSlope <= 0) {
					flowDirorganizer[row][col] = -1;
				}
				else {
					//re-orient the flow direction angles to match North = 0/360 degrees
					flowDir = floor((flowDir * (180 / pi)) * 10) / 10;
					flowDir = 360 - flowDir + 90;
					if (flowDir > 360) {
						flowDir = flowDir - 360;
					}
					flowDirorganizer[row][col] = flowDir;
				}
			}
			else {
				flowDirorganizer[row][col] = NODATA;
			}
		}
	}
}

//calcFlowDirection_D8 function calculates upslope contributing areas using the D8 (O'Callaghan and Mark, 1984) algorithm from Whitebox GAT
//O'Callaghan, J. F., & Mark, D. M. (1984). The Extraction of Drainage Networks from Digital Elevation Data. Computer Vision, Graphics, and Image Processing, 28, 323-344. 
void IndexCalculator::calcFlowDirection_D8()
{
	//e0 and e1 are elevation values
	double e0, e1;
	double slope;
	double maxSlope;
	//initialize flowDir flow direction
	double flowDir = 0;
	int current_row, current_col;

	//Note: Whitebox flow direction vectors are arranged with clockwise indexing, x=1,y=-1 in NorthEast (top right), x=-1,y=1 in SouthWest (bottom left)
	int dx[8] = { 1, 1, 1, 0, -1, -1, -1, 0 };
	int dy[8] = { -1, 0, 1, 1, 1, 0, -1, -1 };

	//Note: ESRI flow direction vectors are arranged with clockwise indexing
	//int dx[8] = { 1, 1, 0, -1, -1, -1, 0, 1 };
	//int dy[8] = { 0, 1, 1, 1, 0, -1, -1, -1 };

	//DX2 is distance between diagnonal folder centers
	double DX2 = DX * sqrt(2.0);
	//dist vector contains distance to neighbors, along diagonal and cardinal directions
	double dist[8] = { DX2, DX, DX2, DX, DX2, DX, DX2, DX };

	//loop through all map rows 
	for (int row = 0; row<rows; row++) {
		//loop through all map columns
		for (int col = 0; col<cols; col++) {
			//e0 is elevation of pixel
			e0 = elevation[row][col];

			//if e0 is real data
			if (e0 != NODATA)
			{
				//initialize variables for algorithm
				maxSlope = -9999999999;
				flowDir = -9999999999;
				//loop through all 8 neighbors
				for (int k = 0; k < 8; k++)
				{
					//current_row and current_col are updated for neighbor
					current_row = row + dy[k];
					current_col = col + dx[k];

					//checkfolder function confirms pixel is within map
					if (checkfolder(row + dy[k], col + dx[k]))
						//e1 updated for neighbor
						e1 = elevation[current_row][current_col];
					else
						e1 = NODATA;

					//if e1 is real data then compute slope
					if (e1 != NODATA)
					{
						//slope is simply rise over run
						slope = (e0 - e1) / dist[k];
						//if slope > maxSlope identify new maxSlope
						if (slope > maxSlope) {
							maxSlope = slope;
							//flowDir is 1 for NE corner, 2 for E, 4 for SE, 8 for S, 16 for SW, 32 for W, 64 for NW, 128 for N
							flowDir = pow(2, k);
						}
					}
				}
				if (maxSlope <= 0)
					flowDirorganizer[row][col] = -1;
				else
					flowDirorganizer[row][col] = flowDir;
			}
			else {
				flowDirorganizer[row][col] = NODATA;
			}
		}
	}
}

//calcFlowDirection_MFD function calculates upslope contributing areas using the multiple flow direction algorithm (Quinn et al., 1991) from Whitebox GAT
//Quinn, P., Beven, K., Chevallier, P., & Planchon, O. (1991). The Prediction of Hillslope Flow Paths for Distributed Hydrological Modelling using Digital Terrain Models. Hydrological Processes, 5, 59-79. 
void IndexCalculator::calcFlowDirection_MFD()
{
	//e0 and e1 are elevation values
	double e0, e1;
	double slope;
	double maxSlope;
	//initialize flowDir flow direction
	double flowDir = 0;
	int current_row, current_col;

	//Note: Whitebox flow direction vectors are arranged with clockwise indexing, x=1,y=-1 in NorthEast (top right), x=-1,y=1 in SouthWest (bottom left)
	int dx[8] = { 1, 1, 1, 0, -1, -1, -1, 0 };
	int dy[8] = { -1, 0, 1, 1, 1, 0, -1, -1 };

	//Note: ESRI flow direction vectors are arranged with clockwise indexing
	//int dx[8] = { 1, 1, 0, -1, -1, -1, 0, 1 };
	//int dy[8] = { 0, 1, 1, 1, 0, -1, -1, -1 };

	//DX2 is distance between diagnonal folder centers
	double DX2 = DX * sqrt(2.0); 
	//dist vector contains distance to neighbors, along diagonal and cardinal directions
	double dist[8] = { DX2, DX, DX2, DX, DX2, DX, DX2, DX };

	for (int row = 0; row<rows; row++) {
		for (int col = 0; col<cols; col++) {

			//e0 is pixel elevation
			e0 = elevation[row][col];
			//if e0 is real data
			if (e0 != NODATA)
			{
				flowDir = 0;
				//loop through all 8 neighbors
				for (int k = 0; k < 8; k++)	{
					//current_row is neighbor row
					current_row = row + dy[k];
					//current_col is neighbor column
					current_col = col + dx[k];

					//checkfolder function confirms pixel is within map
					if (checkfolder(row + dy[k], col + dx[k]))
						//e1 updated for neighbor
						e1 = elevation[current_row][current_col];
					else
						e1 = NODATA;

					// Beginning of the slope calculation loop
					if (e1 != NODATA && e1 < e0)
					{
						flowDir += pow(2, k);
					}
				}
				
				flowDirorganizer[row][col] = flowDir;
			}
			else {
				flowDirorganizer[row][col] = NODATA;
			}
		}
	}
}

//calcSCA_Dinf function calculates Specific Contributing Area (SCA) based on the D-Infinity (Tarboton, 1997) flow pointer organizer from Whitebox GAT
//Tarboton, D. G. (1997). A New Method for the Determination of Flow Directions and Upslope Areas in Grid Digital Elevation Models. Water Resources Research, 33, 309-319. 
void IndexCalculator::calcSCA_Dinf()
{
	//Note: Whitebox flow accumulation facets are arranged with clockwise indexing, startFD=180 endFD=270 goes NorthEast (top right), startFD=0 endFD=90 goes SouthWest (bottom left)
	int dCol[8] = { 1, 1, 1, 0, -1, -1, -1, 0 };
	int dRow[8] = { -1, 0, 1, 1, 1, 0, -1, -1 };
	double organizerRes = 1;
	double startFD[8] = { 180, 225, 270, 315, 0, 45, 90, 135 };
	double endFD[8] = { 270, 315, 360, 45, 90, 135, 180, 225 };

	double flowDir, flowAccum;
	int row, col, newRow, newCol;
	int i;
	int loopNum = 0;

	//maxDepth is maximum number of pixels in flow path length; 1000 used in Whitebox GAT (John Linsdsay)
	maxDepth = 10000;

	//somethingDone is boolean flag used to enter and exit do-while loop tracking flow path 
	bool somethingDone;

	//loop through all map rows 
	for (row = 0; row < rows; row++) {
		//loop through all map columns
		for (col = 0; col < cols; col++) {
			//flowDir is flowDirorganizer map value
			flowDir = flowDirorganizer[row][col];
			//if flowDir is real data
			if (flowDir != NODATA) {
				i = 0;
				//loop through all 8 neighbors
				for (int k = 0; k < 8; k++)
				{
					//newCol is neighbor col
					newCol = col + dCol[k];
					//newRow is neighbor row
					newRow = row + dRow[k];

					//checkfolder function confirms pixel is within map
					if (checkfolder(newRow, newCol))
						//flowDir updated for neighbor
						flowDir = flowDirorganizer[newRow][newCol];
					else
						//continue to next neighbor
						continue;

					//if flowDir is within allowable range
					if (flowDir >= 0 && flowDir <= 360) {
						//if neighbor is not directly south, which has endFD < startFD
						if (k != 3) {
							//flowDir is greater than startFD and lesser than endFD
							if (flowDir > startFD[k] && flowDir < endFD[k]) {
								//increment flow direction counter
								i++;
							}
						}
						//if neighbor is directly south, which has endFD > startFD, divided across north
						else {
							//flowDir is greater than startFD or lesser than endFD
							if (flowDir > startFD[k] || flowDir < endFD[k]) {
								//increment flow direction counter 
								i++;
							}
						}
					}
				}

				//assign flow direction counter to maps
				temporganizer[row][col] = i;
				infloworganizer[row][col] = i;
			}
			else {
				flowAccumorganizer[row][col] = NODATA;
			}
		}
	}

	//Do loop while somethingDone = True to accumulate flow from upslope cells draining to pixel 
	do {
		loopNum++;
		//somethingDone flag to False
		somethingDone = false;
		//loop through all map rows
		for (row = 0; row < rows; row++) {
			//loop through all map columns
			for (col = 0; col < cols; col++)
			{
				//if DEM pixel is real data
				if (getElevation(row, col) != NODATA)
				{
					//if flow has not been computed for pixel
					if (temporganizer[row][col] == 0)
					{   
						//initialize currentDepth to 0 to accumulate pixels
						currentDepth = 0;
						//somethingDone set to True
						somethingDone = true;
						//call DInfAccum function to find flow accumulation area
						DinfAccum(row, col);
					}
				}
				else
					//continue to next pixel
					continue;
			}
		}
	} while (somethingDone);

	//loop through all map rows
	for (row = 0; row < rows; row++) {
		//loop through all map columns
		for (col = 0; col < cols; col++) {
			//flowAccum is flow accumulated pixels, without area
			flowAccum = flowAccumorganizer[row][col];
			if (flowAccum != NODATA) {
				//flowAccumorganizer is product of flowAccum and grid resolution area
				flowAccumorganizer[row][col] = flowAccum * DX;
			}
		}
	}

	// End of Dinf Accum main function

}

//calcSCA_D8 function calculates Specific Contributing Area (SCA) based on the D8 (O'Callaghan and Mark, 1984) flow pointer organizer from Whitebox GAT
//O'Callaghan, J. F., & Mark, D. M. (1984). The Extraction of Drainage Networks from Digital Elevation Data. Computer Vision, Graphics, and Image Processing, 28, 323-344. 
void IndexCalculator::calcSCA_D8()
{
	//Note: Whitebox flow direction vectors are arranged with clockwise indexing, FD=1 in NorthEast (top right), FD=16 in SouthWest (bottom left)
	int dx[8] = { 1, 1, 1, 0, -1, -1, -1, 0 };
	int dy[8] = { -1, 0, 1, 1, 1, 0, -1, -1 };
	double inflowDirVals[8] = { 16, 32, 64, 128, 1, 2, 4, 8 };

	//Note: ESRI flow direction vectors are arranged with clockwise indexing
	//int dx[8] = { 1, 1, 0, -1, -1, -1, 0, 1 };
	//int dy[8] = { 0, 1, 1, 1, 0, -1, -1, -1 };
	//double inflowDirVals[8] = {32, 64, 128, 1, 2, 4, 8, 16 };

	double flowDir, flowAccum;
	int row, col, newRow, newCol, temporganizerVal, i, x, y;
	int z = 0;
	int loopNum = 0;

	//somethingDone is boolean flag used to enter and exit do-while loop tracking flow path 
	bool somethingDone;

	//loop through all map rows 
	for (int row = 0; row<rows; row++)	{
		//loop through all map columns
		for (int col = 0; col<cols; col++)	{
			//flowDir is flowDirorganizer map value
			flowDir = flowDirorganizer[row][col];
			//if flowDir is real data
			if (flowDir != NODATA) {
				//z set to 0
				z = 0;
				//loop through all 8 neighbors
				for (int k = 0; k < 8; k++) {

					//newCol is neighbor col
					newCol = col + dx[k];
					//newRow is neighbor row
					newRow = row + dy[k];

					//checkfolder function confirms pixel is within map
					if (checkfolder(newRow, newCol)) {

						//flowDir updated for neighbor
						flowDir = flowDirorganizer[newRow][newCol];
						//if flowDir is within allowable range
						if (flowDir == inflowDirVals[k]) {
							//flowDir counter updated for neighbor
							z++;
						}
					}
				}

				//assign flow direction counter to maps
				temporganizer[row][col] = z;
				infloworganizer[row][col] = z;
			}
			else {
				temporganizer[row][col] = NODATA;
				infloworganizer[row][col] = NODATA;
				flowAccumorganizer[row][col] = NODATA;
			}
		}
	}

	//For loop while somethingDone = True to accumulate flow from upslope cells draining to pixel 
	for (row = 0; row < rows; row++) {
		for (col = 0; col < cols; col++)
		{
			temporganizerVal = temporganizer[row][col];
			if (temporganizerVal == 0) {
				temporganizer[row][col] = -1;
				somethingDone = false;
				newCol = col;
				newRow = row;
				do {
					flowAccum = flowAccumorganizer[newRow][newCol];
					flowDir = flowDirorganizer[newRow][newCol];
					if (flowDir >= 0) {
						// find which  folder recieves flow direction and pass it down stream
						i = log2(flowDir);
						newCol += dx[i];
						newRow += dy[i];
						//update the output organizers
						flowAccumorganizer[newRow][newCol] += flowAccum;
						temporganizer[newRow][newCol]--;
						if (temporganizer[newRow][newCol] == 0) {
							temporganizer[newRow][newCol] = -1;
							somethingDone = true;
						}
						else {
							somethingDone = false;
						}
					}
					else {
						somethingDone = false;
					}
				} while (somethingDone);
			}
		}
	}

	//loop through all map rows 
	for (row = 0; row < rows; row++) {
		//loop through all map columns
		for (col = 0; col < cols; col++)
		{
			if (flowAccumorganizer[row][col] != NODATA)
				//flowAccumorganizer incremented by product with pixel size
				flowAccumorganizer[row][col] *= DX;
		}
	}
}

//calcSCA_MFD function calculates Specific Contributing Area (SCA) based on the multiple flow direction (Quinn et al., 1991) flow pointer organizer from Whitebox GAT
//Quinn, P., Beven, K., Chevallier, P., & Planchon, O. (1991). The Prediction of Hillslope Flow Paths for Distributed Hydrological Modelling using Digital Terrain Models. Hydrological Processes, 5, 59-79. 
void IndexCalculator::calcSCA_MFD()
{

	//Note: Whitebox flow direction vectors are arranged with clockwise indexing, FD=1 in NorthEast (top right), FD=16 in SouthWest (bottom left)
	int dx[8] = { 1, 1, 1, 0, -1, -1, -1, 0 };
	int dy[8] = { -1, 0, 1, 1, 1, 0, -1, -1 };
	double inflowDirVals[8] = { 16, 32, 64, 128, 1, 2, 4, 8 };

	//Note: ESRI flow direction vectors are arranged with clockwise indexing
	//int dx[8] = { 1, 1, 0, -1, -1, -1, 0, 1 };
	//int dy[8] = { 0, 1, 1, 1, 0, -1, -1, -1 };
	//double inflowDirVals[8] = {32, 64, 128, 1, 2, 4, 8, 16 };

	double flowDir, flowAccum, e0, e1;
	int row, col, newRow, newCol, temporganizerVal, i, x, y;
	int numNeighbors = 0;
	int loopNum = 0;

	//maxDepth is maximum number of pixels in flow path length; 1000 used in Whitebox GAT (John Linsdsay)
	maxDepth = 10000;

	//somethingDone is boolean flag used to enter and exit do-while loop tracking flow path 
	bool somethingDone;

	//loop through all map rows 
	for (int row = 0; row<rows; row++)
	{
		//loop through all map columns
		for (int col = 0; col<cols; col++)
		{
			//e0 is DEM elevation
			e0 = elevation[row][col];
			//if e0 is real data
			if (e0 != NODATA){
				//initialize numNeighbors counter to 0
				numNeighbors = 0;

				//loop through all 8 neighbors
				for (int k = 0; k < 8; k++)
				{
					//newCol is neighbor col
					newCol = col + dx[k];
					//newRow is neighbor row
					newRow = row + dy[k];

					//checkfolder function confirms pixel is within map
					if (checkfolder(newRow, newCol)) {

						//e1 is elevation of neighbor pixel
						e1 = elevation[newRow][newCol];
						//if e1 neighbor is lower than e0 center pixel, and is real data
						if (e1 != NODATA && (e1 > e0)) {
							//increment numNeighbors counter
							numNeighbors++;
						}
					}
					else
						//continue to next neighbor
						continue;			
				}
				//assign numNeighbor counter of downslope neighbors
				temporganizer[row][col] = numNeighbors;
				infloworganizer[row][col] = numNeighbors;

			}
			else {
				temporganizer[row][col] = NODATA;
				infloworganizer[row][col] = NODATA;
			}
		}
	}
	
	//Do loop while somethingDone = True to accumulate flow from upslope cells draining to pixel 
	do {
		//increment loopNum counter
		loopNum++;
		//reset somethingDone = False to exit loop if needed
		somethingDone = false;
		//loop through all map rows 
		for (row = 0; row < rows; row++) {
			//loop through all map columns
			for (col = 0; col < cols; col++)
			{
				//if getElevation is real data
				if (getElevation(row, col) != NODATA)
				{
					//if temporganizer = 0, then pixel has not been checked
					if (temporganizer[row][col] == 0)
					{   
						//initialize currentDepth to 0 to accumulate pixels
						currentDepth = 0;
						//call MFDAccum function to find flow accumulation area
						MFDAccum(row, col);
						//somethingDone set to True
						somethingDone = true;
					}
				}
				else
					//continue to next pixel
					continue;
			}
		}
	} while (somethingDone);

}

//DinfAccum function computes flow accumulation values for map with D-Infinity algorithm (Tarboton, 1997), taken from Whitebox GAT
//Tarboton, D. G. (1997). A New Method for the Determination of Flow Directions and Upslope Areas in Grid Digital Elevation Models. Water Resources Research, 33, 309-319. 
void IndexCalculator::DinfAccum(int row, int col)
{
	double flowAccumVal = flowAccumorganizer[row][col];
	double flowDir = flowDirorganizer[row][col];
	double tempAccum = 0;
	double proportion1 = 0;
	double proportion2 = 0;
	int a1 = 0;
	int b1 = 0;
	int a2 = 0;
	int b2 = 0;

	//currentDepth is incremented by 1 as flow path is evaluated
	currentDepth++;
	//if currentDepth exceeds maxDepth, then recursion exceeds limit and error likely
	if (currentDepth > maxDepth) {
		//break from routine
		return;
	}

	//temporganizer set to -1 to indicate pixel flow path was evaluated; used to avoid revisiting same pixel
	temporganizer[row][col] = -1; 

	//search for flow direction facets that receive flow, and compute proportion of flow received
	if (flowDir >= 0) {
		if (flowDir >= 0 && flowDir < 45) {
			//proportion1 is fraction of flow from 45 degree facet
			proportion1 = (45 - flowDir) / 45;
			//a1 is neighbor column
			a1 = col;
			//b1 is neighbor row
			b1 = row - 1;
			//proportion2 is fraction of flow from 45 degree facet
			proportion2 = flowDir / 45;
			a2 = col + 1;
			b2 = row - 1;
		}
		else if (flowDir >= 45 && flowDir < 90) {
			proportion1 = (90 - flowDir) / 45;
			a1 = col + 1;
			b1 = row - 1;
			proportion2 = (flowDir - 45) / 45;
			a2 = col + 1;
			b2 = row;
		}
		else if (flowDir >= 90 && flowDir < 135) {
			proportion1 = (135 - flowDir) / 45;
			a1 = col + 1;
			b1 = row;
			proportion2 = (flowDir - 90) / 45;
			a2 = col + 1;
			b2 = row + 1;
		}
		else if (flowDir >= 135 && flowDir < 180) {
			proportion1 = (180 - flowDir) / 45;
			a1 = col + 1;
			b1 = row + 1;
			proportion2 = (flowDir - 135) / 45;
			a2 = col;
			b2 = row + 1;
		}
		else if (flowDir >= 180 && flowDir < 225) {
			proportion1 = (225 - flowDir) / 45;
			a1 = col;
			b1 = row + 1;
			proportion2 = (flowDir - 180) / 45;
			a2 = col - 1;
			b2 = row + 1;
		}
		else if (flowDir >= 225 && flowDir < 270) {
			proportion1 = (270 - flowDir) / 45;
			a1 = col - 1;
			b1 = row + 1;
			proportion2 = (flowDir - 225) / 45;
			a2 = col - 1;
			b2 = row;
		}
		else if (flowDir >= 270 && flowDir < 315) {
			proportion1 = (315 - flowDir) / 45;
			a1 = col - 1;
			b1 = row;
			proportion2 = (flowDir - 270) / 45;
			a2 = col - 1;
			b2 = row - 1;
		}
		else if (flowDir >= 315 && flowDir <= 360) {
			proportion1 = (360 - flowDir) / 45;
			a1 = col - 1;
			b1 = row - 1;
			proportion2 = (flowDir - 315) / 45;
			a2 = col;
			b2 = row - 1;
		}

		//checkfolder function confirms pixel is within map
		if (checkfolder(b1, a1)) {
			//if a proportion of flow is assigned to downslope cell along facet of flow path
			if (proportion1 > 0 && flowAccumorganizer[b1][a1] != NODATA)
			{
				tempAccum = flowAccumorganizer[b1][a1];
				//flowAccumorganizer is increased by proportion of accumulated flow 
				flowAccumorganizer[b1][a1] = (tempAccum + (flowAccumVal * proportion1));
				//temporganizer reduced by 1; used as indicator pixel flow path was searched
				temporganizer[b1][a1]--;

				//recursive call to DinfAccum if temporganizer is 0
				if (temporganizer[b1][a1] == 0) {
					DinfAccum(b1, a1);
				}
			}
		}
		//checkfolder function confirms pixel is within map
		if (checkfolder(b2, a2)) {
			//if a proportion of flow is assigned to downslope cell along facet of flow path
			if (proportion2 > 0 && flowAccumorganizer[b2][a2] != NODATA)
			{
				tempAccum = flowAccumorganizer[b2][a2];
				//flowAccumorganizer is increased by proportion of accumulated flow 
				flowAccumorganizer[b2][a2] = (tempAccum + (flowAccumVal * proportion2));
				//temporganizer reduced by 1; used as indicator pixel flow path was searched
				temporganizer[b2][a2]--;
				//temporganizer[b2][a2] = (temporganizer[b2][a2] - 1);

				//recursive call to DinfAccum if temporganizer is 0
				if (temporganizer[b2][a2] == 0) {
					DinfAccum(b2, a2);
				}
			}
		}
	}
	//currentDepth is reduced by 1
	currentDepth--;
}

//MFDAccum function computes flow accumulation values for map with multiple flow direction algorithm (Quinn et al, 1991), taken from Whitebox GAT
//Quinn, P., Beven, K., Chevallier, P., & Planchon, O. (1991). The Prediction of Hillslope Flow Paths for Distributed Hydrological Modelling using Digital Terrain Models. Hydrological Processes, 5, 59-79. 
//Quinn, P. F., Beven, K. J., & Lamb, R. (1995). The ln(a/Tan-Beta) Index - How To Calculate It and How To Use It Within The TOPMODEL Framework. Hydrological Processes, 9(2), 161-182. doi:10.1002/hyp.3360090204
void IndexCalculator::MFDAccum(int row, int col)
{

	//Note: Whitebox flow direction vectors are arranged with clockwise indexing, FD=1 in NorthEast (top right), FD=16 in SouthWest (bottom left)
	int dx[8] = { 1, 1, 1, 0, -1, -1, -1, 0 };
	int dy[8] = { -1, 0, 1, 1, 1, 0, -1, -1 };
	double inflowDirVals[8] = { 16, 32, 64, 128, 1, 2, 4, 8 };

	//Note: ESRI flow direction vectors are arranged with clockwise indexing
	//int dx[8] = { 1, 1, 0, -1, -1, -1, 0, 1 };
	//int dy[8] = { 0, 1, 1, 1, 0, -1, -1, -1 };
	//double inflowDirVals[8] = {32, 64, 128, 1, 2, 4, 8, 16 };

	double flowAccumVal = flowAccumorganizer[row][col];
	double flowDir = flowDirorganizer[row][col];
	double tempAccum = 0;
	double temporganizerVal = 0;
	double proportion = 0;
	double totalRelief = 0;
	double power = 1.0;
	int newRow, newCol = 0;
	double e0 = 0;
	double e1 = 0;

	//currentDepth is incremented by 1 as flow path is evaluated
	currentDepth++;
	//if currentDepth exceeds maxDepth, then recursion exceeds limit and DEM may have error 
	if (currentDepth > maxDepth) {
		return;
	}

	//temporganizer set to -1 to indicate pixel flow path was evaluated; used to avoid revisiting same pixel
	temporganizer[row][col] = -1;

	e0 = elevation[row][col];

	//search for flow direction facets that receive flow, and compute proportion of flow received
	//loop through all 8 neighbors
	for (int k = 0; k < 8; k++)
	{
		//newRow is neighbor row
		newRow = row + dy[k];
		//newCol is neighbor column
		newCol = col + dx[k];

		//checkfolder function confirms pixel is within map
		if (checkfolder(newRow, newCol)) {

			//e1 is neighbor elevation
			e1 = elevation[newRow][newCol];
			//if e1 neighbor is lower than e0 center pixel, and is real data
			if ((e0 > e1) && (e1 != NODATA)) {
				//accumulate totalRelief
				totalRelief += pow((e0 - e1), power);
			}
		}
		else
			//continue to next neighbor
			continue;
	}
	//loop through all 8 neighbors
	for (int k = 0; k < 8; k++)
	{
		//newRow is neighbor row
		newRow = row + dy[k];
		//newCol is neighbor column
		newCol = col + dx[k];

		//checkfolder function confirms pixel is within map
		if (checkfolder(newRow, newCol)) {

			//e1 is neighbor elevation
			e1 = elevation[newRow][newCol];
			//if e1 neighbor is lower than e0 center pixel, and is real data
			if ((e0 > e1) && (e1 != NODATA))
			{
				//Check algorithm with Quinn et al. (1991, 1995)
				//Quinn, P., Beven, K., Chevallier, P., & Planchon, O. (1991). The Prediction of Hillslope Flow Paths for Distributed Hydrological Modelling using Digital Terrain Models. Hydrological Processes, 5, 59-79. 
				//Quinn, P. F., Beven, K. J., & Lamb, R. (1995). The ln(a/Tan-Beta) Index - How To Calculate It and How To Use It Within The TOPMODEL Framework. Hydrological Processes, 9(2), 161-182. doi:10.1002/hyp.3360090204
				proportion = pow((e0 - e1), power) / totalRelief;
				tempAccum = flowAccumorganizer[newRow][newCol];
				double new_FA = (tempAccum + (flowAccumVal * proportion));
				//flowAccumorganizer is increased by proportion of accumulated flow 
				flowAccumorganizer[newRow][newCol] = new_FA;
				temporganizerVal = temporganizer[newRow][newCol];
				//temporganizer reduced by 1; used as indicator pixel flow path was searched
				temporganizer[newRow][newCol]--;

				//recursive call to MFDAccum if temporganizer is 0
				if (temporganizer[newRow][newCol] == 0)
				{
					MFDAccum(newRow, newCol);
				}
			}
		}
		else
			//continue to next neighbor
			continue;
	}
	//currentDepth is reduced by 1
	currentDepth--;
}

//calcTWI shared by exponential and power function decay approaches, both have log taken, but transform() function changes this for power decay function 
//	Computes topographic index for each map pixel, using RHS terms computed in TI_Cal function
void IndexCalculator::calcTWI(double ImperviousArea_total_frac)
{
	//constant conversion from degrees to radians
	double degToRad = pi / 180;

	//loop through all map rows 
	for (int row = 0; row < rows; row++) {
		//loop through all map columns
		for (int col = 0; col < cols; col++) {

			//The flowAccumorganizer term is reduced to remove impervious area, keeping with TOPURBAN design of Valeo and Moin (2000) 
			//Valeo, C., and Moin, S. M. A. (2000). Variable source area modelling in urbanizing watersheds. Journal of Hydrology, 228(1-2), 68-81. 
			//	One option is to flowAccumorganizer * (1-impervious_fraction), the other is to use a map of impervious area when building flowAccumorganizer 

			//if flowAccumorganizer map row and column has data
			if (flowAccumorganizer[row][col] != NODATA) {

				//cte 20210520 need to implement for spatial hydro, get value of (1-ImperviousArea_total_frac)

				//ImperviousArea_total_frac < 1 to prevent (1-ImperviousArea_total_frac) = 0 and mathmatical error of log(0)
				if (ImperviousArea_total_frac >= 1) {
					ImperviousArea_total_frac = 0.99;
				}

				//TWIorganizer is natural log of quotient of contributing area and tangent of slope; c++ log is base e
				TWIorganizer[row][col] = log((flowAccumorganizer[row][col] * (1-ImperviousArea_total_frac)) / (tan(degToRad * slopeorganizer[row][col])));
			}
			else {
				TWIorganizer[row][col] = NODATA;
			}
		}
	}
}

//Topographic index algorithm taken from TOPMODEL code developed by Keith Beven and used by USGS D.M. Wolock https://pubs.er.usgs.gov/publication/wri934124
//Wolock, D. M. (1993). Simulating the variable-source-area concept of watershed hydrology with TOPMODEL, USGS WRI Report 93-4124).
//	The algorithm requires the topographic index (TI) bins (ia) are sorted from highest (wettest) to lowest TI values
//	The algorithm requires when ia = 0, the TI bin Index[0] should be the upper limit and its IndexArea[0] = 0
//	The algorithm presumes for each ia, the area of catchment between discrete TI values Index[ia] and Index[ia+1] equals IndexArea[ia]
//	The algorithm presumes for each ia, the area of catchment bounding Index[ia] equals (IndexArea[ia] + IndexArea[ia+1])/2 

//indexHist function will generate histogram of topographic index
void IndexCalculator::indexHist()
{
	//initialize index as bin number
	int index = 0;
	//initialize TWImax as topographic wetness index maximum, where TWI is another name for TI
	double TWImax = 0.0;
	//initialize TWImin as topographic wetness index minimum, where TWI is another name for TI
	double TWImin = 10000;
	double total = 0.0;

	cout << "Generating the Topographic Index histogram with topographic bins = " << NAC << endl;
	LogMessage::log("Computing the topographic index value of each folder.");

	//looping through NAC, which are breakpoints for bins of topographic index histogram
	for (int i = 0; i<NAC; i++)
	{
		//initializing y, counter of occurrences of TI values in each index bin
		y[i] = 0.0;
		//initializing AC (area of catchment) 
		AC[i] = 0.0;
		//initializing ST (topographic index) values
		ST[i] = 0.0;
	}

	//Find minimum and maximum topographic index values to create histogram
	//loop through rows of topographic index map, along i 
	for (int i = 0; i<rows; i++)
	{
		//loop through cols of topographic index map, along j
		for (int j = 0; j<cols; j++)
		{
			//if real topographic index map values exist, and not NODATA
			if (TWIorganizer[i][j] != NODATA)
			{
				//find new max value
				if (TWIorganizer[i][j] > TWImax)
					TWImax = TWIorganizer[i][j];
				//find new min value
				if (TWIorganizer[i][j] < TWImin)
					TWImin = TWIorganizer[i][j];
			}
		}
	}

	//dTWI is incremental distance between TWI max and min, across the number of bins
	//dTWI will create breakpoints in TI values within histogram
	int ib = NAC - 1;
	double dTWI = (TWImax - TWImin) / (ib);

	//create TI histogram
	//loop through rows of topographic index map, along i
	for (int i = 0; i<rows; i++)
	{
		//loop through cols of topographic index map, along j
		for (int j = 0; j<cols; j++)
		{
			//if real topographic index map values exist, and not NODATA
			if (TWIorganizer[i][j] != NODATA)
			{
				//index is integer of scaled TI values used to put TI values into the NAC topographic bins, ranging from 1 to NAC
				//	Ex w/ NAC=30 for Exponential Decay: TWIOrganizer[i][j] = 15, TWImax = 30, TWImin = 3, dTWI = 0.931, then index = int(13.89) = 14
				//	Ex w/ NAC=30 for Power Decay: TWIOrganizer[i][j] = 1500, TWImax = 3000, TWImin = 3, dTWI = 103.345, then index = int(15.48) = 15
				//	Any TWIorganizer value equal to TWImin becomes an index value of 1
				//	Any TWIorganizer value equal to TWImax becomes an index value of TWImax
				//	Puts entire map of TI values into discrete bins
				index = static_cast<int>(((TWIorganizer[i][j] - TWImin) / dTWI) + 1);

				//index should never be larger than NAC-1 (i.e., equal to NAC) which is reserved for TWImax
				if (index > NAC - 1)
					index = NAC - 1;

				//y[index] counts number of TWIorganizer values going into each bin
				y[index] += 1;
				//total counts all TWIorganizer values, used to scale y[index] into fractions of total
				total += 1;
			}
		}
	}

	//ST[0] takes the maximum TWI value, which can be used later in TI algorithms 
	ST[0] = TWImax;

	//loop through all TI bins in histogram
	for (int i = 1; i<NAC; i++)
	{
		int ib = NAC - i;
		//AC[i] is area (fraction) of catchment for each bin, y[] is counter of occurrence in map of TI values in each bin
		//	N.B. AC[0] = 0, where 0 is bin with TWImax value
		AC[i] = y[ib] / total;
		//ST[i] is TI value for each bin, constructed with dTWI based on incremental distance between TWImax and TWImin, 
		ST[i] = TWImax - (i)*dTWI;
	}

}

//transform function converts topographic index (TI) values in map and histogram from exponential decay ln formulation to power decay formulation
//	TI transformed by taking exponential of ln(a/tanB), raised to power (1/n), units change from ln([L]) to [L]^(1/n)
void IndexCalculator::transform()
{
	//loop through all rows of map
	for (int i = 0; i<rows; i++)
	{
		//loop through all cols of map
		for (int j = 0; j<cols; j++)
		{
			//for pixels with NODATA, keep NODATA
			if (TWIorganizer[i][j] == NODATA)
				TWIorganizer[i][j] = NODATA;
			//for pixels with data, transform 
			else
				//TWIorganizer is TI map value transformed value by taking exponential of ln(a/tanB), raised to power (1/n)
				TWIorganizer[i][j] = pow(exp(TWIorganizer[i][j]), 1 / N);
			//remove zero values
			if (TWIorganizer[i][j] <0)
				TWIorganizer[i][j] = 0.01;

		}
	}

	//loop through all TI bins in histogram
	for (int i = 0; i<NAC; i++)
		//ST is TI histogram value transformed value by taking exponential of ln(a/tanB), raised to power (1/n)
		ST[i] = pow(exp(ST[i]), 1 / N);

}

//createDistorganizerFiles function creates maps of Topographic Index, Slope, Aspect, Flow Direction, Flow Accumulation as .dat files
// maps called TIorganizer.dat, Slope.dat, Aspect.dat, FDorganizer.dat, FlowAccum.dat
void IndexCalculator::createDistorganizerFiles()
{
	std::cout << "Outputing the topography files." << endl;
	LogMessage::log("Outputing the topography files.");

	// Outputting the TIorganizer.dat
	ofstream outfile(directory + "TIorganizer.dat");
	if (!outfile.good())
	{
		LogMessage::log("Could not create"+directory + "TIorganizer.dat");
		return;
	}
	else
	{
		LogMessage::log("Created: " + directory + "TIorganizer.dat");
	}
	outfile << "ncols" << std::setw(10) << cols << endl;
	outfile << "nrows" << std::setw(10) << rows << endl;
	outfile << "xllcorner" << std::setw(20) << xllcorner << endl;
	outfile << "yllcorner" << std::setw(20) << yllcorner << endl;
	outfile << "cellsize" << std::setw(10) << DX << endl;
	outfile << "NODATA_value" << std::setw(10) << NODATA << endl;

	int rem = cols % 8;
	for (int i = 0; i<rows; i++)
	{
		for (int j = 0; j<cols; j++)
		{
			if (elevation[i][j] != NODATA)
				outfile << " " << std::setw(10) << TWIorganizer[i][j];
			else
				outfile << " " << std::setw(10) << NODATA;
		}
		outfile << std::endl;
	}
	// End of outputting the TIorganizer.dat

	// Outputting the slope organizer in radians
	double radToDeg = 180 / pi;
	ofstream outfile2(directory + "Slope.dat");
	if (!outfile2.good())
	{
		LogMessage::log("Could not create" + directory + "Slope.dat");
		return;
	}
	else
	{
		LogMessage::log("Created: " + directory + "Slope.dat");
	}
	outfile2 << left << std::setw(14) << "ncols" << std::setw(15) << cols << endl;
	outfile2 << left << std::setw(14) << "nrows" << std::setw(15) << rows << endl;
	outfile2 << left << std::setw(14) << "xllcorner" << std::setw(15) << setprecision(15) << xllcorner << endl;
	outfile2 << left << std::setw(14) << "yllcorner" << std::setw(15) << setprecision(15) << yllcorner << endl;
	outfile2 << left << std::setw(14) << "cellsize" << std::setw(15) << setprecision(15) << DX << endl;
	outfile2 << left << std::setw(14) << "NODATA_value" << std::setw(15) << NODATA << endl;

	for (int i = 0; i < rows; i++)
	{
		for (int j = 0; j < cols; j++)
		{
			if (elevation[i][j] != NODATA)
			{
				outfile2 << slopeorganizer[i][j] / radToDeg << " ";
			}
			else
				outfile2 << NODATA << " ";
		}
		outfile2 << std::endl;
	}
	outfile2.close();
	// End of outputting the slope organizer

	// Outputting the aspect organizer 
	ofstream outfile2p5(directory + "Aspect.dat");
	if (!outfile2p5.good())
	{
		LogMessage::log("Could not create" + directory + "Aspect.dat");
		return;
	}
	else
	{
		LogMessage::log("Created: " + directory + "Aspect.dat");
	}
	outfile2p5 << left << std::setw(14) << "ncols" << std::setw(15) << cols << endl;
	outfile2p5 << left << std::setw(14) << "nrows" << std::setw(15) << rows << endl;
	outfile2p5 << left << std::setw(14) << "xllcorner" << std::setw(15) << setprecision(15) << xllcorner << endl;
	outfile2p5 << left << std::setw(14) << "yllcorner" << std::setw(15) << setprecision(15) << yllcorner << endl;
	outfile2p5 << left << std::setw(14) << "cellsize" << std::setw(15) << setprecision(15) << DX << endl;
	outfile2p5 << left << std::setw(14) << "NODATA_value" << std::setw(15) << NODATA << endl;

	for (int i = 0; i < rows; i++)
	{
		for (int j = 0; j < cols; j++)
		{
			if (elevation[i][j] != NODATA)
				outfile2p5 << aspectorganizer[i][j] << " ";
			else
				outfile2p5 << NODATA << " ";
		}
		outfile2p5 << std::endl;
	}
	outfile2p5.close();
	// End of outputting the aspect organizer

	// outputting the flowdirection organizer	
	ofstream outfile1(directory + "FDorganizer.dat");
	if (!outfile1.good())
	{
		LogMessage::log("Could not create" + directory + "FDorganizer.dat");
		return;
	}
	else
	{
		LogMessage::log("Created: " + directory + "FDorganizer.dat");
	}
	outfile1 << left << std::setw(14) << "ncols" << std::setw(15) << cols << endl;
	outfile1 << left << std::setw(14) << "nrows" << std::setw(15) << rows << endl;
	outfile1 << left << std::setw(14) << "xllcorner" << std::setw(15) << setprecision(15) << xllcorner << endl;
	outfile1 << left << std::setw(14) << "yllcorner" << std::setw(15) << setprecision(15) << yllcorner << endl;
	outfile1 << left << std::setw(14) << "cellsize" << std::setw(15) << setprecision(15) << DX << endl;
	outfile1 << left << std::setw(14) << "NODATA_value" << std::setw(15) << NODATA << endl;

	for (int i = 0; i<rows; i++)
	{
		for (int j = 0; j<cols; j++)
		{
			if (elevation[i][j] != NODATA)
				outfile1 << flowDirorganizer[i][j] << " ";
			else
				outfile1 << NODATA << " ";
		}
		outfile1 << std::endl;
	}
	outfile1.close();
	// End of outputting the FDorganizer.dat

	// Outputting the flow accumulation organizer	
	ofstream outfile3(directory + "FlowAccum.dat");
	if (!outfile3.good())
	{
		LogMessage::log("Could not create" + directory + "FlowAccum.dat");
		return;
	}
	else
	{
		LogMessage::log("Created: " + directory + "FlowAccum.dat");
	}
	outfile3 << left << std::setw(14) << "ncols" << std::setw(15) << cols << endl;
	outfile3 << left << std::setw(14) << "nrows" << std::setw(15) << rows << endl;
	outfile3 << left << std::setw(14) << "xllcorner" << std::setw(15) << setprecision(15) << xllcorner << endl;
	outfile3 << left << std::setw(14) << "yllcorner" << std::setw(15) << setprecision(15) << yllcorner << endl;
	outfile3 << left << std::setw(14) << "cellsize" << std::setw(15) << setprecision(15) << DX << endl;
	outfile3 << left << std::setw(14) << "NODATA_value" << std::setw(15) << NODATA << endl;

	for (int i = 0; i<rows; i++)
	{
		for (int j = 0; j<cols; j++)
		{
			if (elevation[i][j] != NODATA)
				outfile3 << flowAccumorganizer[i][j] << " ";
			else
				outfile3 << NODATA << " ";
		}
		outfile3 << std::endl;
	}
	outfile3.close();
	// End of outputting the flow accumulation organizer
}

//createIndexFile function writes topographic index (TI) histogram, either as Power Decay or Exponential Decay
void IndexCalculator::createIndexFile(bool powdecay)
{
	//determine name of TI histogram output file, initialize string name 
	std::string name = "";
	//if powdecay is True, then
	if (powdecay)
	{
		//TI histogram is Power Decay, which has much larger values than Exponential Decay
		name = "powdecayTI.dat";
	}
	//if powdecay is False, then
	else
	{
		//TI histogram is Exponential Decay, which has much smaller values than Power Decay
		name = "expdecayTI.dat";
	}

	//outfile has directory paty and file name
	ofstream outfile(directory + name);

	//if outfile.good returns a False value, meaning it did not work, then
	if (!outfile.good())
	{
		cout << "createIndexFile: Could not write topographic index histogram file. Check your DEM data against a Test Case dem.dat to troubleshoot." << endl;
		//exit function with False
		return;
	}
	//if outfile.good returns a True value, meaning it did work, then
	else
	{
		cout << "createIndexFile: Created topographic index histogram file." << endl;
	}

	//outfile header not writing ncols or NCOLS from _catName header vector, only writing %ofProjectArea TIValueBin
	outfile << ((_catName == "ncols" || _catName == "NCOLS") ? "%ofProjectArea TIValueBin" : _catName) << endl;
	//outfile header writing number of bins and total catchment area as fraction, value 1
	outfile << NAC << " " << fArea << endl;

	//loop through all bin values of topographic index (TI) histogram
	for (int i = 0; i<NAC; i++)
		//outfile is written line by line, adding formatted AC variable and ST variable for each i 
		outfile << "  " << std::setw(10) << AC[i] << "  " << std::setw(10) << ST[i] << endl;
	//close outfile
	outfile.close();
}