#ifndef INDEX_H
#define INDEX_H

//////////////////////////////////////////////////////////////////////////
//                                                                      //
//  index.h -- Calculate topgraphic index                               //
//                                                                      //
//                                                                      //
//                                                                      //
//  Language:     Visual C++, ver 6.0                                   //
//  Platform:     Pentium III,IV, Windows 2000, Windows XP              //
//  Application:  iTree Hydro, Summer 2003, 2015//
//  Author:       Jun Wang                                              //
//                Thomas P. Taggart, traggatmot@gmail.com//
//                                                                      //
//////////////////////////////////////////////////////////////////////////


#include <iostream>
#include <fstream>
#include <vector>
#include <string>
#include <math.h>
#include <sstream>
#include "..\LogMessage.h"

using namespace std;                    

class IndexCalculator
{
public:
	//Power Decay: passing directory, N parameter for power decay, NAC number of topographic index bins, simulation mode , routing algorithm
	//cte 20210520 adding IA to function
	//IndexCalculator(std::string directory, double N, int NAC, std::string simulationMode, std::string routingAlgorithm);
	IndexCalculator(std::string directory, double N, int NAC, std::string simulationMode, std::string routingAlgorithm, double ImperviousArea_total_frac);

	//Exponential Decay: passing directory, NAC number of topographic index bins, simulation mode , routing algorithm
	//cte 20210520 adding IA to function
	//IndexCalculator(std::string directory, int NAC, std::string simulationMode, std::string routingAlgorithm);
	IndexCalculator(std::string directory, int NAC, std::string simulationMode, std::string routingAlgorithm, double ImperviousArea_total_frac);

	void createDistorganizerFiles();
	void createIndexFile(bool powerdecay);
	bool fileExists(const std::string& filename);
	bool getParseSucess() {return success;}
	        
	void getElevation(vector<vector<double>> &elevation){elevation = elevation;}
	void getElevation1D(vector<double> &elevation){elevation = elevation1D;}
	int getcols() { return cols; }
	int getrows() { return rows; }
	double getNODATA() { return NODATA; }
	double getxllcorner() { return xllcorner; }
	double getyllcorner() { return yllcorner; }

private:

	vector<double> elevation1D;
	vector<vector<double>> elevation;	// elevation of each folder, elevation(i,j)
	vector<vector<double>> slopeorganizer;	// slope of each organizer folder
	vector<vector<double>> flowDirorganizer;	// Dinf flow direction organizer
	vector<vector<double>> flowAccumorganizer;// Dinf flow accumulation organizer
	vector<vector<double>> temporganizer;	// temporary organizer for the flow accumulation function
	vector<vector<double>> infloworganizer;	// permanent organizer for the checking the flow accumulation function
	vector<vector<double>> TWIorganizer;// final topographic wetness index  
	vector<vector<double>> aspectorganizer;

	//ATB array (i,j), as acronym for "a tangent Beta", the topographic index (exponential = ln(a/tanB) or power = (a/tanB)^n) 
	vector<vector<double>> ATB;
	//Area array (i,j) is upslope area draining per contour (pixel) width, referred to as a
	vector<vector<double>> Area;     
	//y is counter of index bins
	vector<double> y;

	//Statistically distributed topographic index algorithm taken from TOPMODEL code developed by Keith Beven and used by USGS D.M. Wolock https://pubs.er.usgs.gov/publication/wri934124
	//Wolock, D. M. (1993). Simulating the variable-source-area concept of watershed hydrology with TOPMODEL (USGS Water-Resources Investigation Report 93-4124).
	//	The algorithm requires the topographic index (TI) bins (ia) are sorted from highest (wettest) to lowest TI values
	//	The algorithm requires when ia = 0, the TI bin Index[0] should be the upper limit and its IndexArea[0] = 0
	//	The algorithm presumes for each ia, the area of catchment between discrete TI values Index[ia] and Index[ia+1] equals IndexArea[ia]
	//	The algorithm presumes for each ia, the area of catchment bounding Index[ia] equals (IndexArea[ia] + IndexArea[ia+1])/2 
	//	The algorithm in C++ has TI values for ia = 0 to ia = [IndexSize - 1], yet requires use of IndexArea[IndexSize] = 0 in the above fractional area equation

	//AC[ia] is vector, for "area catchment", the proportion of catchment area between two topographic index (TI) values, where AC(1) = 0, where TI(1) is maximum possible value above largest actual value
	vector<double> AC;                  
	//ST[ia] is vector, for "soil topographic", the topographic index (exponential = ln(a/tanB) or power = (a/tanB)^n) 
	vector<double> ST;                  
	//rows, cols are size of map input; NATB is number of ST bins and not used
	int rows, cols, NATB;

	std::string simulationMode;
	std::string routingAlgorithm;

	//xll and yll are lower left map coordinates in x (easting) and y (northing) directions
	double xllcorner, yllcorner;        

	// simulation method (0;1): 1: need to load impervious coverage data
	int simMethod;                      
	// Number of ln(a/tanB) or (a/tanB)^n histogram ordinates
	string _catName;                    
	// DX, organizer spacing in elevation array, fArea: fractional area of the subcatchment
	double DX, fArea, NODATA;	
	// total fractional impervious area of the simulated subcatchment
	double totalFImpArea;               
    std::string directory;
	//N is Power Decay parameter
	double N;
	//NAC is number of topographic index bins
	int NAC;
	//currentDepth is parameter for flow accumulation routing, used in interation through DEM cells
	int currentDepth;
	//maxDepth is parameter threshold for flow accumulation routing
	int maxDepth;
	const double pi;

	//Functions with IndexCalculator

	//readDEMData function checks if DEM was read, returns 1 if True, 0 if False
	bool readDEMData();

	//init() function will set initial values for the following variables used in the topographic index
	void init() { rows = 0; cols = 0;  NATB = 0; DX = 0.0; fArea = 1.0; totalFImpArea = 0.0;}

	//checkfolder function checks if a folder exists, returns 1 if True, NODATA values if False
	bool checkfolder(int row, int col);    

	double getElevation(int row, int col);
	//cte 20210520 adding IA to function
	//void TI_Cal();
	void TI_Cal(double ImperviousArea_total_frac);
	void calcSlope();
	void calcFlowDirection_DInf();
	void calcFlowDirection_MFD();
	void calcFlowDirection_D8();
	void calcSCA_Dinf();
	void calcSCA_D8();
	void calcSCA_MFD();
	
	//DinfAccum function is recursive Dinfinity flow accumulation algorithm
	void DinfAccum(int row, int col); 
	//MFDAccum function is recursive MFD (multiple flow direction) flow accumulation algorithm
	void MFDAccum(int row, int col);  

	//cte 20210520 adding IA to function
	//void calcTWI();
	void calcTWI(double ImperviousArea_total_frac);
	void indexHist();
	//cte 20210520 adding IA to function
	//void indexCalPow();
	void indexCalPow(double ImperviousArea_total_frac);
	//void indexCalExp();
	void indexCalExp(double ImperviousArea_total_frac);
	void transform();
	void clear();
	bool success;
};

#endif

