#include "InputTopographyData.h"

//getAveSoilMoistureDeficit function only used at start of simulation
//Initializes AveSMD (m), average soil moisture deficit for folder, for use during the simulation
//receiving topographic index average, number of topographic index bins, topographic index scaling parameter, power decay parameter, transmissivity at saturation, initial discharge
double TopographyData::getAveSoilMoistureDeficit(double topAveIndex, double mM, double nN, double T0, double Q0)
{
	//AveSMD (m) is soil moisture deficit, or capacity to store water when positive, for the area between top of soil and grounwater table
	//AveSMD (m) is recomputed at each time step after updating and averaging Local SMD vector values based on their fraction of the catchment area
	//AveSMD (m) increases toward greater storage deficit or capacity with subsurface runoff removing water from storage and lowering the groundwater table
	//AveSMD (m) decreases toward smaller storage deficit or capacity with flow from Drainage_UnsatZone_m adding water into storage and raising the groundwater table
	//AveSMD (m) is used to compute local SMD as vector of values for each of the 30 bins of the Topographic Index during the subsequent simulation
	//	Local SMD values are computed in 	LocalSoilMoistureCalc.cpp or LocalSMDPowDecayCalc.cpp
	//	Local SMD values are updated with vertical water balance of precipitation, infiltration, evapotranspiration, and unsaturated zone flow, and subsurface flow
	//AveSMD (m) is related to the depth to groundwater table, Zi
	//	Zi = AveSMD / EffectivePorosity, Eq 16 in Wolock, D. M. (1993). Simulating the variable-source-area concept of watershed hydrology with TOPMODEL (USGS Water-Resources Investigation Report 93-4124). 

	//compute AveSMD (m), average soil moisture deficit, for the folder with getAveSoilMoistureDeficit function
	//	AverageIndex = average Topographic Index, T0 = transmissivity, Q0 = baseflow at start of simulation
	//	mM = scaling parameter for decay in saturated hydraulic conductivity (Ks), nN = power exponent for decay in Ks
	double aveSMD = 0.0;

	//Power function decay computation of AveSMD
	if (simulationMode.find("PowerDecay") != std::string::npos)
	{
		// used for power decay calcualtions
		double SZQ = 0.0;

		//Eq for SZQ is RHS of Eq for S_bar, aveSMD, given in text below Eq 9 of Wang, J., Endreny, T. A., & Hassett, J. M. (2006). Power function decay of hydraulic conductivity for a TOPMODEL-based infiltration routine Hydrological Processes, 20(18), 3825-3834. 
		//	From Wang et al. (2006) S_bar = m[1-(R/T0)^(1/n)*Lambda], where Lambda = topAveIndex, R = Q0 below; and SZQ = (R/T0)^(1/n)*Lambda
		SZQ = pow(Q0 / T0, 1 / nN)*(topAveIndex);

		if (SZQ >1.0)
			SZQ = 1.0;
		if (SZQ <0.0)
			SZQ = 0.0;

		//Compute initial average soil moisture deficit, aveSMD (m) for watershed
		//Eq for aveSMD is below Eq 9 of Wang, J., Endreny, T. A., & Hassett, J. M. (2006). Power function decay of hydraulic conductivity for a TOPMODEL-based infiltration routine Hydrological Processes, 20(18), 3825-3834. 
		//	From Wang et al. (2006) S_bar = m[1-(R/T0)^(1/n)*Lambda], where SZQ = (R/T0)^(1/n)*Lambda, Lambda = topAveIndex, R = Q0 above
		//Eq for aveSMD is below Eq 5 of Wang, J., Endreny, T. A., & Hassett, J. M. (2005). Flexible Modeling Package for Topographically Based Watershed Hydrology. Journal of Hydrology, 314(1-4), 78-91. 
		//	From Wang et al. (2005) S_bar = m[1-(R/T0)^(1/n)*Lambda], where SZQ = (R/T0)^(1/n)*Lambda, Lambda = topAveIndex, R = Q0 above
		aveSMD = mM * (1.0 - SZQ);
	}

	//Exponential decay computation of AveSMD
	else
	{
		double SZQ = 0.0;

		//Eq for SZQ similar to Eq 31b for Q0 in Famiglietti, J. S., & Wood, E. F. (1994). Multiscale Modeling of Spatially Variable Water and Energy Balance Processes. Water Resources Research, 30(11), 3061-3078. 
		//Eq for SZQ embedded in S_bar Eq below Eq 3 of Wang, J., Endreny, T. A., & Hassett, J. M. (2005). Flexible Modeling Package for Topographically Based Watershed Hydrology. Journal of Hydrology, 314(1-4), 78-91. 
		//	From Wang et al. (2005) S_bar = -m*ln(R/T0) -m*Lambda, where Lambda = topAveIndex, R = Q0 below 
		//	From Wang et al. (2005) given aveSMD Eq below on RHS takes log of SZQ, SZQ becomes T0 * exp(-Lambda)
		SZQ = T0 * exp(-topAveIndex);

		//Compute average soil moisture deficit, aveSMD (m) for watershed
		//Eq 4 derived from Hornberger, G. M., Beven, K. J., Cosby, B. J., & Sappington, D. E. (1985). Shenandoah Watershed Study: Calibration of a Topography-Based, Variable Contributing Area Hydrological Model to a Small Forested Catchment. Water Resources Research, 21(12), 1841-1850.
		//	Eq 4 in Horberger et al. (1985) Qb = SZQ * EXP(-S/SZM)
		//Eq given as SBAR=-SZM*ALOG(Q0/SZQ) in TMOD9502.FOR TOPMODEL DEMONSTRATION PROGRAM VERSION 95.02, By Keith Beven or Eq given as misc.S_mean[0] = - params.m * log(misc.qs0 / misc.qss) in param.init.c https://cran.r-project.org/web/packages/topmodel/index.html
		//Eq similar to aveSMD below Eq 3 of Wang, J., Endreny, T. A., & Hassett, J. M. (2005). Flexible Modeling Package for Topographically Based Watershed Hydrology. Journal of Hydrology, 314(1-4), 78-91. 
		//	From Wang et al. (2005) S_bar = -m*ln(R/T0) -m*Lambda, = -m[log(Q0/T0) / exp(-Lambda)] where Lambda = topAveIndex, R = Q0 below
		aveSMD = -mM * log(Q0 / SZQ);
	}
	return aveSMD;
}

//init function coordinates model type (Statistically or Spatially Distributed), infiltration type (Power Decay with N, or Exponential Decay), and routing type
//cte 20210715 Need to ensure variable ImperviousArea_total_frac is either statistical Hydro sum or spatial Cool Air sum of impervious area
//cte currently, statistical Hydro sum is ImperviousArea_total_frac = IA_Land_NoTC_frac + IA_Land_TC_frac + IA_Water_frac, but if Cool Air is running, it should be imperviousArea_mapAverage_frac
//void TopographyData::init(double N)
void TopographyData::init(double N, double IA_Land_NoTC_frac, double IA_Land_TC_frac, double IA_Water_frac, double IA_Map_frac)
{
	//cte 20210501 adding intput to function
	double ImperviousArea_total_frac = IA_Land_NoTC_frac + IA_Land_TC_frac + IA_Water_frac;
	//cte 20210520 update routine to compute average impervious cover for catchment area, then transfer to IndexCalculator::calcTW for adjusting TI map and histogram
	cout << "InputTopographyData.cpp init IA_Map_frac: " << IA_Map_frac << endl;

	//model type is Statistically Distributed with infiltration type Power Decay, using topographic index histogram
	if (simulationMode == "StatisticalHydroPowerDecay")
	{
		//if calcIndexFile function for Power Decay returns a False value, meaning it did not work, then
		//cte 20210520 adding IA to function
		//if (!calcIndexFile(directory, N, simulationMode, routingAlgorithm))
		if (!calcIndexFile(directory, N, simulationMode, routingAlgorithm, ImperviousArea_total_frac))
		{
			//exit with value of 1
			throw 1;
		}
	}
	//model type is Statistically Distributed with infiltration type Exponential Decay, using topographic index histogram
	else if (simulationMode == "StatisticalHydroExponentialDecay")
	{
		//if calcIndexFile function for Exponential Decay returns a False value, meaning it did not work, then
		//cte 20210520 adding IA to function
		//if (!calcIndexFile(directory, simulationMode, routingAlgorithm))
		if (!calcIndexFile(directory, simulationMode, routingAlgorithm, ImperviousArea_total_frac))
		{
			//exit with value of 1
			throw 1;
		}
	}
	//model type is Land Cover Scaling Spatially Distributed with infiltration type Power Decay, using topographic index histogram
	else if (simulationMode == "LCScalingTempSpatialPowerDecay" || simulationMode == "SpatialTemperatureHydroPowerDecay"|| simulationMode == "CoolRiverPowerDecay")
	{
		//if calcIndexFile function for Power Decay returns a False value, meaning it did not work, then
		//cte 20210520 adding IA to function
		//if (!calcDistIndexFile(directory, N, NAC, simulationMode, routingAlgorithm))
		if (!calcDistIndexFile(directory, N, NAC, simulationMode, routingAlgorithm, ImperviousArea_total_frac))
		{
			//exit with value of 1
			throw 1;
		}
	}
	//model type is Land Cover Scaling Spatially Distributed with infiltration type Power Decay, using topographic index histogram
	else if (simulationMode == "LCScalingTempSpatialExponentialDecay" || simulationMode == "SpatialTemperatureHydroExponentialDecay")
	{
		//if calcIndexFile function for Exponential Decay returns a False value, meaning it did not work, then
		//cte 20210520 adding IA to function
		//if (!calcDistIndexFile(directory, NAC, simulationMode, routingAlgorithm))
		if (!calcDistIndexFile(directory, NAC, simulationMode, routingAlgorithm, ImperviousArea_total_frac))
		{
			//exit with value of 1
			throw 1;
		}
	}
}

//readSlopeAspectFile function reads prepared topographic data
bool TopographyData::readSlopeAspectFile()
{
	double temp, NODATA;
	std::string space, line;
	
	//ifstream function readSlopeFile diretory path and file name to read file 
	std::ifstream readSlopeFile(directory + "\\Slope.dat");

	//if readSlopeFile function returns a False value, meaning it did not work, then
	if (!readSlopeFile)
	{
		cout << "readSlopeAspectFile: Slope.dat could not be opened. Delete TIorganizer.dat and FDorganizer.dat and try again." << endl;
		//exit the function with a False value
		return false;
	}

	//getline readFDorganizer is reading in the 6 header lines of data from map file
	int xllcorner, yllcorner;
	getline(readSlopeFile, line);
	istringstream(line) >> space >> ncols;
	getline(readSlopeFile, line, '\n');
	istringstream(line) >> space >> nrows;
	getline(readSlopeFile, line, '\n');
	istringstream(line) >> space >> xllcorner;
	getline(readSlopeFile, line, '\n');
	istringstream(line) >> space >> yllcorner;
	getline(readSlopeFile, line, '\n');
	istringstream(line) >> space >> foldersize;
	getline(readSlopeFile, line, '\n');
	istringstream(line) >> space >> NODATA;

	//while loop to readSlopeFile all pixel data values, assigned to temp variable
	while (readSlopeFile >> temp)
	{
		//if pixel value is real data then
		if (temp != NODATA)
		{
			//slope1D vector has temp value added
			slope1D.push_back(temp);
		}
		//if pixel is NODATA then
		else
		{
			//slope1D vector has NODATA value added
			slope1D.push_back(simulationNoDataVal);
		}
	}
	//close readSlopeFile file
	readSlopeFile.close();

	//ifstream function readAspectFile diretory path and file name to read file 
	std::ifstream readAspectFile(directory + "\\Aspect.dat");

	//if readAspectFile function returns a False value, meaning it did not work, then
	if (!readAspectFile)
	{
		cout << "readSlopeAspectFile: Aspect.dat could not be opened. Delete TIorganizer.dat and FDorganizer.dat and try again." << endl;
		//exit the function with a False value
		return false;
	}

	//getline readFDorganizer is reading in the 6 header lines of data from map file
	getline(readAspectFile, line);
	istringstream(line) >> space >> ncols;
	getline(readAspectFile, line, '\n');
	istringstream(line) >> space >> nrows;
	getline(readAspectFile, line, '\n');
	istringstream(line) >> space >> xllcorner;
	getline(readAspectFile, line, '\n');
	istringstream(line) >> space >> yllcorner;
	getline(readAspectFile, line, '\n');
	istringstream(line) >> space >> foldersize;
	getline(readAspectFile, line, '\n');
	istringstream(line) >> space >> NODATA;

	//while loop to readAspectFile all pixel data values, assigned to temp variable
	while (readAspectFile >> temp)
	{
		//if pixel value is real data then
		if (temp != NODATA)
		{
			//aspect1D vector has temp value added
			aspect1D.push_back(temp);
		}
		//if pixel is NODATA then
		else
		{
			//aspect1D vector has NODATA value added
			aspect1D.push_back(simulationNoDataVal);
		}
	}
	//close readAspectFile file
	readAspectFile.close();

	//exit and return True
	return true;
}

//calcDistIndexFile for Spatially Distributed Power Decay applications, i.e., CoolAir, with map based topographic index (TI) values
//calcDistIndexFile not called by the statistically distributed applications using the topographic index (TI) histogram
//cte 20210520 adding IA to function
//bool TopographyData::calcDistIndexFile(std::string &directory, double N, int NAC, std::string simulationMode, std::string routingAlgorithm)
bool TopographyData::calcDistIndexFile(std::string& directory, double N, int NAC, std::string simulationMode, std::string routingAlgorithm, double ImperviousArea_total_frac)
{
	//subFarea is set to 1 for each folder
	subFarea = 1.0;
	//averageIndex for map will be computed in distAveIndex function using map topographic index values
	averageIndex = 0.0;
	indexSize = 0;
	indexAreaSize = 0;

	double temp, NODATA;
	std::string space;
	std::string line, temp2;

	//if TIorganizer.data and FDorganizer.dat do not exist, then compute them, otherwise save computational time
	if (!fileExists(directory + "\\TIorganizer.dat") && !fileExists(directory + "\\FDorganizer.dat"))
	{
		//calc function computes topographic index (TI) using directory, simulationMode, and routingAlgorithm inputs
		IndexCalculator calc(directory, N, NAC, simulationMode, routingAlgorithm, ImperviousArea_total_frac);

		//if getParseSuccess function in IndexCalculator.h returns success then TIorganizer.dat was likely created
		if (calc.getParseSucess())
		{
			//getElevation1D function in IndexCalculator.h is pointer to DEM data 
			calc.getElevation1D(elevation1D);
			setcols(calc.getcols());
			setrows(calc.getrows());
			setxllcorner(calc.getxllcorner());
			setyllcorner(calc.getyllcorner());
		}
		//getParseSuccess function in IndexCalculator.h does not return success then
		else
		{
			cout << "calcDistIndexFile Power Decay: TIorganizer.dat not created. Make sure that the dem.dat is the input directory and check its format." << endl;
			//exit function with False value
			return false;
		}
	}

	//cte 20210225 Test if this could be set to avoid computing TIorganizer.dat and FDorganizer.dat, given they likely exist
	//if TIorganizer.data and FDorganizer.dat do exist, then do not compute them to save computational time
	else
	{
		//calc function computes topographic index (TI) using directory, simulationMode, and routingAlgorithm inputs
		IndexCalculator calc(directory, N, NAC, simulationMode, routingAlgorithm, ImperviousArea_total_frac);

		//getElevation1D function in IndexCalculator.h is pointer to DEM data 
		calc.getElevation1D(elevation1D);
	}

	//ifstream function readTIorganizer diretory path and file name to read file, which contains topographic index values for each map pixel
	std::ifstream readTIorganizer(directory + "\\TIorganizer.dat");

	//if readTIorganizer function returns a False value, meaning it did not work, then
	if (!readTIorganizer)
	{
		cout << "readTIorganizer Power Decay: TIorganizer.dat could not be opened." << endl;
		//exit the function with a False value
		return false;
	}

	//getline readTIorganizer is reading in the 6 header lines of data from map file
	int xllcorner, yllcorner;
	getline(readTIorganizer, line);
	istringstream(line) >> space >> ncols;
	getline(readTIorganizer, line, '\n');
	istringstream(line) >> space >> nrows;
	getline(readTIorganizer, line, '\n');
	istringstream(line) >> space >> xllcorner;
	getline(readTIorganizer, line, '\n');
	istringstream(line) >> space >> yllcorner;
	getline(readTIorganizer, line, '\n');
	istringstream(line) >> space >> foldersize;
	getline(readTIorganizer, line, '\n');
	istringstream(line) >> space >> NODATA;

	//while loop to readTIorganizer all pixel data values, assigned to temp variable
	while (readTIorganizer >> temp)
	{
		//if pixel value is real data then
		if (temp != NODATA)	{
			//index vector has temp value added
			index.push_back(temp);
		}
		//if pixel is NODATA then
		else {
			//index vector has NODATA value added
			index.push_back(simulationNoDataVal);
		}
	}
	//indexArea vector stores value of 1 for each pixel; only 1 pixel of area to be used by all spatially distributed map folders
	indexArea.push_back(1.0);

	//close the readTIorganizer file
	readTIorganizer.close();

	//ifstream function readFDorganizer diretory path and file name to read file 
	std::ifstream readFDorganizer(directory + "\\FDorganizer.dat");

	//if readFDorganizer function returns a False value, meaning it did not work, then
	if (!readFDorganizer)
	{
		cout << "readTIorganizer Power Decay: FDorganizer.dat could not be opened." << endl;
		//exit the function with a False value
		return false;
	}

	//getline readFDorganizer is reading in the 6 header lines of data from map file
	getline(readFDorganizer, line);
	istringstream(line) >> space >> ncols;
	getline(readFDorganizer, line, '\n');
	istringstream(line) >> space >> nrows;
	getline(readFDorganizer, line, '\n');
	istringstream(line) >> space >> xllcorner;
	getline(readFDorganizer, line, '\n');
	istringstream(line) >> space >> yllcorner;
	getline(readFDorganizer, line, '\n');
	istringstream(line) >> space >> foldersize;
	getline(readFDorganizer, line, '\n');
	istringstream(line) >> space >> NODATA;

	//while loop to readFDorganizer all pixel data values, assigned to temp variable
	while (readFDorganizer >> temp)
	{
		//if pixel value is real data then
		if (temp != NODATA)
		{
			//FDorganizer vector has temp value added
			FDorganizer_1D.push_back(temp);
		}
		//if pixel is NODATA then
		else
		{
			//FDorganizer vector has NODATA value added
			FDorganizer_1D.push_back(simulationNoDataVal);
		}
	}

	//close readFDorganizer file
	readFDorganizer.close();

	//calcPropVector function calculates the proportionVector that stores the proportion of runoff that flows to downstream folders
	calcPropVector();

	//if index.size is less than numfolders, then count of map pixels (strActual) in TIorganizer.dat is different than rows and cols specified in input .xml (strExpected)
	if (index.size() < static_cast<size_t>(numfolders))
	{
		std::string strExpected, strActual;
		//exit function with a False value
		return false;
	}
	//if distAveIndex function returns a False value, meaning it did not work, then
	else if (!distAveIndex())
		//exit function with a False value
		return false;
	//if distAveIndex function returns a True value, meaning it did work, then
	else
		//exit function with a True value
		return true;
}

//calcDistIndexFile for Spatially Distributed Exponential Decay applications, i.e., CoolAir, with map based topographic index (TI) values
//calcDistIndexFile not called by the statistically distributed applications using the topographic index (TI) histogram
bool TopographyData::calcDistIndexFile(std::string &directory, int NAC, std::string simulationMode, std::string routingAlgorithm, double ImperviousArea_total_frac)
{
	//subFarea is set to 1 for each folder
	subFarea = 1.0;
	//averageIndex for map will be computed in distAveIndex function using map topographic index values
	averageIndex = 0.0;
	indexSize = 0;

	double temp, NODATA;
	std::string space;
	std::string line, temp2;

	//if TIorganizer.data and FDorganizer.dat do not exist, then compute them, otherwise save computational time
	if (!fileExists(directory + "\\TIorganizer.dat") && !fileExists(directory + "\\FDorganizer.dat"))
	{
		//calc function computes topographic index (TI) using directory, simulationMode, and routingAlgorithm inputs
		IndexCalculator calc(directory, NAC, simulationMode, routingAlgorithm, ImperviousArea_total_frac);

		//if getParseSuccess function in IndexCalculator.h returns success then TIorganizer.dat was likely created
		if (calc.getParseSucess())
		{
			//getElevation1D function in IndexCalculator.h is pointer to DEM data 
			calc.getElevation1D(elevation1D);
		}
		//getParseSuccess function in IndexCalculator.h does not return success then
		else
		{
			cout << "calcDistIndexFile Exponential Decay: TIorganizer.dat not created. Make sure that the dem.dat is the input directory and check its format." << endl;
			//exit function with False value
			return false;
		}
	}
	//cte 20210225 Test if this could be set to avoid computing TIorganizer.dat and FDorganizer.dat, given they likely exist
	//if TIorganizer.data and FDorganizer.dat do exist, then do not compute them to save computational time
	else
	{
		//calc function computes topographic index (TI) using directory, simulationMode, and routingAlgorithm inputs
		IndexCalculator calc(directory, NAC, simulationMode, routingAlgorithm, ImperviousArea_total_frac);

		//getElevation1D function in IndexCalculator.h is pointer to DEM data 
		calc.getElevation1D(elevation1D);
	}

	//ifstream function readTIorganizer diretory path and file name to read file, which contains topographic index values for each map pixel
	std::ifstream readTIorganizer(directory + "\\TIorganizer.dat");

	//if readTIorganizer function returns a False value, meaning it did not work, then
	if (!readTIorganizer)
	{
		cout << "readTIorganizer Exponential Decay: TIorganizer.dat could not be opened." << endl;
		//exit the function with a False value
		return false;
	}

	//getline readTIorganizer is reading in the 6 header lines of data from map file
	int xllcorner, yllcorner;
	getline(readTIorganizer, line);
	istringstream(line) >> space >> ncols;
	getline(readTIorganizer, line, '\n');
	istringstream(line) >> space >> nrows;
	getline(readTIorganizer, line, '\n');
	istringstream(line) >> space >> xllcorner;
	getline(readTIorganizer, line, '\n');
	istringstream(line) >> space >> yllcorner;
	getline(readTIorganizer, line, '\n');
	istringstream(line) >> space >> foldersize;
	getline(readTIorganizer, line, '\n');
	istringstream(line) >> space >> NODATA;

	//while loop to readTIorganizer all pixel data values, assigned to temp variable
	while (readTIorganizer >> temp)
	{
		//if pixel value is real data then
		if (temp != NODATA)
		{
			//index vector has temp value added
			index.push_back(temp);
		}
		//if pixel is NODATA then
		else
		{
			//index vector has NODATA value added
			index.push_back(simulationNoDataVal);
		}
	}
	//indexArea vector stores value of 1 for each pixel; only 1 pixel of area to be used by all spatially distributed map folders
	indexArea.push_back(1.0);

	//close the readTIorganizer file
	readTIorganizer.close();

	//ifstream function readFDorganizer diretory path and file name to read file 
	std::ifstream readFDorganizer(directory + "\\FDorganizer.dat");

	//if readFDorganizer function returns a False value, meaning it did not work, then
	if (!readFDorganizer)
	{
		cout << "readTIorganizer Exponential Decay: FDorganizer.dat could not be opened." << endl;
		//exit the function with a False value
		return false;
	}

	//getline readFDorganizer is reading in the 6 header lines of data from map file
	getline(readFDorganizer, line);
	istringstream(line) >> space >> ncols;
	getline(readFDorganizer, line, '\n');
	istringstream(line) >> space >> nrows;
	getline(readFDorganizer, line, '\n');
	istringstream(line) >> space >> xllcorner;
	getline(readFDorganizer, line, '\n');
	istringstream(line) >> space >> yllcorner;
	getline(readFDorganizer, line, '\n');
	istringstream(line) >> space >> foldersize;
	getline(readFDorganizer, line, '\n');
	istringstream(line) >> space >> NODATA;

	//while loop to readFDorganizer all pixel data values, assigned to temp variable
	while (readFDorganizer >> temp)
	{
		//if pixel value is real data then
		if (temp != NODATA)
		{
			//FDorganizer vector has temp value added
			FDorganizer_1D.push_back(temp);
		}
		//if pixel is NODATA then
		else
		{
			//FDorganizer vector has NODATA value added
			FDorganizer_1D.push_back(simulationNoDataVal);
		}

	}
	//close readFDorganizer file
	readFDorganizer.close();

	//calcPropVector function calculates the proportionVector that stores the proportion of runoff that flows to downstream folders
	calcPropVector();

	//if index.size is less than numfolders, then count of map pixels (strActual) in TIorganizer.dat is different than rows and cols specified in input .xml (strExpected)
	if (index.size() < static_cast<size_t>(numfolders))
	{
		std::string strExpected, strActual;
		//exit function with a False value
		return false;
	}
	//if distAveIndex function returns a False value, meaning it did not work, then
	else if (!distAveIndex()) {
		//exit function with a False value
		return false;
	}
	//if distAveIndex function returns a True value, meaning it did work, then
	else {
		//exit function with a True value
		return true;
	}
}


//distAveIndex function for Spatially Distributed applications, i.e., CoolAir, using a discrete  topographic index (TI) value for each pixel, i.e. folder, of map
//	distAveIndex not called by the Statistically Distributed applications, using the topographic index (TI) histogram
bool TopographyData::distAveIndex()
{
	//initialize variables before summing
	int cnt = 0;
	averageIndex = 0;

	//loop through all pixels, i.e., folders of map
	for (size_t i = 0; i < index.size(); i++) 
	{
		//if pixel is real data, not NODATA
		if (index[i] != simulationNoDataVal) 
		{
			//averageIndex is sum of all pixel topographic index value
			averageIndex = averageIndex + index[i];
			//cnt counter is advanced + 1
			cnt++;
		}
	}
	//if all pixels were NODATA and cnt = 0
	if (cnt == 0)
	{
		cout << "distAveIndex: Error calculating distributed average index. \n All map pixels are NODATA." << endl;
		//exit function with False
		return false;
	}
	//if some pixels were real data and cnt > 0
	else
	{
		//averageIndex is quotient of summed index values and summed number of pixels
		//averageIndex stored as input->AverageIndex
		averageIndex = averageIndex / cnt;
		//exit function with True
		return true;
	}
}

//cte 20210917 Can we replace checkFile function (where is it?) with use ifstream::is_open functions?
//fileExists function tests if file path exists using the checkFile function
bool TopographyData::fileExists(std::string path)
{
	//call checkFile function with path variable
	std::ifstream checkFile(path);

	//if checkFile returns a False value, meaning it did not work, then
	if (!checkFile)
	{
		//exit function with False value
		return false;
	}
	//if checkFile returns a True value, meaning it did work, then
	else
	{
		//close checkFile
		checkFile.close();
		//exit function with True value
		return true;
	}
}

//calcIndexFile function computes Topographic Index (TI) histogram for Statistically Distributed Exponential Decay applications, i.e., Hydro w/ BulkArea, with TI histogram assigned to each bulk area, i.e. folder
//	calcIndexFile not called by the Spatially Distributed applications, using the topographic index (TI) values for map pixels
bool TopographyData::calcIndexFile(std::string &directory, std::string simulationMode, std::string routingAlgorithm, double ImperviousArea_total_frac)
{
	//Statistically distributed topographic index algorithm taken from TOPMODEL code developed by Keith Beven and used by USGS D.M. Wolock https://pubs.er.usgs.gov/publication/wri934124
	//Wolock, D. M. (1993). Simulating the variable-source-area concept of watershed hydrology with TOPMODEL (USGS Water-Resources Investigation Report 93-4124).
	//	The algorithm requires the topographic index (TI) bins (ia) are sorted from highest (wettest) to lowest TI values
	//	The algorithm requires when ia = 0, the TI bin Index[0] should be the upper limit and its IndexArea[0] = 0
	//	The algorithm presumes for each ia, the area of catchment between discrete TI values Index[ia] and Index[ia+1] equals IndexArea[ia]
	//	The algorithm presumes for each ia, the area of catchment bounding Index[ia] equals (IndexArea[ia] + IndexArea[ia+1])/2 

	//if fileExists returns a False value, meaning it did not work, then
	if (!fileExists(directory + "expdecayTI.dat"))
	{
		//calc function computes topographic index (TI) for Exponential Decay using directory, simulationMode, and routingAlgorithm inputs
		IndexCalculator calc(directory, NAC, simulationMode, routingAlgorithm, ImperviousArea_total_frac);

		//if getParseSuccess function in IndexCalculator.h returns success then TIorganizer.dat was likely created
		if (calc.getParseSucess())

			//createIndexFile function in IndexCalculator.cpp writes topographic index histogram file and exits function with False
			calc.createIndexFile(false);
		//if getParseSuccess function in IndexCalculator.h does not return success then TIorganizer.dat was not created
		else
		{
			cout << "calcIndexFile: Error creating Topographic Index Exponential Decay histogram file. Delete any files from input directory that were created during model run, and check for errors in dem.dat against TestCase example files." << endl;
			//exit function with False
			return false;
		}
	}

	//ifstream readIndex to read existing topographic index histogram file, Exponential Decay version
	std::ifstream readIndex(directory + "expdecayTI.dat");

	//if readIndex returns a False value, meaning it did not work, then
	if (!readIndex)
	{
		cout << "calcIndexFile: Error opening Topographic Index Expontial Decay histogram file. Delete any files from input directory that were created during model run, and check for errors in dem.dat against TestCase example files." << endl;
		//exit function with False
		return false;
	}

	//initialize the tempArea and tempIndex with low values
	double tempArea = -1111.0;
	double tempIndex = -1111.0;
	std::string space;
	
	//getline reads first line of header from readIndex file
	getline(readIndex, space);

	//>> reads next line from readIndex file, as variable indexSize (number of rows of data), and subFarea (fraction of catchment)
	//	Note: these variables are not headers for the TI values that follow
	readIndex >> indexSize >> subFarea;
	
	//while loop with >> to read tempArea (fraction; can be area units such as m^2) as bin fraction of catchment area, and tempIndex as bin TI value
	while (readIndex >> tempArea && readIndex >> tempIndex)
	{
		//index vector has tempIndex value added; index.size() is determined based on rows of data within expdecayTI.dat or powdecayTI.dat
		index.push_back(tempIndex);
		//indexArea vector has tempArea value added
		indexArea.push_back(tempArea);
	}
	//indexArea vector extended to hold a 0 value at index=NAC, after reading last item in histogram NAC-1 given C++ starts vector at 0
	//	This final indexArea value is used in algorithms computing TI_binArea_frac (ACF in Topmodel)
	indexArea.push_back(0);

	//close the readIndex file
	readIndex.close();

	//if aveIndex returns a False value, meaning it did not work, then
	if (!aveIndex())
		//aveIndex function failed, and the average TI value was not computed
		//exit function with False value
		return false;
	//if aveIndex returns a True value, meaning it did work, then
	else
		//aveIndex function worked, and the average TI value was computed
		//exit function with True value
		return true;
}

//calcIndexFile function computes Topographic Index (TI) histogram for Statistically Distributed Power Decay applications, i.e., Hydro w/ BulkArea, with TI histogram assigned to each bulk area, i.e. folder
//	calcIndexFile not called by the Spatially Distributed applications, using the topographic index (TI) values for map pixels
bool TopographyData::calcIndexFile(std::string &directory, double N, std::string simulationMode, std::string routingAlgorithm, double ImperviousArea_total_frac)
{
	//Statistically distributed topographic index algorithm taken from TOPMODEL code developed by Keith Beven and used by USGS D.M. Wolock https://pubs.er.usgs.gov/publication/wri934124
	//Wolock, D. M. (1993). Simulating the variable-source-area concept of watershed hydrology with TOPMODEL (USGS Water-Resources Investigation Report 93-4124).
	//	The algorithm requires the topographic index (TI) bins (ia) are sorted from highest (wettest) to lowest TI values
	//	The algorithm requires when ia = 0, the TI bin Index[0] should be the upper limit and its IndexArea[0] = 0
	//	The algorithm presumes for each ia, the area of catchment between discrete TI values Index[ia] and Index[ia+1] equals IndexArea[ia]
	//	The algorithm presumes for each ia, the area of catchment bounding Index[ia] equals (IndexArea[ia] + IndexArea[ia+1])/2 

	//if fileExists returns a False value, meaning it did not work, then
	if (!fileExists(directory + "powdecayTI.dat"))
	{
		//calc function computes topographic index (TI) for Power Decay using directory, simulationMode, and routingAlgorithm inputs
		IndexCalculator calc(directory, N, NAC, simulationMode, routingAlgorithm, ImperviousArea_total_frac);

		//if getParseSuccess function in IndexCalculator.h returns success then TIorganizer.dat was likely created
		if (calc.getParseSucess())

			//createIndexFile function in IndexCalculator.cpp writes topographic index histogram file and exits function with True
			calc.createIndexFile(true);
		//if getParseSuccess function in IndexCalculator.h does not return success then TIorganizer.dat was not created
		else
		{
			cout << "calcIndexFile: Error creating Topographic Index Power Decay histogram file. Delete any files from input directory that were created during model run, and check for errors in dem.dat against TestCase example files." << endl;
			//exit function with False
			return false;
		}
	}

	//ifstream readIndex to read existing topographic index histogram file, Power Decay version
	std::ifstream readIndex(directory + "powdecayTI.dat");

	//if readIndex returns a False value, meaning it did not work, then
	if (!readIndex)
	{
		cout << "calcIndexFile: Error opening Topographic Index Power Decay histogram file. Delete any files from input directory that were created during model run, and check for errors in dem.dat against TestCase example files." << endl;
		//exit function with False
		return false;
	}

	//initialize the tempArea and tempIndex with low values
	double tempArea = -1111.0;
	double tempIndex = -1111.0;
	std::string space;

	//getline reads first line of header from readIndex file
	getline(readIndex, space);

	//>> reads next line from readIndex file, as variable indexSize (number of rows of data), and subFarea (fraction of catchment)
	//	Note: these variables are not headers for the TI values that follow
	readIndex >> indexSize >> subFarea;

	//while loop with >> to read tempArea (fraction; can be area units such as m^2) as bin fraction of catchment area, and tempIndex as bin TI value
	while (readIndex >> tempArea && readIndex >> tempIndex)
	{
		//index vector has tempIndex value added
		index.push_back(tempIndex);
		//indexArea vector has tempArea value added
		indexArea.push_back(tempArea);
	}
	//indexArea vector extended to hold a 0 value at index=NAC, after reading last item in histogram NAC-1 given C++ starts vector at 0
	//	This final indexArea value is used in algorithms computing TI_binArea_frac (ACF in Topmodel)
	indexArea.push_back(0);

	//close the readIndex file
	readIndex.close();

	//if aveIndex returns a False value, meaning it did not work, then
	if (!aveIndex())
		//aveIndex function failed, and the average TI value was not computed
		//exit function with False value
		return false;
	//if aveIndex returns a True value, meaning it did work, then
	else
		//aveIndex function worked, and the average TI value was computed
		//exit function with True value
		return true;
}

//aveIndex function computes average of Topographic Index (TI) histogram for Statistically Distributed applications, i.e., Hydro w/ BulkArea, with TI histogram assigned to each bulk area, i.e. folder
//	aveIndex not called by the Spatially Distributed applications, using the topographic index (TI) values for map pixels
bool TopographyData::aveIndex()
{
	//Statistically distributed topographic index algorithm taken from TOPMODEL code developed by Keith Beven and used by USGS D.M. Wolock https://pubs.er.usgs.gov/publication/wri934124
	//Wolock, D. M. (1993). Simulating the variable-source-area concept of watershed hydrology with TOPMODEL (USGS Water-Resources Investigation Report 93-4124).
	//	The algorithm requires the topographic index (TI) bins (ia) are sorted from highest (wettest) to lowest TI values//The algorithm requires when ia = 0, the TI bin Index[0] should be the upper limit and its IndexArea[0] = 0
	//	The algorithm requires when ia = 1, the fractional Area = the area between [IndexArea[1] + IndexArea[1+1]]/2
	//	The algorithm presumes for each ia, the area of catchment between discrete TI values Index[ia] and Index[ia+1] equals IndexArea[ia]
	//	The algorithm presumes for each ia, the area of catchment bounding Index[ia] equals (IndexArea[ia] + IndexArea[ia+1])/2 

	//initialize the averageIndex and totalArea values
	averageIndex = 0.0;
	double totalArea = 0.0;

	//loop through all TI histogram bins, counter starting at 0
	for (size_t i = 0; i< index.size(); i++)
	{
		//totalArea is sum of indexArea (fraction; can be area units such as m^2) for each bin; N.B. The 1st bin, should contain the upper limit for TI values and have indexArea = 0
		totalArea = totalArea + indexArea[i]; 
	}

	//if totalArea is equal to or less than 0
	if (totalArea <= 0.0)
	{
		cout << "aveIndex: Error reading Topographic Index file; catchment fractional area is negative or 0. Delete any files from input directory that were created during model run, and check for errors in dem.dat against TestCase example files." << endl;
		//exit function with False
		return false;
	}

	//if totalArea is not equal to 0
	if (totalArea != 0.0)

		//indexArea for 1st item in vector (largest TI value) should be zero, so this division should be doubly superfluous
		//	Algorithms in subsurface moisture routines presume the 1st bin should contain the upper limit for TI values and have indexArea = 0
		indexArea[0] = indexArea[0] / totalArea;   

	//loop through all remaining TI histogram bins, counter starting at 1
	for (size_t i = 1; i < index.size(); i++)
	{
		//indexArea enforced as fraction by division with totalArea; algorithm works for values originally stored as fractions (totalArea = 1) or as area units
		indexArea[i] = indexArea[i] / totalArea;

		//averageIndex is the final sum of the (average of neighboring discrete TI values defining bin edges), weighted by the (fractional area within each bin)
		//averageIndex stored as input->AverageIndex
		averageIndex = averageIndex + indexArea[i] * (index[i] + index[i - 1]) / 2;
		//Original TOPMODEL Eq TL = TL + AC(J) * (ST(J) + ST(J - 1)) / 2; find algorithm by searching Entire Solution for "CALCULATE AREAL INTEGRAL OF LN(A / TANB)"
	}

	//exit function with True value
	return true;
}

//function may transform linearized organizer into [Row,Col]
void TopographyData::calcPropVector()
{
	double flowDir;
	int location1, location2;

	for (int folder = 0; folder < FDorganizer_1D.size(); folder++)
	{
		flowDir = FDorganizer_1D[folder];
		if (flowDir != NODATA)
		{

			//function may transform linearized organizer into [Row,Col]
			int col = folder % ncols;
			int row = folder / ncols;

			double proportion1 = 0;
			double proportion2 = 0;
			int a1 = 0;
			int b1 = 0;
			int a2 = 0;
			int b2 = 0;


			if (flowDir >= 0) {
				// find which two folders receive flow and the proportion to each
				if (flowDir >= 0 && flowDir < 45) {
					proportion1 = (45 - flowDir) / 45;
					a1 = col;
					b1 = row - 1;
					proportion2 = flowDir / 45;
					a2 = col + 1;
					b2 = row - 1;
				}
				else if (flowDir >= 45 && flowDir < 90) {
					proportion1 = (90 - flowDir) / 45;
					a1 = col + 1;
					b1 = row - 1;
					proportion2 = (flowDir - 45) / 45;
					a2 = col + 1;
					b2 = row;
				}
				else if (flowDir >= 90 && flowDir < 135) {
					proportion1 = (135 - flowDir) / 45;
					a1 = col + 1;
					b1 = row;
					proportion2 = (flowDir - 90) / 45;
					a2 = col + 1;
					b2 = row + 1;
				}
				else if (flowDir >= 135 && flowDir < 180) {
					proportion1 = (180 - flowDir) / 45;
					a1 = col + 1;
					b1 = row + 1;
					proportion2 = (flowDir - 135) / 45;
					a2 = col;
					b2 = row + 1;
				}
				else if (flowDir >= 180 && flowDir < 225) {
					proportion1 = (225 - flowDir) / 45;
					a1 = col;
					b1 = row + 1;
					proportion2 = (flowDir - 180) / 45;
					a2 = col - 1;
					b2 = row + 1;
				}
				else if (flowDir >= 225 && flowDir < 270) {
					proportion1 = (270 - flowDir) / 45;
					a1 = col - 1;
					b1 = row + 1;
					proportion2 = (flowDir - 225) / 45;
					a2 = col - 1;
					b2 = row;
				}
				else if (flowDir >= 270 && flowDir < 315) {
					proportion1 = (315 - flowDir) / 45;
					a1 = col - 1;
					b1 = row;
					proportion2 = (flowDir - 270) / 45;
					a2 = col - 1;
					b2 = row - 1;
				}
				else if (flowDir >= 315 && flowDir <= 360) {
					proportion1 = (360 - flowDir) / 45;
					a1 = col - 1;
					b1 = row - 1;
					proportion2 = (flowDir - 315) / 45;
					a2 = col;
					b2 = row - 1;
				}

				// recording the proportion that flows to each downstream folder
				location1 = ncols * b1 + a1;
				location2 = ncols * b2 + a2;
				std::pair<int, int> downstreamfolders = std::make_pair(location1, location2);
				downstreamfoldermap.emplace(folder, downstreamfolders);
				std::pair<int, int> downstreamProportion = std::make_pair(proportion1, proportion2);
				downstreamfoldermap.emplace(folder, downstreamProportion);
			}

		}
		else {

			std::pair<int, int> downstreamfolders = std::make_pair(NODATA, NODATA);
			downstreamfoldermap.emplace(folder, downstreamfolders);
			std::pair<int, int> downstreamProportion = std::make_pair(NODATA, NODATA);
			downstreamfoldermap.emplace(folder, downstreamProportion);
		}
	}


}


