#include "Inputs.h"
#include <random>
#include <iostream>
#include <string>

//Input function carries string pointer called path
Inputs::Inputs(std::string& path) : directory(path)
{
	std::cout << "===" << std::endl;
	std::cout << "Reading input from " << std::endl;
	std::cout << directory << std::endl;
	LogMessage::log("Reading input from " + directory);
	std::cout << "===" << std::endl;

	//Call function to read HydroPlusConfig.xml, which populates SimulationStringParam[""], SimulationNumericalParams[""], and LocationParams[""] terms
	readHydroPlusConfigFile();

	directory += "\\";
	SimulationStringParams["InputDirectory"] = directory;
	SimulationStringParams["OutputDirectory"].erase(std::remove(SimulationStringParams["OutputDirectory"].begin(), SimulationStringParams["OutputDirectory"].end(), '\"'), 
				SimulationStringParams["OutputDirectory"].end());

	if (SimulationStringParams["Model"] == "CoolRiver")
	{
		readWeatherFile(SimulationNumericalParams["StartDay"], SimulationNumericalParams["EndDay"]);
	}
	//Li: ECDynamic don't need to read weather input
	else if (SimulationStringParams["Model"] != "ECDynamic")
	{
		readWeatherFile(SimulationNumericalParams["StartDay"], SimulationNumericalParams["EndDay"]);
	}
	//Li: ECDynamic don't need to read Evap input, all the others models will read
	if (SimulationStringParams["Model"] != "ECDynamic") {
		readEvaporationFile(SimulationNumericalParams["StartDay"], SimulationNumericalParams["EndDay"]);
		readPollutionData(directory);
	}

	double imperviousArea_mapAverage_frac = 0.0;
	if (SimulationStringParams["Model"] == "SpatialTemperatureHydro")
	{
		readNLCDLandCoverFile(directory, LocationParams["NODATA"]);

		//cte 20210520 update routine to compute average impervious cover for catchment area, then transfer to IndexCalculator::calcTW for adjusting TI map and histogram
		//readImperviousFile(directory, LocationParams["NODATA"]);
		readImperviousFile(directory, LocationParams["NODATA"], imperviousArea_mapAverage_frac);

		readTreeCoverFile(directory, LocationParams["NODATA"]);
		readBlockGroupOutputFileMap(directory);
		calcNLCDLandCoverData(LocationParams["NumberOfLocations"], LocationParams["NODATA"]);
	}
	if (SimulationStringParams["Model"] == "LCScalingTempSpatial")
	{
		readBlockGroupOutputFileMap(directory);
	}
	if (SimulationStringParams["Model"] == "CoolRiver")
	{
		readHecRasFile(SimulationNumericalParams["StartDay"], SimulationNumericalParams["EndDay"]);
		readRiverSWTimeSeriesFile(SimulationNumericalParams["StartDay"], SimulationNumericalParams["EndDay"]);
		readRiverGWTimeSeriesFile(SimulationNumericalParams["StartDay"], SimulationNumericalParams["EndDay"]);
		readRiverTimeSeriesFile(SimulationNumericalParams["StartDay"], SimulationNumericalParams["EndDay"]);
		readRiverStationSeriesFile(directory, LocationParams["NODATA"]);
	}
	//02012020 ECDynamic 
	if (SimulationStringParams["Model"] == "ECDynamic")
	{
		readNLCDLandCoverFile(directory, LocationParams["NODATA"]);
		readAOIECDatabase(directory);
		readDischargeData(directory);
	}
	//For now, ECDynamic don't need to proceed topoindex calculation
	if (SimulationStringParams["Model"] != "ECDynamic") {
		//cte 20210520 update routine to compute average impervious cover for catchment area, then transfer to IndexCalculator::calcTW for adjusting TI map and histogram
		//ProcessTopographyData();
		ProcessTopographyData(imperviousArea_mapAverage_frac);
		
		CalculateLAISeries();
	}
}

//cte 20210520 update routine to compute average impervious cover for catchment area, then transfer to IndexCalculator::calcTW for adjusting TI map and histogram
//void Inputs::ProcessTopographyData()
void Inputs::ProcessTopographyData(double imperviousArea_mapAverage_frac)
{
	//initialize numfolders as number of folders, each folder with a unique water balance (e.g, map pixel in spatial distribution, or bulk area in statistical distribution)
	double numfolders = LocationParams["NumberOfLocations"];
	//initialize simulationNoDataVal as NODATA
	double simulationNoDataVal = LocationParams["NODATA"];
	//initialize NAC as number of topographic index bins TopographicIndexBins
	//	index.size() determined by NAC when reading in dem.dat, 
	//	otherwise, index.size() determined based on rows of data within expdecayTI.dat or powdecayTI.dat
	int NAC = SimulationNumericalParams["TopographicIndexBins"];
	//if NAC value is not greater than 0 
	if (NAC * 1 > 0) { }
	//reset NAC to default size of 30 bins
	else {
		NAC = 30;
		cout << "The number of topographic index bins, NAC, was reset to: " << NAC << endl;}

	//call TopographyData function in InputTopographyData.cpp sending directory, model type, infiltration type, routing type, number of folders, and no data value
	//	This call formalizes naming of routingAlgorithm = SimulationStringParams["Routing"]
	//	This call formalizes naming of simulationMode = SimulationStringParams["Model"] + SimulationStringParams["Infiltration"]
	TopographyData inputTopographyData(directory + "\\", SimulationStringParams["Model"] + SimulationStringParams["Infiltration"],
		SimulationStringParams["Routing"], numfolders, simulationNoDataVal, NAC);

	//n is coefficient for power function decay of saturated hydrualic conductivity with depth, due to decreasing number of macropores in soil at depth
	//ImperviousCover_frac and ImperviousCoverUnderTreeCanopy_frac are impervious cover for catchment
	//cte 20210520 adding IA to function, determine how to obtain imperviousArea_mapAverage_frac from readImperviousFile and send to inputTopographyData.init
	//double imperviousArea_mapAverage_frac = readImperviousFile.imperviousArea_mapAverage_frac;
	//inputTopographyData.init(RefParams["n"]);
	inputTopographyData.init(RefParams["n"],RefParams["ImperviousCover_frac"], RefParams["ImperviousCoverUnderTreeCanopy_frac"], RefParams["SurfaceWaterIA_frac"], imperviousArea_mapAverage_frac);

	//AverageIndex is watershed area weighted average of Topographic Index, often called Lambda, computed with inputTopographyData.averageIndex
	AverageIndex = inputTopographyData.averageIndex;

	//AveSMD (m) is soil moisture deficit, or capacity to store water when positive, for the area between top of soil and grounwater table
	//compute AveSMD, average soil moisture deficit (m), for the folder with getAveSoilMoistureDeficit function
	//	AverageIndex = average Topographic Index, T0_m2ph = transmissivity, Q0_mph = baseflow at start of simulation, both adjusted to per time step with TimeStep_sec input key
	//	m = scaling parameter (m) for decay in saturated hydraulic conductivity (Ks), n = power exponent for decay in Ks
	AveSMD = inputTopographyData.getAveSoilMoistureDeficit(AverageIndex, RefParams["m"], RefParams["n"], RefParams["T0_m2ph"], RefParams["Q0_mph"]);

	//IndexSize is number of index values in TI index vector (equal to bins in statistically distributed, equal to count of map pixels with real data in spatially distributed)
	//	index.size() determined by NAC when reading in dem.dat, 
	//	otherwise, index.size() determined based on rows of data within expdecayTI.dat or powdecayTI.dat, within TopographyData::calcIndexFile
	IndexSize = inputTopographyData.index.size();
	//Index is topographic index (TI) vector
	Index = inputTopographyData.index;
	//IndexAreaSize is number of index values in TI fractional area vector (equal to bins + 1 in statistically distributed, equal to count of map pixels with real data in spatially distributed)
	IndexAreaSize = inputTopographyData.indexArea.size();
	//IndexArea is fraction of catchment area associated with index value
	IndexArea = inputTopographyData.indexArea;

	setrows(inputTopographyData.getrows());
	setcols(inputTopographyData.getcols());
	setxllcorner(inputTopographyData.getxllcorner());
	setyllcorner(inputTopographyData.getyllcorner());
	if (SimulationStringParams["Model"] == "SpatialTemperatureHydro" || SimulationStringParams["Model"] == "LCScalingTempSpatial" || SimulationStringParams["Model"] == "CoolRiver")
	{
		readSolarRadiationFile(SimulationNumericalParams["StartDay"], SimulationNumericalParams["EndDay"]);
		processTemperatureExecutionParams();
		DEM = inputTopographyData.elevation1D;
		bool filesExist = inputTopographyData.readSlopeAspectFile();
		if (!filesExist)
		{
			throw 0;
		}
		Slope = inputTopographyData.slope1D;
		Aspect = inputTopographyData.aspect1D;
	}
}

int Inputs::rdn(int y, int m, int d)
{ /* Rata Die day one is 0001-01-01 */
	if (m < 3)
		y--, m += 12;
	return 365 * y + y / 4 - y / 100 + y / 400 + (153 * m - 457) / 5 + d - 306;
}

//Subtract function finds number of days between dateStart and dateEnd
int Inputs::Subtract(int dateStart, int dateEnd)
{
	char buffer[30];
	_itoa_s(dateStart, buffer, 10);
	std::string d1 = buffer;
	int I1 = atoi(d1.substr(0, 4).c_str()); //I stores current year
	int J1 = atoi(d1.substr(4, 2).c_str()); //J stores current month
	int K1 = atoi(d1.substr(6, 2).c_str()); //K stores current day

	char buffer2[30];
	_itoa_s(dateEnd, buffer2, 10);
	std::string d2 = buffer2;
	int I2 = atoi(d2.substr(0, 4).c_str()); //I stores current year
	int J2 = atoi(d2.substr(4, 2).c_str()); //J stores current month
	int K2 = atoi(d2.substr(6, 2).c_str()); //K stores current day
	return rdn(I1, J1, K1) - rdn(I2, J2, K2);
}

void Inputs::readHydroPlusConfigFile()
{

	/*
	//cte 20210916 Testing Sean Zhu's XML reader code which uses pugixml
	// Create path3 to contain path1 & path2
	std::string path3;
	//define path1 as current directory given at command prompt
	const char* path1 = directory.c_str();
	//define path2 as HydroPlusConfig.xml
	const char* path2 = "\\HydroPlusConfig.xml";
	//define path3 as path1 + path2
	path3 += path1;
	path3 += path2;
	//create fullpath as pointer to path3 character string
	const char* fullpath = path3.c_str();
	//create pugi xml document called docu, representing the HydroPlusConfig.xml input file
	pugi::xml_document docu;
	//initiate counters to loop through HydroPlusConfig.xml DataDrawers and DataFolders
	int i = 0;
	int j = 0;
 
	//if not able to load file at fullpath, then HydroPlusConfig.xml file does not exit, and then leave function and return
	if (!docu.load_file(fullpath, pugi::parse_default, pugi::encoding_utf8))
	{ return;
	}

	//Define node named after HydroPlusConfig, 1st child in HydroPlusConfig.xml file; read all children in this node
	pugi::xml_node HydroPlusConfig_node = docu.child("HydroPlusConfig");
	//Define node named after SimulationStringParams, child in HydroPlusConfig node; read all children in this node
	pugi::xml_node SimulationStringParams_node = HydroPlusConfig_node.child("SimulationStringParams");
	//Define node named after SimulationNumericalParams, child in HydroPlusConfig node; read all children in this node
	pugi::xml_node SimulationNumericalParams_node = HydroPlusConfig_node.child("SimulationNumericalParams");

	//example of calling a key within a node read by pugi xml
	//cout << "StartDay: " << SimulationNumericalParams_node.child("StartDay").first_child().value() << endl;

	//Define node named after DataOrganizer, child in HydroPlusConfig node, then loop within this node to find DataDrawers and DataFolders
	pugi::xml_node DataOrganizer_node = HydroPlusConfig_node.child("DataOrganizer");
	//Loop within DataOrganizer node to find child and next sibling nodes labeled as DataDrawers
	for (pugi::xml_node DataDrawers_node = DataOrganizer_node.child("DataDrawer"); DataDrawers_node; DataDrawers_node = DataDrawers_node.next_sibling("DataDrawer"))
	{
		++i;
		cout << "DataDrawer: " << i << endl;
		// Using for loop to search all the files called DataFolder in DataDrawer
		for (pugi::xml_node DataFolder_node = DataDrawers_node.child("DataFolder"); DataFolder_node; DataFolder_node = DataFolder_node.next_sibling("DataFolder"))
		{
			//count how many DataFolder in DataDrawer
			++j;
			cout << "DataFolder: " << j << endl;

			//cte perhaps use  DataDrawers[row]. to simultaneously store multiple data sets

			// Example for work without pre-define name for each key.
			cout << "Type: " << DataFolder_node.child("Type").first_child().value() << endl;
			cout << "Area: " << DataFolder_node.child("Area").first_child().value() << endl;
			
			// search for all the values which are we needed.
			// When value is empty, it skep over.
			if (pugi::xml_node Type_node = DataFolder_node.child("Type")) {
				// Otherwise get the value which are we needed
				cout << "Type: " << Type_node.first_child().value() << endl;
			}

			if (pugi::xml_node Area_node = DataFolder_node.child("Area")) {
				cout << "Area: " << Area_node.first_child().value() << endl;
			}

			if (pugi::xml_node Zn_node = DataFolder_node.child("Zn")) {
				cout << "Zn: " << Zn_node.first_child().value() << endl;
			}

			cout << endl;
		}
	}
	*/

	//cte 20210916
	//XMLReader contains six interacting .cpp files intended to extract data from inputs, such as HydroPlusConfig.xml, and build new document tree of element nodes: 
	// Tokenizer.cpp remove whitespace and constructs new doucment using XMLElementParts.cpp
	// Node.cpp creates nodes which Tree.cpp uses to create document tree with XMLDocument.cpp

	//Can we replace t.WalkTree functions and create structures like SimulationNumericalParams["TotalTimeSteps"].?

	//Create t as Tree class for organizing HydroPlusConfig.xml input
	Tree t;
	//Create doc as XMLDocument class for containing HydroPlusConfig.xml file
	XMLDocument doc;
	//Create root as pointer to Node within XML document
	Node* root;
	//Define root as XMLDocument HydroPlusConfig.xml file, loaded from diretory given by command prompt
	root = doc.Load(directory + "\\HydroPlusConfig.xml", true);
	//Define temp as pointer to HydroPlusConfig node, known as child within XMLDocument
	Node* temp = root->getChildByTagName("HydroPlusConfig");

	//Define SimulationStringParams and SimulationNumericalParams nodes in HydroPlusConfig.xml, and once defined they are used to call specific keys and their values
	//Define SimulationStringParams as branch off t Tree using WalkTree, found as child within HydroPlusConfig node called temp
	t.WalkTree(temp->getChildByTagName("SimulationStringParams"), SimulationStringParams);
	//Define SimulationNumericalParams as branch off t Tree using WalkTree, found as child within HydroPlusConfig node called temp
	t.WalkTree(temp->getChildByTagName("SimulationNumericalParams"), SimulationNumericalParams);

	//String reader functions
	//refParamLoc string defined as key RefParamFolder, contained within node SimulationStringParams of HydroPlusConfig.xml
	std::string refParamLoc = SimulationStringParams["RefParamFolder"];
	//row defined using atof function from substring of refParamLoc key, as value between starting point 0 and comma string
	int row = atof(refParamLoc.substr(0, refParamLoc.find(",")).c_str());
	//col defined using atof function from substring of refParamLoc key, as value between comma string + 1 size of refParamLoc string minus location of comma
	int col = atof(refParamLoc.substr(refParamLoc.find(",") + 1, refParamLoc.size() - refParamLoc.find(",")).c_str());

	//SimulationNumericalParams["TotalTimeSteps"] initially defined in HydroPlusConfig.xml, and redefined in Inputs::readHydroPlusConfigFile() 
	//if TotalTimeSteps value in HydroPlusConfig.xml = 0 or empty, then TotalTimeSteps = daysDiff * 24.0
	if (SimulationNumericalParams["TotalTimeSteps"] == 0)
	{
		//daysDiff computed with Inputs:Subtract function, number of days between YYYYMMDD EndDay and YYYYMMDD StartDay, accounting for changes in year, month and day
		int daysDiff = Subtract(SimulationNumericalParams["EndDay"], SimulationNumericalParams["StartDay"]);
		//daysDiff adds 1 day to account for initial day; there is no zero value for initial day
		daysDiff += 1;
		//TotalTimeSteps set to default hours, with 24 hours in each day, if HydroPlusConfig.xml did not define TotalTimeSteps > 0
		SimulationNumericalParams["TotalTimeSteps"] = daysDiff * 24.0;
	}

	//Define DataOrganizer nodes in HydroPlusConfig.xml, and once defined they are used to call specific keys and their values
	//Define DataOrganizer as branch off t Tree using WalkTree, found as child within HydroPlusConfig node called temp
	t.WalkTree(temp->getChildByTagName("DataOrganizer"), DataDrawers, StringDataDrawers);
	if (row > DataDrawers.size() - 1 || row > StringDataDrawers.size() - 1)
	{
		LogMessage::log("RefParamFolder coordinates out of bounds. Check drawer index.");
		exit(0);
	}
	if (col > DataDrawers[row].size() - 1 || col > StringDataDrawers[row].size() - 1)
	{
		LogMessage::log("RefParamFolder coordinates out of bounds. Check row index.");
		exit(0);
	}
	RefParams = DataDrawers[row][col];

	RefStringParams = StringDataDrawers[row][col];

	if (SimulationStringParams["Model"] == "SpatialTemperatureHydro" || SimulationStringParams["Model"] == "LCScalingTempSpatial")
	{
		t.WalkTree(temp->getChildByTagName("TemperatureCalculationParams"), TemperatureCalculationParams);
		t.WalkTree(temp->getChildByTagName("TemperatureExecutionParams"), TemperatureExecutionParams);
		t.WalkTree(temp->getChildByTagName("TemperatureLocationParams"), LocationParams);
		LocationParams["NumberOfLocations"] = LocationParams["nCols"] * LocationParams["nRows"];
	}
	else
	{
		LocationParams["NumberOfLocations"] = 1.0;
		LocationParams["nCols"] = 1.0;
		LocationParams["nRows"] = 1.0;
		LocationParams["NODATA"] = -9999.0;
		LocationParams["cellsize"] = 1.0;
		LocationParams["xllcorner"] = 1.0;
		LocationParams["yllcorner"] = 1.0;
	}
	if (SimulationStringParams["Model"] == "CoolRiver")
	{
		t.WalkTree(temp->getChildByTagName("CoolRiver_Variables"), CoolRiverVariables);
	}
	if (SimulationStringParams["Model"] == "ECDynamic")
	{
		t.WalkTree(temp->getChildByTagName("ECDynamic_Variables"), ECDynamicVariables);
	}
}

void Inputs::processTemperatureExecutionParams()
{
	if (TemperatureExecutionParams["PrintBlockGroupRange"] == "all" || TemperatureExecutionParams["PrintBlockGroupRange"] == "All")
	{
		TemperatureCalculationParams["PrintAllBlockGroups"] = 1;
	}
	else
	{
		TemperatureCalculationParams["PrintAllBlockGroups"] = 0;
	}
	if (TemperatureExecutionParams["RowColOutputPeriod"] == "all" || TemperatureExecutionParams["RowColOutputPeriod"] == "All")
	{
		TemperatureCalculationParams["RowColOutputAllTimeSteps"] = 1;
	}
	if (TemperatureExecutionParams["PrintRowCol"] == "all" || TemperatureExecutionParams["PrintRowCol"] == "All")
	{
		TemperatureCalculationParams["PrintAllRowColOutput"] = 1;
	}
	if (TemperatureExecutionParams["TimeBasedOutputPeriod"] == "all" || TemperatureExecutionParams["TimeBasedOutputPeriod"] == "All")
	{
		TemperatureCalculationParams["TimeBasedOutputAllTimeSteps"] = 1;
	}
	if (TemperatureExecutionParams["PrintTimeBasedResults"] == "all" || TemperatureExecutionParams["PrintTimeBasedResults"] == "All")
	{
		TemperatureCalculationParams["PrintAllTimeBasedResults"] = 1;
	}

	for (auto i : TemperatureExecutionParams)
	{
		if (i.second == "")
			continue;
		if (!TemperatureCalculationParams["PrintAllRowColOutput"] && i.first.find("PrintRowCol") != std::string::npos)
		{
			int row = atof(i.second.substr(0, i.second.find(",")).c_str());
			int col = atof(i.second.substr(i.second.find(",") + 1, i.second.size() - i.second.find(",")).c_str());
			//row*maxcol+col
			int index = row * LocationParams["nCols"] + col;
			LocationIdsToPrintTempOutput.insert(index);
		}
		else if (!TemperatureCalculationParams["RowColOutputAllTimeSteps"] && i.first.find("RowColOutputPeriod") != std::string::npos)
		{
			int startDateHour = atof(i.second.substr(0, i.second.find(",")).c_str());
			int endDateHour = atof(i.second.substr(i.second.find(",") + 1, i.second.size() - i.second.find(",")).c_str());
			StartDatesForRowColTempOutput.push_back(startDateHour);
			EndDatesForRowColTempOutput.push_back(endDateHour);
		}
		else if (!TemperatureCalculationParams["TimeBasedOutputAllTimeSteps"] && i.first.find("TimeBasedOutputPeriod") != std::string::npos)
		{
			int startDateHour = atof(i.second.substr(0, i.second.find(",")).c_str());
			int endDateHour = atof(i.second.substr(i.second.find(",") + 1, i.second.size() - i.second.find(",")).c_str());
			StartDatesForTimeBasedTempOutput.push_back(startDateHour);
			EndDatesForTimeBasedTempOutput.push_back(endDateHour);
		}
		else if (!TemperatureCalculationParams["PrintAllBlockGroups"] && i.first.find("PrintBlockGroupRange") != std::string::npos)
		{
			int startBlockId = atof(i.second.substr(0, i.second.find(",")).c_str());
			int endBlockId = atof(i.second.substr(i.second.find(",") + 1, i.second.size() - i.second.find(",")).c_str());
			StartingBlockIdsToPrintTempOutput.push_back(startBlockId);
			EndingBlockIdsToPrintTempOutput.push_back(endBlockId);
		}
		else if (i.first.find("RefWeatherLocation") != std::string::npos)
		{
			int row = atof(i.second.substr(0, i.second.find(",")).c_str());
			int col = atof(i.second.substr(i.second.find(",") + 1, i.second.size() - i.second.find(",")).c_str());
			//row*maxcol+col
			int index = row * LocationParams["nCols"] + col;
			TemperatureCalculationParams["RefWeatherLocation"] = index;
		}
	}
}

//---------------------------------------<read Evaporation.dat>-----------------------------------------------------------
bool Inputs::readEvaporationFile(double startDate, double endDate)
{
	std::fstream readEvap(directory + "\\Evaporation.dat");
	if (!readEvap)
	{
		std::cout << "Evaporation.dat could not be opened!" << std::endl;
		LogMessage::log("Evaporation.dat could not be opened!");
		return false;
	}
	std::string tempStr = "";
	double tempDbl = -1.0;
	getline(readEvap, tempStr);
	while (readEvap.good())
	{
		std::string temp;
		double date = -1.0;
		double tempPET = -1.0;
		double tempPE = -1.0;
		double tempPETree = -1.0;
		double tempPESnow = -1.0;
		double tempPETreeSnow = -1.0;
		std::string HrMinSec;
		std::getline(readEvap, temp, ',');
		std::istringstream ss(temp);
		ss >> date;
		if (date <= endDate && date >= startDate)
		{
			EvapCalendarDates.push_back(static_cast<int>(date));
			std::getline(readEvap, temp, ',');
			std::istringstream ssTime(temp);
			ssTime >> HrMinSec;
			EvapTimeSeries.push_back(HrMinSec);
			std::getline(readEvap, temp, ',');
			std::istringstream ssPet(temp);
			ssPet >> tempPET;
			PET.push_back(tempPET);
			std::getline(readEvap, temp, ',');
			std::istringstream ssPe(temp);
			ssPe >> tempPE;
			PE.push_back(tempPE);
			std::getline(readEvap, temp, ',');
			std::istringstream ssPetree(temp);
			ssPetree >> tempPETree;
			PETree.push_back(tempPETree);
			std::getline(readEvap, temp, ',');
			std::istringstream ssPesnow(temp);
			ssPesnow >> tempPESnow;
			PESnow.push_back(tempPESnow);
			std::getline(readEvap, temp, '\n');
			std::istringstream ssPetreesnow(temp);
			ssPetreesnow >> tempPETreeSnow;
			PETreeSnow.push_back(tempPETreeSnow);
		}
		else
		{
			std::getline(readEvap, temp);
		}
	}
	readEvap.close();
	return true;
}
//---------------------------------------<read Weather.dat>---------------------------------------------------------------

//cte 20210917 Can we replace Tokenizer functions with standard C++ functions, e.g., .good, .getline
bool Inputs::readWeatherFile(double startDate, double endDate)
{
	std::random_device rd;
	std::mt19937 mt(rd());
	std::uniform_real_distribution<double> dist(0.0, 1.0);
	// Sensitive variable. If removed or uncommented, slightly changes temperature results. Replaced by minimum winds in ResistanceCal.cpp
	//MinAllowableWindSpeed = 1;

	std::fstream readWeather(directory + "\\weather.dat");
	if (!readWeather)
	{
		std::cout << "Weather.dat could not be opened!" << std::endl;
		LogMessage::log("Weather.dat could not be opened!");
		return false;
	}
	TotalRain_m = 0.0;
	TotalPrecip_m = 0.0;
	TotalSnow_m = 0.0;
	std::string temp;
	double date = -1.0;
	std::string HrMinSec;
	double tempTair = -99999;
	double tempTdew = -99999;
	double tempRn = -99999;
	double tempWindSp = -99999;
	double tempPrecip = -99999;
	double tempSnowDepth = -99999;
	std::string startline;
	std::getline(readWeather, temp);

	//cte 20210514 debug weather inputs repeatedly reading last row of data if there are blank lines at end of weather.dat file
	//int icnt = 0;

	while (readWeather.good())
	{
		std::getline(readWeather, temp, ',');
		std::istringstream ss(temp);
		ss >> date;
		if (date <= endDate && date >= startDate)
		{
			CalendarDates.push_back(static_cast<int>(date));
			std::getline(readWeather, temp, ',');
			std::istringstream ssTime(temp);
			ssTime >> HrMinSec;
			TimeSeries.push_back(HrMinSec);

			std::getline(readWeather, temp, ',');
			std::istringstream ssTair(temp);
			ssTair >> tempTair;
			Tair.push_back(tempTair);
			std::getline(readWeather, temp, ',');
			std::istringstream ssDew(temp);
			ssDew >> tempTdew;
			Tdew.push_back(tempTdew);
			std::getline(readWeather, temp, ',');
			std::istringstream ssRn(temp);
			ssRn >> tempRn;
			RadiationNet_Wpm2.push_back(tempRn);
			std::getline(readWeather, temp, ',');
			std::istringstream ssWind(temp);
			ssWind >> tempWindSp;
			if ((SimulationStringParams["Model"] == "SpatialTemperatureHydro" || SimulationStringParams["Model"] == "LCScalingTempSpatial") && tempWindSp < MinAllowableWindSpeed)
			{
				tempWindSp = MinAllowableWindSpeed;
			}
			WindSp_mps.push_back(tempWindSp);
			std::getline(readWeather, temp, ',');
			std::istringstream ssPrecip(temp);
			ssPrecip >> tempPrecip;
			Precip_mpdt.push_back(tempPrecip);
			TotalPrecip_m += tempPrecip;
			//cte 20210514 debug weather inputs repeatedly reading last row of data if there are blank lines at end of weather.dat file
			//	may be able to use input->SimulationNumericalParams["TotalTimeSteps"] to limit loop
			//icnt += 1;
			//cout << fixed << icnt << " " << date << " " << HrMinSec << " TotalPrecip_m: " << TotalPrecip_m << endl;

			getline(readWeather, temp, '\n');
			if (tempTair > 32.0)
			{
				Rain.push_back(tempPrecip);
				Snow.push_back(0);
				TotalRain_m += tempPrecip;
			}
			else
			{
				Rain.push_back(0);
				Snow.push_back(tempPrecip);
				TotalSnow_m += tempPrecip;
			}
		}
		else
		{
			std::getline(readWeather, temp);
		}
	}
	readWeather.close();
	return true;
}
//---------------------------------------<read SolarRadiation.dat>-----------------------------------------------------------------

bool Inputs::readSolarRadiationFile(double startDate, double endDate)
{
	int date = 0;
	std::string HrMinSec;
	double tempDirSW = 0;
	double tempDiffSW = 0;
	double tempDownLW = 0;
	double tempUpLW = 0;
	double tempLatitude = 0;
	std::string temphourAngle;
	std::string tempDecline;
	double ntemphourAngle = 0.0;
	double ntempDecline = 0.0;
	std::string temp;

	ifstream infile(directory + "\\SolarRadiation.dat");
	if (!infile.good())
	{
		std::cout << "Cannot find SolarRadiation.dat file" << std::endl;
		LogMessage::log("Cannot find SolarRadiation.dat file");
		return false;
	}
	std::getline(infile, temp);
	while (infile.good())
	{
		std::getline(infile, temp, ',');
		std::istringstream ss(temp);
		ss >> date;
		if (date <= endDate && date >= startDate)
		{
			SolarRadCalendarDates.push_back(static_cast<int>(date));
			std::getline(infile, temp, ',');
			std::istringstream time(temp);
			time >> HrMinSec;
			SolarRadTimeSeries.push_back(HrMinSec);
			std::getline(infile, temp, ',');
			std::istringstream ssDirsw(temp);
			ssDirsw >> tempDirSW;
			if (!DirectShortWaveRadiation.empty())
				if ((DirectShortWaveRadiation.back() - tempDirSW) > 250)
					tempDirSW = DirectShortWaveRadiation.back() - 250;
				else if ((DirectShortWaveRadiation.back() - tempDirSW) < -250.0)
					tempDirSW = DirectShortWaveRadiation.back() + 250;
			DirectShortWaveRadiation.push_back(tempDirSW);
			std::getline(infile, temp, ',');
			std::istringstream ssDiffsw(temp);
			ssDiffsw >> tempDiffSW;
			DiffuseShortWaveRadiation.push_back(tempDiffSW);
			std::getline(infile, temp, ',');
			std::istringstream ssDownlw(temp);
			ssDownlw >> tempDownLW;
			DownLongWaveRadiation.push_back(tempDownLW);
			std::getline(infile, temp, ',');
			std::istringstream ssUplw(temp);
			ssUplw >> tempUpLW;
			UpLongWaveRadiation.push_back(tempUpLW);
			std::getline(infile, temp, ',');
			std::istringstream latitude(temp);
			latitude >> tempLatitude;
			Latitude.push_back(tempLatitude);
			std::getline(infile, temp, ',');
			std::istringstream hourangle(temp);
			hourangle >> ntemphourAngle;
			HourAngle.push_back(ntemphourAngle);
			std::getline(infile, temp, '\n');
			std::istringstream tempdecline(temp);
			tempdecline >> ntempDecline;
			Decline.push_back(ntempDecline);
		}
		else
		{
			std::getline(infile, temp, '\n');
		}
	}

	infile.close();
	solarRadiationCalZenAzim();
	return true;
}
//---------------------------------------<calculate Solar Zenith and Azimuth Params>-------------------------------------
void Inputs::solarRadiationCalZenAzim()
{
	double tempZenith = 0.0;
	double tempCosAzimuth = 0.0;
	double tempSinAzimuth = 0.0;

	for (int i = 0; i < HourAngle.size(); ++i)
	{

		tempZenith = acos(sin(Decline[i]) * sin(Latitude[i]) + cos(Decline[i]) * cos(Latitude[i]) * cos(HourAngle[i]));
		tempCosAzimuth = (sin(Decline[i]) * cos(Latitude[i]) - cos(Decline[i]) * sin(Latitude[i]) * cos(HourAngle[i])) / sin(tempZenith);
		if (HourAngle[i] < 0) // in the morning
			tempSinAzimuth = sqrt(1 - tempCosAzimuth * tempCosAzimuth);
		if (HourAngle[i] > 0) //in the afternoon
			tempSinAzimuth = -sqrt(1 - tempCosAzimuth * tempCosAzimuth);
		Zenith.push_back(tempZenith);
		CosAzimuth.push_back(tempCosAzimuth);
		SinAzimuth.push_back(tempSinAzimuth);
	}
}
//---------------------------------------<calculate LAI series>-----------------------------------------------------------------
void Inputs::CalculateLAISeries()
{
	if (SimulationStringParams["Model"] == "CoolRiver")
		return;
	LaiVariables["dayStartTransLOn"] = RefParams["LeafOnDay"] - RefParams["LeafTransDays"] * 0.5;
	LaiVariables["dayEndTransLOn"] = RefParams["LeafOnDay"] + RefParams["LeafTransDays"] * 0.5;
	LaiVariables["dayStartTransLOff"] = RefParams["LeafOffDay"] - RefParams["LeafTransDays"] * 0.5;
	LaiVariables["dayEndTransLOff"] = RefParams["LeafOffDay"] + RefParams["LeafTransDays"] * 0.5;
	LaiVariables["treeminLAI"] = RefParams["TreeLAI"] * RefParams["EvergreenTreeCover_frac"] + RefParams["TreeBAI"];
	LaiVariables["treemaxLAI"] = RefParams["TreeLAI"] + RefParams["TreeBAI"];
	LaiVariables["shortVegminLAI"] = RefParams["ShortVegLAI"] * RefParams["EverGreenShrubCover_frac"] + RefParams["ShrubBAI"];
	LaiVariables["shortVegmaxLAI"] = RefParams["ShortVegLAI"] + RefParams["ShrubBAI"];
	int date = static_cast<int>(SimulationNumericalParams["StartDay"]);
	int i = 1;
	int size = SimulationNumericalParams["TotalTimeSteps"];
	int julianDay = ToJulianDate(date);
	while (date <= static_cast<int>(SimulationNumericalParams["EndDay"]))
	{
		CalcTreeLAI(julianDay);
		CalcShortVegLAI(julianDay);
		if (i >= size)
			break;
		date = CalendarDates[i];
		julianDay = ToJulianDate(date);
		++i;
	}
}

void Inputs::CalcTreeLAI(int JulianDay)
{
	double tempTreeLAI = 0;
	double x = 0;
	if (JulianDay <= LaiVariables["dayStartTransLOn"])
		tempTreeLAI = LaiVariables["treeminLAI"];
	else if (JulianDay > LaiVariables["dayStartTransLOn"] && JulianDay < LaiVariables["dayEndTransLOn"])
	{
		x = (JulianDay - RefParams["LeafOnDay"]) * 0.37;
		tempTreeLAI = 1 / (1 + exp(-x)) * (LaiVariables["treemaxLAI"] - LaiVariables["treeminLAI"]) + LaiVariables["treeminLAI"];
	}
	else if (JulianDay >= LaiVariables["dayEndTransLOn"] && JulianDay <= LaiVariables["dayStartTransLOff"])
	{
		tempTreeLAI = LaiVariables["treemaxLAI"];
	}
	else if (JulianDay > LaiVariables["dayStartTransLOff"] && JulianDay < LaiVariables["dayEndTransLOff"])
	{
		x = (RefParams["LeafOffDay"] - JulianDay) * 0.37;
		tempTreeLAI = 1 / (1 + exp(-x)) * (LaiVariables["treemaxLAI"] - LaiVariables["treeminLAI"]) + LaiVariables["treeminLAI"];
	}
	else if (JulianDay >= LaiVariables["dayEndTransLOff"])
	{
		tempTreeLAI = LaiVariables["treeminLAI"];
	}
	TreeLAI.push_back(tempTreeLAI);
}

void Inputs::CalcShortVegLAI(int JulianDay)
{
	double tempShortVegLAI = 0;
	double x = 0;
	if (JulianDay <= LaiVariables["dayStartTransLOn"])
		tempShortVegLAI = LaiVariables["shortVegminLAI"];
	else if (JulianDay > LaiVariables["dayStartTransLOn"] && JulianDay < LaiVariables["dayEndTransLOn"])
	{
		x = (JulianDay - RefParams["LeafOnDay"]) * 0.37;
		tempShortVegLAI = 1 / (1 + exp(-x)) * (LaiVariables["shortVegmaxLAI"] - LaiVariables["shortVegminLAI"]) + LaiVariables["shortVegminLAI"];
	}
	else if (JulianDay >= LaiVariables["dayEndTransLOn"] && JulianDay <= LaiVariables["dayStartTransLOff"])
		tempShortVegLAI = LaiVariables["shortVegmaxLAI"];
	else if (JulianDay > LaiVariables["dayStartTransLOff"] && JulianDay < LaiVariables["dayEndTransLOff"])
	{
		x = (RefParams["LeafOffDay"] - JulianDay) * 0.37;
		tempShortVegLAI = 1 / (1 + exp(-x)) * (LaiVariables["shortVegmaxLAI"] - LaiVariables["shortVegminLAI"]) + LaiVariables["shortVegminLAI"];
	}
	else if (JulianDay >= LaiVariables["dayEndTransLOff"])
		tempShortVegLAI = LaiVariables["shortVegminLAI"];
	ShortVegLAI.push_back(tempShortVegLAI);
}

//-------------------------------------------<convert to JulianDate>---------------------------------------------------------------------
int Inputs::ToJulianDate(int date)
{
	//http://aa.usno.navy.mil/faq/docs/JD_Formula.php
	char buffer[30];
	_itoa_s(date, buffer, 10);
	std::string currdate = buffer;
	int K = atoi(currdate.substr(6, 2).c_str()); //K stores current day
	int I = atoi(currdate.substr(0, 4).c_str()); //I stores current year
	int J = atoi(currdate.substr(4, 2).c_str()); //J stores current month
	int julianDate = K - 32075 + 1461 * (I + 4800 + (J - 14) / 12) / 4 + 367 * (J - 2 - (J - 14) / 12 * 12) / 12 - 3 * ((I + 4900 + (J - 14) / 12) / 100) / 4;
	J = 1;
	K = 1;
	int julianDateFirstDayOfYear = K - 32075 + 1461 * (I + 4800 + (J - 14) / 12) / 4 + 367 * (J - 2 - (J - 14) / 12 * 12) / 12 - 3 * ((I + 4900 + (J - 14) / 12) / 100) / 4;
	int dayNumberOfYear = (julianDate - julianDateFirstDayOfYear) + 1;
	return dayNumberOfYear;
}

//read landcover.dat file
bool Inputs::readNLCDLandCoverFile(std::string& directory, double simulationNoDataVal)
{
	std::ifstream readLandCover(directory + "\\LandCover.txt");
	if (!readLandCover)
	{
		LogMessage::log("Could not read" + directory + "\\LandCover.txt");
		return false;
	}
	std::string buffer;
	readLandCover >> buffer >> ncols >> buffer >> nrows >> buffer >> xllcorner >> buffer >> yllcorner >> buffer >> cellsize >> buffer >> NODATA;
	double tempdata = 0.0;
	int count = 0;
	
	while (readLandCover >> tempdata && count < nrows * ncols)
	{
		if (tempdata != NODATA)
		{
			NLCDLandCoverData.push_back(tempdata);
			TotalCover.push_back(0.0);
			SoilCover.push_back(0.0);
			ImperviousNOTree.push_back(0.0);
			ShortVegCover.push_back(0.0);
			ConnectedImpervious.push_back(0.0);
			FracEgreenTree.push_back(0.0);
			FracEgreenShortVeg.push_back(0.0);
			TreeOnPervious.push_back(0.0);
			TreeOnImpervious.push_back(0.0);
		}
		else
		{
			NLCDLandCoverData.push_back(simulationNoDataVal);
			TotalCover.push_back(simulationNoDataVal);
			SoilCover.push_back(simulationNoDataVal);
			ImperviousNOTree.push_back(simulationNoDataVal);
			ShortVegCover.push_back(simulationNoDataVal);
			ConnectedImpervious.push_back(simulationNoDataVal);
			FracEgreenTree.push_back(simulationNoDataVal);
			FracEgreenShortVeg.push_back(simulationNoDataVal);
			TreeOnPervious.push_back(simulationNoDataVal);
			TreeOnImpervious.push_back(simulationNoDataVal);
		}

		//Li 020212020 ECDynamic
		
		if (tempdata == 21) { LC21 += 1;}
		if (tempdata == 22) { LC22 += 1; }
		if (tempdata == 23) { LC23 += 1; }
		if (tempdata == 24) { LC24 += 1; }
		if (tempdata == 31) { LC31 += 1; } 
		if (tempdata == 41) { LC41 += 1; }
		if (tempdata == 42) { LC42 += 1; }
		if (tempdata == 43) { LC43 += 1; }
		if (tempdata == 52) { LC52 += 1; }
		if (tempdata == 71) { LC71 += 1; }
		if (tempdata == 81) { LC81 += 1; }
		if (tempdata == 82) { LC82 += 1; }
		++count;

		//cell number of all watershed area including open water, wetland...
		if (tempdata > 0) { ++cell_count; }
	}
	// LC 31 Barren Land -> LC71 Grassland
	// LC 52 Shrub/Scrub ->LC 41 Deciduous Forest
	Cell_number.push_back(LC21);
	Cell_number.push_back(LC22);
	Cell_number.push_back(LC23);
	Cell_number.push_back(LC24);
	Cell_number.push_back(int(LC41+LC52));
	Cell_number.push_back(LC42);
	Cell_number.push_back(LC43);
	Cell_number.push_back(int(LC71+LC31));
	Cell_number.push_back(LC81);
	Cell_number.push_back(LC82);

	//cout << LC21 << " " << LC22 << " " << LC23 << " " << LC24 << " " << LC41 << " " << LC42 << " " << LC43 << " " << LC71 << " " << LC81 << " " << LC82 << endl;
	//cells number of contribution area
	//cell_count = std::accumulate(Cell_number.begin(), Cell_number.end(), 0);
	
	readLandCover.close();
	return true;
}

//read treecover.dat file
bool Inputs::readTreeCoverFile(std::string& directory, double simulationNoDataVal)
{
	std::ifstream readTreeCover(directory + "\\TreeCover.txt");
	if (!readTreeCover)
	{
		LogMessage::log("Could not read" + directory + "\\TreeCover.txt");
		return false;
	}
	std::string buffer;
	readTreeCover >> buffer >> ncols >> buffer >> nrows >> buffer >> xllcorner >> buffer >> yllcorner >> buffer >> foldersize >> buffer >> NODATA;
	double tempdata = 0.0;
	int count = 0;
	while (readTreeCover >> tempdata && count < nrows * ncols)
	{
		if (tempdata != NODATA)
		{
			TreeCoverdata.push_back(tempdata);
		}
		else
		{
			TreeCoverdata.push_back(simulationNoDataVal); //use single uniform value for NODATA but let input files have diff NODATA vals
		}
		++count;
	}
	readTreeCover.close();
	return true;
}

//read impervious.dat file
//cte 20210520 update routine to compute average impervious cover for catchment area, then transfer to IndexCalculator::calcTW for adjusting TI map and histogram
//bool Inputs::readImperviousFile(std::string& directory, double simulationNoDataVal)
bool Inputs::readImperviousFile(std::string& directory, double simulationNoDataVal, double imperviousArea_mapAverage_frac)
{
	std::ifstream readImperviousFile(directory + "\\ImperviousCover.txt");
	if (!readImperviousFile)
	{
		LogMessage::log("Could not read" + directory + "\\ImperviousCover.txt");
		return false;
	}
	std::string buffer;
	readImperviousFile >> buffer >> ncols >> buffer >> nrows >> buffer >> xllcorner >> buffer >> yllcorner >> buffer >> foldersize >> buffer >> NODATA;

	double tempdata = 0.0;
	int count = 0;
	//cte 20210520 Need to pass impervious area to IndexCalculator and update TI for spatial hydro
	int imperviousArea_mapCells_count = 0;
	double imperviousArea_mapTotal_percent = 0;
	//double imperviousArea_mapAverage_frac = 0;

	while (readImperviousFile >> tempdata && count < ncols * nrows)
	{
		if (tempdata != NODATA)
		{
			ImperCoverdata.push_back(tempdata);
			imperviousArea_mapCells_count = imperviousArea_mapCells_count + 1;
			imperviousArea_mapTotal_percent = imperviousArea_mapTotal_percent + tempdata;
		}
		else
		{
			//use single uniform value (set by the Param.dat, typically is -9999) for NODATA but let input files have diff NODATA vals
			ImperCoverdata.push_back(simulationNoDataVal);
		}
		++count;
	}

	//imperviousArea_mapAverage_frac (frac) used for TOPURBAN routine to compute topographic index for pervious drainage area
	//quotient to compute impervious area percent
	imperviousArea_mapAverage_frac = imperviousArea_mapTotal_percent / imperviousArea_mapCells_count;
	//cte 20210520 Test if imperviousArea_mapAverage_frac can transfer to IndexCalculator::calcTW for adjusting TI map and histogram
	cout << "Inputs.cpp readImperviousFile imperviousArea_mapAverage_frac: " << imperviousArea_mapAverage_frac << endl;

	readImperviousFile.close();
	return true;
}

bool Inputs::readBlockGroupOutputFileMap(std::string& directory)
{
	std::ifstream readBlockGroupOutputFileMap(directory + "\\blockgroupmap.txt");
	if (!readBlockGroupOutputFileMap)
	{
		LogMessage::log("Could not read" + directory + "\\blockgroupmap.txt");
		return false;
	}
	std::string buffer;
	readBlockGroupOutputFileMap >> buffer >> ncols >> buffer >> nrows >> buffer >> xllcorner >> buffer >> yllcorner >> buffer >> foldersize >> buffer >> NODATA;

	std::string tempdata = "";
	int count = 0;

	while (readBlockGroupOutputFileMap >> tempdata && count < ncols * nrows)
	{
		EachfoldersBlockGroup.push_back(atoi(tempdata.c_str()));
	}
	readBlockGroupOutputFileMap.close();
	return true;
}

//Warning - this function's calcs may be replaced by LandCoverPCal::CalLandCover, as tested by R Coville 10/2020. Comment below about LCScalingTempSpatial is older but still seems valid.
//calculate landcover data for Model == SpatialTemperatureHydro and not for LCScalingTempSpatial (which uses BuildLCScalingDataOrganizer and AddFolderAreaAlternativeCoverData)
void Inputs::calcNLCDLandCoverData(int NumberOfLocations, double simulationNoDataVal)
{
	for (int i = 0; i < NumberOfLocations; i++)
	{
		//Check if TC and IC are actual data values
		if (TreeCoverdata[i] != simulationNoDataVal && ImperCoverdata[i] != simulationNoDataVal)
			TotalCover[i] = TreeCoverdata[i] + ImperCoverdata[i];
		
		if (TreeCoverdata[i] != simulationNoDataVal && ImperCoverdata[i] != simulationNoDataVal && NLCDLandCoverData[i] != simulationNoDataVal)
		{
			//FracEgreenTree and FracEgreenShortVeg not currently used by SpatialTemperatureHydro, could be used to set treeminLAI
			//TreeLAI and ShortVegLAI are set in SpatialTemperatureHydro using UnfiedHydroConfig.xml key of EvergreenTreeCover_frac and EverGreenShrubCover_frac
			FracEgreenTree[i] = 0.05;
			FracEgreenShortVeg[i] = 0.10;
			//DCIA fraction is ConnectedImpervious for SpatialTemperatureHydro, set here and not read from HydroPlusConfig.xml
			//DCIA could be set to a unique value for different NLCD types
			ConnectedImpervious[i] = 0.65;

			// If land cover classification = 42 = Evergreen forest, fraction of evergreen in respective covers = 100%
			if (NLCDLandCoverData[i] == 42)
			{
				FracEgreenTree[i] = 1;
				FracEgreenShortVeg[i] = 1;
			}
			// If land cover classification = 43 = Mixed (Deciduous and Evergreen) forest, fraction of evergreen in respective covers = 50%
			if (NLCDLandCoverData[i] == 43)
			{
				FracEgreenTree[i] = 0.50;
			}
			// If impervious cover + pervious cover > 100%
			if (TotalCover[i] > 100)
			{

				SoilCover[i] = 0;
				ShortVegCover[i] = 0;

				// Cover above 100% is assumed to be tree cover above impervious area
				// Ex: 60% TC + 60% IC = 120% - so TreeOnImpervious = 120% -100% = 20%
				TreeOnImpervious[i] = TreeCoverdata[i] + ImperCoverdata[i] - 100;

				// Then TreeOnPervious = TC% - TreeOnImpervious = 60% - 20% = 40%
				TreeOnPervious[i] = TreeCoverdata[i] - TreeOnImpervious[i];
				TreeOnPervious[i] = TreeOnPervious[i] / 100;

				// ImperviousNOTree = IC% - TreeOnImpervious = 60% - 20% = 40%
				ImperviousNOTree[i] = ImperCoverdata[i] - TreeOnImpervious[i];
				ImperviousNOTree[i] = ImperviousNOTree[i] / 100;

				// Adjusting the TreeOnImpervious to be a decimal
				TreeOnImpervious[i] = TreeOnImpervious[i] / 100;
			}
			else
			{
				// If the Open Water (11) or Perennial Ice/Snow (12) - not sure what 10 is - not in either the '01 or '06 NLCD classification
				if (NLCDLandCoverData[i] == 10 || NLCDLandCoverData[i] == 11 || NLCDLandCoverData[i] == 12)
				{

					SoilCover[i] = 0;
					ShortVegCover[i] = 0;
					TreeOnImpervious[i] = 0;

					TreeOnPervious[i] = TreeCoverdata[i];
					TreeOnPervious[i] = TreeOnPervious[i] / 100;

					ImperviousNOTree[i] = ImperCoverdata[i] + 100 - TotalCover[i];
					ImperviousNOTree[i] = ImperviousNOTree[i] / 100;
					continue;
				}
				if (NLCDLandCoverData[i] == 20 || NLCDLandCoverData[i] == 21 || NLCDLandCoverData[i] == 22 || NLCDLandCoverData[i] == 23 || NLCDLandCoverData[i] == 24)
				//Fill Gap with 50% ShortVeg and 50% Soil cover; T. Endreny and R. Coville May 12, 2020 
				{

					SoilCover[i] = 0;
					TreeOnImpervious[i] = 0.;
					TreeOnPervious[i] = TreeCoverdata[i];

					if (ImperCoverdata[i] < TreeOnImpervious[i])
					{
						TreeOnImpervious[i] = ImperCoverdata[i];
						ImperviousNOTree[i] = 0.0;
					}
					else
						ImperviousNOTree[i] = ImperCoverdata[i] - TreeOnImpervious[i];

					ShortVegCover[i] = 100 - SoilCover[i] - TreeOnImpervious[i] - TreeOnPervious[i] - ImperviousNOTree[i];
					SoilCover[i] = ShortVegCover[i] / 2;
					ShortVegCover[i] = ShortVegCover[i] / 2;

					TreeOnImpervious[i] = TreeOnImpervious[i] / 100;
					TreeOnPervious[i] = TreeOnPervious[i] / 100;
					ImperviousNOTree[i] = ImperviousNOTree[i] / 100;
					ShortVegCover[i] = ShortVegCover[i] / 100;
					SoilCover[i] = SoilCover[i] / 100;
					continue;
				}
				if (NLCDLandCoverData[i] == 30 || NLCDLandCoverData[i] == 31 || NLCDLandCoverData[i] == 32)
				{

					SoilCover[i] = 100 - TotalCover[i];
					SoilCover[i] = SoilCover[i] / 100;

					ShortVegCover[i] = 0;
					TreeOnImpervious[i] = 0;

					TreeOnPervious[i] = TreeCoverdata[i];
					TreeOnPervious[i] = TreeOnPervious[i] / 100;

					ImperviousNOTree[i] = ImperCoverdata[i];
					ImperviousNOTree[i] = ImperviousNOTree[i] / 100;
					continue;
				}

				if (NLCDLandCoverData[i] == 40 || NLCDLandCoverData[i] == 41 || NLCDLandCoverData[i] == 42 || NLCDLandCoverData[i] == 43)
				{

					SoilCover[i] = 100 - TotalCover[i];
					SoilCover[i] = SoilCover[i] / 100;

					ShortVegCover[i] = 0;
					TreeOnImpervious[i] = 0;

					TreeOnPervious[i] = TreeCoverdata[i];
					TreeOnPervious[i] = TreeOnPervious[i] / 100;

					ImperviousNOTree[i] = ImperCoverdata[i];
					ImperviousNOTree[i] = ImperviousNOTree[i] / 100;
					continue;
				}

				if (TreeCoverdata[i] != simulationNoDataVal && ImperCoverdata[i] != simulationNoDataVal && NLCDLandCoverData[i] != simulationNoDataVal)
				{
					SoilCover[i] = 0;

					ShortVegCover[i] = 100 - TotalCover[i];
					ShortVegCover[i] = ShortVegCover[i] / 100;

					TreeOnImpervious[i] = 0;
					TreeOnPervious[i] = TreeCoverdata[i];
					TreeOnPervious[i] = TreeOnPervious[i] / 100;

					ImperviousNOTree[i] = ImperCoverdata[i];
					ImperviousNOTree[i] = ImperviousNOTree[i] / 100;
				}
			}
		}
	}
}

bool Inputs::readRiverTimeSeriesFile(double startDate, double endDate)
{
	std::fstream readRiverTimeSeries(directory + "\\RiverTimeSeries.txt");
	if (!readRiverTimeSeries)
	{
		std::cout << "RiverTimeSeries.txt could not be opened!" << std::endl;
		LogMessage::log("RiverTimeSeries.txt could not be opened!");
		return false;
	}
	std::string temp;
	double date = -1.0;
	std::string HrMinSec;
	double tempSedi = -99999;
	double tempHumid = -99999;
	double tempCloudy = -99999;

	std::string startline;
	std::getline(readRiverTimeSeries, temp);

	while (readRiverTimeSeries.good())
	{
		std::getline(readRiverTimeSeries, temp, ',');
		std::istringstream ss(temp);
		ss >> date;
		if (date <= endDate && date >= startDate)
		{
			if (readRiverTimeSeries.eof())
				break;
			DatesRiverTime.push_back(static_cast<int>(date));
			std::getline(readRiverTimeSeries, temp, ',');
			std::istringstream ssTime(temp);
			ssTime >> HrMinSec;
			RiverTimeSeries.push_back(HrMinSec);

			std::getline(readRiverTimeSeries, temp, ',');
			std::istringstream ssSedimentTC(temp);
			ssSedimentTC >> tempSedi;
			Sediment_T_C.push_back(tempSedi);

			std::getline(readRiverTimeSeries, temp, ',');
			std::istringstream ssCloudinessF(temp);
			ssCloudinessF >> tempCloudy;
			Cloudiness_F.push_back(tempCloudy);

			std::getline(readRiverTimeSeries, temp, '\n');
			std::istringstream ssHumidityP(temp);
			ssHumidityP >> tempHumid;
			Humidity_Per.push_back(tempHumid);
		}
		else
		{
			std::getline(readRiverTimeSeries, temp);
		}
	}
	readRiverTimeSeries.close();
	return true;
}

bool Inputs::readRiverStationSeriesFile(std::string& directory, double simulationNoDataVal)
{
	std::fstream readRiverStationSeries(directory + "\\RiverStationSeries.txt");
	if (!readRiverStationSeries)
	{
		std::cout << "RiverStationSeries.txt could not be opened!" << std::endl;
		LogMessage::log("RiverStationSeries.txt could not be opened!");
		return false;
	}
	std::string temp;
	std::string tempType;
	double stationNumber = -1.0;
	double stationDistance = -1.0;
	double tempDepthOfMesu = -99999;
	double tempHorizontalBedCon = -99999;
	double tempBedParticle = -99999;
	double tempEmbeddedness = -99999;
	double tempArea = -99999;
	double tempWidth = -99999;
	double tempDepth = -99999;
	double tempSlope = -99999;
	double tempRow = -99999;
	double tempColumn = -99999;
	double tempLongitude = -99999;
	double tempLatitude = -99999;
	double tempssZRiver = -99999;

	double tempEastBankH = -99999;
	double tempEastTreeH = -99999;
	double tempEastBuildingH = -99999;
	double tempEastBankDist = -99999;
	double tempEastCanDist = -99999;
	double tempEastBuildingDist = -99999;
	double tempEastBuffer = -99999;
	double tempWestBankH = -99999;
	double tempWestTreeH = -99999;
	double tempWestBuildingH = -99999;
	double tempWestBankDist = -99999;
	double tempWestCanDist = -99999;
	double tempWestBuildingDist = -99999;
	double tempWestBuffer = -99999;
	double tempElevation = -99999;
	double tempStreamAzi = -99999;
	double tempShadeFactor = -99999;
	double tempViewToSky = -99999;
	double stationDistanceMorphology = -1.0;
	double stationDistanceShading = -1.0;
	double stationDistanceShadingPercent = -1.0;

	std::string startline;
	std::getline(readRiverStationSeries, temp);

	while (readRiverStationSeries.good())
	{
		if (readRiverStationSeries.eof())
			break;
		std::getline(readRiverStationSeries, temp, ',');
		if (temp == "")
			break;
		std::istringstream ss(temp);
		ss >> stationNumber;
		StationNumber.push_back(static_cast<int>(stationNumber));
		std::getline(readRiverStationSeries, temp, ',');
		std::istringstream ssDistance(temp);
		ssDistance >> stationDistance;
		StationDistanceBedData.push_back(static_cast<int>(stationDistance));

		std::getline(readRiverStationSeries, temp, ',');
		std::istringstream ssDepthOfMes(temp);
		ssDepthOfMes >> tempDepthOfMesu;
		DepthMesurments.push_back(tempDepthOfMesu);
		std::getline(readRiverStationSeries, temp, ',');
		std::istringstream ssType(temp);
		ssType >> tempType;
		TypeOfRiver.push_back(tempType);
		std::getline(readRiverStationSeries, temp, ',');
		std::istringstream ssHorizontalBedCon(temp);
		ssHorizontalBedCon >> tempHorizontalBedCon;
		HorizontalBedConnectivity.push_back(tempHorizontalBedCon);

		std::getline(readRiverStationSeries, temp, ',');
		std::istringstream ssBedParticle(temp);
		ssBedParticle >> tempBedParticle;
		BedParticleSize.push_back(tempBedParticle);

		std::getline(readRiverStationSeries, temp, ',');
		std::istringstream ssEmbeddedness(temp);
		ssEmbeddedness >> tempEmbeddedness;
		Embeddedness.push_back(tempEmbeddedness);

		std::getline(readRiverStationSeries, temp, ',');
		std::istringstream ssstationDistanceMorphology(temp);
		ssstationDistanceMorphology >> stationDistanceMorphology;
		StationDistanceMorphology.push_back(static_cast<int>(stationDistanceMorphology));

		std::getline(readRiverStationSeries, temp, ',');
		std::istringstream ssArea(temp);
		ssArea >> tempArea;
		AreaR_m2.push_back(tempArea);

		std::getline(readRiverStationSeries, temp, ',');
		std::istringstream ssWidth(temp);
		ssWidth >> tempWidth;
		WidthR_m.push_back(tempWidth);

		std::getline(readRiverStationSeries, temp, ',');
		std::istringstream ssDepth(temp);
		ssDepth >> tempDepth;
		DepthR_m.push_back(tempDepth);

		std::getline(readRiverStationSeries, temp, ',');
		std::istringstream ssSlope(temp);
		ssSlope >> tempSlope;
		SlopeR_f.push_back(tempSlope);

		std::getline(readRiverStationSeries, temp, ',');
		std::istringstream ssRow(temp);
		ssRow >> tempRow;
		RowRiver.push_back(static_cast<int>(tempRow));

		std::getline(readRiverStationSeries, temp, ',');
		std::istringstream sscolumn(temp);
		sscolumn >> tempColumn;
		ColRiver.push_back(static_cast<int>(tempColumn));

		std::getline(readRiverStationSeries, temp, ',');
		std::istringstream ssLongitude(temp);
		ssLongitude >> tempLongitude;
		LongitudeR_frac.push_back(tempLongitude);

		std::getline(readRiverStationSeries, temp, ',');
		std::istringstream ssLatitude(temp);
		ssLatitude >> tempLatitude;
		LatitudeR_frac.push_back(tempLatitude);

		std::getline(readRiverStationSeries, temp, ',');
		std::istringstream ssZRiver(temp);
		ssZRiver >> tempssZRiver;
		ZRiver_m.push_back(tempssZRiver);

		std::getline(readRiverStationSeries, temp, ',');
		std::istringstream ssstationDistanceShading(temp);
		ssstationDistanceShading >> stationDistanceShading;
		StationDistanceShading.push_back(static_cast<int>(stationDistanceShading));

		std::getline(readRiverStationSeries, temp, ',');
		std::istringstream ssEastBankH(temp);
		ssEastBankH >> tempEastBankH;
		EastBankHR_m.push_back(tempEastBankH);
		std::getline(readRiverStationSeries, temp, ',');
		std::istringstream ssEastTreeH(temp);
		ssEastTreeH >> tempEastTreeH;
		EastTreeHR_m.push_back(tempEastTreeH);
		std::getline(readRiverStationSeries, temp, ',');
		std::istringstream ssEastBuildingH(temp);
		ssEastBuildingH >> tempEastBuildingH;
		EastBuildingHR_m.push_back(tempEastBuildingH);
		std::getline(readRiverStationSeries, temp, ',');
		std::istringstream ssEastBankDist(temp);
		ssEastBankDist >> tempEastBankDist;
		EastBankDistR_m.push_back(tempEastBankDist);
		std::getline(readRiverStationSeries, temp, ',');
		std::istringstream ssEastCanDist(temp);
		ssEastCanDist >> tempEastCanDist;
		EastCanDistR_m.push_back(tempEastCanDist);
		std::getline(readRiverStationSeries, temp, ',');
		std::istringstream ssEastBuildingDist(temp);
		ssEastBuildingDist >> tempEastBuildingDist;
		EastBuildingDistR_m.push_back(tempEastBuildingDist);
		std::getline(readRiverStationSeries, temp, ',');
		std::istringstream ssEastBuffer(temp);
		ssEastBuffer >> tempEastBuffer;
		EastBufferWR_m.push_back(tempEastBuffer);

		std::getline(readRiverStationSeries, temp, ',');
		std::istringstream ssWestBankH(temp);
		ssWestBankH >> tempWestBankH;
		WestBankHR_m.push_back(tempWestBankH);
		std::getline(readRiverStationSeries, temp, ',');
		std::istringstream ssWestTreeH(temp);
		ssWestTreeH >> tempWestTreeH;
		WestTreeHR_m.push_back(tempWestTreeH);
		std::getline(readRiverStationSeries, temp, ',');
		std::istringstream ssWestBuildingH(temp);
		ssWestBuildingH >> tempWestBuildingH;
		WestBuildingHR_m.push_back(tempWestBuildingH);
		std::getline(readRiverStationSeries, temp, ',');
		std::istringstream ssWestBankDist(temp);
		ssWestBankDist >> tempWestBankDist;
		WestBankDistR_m.push_back(tempWestBankDist);
		std::getline(readRiverStationSeries, temp, ',');
		std::istringstream ssWestCanDist(temp);
		ssWestCanDist >> tempWestCanDist;
		WestCanDistR_m.push_back(tempWestCanDist);
		std::getline(readRiverStationSeries, temp, ',');
		std::istringstream ssWestBuildingDist(temp);
		ssWestBuildingDist >> tempWestBuildingDist;
		WestBuildingDistR_m.push_back(tempWestBuildingDist);
		std::getline(readRiverStationSeries, temp, ',');
		std::istringstream ssWestBuffer(temp);
		ssWestBuffer >> tempWestBuffer;
		WestBufferWR_m.push_back(tempWestBuffer);
		std::getline(readRiverStationSeries, temp, ',');
		std::istringstream ssElevation(temp);
		ssElevation >> tempElevation;
		ElevationR_m.push_back(tempElevation);
		std::getline(readRiverStationSeries, temp, ',');
		std::istringstream ssStreamAzi(temp);
		ssStreamAzi >> tempStreamAzi;
		StreamAzimuthR_deg.push_back(tempStreamAzi);

		std::getline(readRiverStationSeries, temp, ',');
		std::istringstream ssstationDistanceShadingPercent(temp);
		ssstationDistanceShadingPercent >> stationDistanceShadingPercent;
		StationDistanceShadingPercent.push_back(static_cast<int>(stationDistanceShadingPercent));

		std::getline(readRiverStationSeries, temp, ',');
		std::istringstream ssShadeFactor(temp);
		ssShadeFactor >> tempShadeFactor;
		ShadeFactorR_f.push_back(tempShadeFactor);
		std::getline(readRiverStationSeries, temp, '\n');
		std::istringstream ssViewToSky(temp);
		ssViewToSky >> tempViewToSky;
		ViewToSkyR_f.push_back(tempViewToSky);
		if (readRiverStationSeries.eof())
			break;
	}

	readRiverStationSeries.close();
	return true;
}

bool Inputs::readRiverGWTimeSeriesFile(double startDate, double endDate)
{
	std::fstream readRiverGWTimeSeries(directory + "\\RiverGWTimeSeries.txt");
	if (!readRiverGWTimeSeries)
	{
		std::cout << "RiverGWTimeSeries.txt could not be opened!" << std::endl;
		LogMessage::log("RiverGWTimeSeries.txt could not be opened!");
		return false;
	}
	std::string temp;
	double date = -1.0;
	std::string HrMinSec;
	double tempGWQcms = -99999;
	double tempTCcms = -99999;
	int indexValueForGW = -99999;
	int indexValueForTC = -99999;

	std::string delimiter = ",";
	vector<double> headerValues;
	std::getline(readRiverGWTimeSeries, temp);
	int size = temp.size();
	size_t pos = 0;
	int counter = 0;
	std::string token;
	bool flag = true;
	int maxNumberOfVectors = 10;
	while ((pos = temp.find(delimiter)) != std::string::npos)
	{
		token = temp.substr(0, pos);
		int pos1 = token.find("#");
		if (pos1 != -1)
		{
			std::string str3 = token.substr(pos1 + 1);
			int val = stoi(str3);
			headerValues.push_back(val);
			counter++;
		}
		temp.erase(0, pos + delimiter.length());
	}
	int pos1 = token.find("#"); // position of "live" in str
	if (pos1 != -1)
	{
		std::string str3 = token.substr(pos1 + 1);
		int val = stoi(str3);
		headerValues.push_back(val);
		counter++;
	}
	while (readRiverGWTimeSeries.good())
	{
		std::getline(readRiverGWTimeSeries, temp, ',');
		std::istringstream ss(temp);
		ss >> date;
		if (date <= endDate && date >= startDate)
		{
			tempGWQcms = -99999;
			tempTCcms = -99999;
			if (readRiverGWTimeSeries.eof())
				break;
			std::getline(readRiverGWTimeSeries, temp, ',');
			for (int i = 0; i < counter;)
			{
				std::getline(readRiverGWTimeSeries, temp, ',');
				std::istringstream ssGWQcms(temp);
				ssGWQcms >> tempGWQcms;
				indexValueForGW = headerValues.at(i);
				map<int, std::vector<double>>::iterator it = GW_Q_cms1.find(indexValueForGW);
				vector<double> tempVector;
				if (it == GW_Q_cms1.end())
				{
					tempVector.push_back(tempGWQcms);
					GW_Q_cms1.insert(pair<int, std::vector<double>>(indexValueForGW, tempVector));
				}
				else
				{
					GW_Q_cms1[indexValueForGW].push_back(tempGWQcms);
				}
				if (i != counter - 2)
				{
					std::getline(readRiverGWTimeSeries, temp, ',');
				}
				else
				{
					std::getline(readRiverGWTimeSeries, temp, '\n');
				}
				i = i + 1;
				std::istringstream ssTCcms(temp);
				ssTCcms >> tempTCcms;
				indexValueForTC = headerValues.at(i);

				map<int, std::vector<double>>::iterator it1 = GW_T_C1.find(indexValueForTC);
				vector<double> tempVector1;
				if (it1 == GW_T_C1.end())
				{
					tempVector1.push_back(tempTCcms);
					GW_T_C1.insert(pair<int, std::vector<double>>(indexValueForTC, tempVector1));
				}
				else
				{
					GW_T_C1[indexValueForTC].push_back(tempTCcms);
				}
				i = i + 1;
			}
		}
		else
		{
			std::getline(readRiverGWTimeSeries, temp);
		}
	}
	readRiverGWTimeSeries.close();
	return true;
}

bool Inputs::readRiverSWTimeSeriesFile(double startDate, double endDate)
{
	std::fstream readRiverSWTimeSeries(directory + "\\RiverSWTimeSeries.txt");
	if (!readRiverSWTimeSeries)
	{
		std::cout << "RiverSWTimeSeries.txt could not be opened!" << std::endl;
		LogMessage::log("RiverSWTimeSeries.txt could not be opened!");
		return false;
	}
	std::string temp;
	double date = -1.0;
	std::string HrMinSec;
	double tempGWQcms = -99999;
	double tempTCcms = -99999;
	int indexValueForGW = -99999;
	int indexValueForTC = -99999;

	std::string delimiter = ",";
	vector<double> headerValues;
	std::getline(readRiverSWTimeSeries, temp);
	size_t pos = 0;
	int counter = 0;
	std::string token;
	int maxNumberOfVectors = 10;
	while ((pos = temp.find(delimiter)) != std::string::npos)
	{
		token = temp.substr(0, pos);
		int pos1 = token.find("#");
		if (pos1 != -1)
		{
			std::string str3 = token.substr(pos1 + 1);
			int val = stoi(str3);
			if (val > maxNumberOfVectors)
			{
				maxNumberOfVectors = val;
			}
			headerValues.push_back(val);
			counter++;
		}
		temp.erase(0, pos + delimiter.length());
	}
	int pos1 = token.find("#"); // position of "live" in str
	if (pos1 != -1)
	{
		std::string str3 = token.substr(pos1 + 1);
		int val = stoi(str3);
		if (val > maxNumberOfVectors)
		{
			maxNumberOfVectors = val;
		}
		headerValues.push_back(val);
		counter++;
	}
	while (readRiverSWTimeSeries.good())
	{
		std::getline(readRiverSWTimeSeries, temp, ',');
		std::istringstream ss(temp);
		ss >> date;
		tempGWQcms = -99999;
		tempTCcms = -99999;
		if (readRiverSWTimeSeries.eof())
			break;
		std::getline(readRiverSWTimeSeries, temp, ',');
		for (int i = 0; i < counter;)
		{
			std::getline(readRiverSWTimeSeries, temp, ',');
			std::istringstream ssGWQcms(temp);
			ssGWQcms >> tempGWQcms;
			indexValueForGW = headerValues.at(i);
			map<int, std::vector<double>>::iterator it = SW_Q_cms1.find(indexValueForGW);
			vector<double> tempVector;
			if (it == SW_Q_cms1.end())
			{
				tempVector.push_back(tempGWQcms);
				SW_Q_cms1.insert(pair<int, std::vector<double>>(indexValueForGW, tempVector));
			}
			else
			{
				SW_Q_cms1[indexValueForGW].push_back(tempGWQcms);
			}
			if (i != counter - 2)
			{
				std::getline(readRiverSWTimeSeries, temp, ',');
			}
			else
			{
				std::getline(readRiverSWTimeSeries, temp, '\n');
			}
			i = i + 1;
			std::istringstream ssTCcms(temp);
			ssTCcms >> tempTCcms;
			indexValueForTC = headerValues.at(i);

			map<int, std::vector<double>>::iterator it1 = SW_T_C1.find(indexValueForTC);
			vector<double> tempVector1;
			if (it1 == SW_T_C1.end())
			{
				tempVector1.push_back(tempTCcms);
				SW_T_C1.insert(pair<int, std::vector<double>>(indexValueForTC, tempVector1));
			}
			else
			{
				SW_T_C1[indexValueForTC].push_back(tempTCcms);
			}
			i = i + 1;
		}
	}
	readRiverSWTimeSeries.close();
	return true;
}
bool Inputs::readHecRasFile(double startDate, double endDate)
{
	fstream infile(directory + "\\HecRasData.dat");
	if (!infile.good()) {
		std::ofstream logfile("log.txt", ios::app);
		logfile << "Cannot find HecRasData.dat file needed to run simulation.\n";
		logfile.close();
		std::cout << "Cannot find HecRasData.dat file" << std::endl;
		return false;
	}
	//std::cout << "Reading HecRasData.dat file\n" << std::endl;

	std::string temp;
	double tempDummy = -1.0;

	double temp_dist = -99999;
	double temp_totalQ = -99999;
	double temp_chElev = -99999;
	double temp_waterSurElev = -99999;
	double temp_velChnl = -99999;
	double temp_xArea = -99999;
	double temp_topWidth = -99999;
	double temp_waterSurSlope = -99999;
	double temp_wettedPerimeter = -99999;

	// getting the header line and the blank line out of the way
	getline(infile, temp);

	// reading in a CSV
	while (infile.good())
	{
		getline(infile, temp, ',');
		getline(infile, temp, ',');
		std::istringstream temp1(temp);
		temp1 >> temp_dist;
		downStreamDist.push_back(temp_dist);
		getline(infile, temp, ',');
		std::istringstream temp2(temp);
		temp2 >> temp_totalQ;
		totalQ.push_back(temp_totalQ);
		getline(infile, temp, ',');
		std::istringstream temp3(temp);
		temp3 >> temp_chElev;
		chElev.push_back(temp_chElev);
		getline(infile, temp, ',');
		std::istringstream temp4(temp);
		temp4 >> temp_waterSurElev;
		waterSurElev.push_back(temp_waterSurElev);
		getline(infile, temp, ',');
		std::istringstream temp5(temp);
		temp5 >> temp_velChnl;
		velChnl.push_back(temp_velChnl);
		getline(infile, temp, ',');
		std::istringstream temp6(temp);
		temp6 >> temp_xArea;
		xArea.push_back(temp_xArea);
		getline(infile, temp, ',');
		std::istringstream temp7(temp);
		temp7 >> temp_topWidth;
		topWidth.push_back(temp_topWidth);
		getline(infile, temp, ',');
		std::istringstream temp8(temp);
		temp8 >> temp_waterSurSlope;
		waterSurSlope.push_back(temp_waterSurSlope);
		getline(infile, temp, '\n');
		std::istringstream temp9(temp);
		temp9 >> temp_wettedPerimeter;
		wettedPerimeter.push_back(temp_wettedPerimeter);
	}
	totalCount = downStreamDist.size();
	infile.close();
}

//02012020 ECDynamic Read White's data, in Metric Ton/ha
bool Inputs::readAOIECDatabase(std::string& directory)
{
	std::ifstream readECdb(directory + "\\ECdb5_95.txt");
	if (!readECdb)
	{
		LogMessage::log("Could not read" + directory + "\\ECdb5_95.txt");
		return false;
	}
	std::string header;
	//std::vector<double> TP21, TP22, TP23, TP24, TP41, TP42, TP43, TP71, TP81, TP82, TN21, TN22, TN23, TN24, TN41, TN42, TN43, TN71, TN81, TN82, TSS21, TSS22, TSS23, TSS24, TSS41, TSS42, TSS43, TSS71, TSS81, TSS82;
	std::string ECLable;
	std::vector<double> tempECVec;
	double ECValue0,ECValue1,ECValue2,ECValue3 ,ECValue4,ECValue5 ,ECValue6;
	double percentile0, percentile1, percentile2, percentile3, percentile4, percentile5, percentile6;
	//read the fist line of the database: the header
	//getline(readECdb, header);
	readECdb >> header>> percentile0 >> percentile1 >> percentile2 >> percentile3 >> percentile4 >> percentile5 >> percentile6;
	percentileVec.push_back(percentile0 / 100);
	percentileVec.push_back(percentile1 / 100);
	percentileVec.push_back(percentile2 / 100);
	percentileVec.push_back(percentile3 / 100);
	percentileVec.push_back(percentile4 / 100);
	percentileVec.push_back(percentile5 / 100);
	percentileVec.push_back(percentile6 / 100);
	//readECdb >> header >> header >> header >> header >> header >> header >> header >> header;
	//If the next data exist, read it in as the ECLable, then continue the looping
	while (readECdb >> ECLable)
	{ 
		int j = 0;
		//read each row: 1 lable & 7 values, then saving the EC values into a temp vector:ECValue
		readECdb >> ECValue0 >> ECValue1 >> ECValue2 >> ECValue3 >> ECValue4 >> ECValue5 >> ECValue6;
		tempECVec.push_back(ECValue0);
		tempECVec.push_back(ECValue1);
		tempECVec.push_back(ECValue2);
		tempECVec.push_back(ECValue3);
		tempECVec.push_back(ECValue4);
		tempECVec.push_back(ECValue5);
		tempECVec.push_back(ECValue6);
		//The assignments were devided into 30 (3 pollutants *10 LC) conditions, so the order of LC in ECdb.dat will not cause the error
		//Push back 7 times for each vector
		if (ECLable == "TP21") {
			
			while (j < 7) {
				TP21.push_back(tempECVec[j]);
				++j;
			}
			tempECVec.clear();
		}
		if (ECLable == "TN21") {

			while (j < 7) {
				TN21.push_back(tempECVec[j]);
				++j;
			}
			tempECVec.clear();
		}
		if (ECLable == "TSS21") {

			while (j < 7) {
				TSS21.push_back(tempECVec[j]/1000);//raw data for Tss is in ton/ha,convert to kg/ha
				++j;
			}
			tempECVec.clear();
		}
		if (ECLable == "TP22") {
			while (j < 7) {
				TP22.push_back(tempECVec[j]);
				++j;
			}
			tempECVec.clear();
		}
		if (ECLable == "TN22") {
			while (j < 7) {
				TN22.push_back(tempECVec[j]);
				++j;
			}
			tempECVec.clear();
		}
		if (ECLable == "TSS22") {
			while (j < 7) {
				TSS22.push_back(tempECVec[j]/1000); //raw data for Tss is in ton/ha,convert to kg/ha
				++j;
			}
			tempECVec.clear();
		}
		if (ECLable == "TP23") {
			while (j < 7) {
				TP23.push_back(tempECVec[j]);
				++j;
			}
			tempECVec.clear();
		}
		if (ECLable == "TN23") {
			while (j < 7) {
				TN23.push_back(tempECVec[j]);
				++j;
			}
			tempECVec.clear();
		}
		if (ECLable == "TSS23") {
			while (j < 7) {
				TSS23.push_back(tempECVec[j]/1000);//raw data for Tss is in ton/ha,convert to kg/ha
				++j;
			}
			tempECVec.clear();
		}
		if (ECLable == "TP24") {
			while (j < 7) {
				TP24.push_back(tempECVec[j]);
				++j;
			}
			tempECVec.clear();
		}
		if (ECLable == "TN24") {
			while (j < 7) {
				TN24.push_back(tempECVec[j]);
				++j;
			}
			tempECVec.clear();
		}
		if (ECLable == "TSS24") {
			while (j < 7) {
				TSS24.push_back(tempECVec[j]/1000);//raw data for Tss is in ton/ha,convert to kg/ha
				++j;
			}
			tempECVec.clear();
		}
		if (ECLable == "TP71") {
			while (j < 7) {
				TP71.push_back(tempECVec[j]);
				++j;
			}
			tempECVec.clear();
		}
		if (ECLable == "TN71") {
			while (j < 7) {
				TN71.push_back(tempECVec[j]);
				++j;
			}
			tempECVec.clear();
		}
		if (ECLable == "TSS71") {
			while (j < 7) {
				TSS71.push_back(tempECVec[j]/1000);//raw data for Tss is in ton/ha,convert to kg/ha
				++j;
			}
			tempECVec.clear();
		}
		if (ECLable == "TP81") {
			while (j < 7) {
				TP81.push_back(tempECVec[j]);
				++j;
			}
			tempECVec.clear();
		}
		if (ECLable == "TN81") {
			while (j < 7) {
				TN81.push_back(tempECVec[j]);
				++j;
			}
			tempECVec.clear();
		}
		if (ECLable == "TSS81") {
			while (j < 7) {
				TSS81.push_back(tempECVec[j]/1000);//raw data for Tss is in ton/ha,convert to kg/ha
				++j;
			}
			tempECVec.clear();
		}
		if (ECLable == "TP82") {
			while (j < 7) {
				TP82.push_back(tempECVec[j]);
				++j;
			}
			tempECVec.clear();
		}
		if (ECLable == "TN82") {
			while (j < 7) {
				TN82.push_back(tempECVec[j]);
				++j;
			}
			tempECVec.clear();
		}
		if (ECLable == "TSS82") {
			while (j < 7) {
				TSS82.push_back(tempECVec[j]/1000);//raw data for Tss is in ton/ha,convert to kg/ha
				++j;
			}
			tempECVec.clear();
		}
		if (ECLable == "TP41") {
			while (j < 7) {
				TP41.push_back(tempECVec[j]);
				++j;
			}
			tempECVec.clear();
		}
		if (ECLable == "TN41") {
			while (j < 7) {
				TN41.push_back(tempECVec[j]);
				++j;
			}
			tempECVec.clear();
		}
		if (ECLable == "TSS41") {
			while (j < 7) {
				TSS41.push_back(tempECVec[j]/1000);//raw data for Tss is in ton/ha,convert to kg/ha
				++j;
			}
			tempECVec.clear();
		}
		if (ECLable == "TP42") {
			while (j < 7) {
				TP42.push_back(tempECVec[j]);
				++j;
			}
			tempECVec.clear();
		}
		if (ECLable == "TN42") {
			while (j < 7) {
				TN42.push_back(tempECVec[j]);
				++j;
			}
			tempECVec.clear();
		}
		if (ECLable == "TSS42") {
			while (j < 7) {
				TSS42.push_back(tempECVec[j]/1000);//raw data for Tss is in ton/ha,convert to kg/ha
				++j;
			}
			tempECVec.clear();
		}
		if (ECLable == "TP43") {
			while (j < 7) {
				TP43.push_back(tempECVec[j]);
				++j;
			}
			tempECVec.clear();
		}
		if (ECLable == "TN43") {
			while (j < 7) {
				TN43.push_back(tempECVec[j]);
				++j;
			}
			tempECVec.clear();
		}
		if (ECLable == "TSS43") {
			while (j < 7) {
				TSS43.push_back(tempECVec[j]/1000);//raw data for Tss is in ton/ha,convert to kg/ha
				++j;
			}
			tempECVec.clear();
		}
	}
	//The EC database do not have LC71 Grassland data in some HUCs, so use LC71 <- LC81 (Pasture/Hay)
	if (TP71.size() == 0) {TP71 = TP81;}
	if (TN71.size() == 0) {TN71 = TN81;}
	if (TSS71.size() == 0) {TSS71 = TSS81;}
		
	readECdb.close();
	return true;
}

/*void Inputs::Print(const std::vector<double>& arrays)
{
	for (auto it = arrays.begin(); it != arrays.end(); ++it)
	{
		std::cout << *it << ' '; 
	}
	std::cout << endl;
}*/

//02012020 ECDynamic Read precipitation/discharge data
bool Inputs::readDischargeData(std::string& directory)
{
	std::ifstream readDischarge(directory + "\\DailyDischarge.txt");
	if (!readDischarge)
	{
		LogMessage::log("Could not read" + directory + "\\DailyDischarge.txt");
		return false;
	}

	std::string header, info, HUC;
	double tempDischarge, tempmonth, tempday, tempyear;

	//read the fist line of the database: the header
	getline(readDischarge, header);
	//Continue reading while data exist
	while (readDischarge>> info) {
		readDischarge >> HUC >> tempmonth >> tempday >> tempyear >> tempDischarge;
		//Remove the NA data -9999, and set up the Discharge threshold, for this case, we consider all events, th=0.
		if (tempDischarge != -9999){
			DischargeRecord.push_back(tempDischarge);
			year.push_back(tempyear);
			month.push_back(tempmonth);
			day.push_back(tempday);
		}
	}
	/*cout << "size:" << DischargeRecord.size() << " ,1st data: "<< DischargeRecord.front() <<",last data:" << DischargeRecord.back() << endl;
	cout << "size:" << year.size() << " ,1st data: " << year.front() << ",last data:" << year.back() << endl;
	cout << "size:" << month.size() << " ,1st data: " << month.front() << ",last data:" << month.back() << endl;
	cout << "size:" << day.size() << " ,1st data: " << day.front() << ",last data:" << day.back() << endl;*/
}

//Li 03112020 For GI water quality, modified from Pollutant.cpp
bool Inputs::readPollutionData(std::string& directory)
{

	std::string headerline;
	std::string Medline;
	std::string Meanline;
	std::string element;
	double value;
	std::string type;

	std::ifstream pollutant(directory + "Pollutiondata.dat");
	if (!pollutant.good())
	{
		PollutionDataFileExists = false;
		return false;
	}
	else
	{
		PollutionDataFileExists = true;
	}
	//read the 1st line, get the elements from the 1st line
	getline(pollutant, headerline);
	istringstream iss(headerline);
	//within the 1st line, read in the 1st element as a string info, then read the remaining values into headerNames
	iss >> type;
	while (iss >> element) {
		headerNames.push_back(element);
	}
	
	//to get the 2nd line, the med concentration
	getline(pollutant, Medline);
	istringstream med(Medline);
	med >> type;
	while (med >> value)
	{
		headermedValues.push_back(value);
	}

	//to get the 3rd line, the mean concentration
	getline(pollutant, Meanline);
	istringstream mean(Meanline);
	mean >> type;
	while (mean >> value)
	{
		headermeanValues.push_back(value);
	}
	pollutant.close();
}