#include "ShortVegInterceptionCalc.h"

//cte 20210505 Determine logic of intercepting snow on short vegetation, and melting it beneath short vegetation, given this may just be short grass. 
//	If short vegetation is actually short grass, it is not clear that this logic is same as for trees; even for shrubs, there is limited beneath veg snow storage

void ShortVegInterceptionCalc::calculate(Inputs *input, DataFolder *folder, int timeStep)
{
	//shortVegThruFlowW (m) initialized to 0 at start of each time step;
	folder->VarDict["shortVegThruFlowW"] = 0.0;
	//shortVegThruFlowSWE (m) initialized to 0 at start of each time step;
	folder->VarDict["shortVegThruFlowSWE"] = 0.0;
	//shortVegIntW (m) initialized to 0 at start of each time step;
	folder->VarDict["shortVegIntW"] = 0.0;
	//shortVegIntSWE (m) initialized to 0 at start of each time step;
	folder->VarDict["shortVegIntSWE"] = 0.0;
	//snowMeltSVegCanopy_m (m) initialized to 0 at start of each time step;
	folder->VarDict["snowMeltSVegCanopy_m"] = 0.0;

	// Decides whether to treat the incoming precipitation as rain or snow
	if (input->Tair[timeStep] > 32.0) {
		// heads to the treeInterceptionRain function defined below
		shortVegInterceptionRain(input, folder, timeStep);
	}
	else {
		// heads to the treeInterceptionRain function defined below
		shortVegInterceptionSnow(input, folder, timeStep);
	}

	//treeUnloadSnow function provides snow unloaded by Wind
	shortVegUnloadSnow(input, folder, timeStep);
	
	//Only allow for snowmelt by radiation if snow storage on shortvegetation 
	if (folder->VarDict["shortVegStoredSWE"] > 0.0) {
		// Snow melted by Radiation
		shortVegSnowMeltbyRn(input, folder, timeStep);
	}

	// New function added to keep updating/storage in one place
	storeValues(input, folder, timeStep);
}


void ShortVegInterceptionCalc::storeValues(Inputs *input, DataFolder *folder, int timeStep)
{

	// Updating the combined throughfall to include the water throughfall and snow-water equivalent throughfall
	folder->VarDict["shortVegThroughQ"] = folder->VarDict["shortVegThruFlowW"] + folder->VarDict["shortVegThruFlowSWE"];

	// Updating total variables
	if (input->SimulationStringParams["Model"]=="StatisticalHydro")
	{
		// Updating vector variables
		folder->VarVecDict["shortVegIntW"].push_back(folder->VarDict["shortVegIntW"]);
		folder->VarVecDict["shortVegIntSWE"].push_back(folder->VarDict["shortVegIntSWE"]);
		folder->VarVecDict["shortVegThroughW"].push_back(folder->VarDict["shortVegThruFlowW"]);
		folder->VarVecDict["shortVegThroughSWE"].push_back(folder->VarDict["shortVegThruFlowSWE"]);
	}
}


// History: introduced by Yang Yang for use in iTree V5, adapted from Roesch et al. (2001)
// Reference: iTree Hydro: Snow Hydrology Update for the Impervious Forest Hydrology Model
// Authors: Yang Yang, Ted Endreny, Dave Nowak
//cte 20210517 Not clear if full percentage of short veg (such as grass) should be able to hold snow above ground
void ShortVegInterceptionCalc::shortVegUnloadSnow(Inputs *input, DataFolder *folder, int timeStep)
{
	double shortVegUnloadSWE = 0.0231 * input->WindSp_mps[timeStep] * folder->VarDict["shortVegStoredSWE"];
	
	if (shortVegUnloadSWE > folder->VarDict["shortVegStoredSWE"])
	{
		shortVegUnloadSWE = folder->VarDict["shortVegStoredSWE"];
	}

	folder->VarDict["shortVegStoredSWE"] = folder->VarDict["shortVegStoredSWE"] - shortVegUnloadSWE;
	folder->VarDict["shortVegThruFlowSWE"] = folder->VarDict["shortVegThruFlowSWE"] + shortVegUnloadSWE;

}

// treeSnowMeltbyRn function 
// History: introduced by Yang Yang for use in iTree V5, adapted from Roesch et al. (2001)
// Reference: iTree Hydro: Snow Hydrology Update for the Impervious Forest Hydrology Model
// Authors: Yang Yang, Ted Endreny, Dave Nowak
void ShortVegInterceptionCalc::shortVegSnowMeltbyRn(Inputs *input, DataFolder *folder, int timeStep)
{
	//cte 20210517 check the coefficients on these equations
	// Mn = SWE melted by shortwave radiation - depends on cloudiness of sky and albedo (0.85)
	double Mn = 0.000011 * input->RadiationNet_Wpm2[timeStep] * (1 - 0.85);     // assume the snow surface albedo is 0.85
	// Ml = SWE melted by longwave radiation - if it is negative, it is set equal to 0
	double Ml = 0.0212 * 0.0010583 * (input->Tair[timeStep] - 32.0) - 0.84 * 0.0010583;
	if (Ml < 0)
		Ml=0;
	double RnMeltedSWE = Mn+Ml; // it is snow water equivalent

	//double RnMeltedSnow=WaterDToSnowD(W,RnMeltedSnowW,i);
	if (RnMeltedSWE > folder->VarDict["shortVegStoredSWE"])
	{
		RnMeltedSWE = folder->VarDict["shortVegStoredSWE"];
	}
	
	//snowMeltSVegCanopy_m (m) is snow melted in short veg canopy due to radiation, added to that melted earlier by rain
	folder->VarDict["snowMeltSVegCanopy_m"] = RnMeltedSWE;

	folder->VarDict["shortVegStoredSWE"] = folder->VarDict["shortVegStoredSWE"] - RnMeltedSWE;
	
	folder->VarDict["shortVegStoredW"] = folder->VarDict["shortVegStoredW"] + RnMeltedSWE;

	//canopy water overflows storage
	if (folder->VarDict["shortVegStoredW"] >folder->VarDict["storMaxShortVeg"])  
	{   
		//shortVegThruFlowSWE (m) increased by shortVegStoredSWE (m) due to overflowing water pushing snow off of canopy
		folder->VarDict["shortVegThruFlowSWE"] = folder->VarDict["shortVegThruFlowSWE"] + folder->VarDict["shortVegStoredSWE"];
		folder->VarDict["shortVegStoredSWE"] = 0.0;
		folder->VarDict["shortVegThruFlowW"] = folder->VarDict["shortVegThruFlowW"] + (folder->VarDict["shortVegStoredW"] - folder->VarDict["storMaxShortVeg"]);
		folder->VarDict["shortVegStoredW"] = folder->VarDict["storMaxShortVeg"];
	}
	folder->VarDict["shortVegStoredSWE"] = (folder->VarDict["shortVegStoredSWE"] >= 0.0) ? folder->VarDict["shortVegStoredSWE"] : 0.0;
	folder->VarDict["shortVegStoredW"] = (folder->VarDict["shortVegStoredW"] >= 0.0) ? folder->VarDict["shortVegStoredW"] : 0.0;
}

// treeInterceptionRain function 
// History: introduced by Jun Wang for use in iTree V4, adapted from van Dijk and Bruijnzeel (2001)
// Reference: 1. Wang, Jun, Theodore A. Endreny, and David J. Nowak, 2008.
//                Mechanistic Simulation of Tree Effects in an Impervious Water Balance Modell 
//                Journal of the American Water Resources Association (JAWRA) 44(1): 75-85
//                DOI: 10.1111/j.1752-1688.2007.00139.x
// Authors: Jun Wang, Ted Endreny, Dave Nowak
// Modified by: Yang Yang, Thomas Taggart, Shannon Conley
void ShortVegInterceptionCalc::shortVegInterceptionRain(Inputs *input, DataFolder *folder, int timeStep)
{
	double tempShortVegLAI = input->ShortVegLAI[timeStep];
	if (folder->ParamStringDict["Type"]  !="BulkArea")
	{
		//cte 20210517 address this hard coded 2.2 value, presuming a maximum LAI
		double factor = folder->ParamDict["MaxShortVegLAI"] / 2.2;
		tempShortVegLAI *= factor;
	}

	// The current Storage Max for a tree leaf (SmaxTree_m) is TreeLeafStorage_mm * 0.001 m/mm * LAI for this timestep
	// The storage max is therefore a volume per area measurement (m^3/m^2)
	folder->VarDict["storMaxShortVeg"] = (0.15 * 0.001 * tempShortVegLAI);

	//Modified form of Eq 2a & Eq 2b from Wang, Endreny, Nowak (2008, JAWRA) based on Dijk and Bruijnzeel (2001) for free throughfall
	//Wang et al. (2008) after Eq 2b assign extinction coefficient k = 0.3 for short veg
	// fTreeTQ (the short veg throughfall coefficiant) =  e^(-0.3 * LAI)
	double fShortVegTQ  =  exp(-0.3 *  tempShortVegLAI);

	// Current throughfall  = fTreeTQ * current precip
	folder->VarDict["shortVegThruFlowW"] = fShortVegTQ * input->Precip_mpdt[timeStep];
	// Current interception = (1-fTreeTQ)*current precip
	folder->VarDict["shortVegIntW"] = (1-fShortVegTQ) * input->Precip_mpdt[timeStep];

	//cte 20210517 check: Reza, making a new variable for the rainfall in every time step to use in the GI routine
	folder->VarDict["rainfall"] = input->Precip_mpdt[timeStep];

	//cte 20210517 check coefficient against original documentation
	// The amount of snow melted by rain (for this timestep) = 0.0129 * intercepted rain * current temp(F) - 32(F)
	double MeltedSnowbyRain = 0.0129 * folder->VarDict["shortVegIntW"] * (input->Tair[timeStep] - 32);

	// melted snow cannot be larger than the stored snow
	if (MeltedSnowbyRain > folder->VarDict["shortVegStoredSWE"]) {
		MeltedSnowbyRain = folder->VarDict["shortVegStoredSWE"];
	}

	//snowMeltSVegCanopy_m (m) is snow melted in short veg canopy due to rain, later added to that melted by radiation
	folder->VarDict["snowMeltSVegCanopy_m"] = MeltedSnowbyRain;
	
	// Update the stored snow-water-equivalent (within the folder) to reflect the melted snow
	folder->VarDict["shortVegStoredSWE"] = folder->VarDict["shortVegStoredSWE"] - MeltedSnowbyRain;

	double tempShortVegStoredW = folder->VarDict["shortVegStoredW"] + folder->VarDict["shortVegIntW"] + MeltedSnowbyRain;

	if (tempShortVegStoredW > folder->VarDict["storMaxShortVeg"]) {
		// reset the current intercepted rain to be (storMaxShortVeg - current stored W (coming into the timestep) - melted snow) - so the intercepted rain is the amount available to fill up storMaxShortVeg
		folder->VarDict["shortVegIntW"] = folder->VarDict["storMaxShortVeg"] - folder->VarDict["shortVegStoredW"] - MeltedSnowbyRain;
		// Throughfall  = calculated throughfall + (temp stored W - storMaxShortVeg) (the amount of water above storMaxShortVeg, gets added to throughfall)
		folder->VarDict["shortVegThruFlowW"] = folder->VarDict["shortVegThruFlowW"] + (tempShortVegStoredW - folder->VarDict["storMaxShortVeg"]);

		folder->VarDict["shortVegThruFlowSWE"] = folder->VarDict["shortVegThruFlowSWE"] + folder->VarDict["shortVegStoredSWE"];
		folder->VarDict["shortVegStoredSWE"] = 0.0;
		tempShortVegStoredW = folder->VarDict["storMaxShortVeg"];
	}

	folder->VarDict["shortVegStoredW"] = tempShortVegStoredW;
	
	if(folder->VarDict["shortVegIntW"] < 0)
		folder->VarDict["shortVegIntW"] = 0.0;
}

void ShortVegInterceptionCalc::shortVegInterceptionSnow(Inputs *input, DataFolder *folder, int timeStep)
{
	double tempShortVegLAI = input->ShortVegLAI[timeStep];
	if (folder->ParamStringDict["Type"]  !="BulkArea")
	{
		double factor = folder->ParamDict["MaxShortVegLAI"] / 2.2;
		tempShortVegLAI *= factor;
	}
	folder->VarDict["storMaxShortVeg"] = 0.15 * 0.001 * tempShortVegLAI;   
	
	double Tair = 5.0/9.0 * (input->Tair[timeStep] - 32); // should be 0C
	double SnowDensity = 67.92 + 51.25 * exp(Tair/2.59);
	double S = 6.0 * (0.27 + 46.0/SnowDensity)/SnowDensity;    // 6.0 can be changed
	double storMaxShortVegSnow = S * tempShortVegLAI;
	double storMaxShortVegSWE = storMaxShortVegSnow * (SnowDensity/1000);
	//Eq 2a & Eq 2b from Wang, Endreny, Nowak (2008, JAWRA) based on Dijk and Bruijnzeel (2001) for free throughfall
	//Wang et al. (2008) after Eq 2b assign extinction coefficient k = 0.3 to short vegetation
	double fShortVegTQ = (1-(1-exp(-0.3 * tempShortVegLAI)));
	double SnowP = input->Precip_mpdt[timeStep];
	
	folder->VarDict["shortVegThruFlowSWE"] = fShortVegTQ * SnowP;   //SnowP is the Snow water equivalent depth
	folder->VarDict["shortVegIntSWE"] = (1-fShortVegTQ) * SnowP; // need to convert precip to snow depth first

	double  tempShortVegStoredSWE = folder->VarDict["shortVegStoredSWE"] + folder->VarDict["shortVegIntSWE"];
	if (tempShortVegStoredSWE > storMaxShortVegSWE)
	{	
		folder->VarDict["shortVegIntSWE"] = storMaxShortVegSWE - folder->VarDict["shortVegStoredSWE"];
		folder->VarDict["shortVegThruFlowSWE"] = folder->VarDict["shortVegThruFlowSWE"] + (tempShortVegStoredSWE - storMaxShortVegSWE);
		tempShortVegStoredSWE = storMaxShortVegSWE;
	} 

	folder->VarDict["shortVegStoredSWE"] = tempShortVegStoredSWE;


	if(folder->VarDict["shortVegIntSWE"] < 0)
		folder->VarDict["shortVegIntSWE"] = 0.0;
}
