#include "TreeInterceptionCalc.h"

void TreeInterceptionCalc::calculate(Inputs *input, DataFolder *folder, int timeStep)
{
	// initializing these values at the start of each timestep
	//treeThruFlowW (m) initialized to 0 at start of each time step;
	folder->VarDict["treeThruFlowW"] = 0.0;
	//treeThruFlowSWE (m) initialized to 0 at start of each time step;
	folder->VarDict["treeThruFlowSWE"] = 0.0;
	//treeIntW (m) initialized to 0 at start of each time step;
	folder->VarDict["treeIntW"] = 0.0;
	//treeIntSWE (m) initialized to 0 at start of each time step;
	folder->VarDict["treeIntSWE"] = 0.0;
	//snowMeltTreeCanopy_m (m) initialized to 0 at start of each time step;
	folder->VarDict["snowMeltTreeCanopy_m"] = 0;

	//folder->VarDict[treeIntW[timeStep] = 0.0;	// intializing the intercepted rain at this timestep in this folder to 0 
	//folder->VarDict[treeIntW[timeStep] = 0.0;		// intializing the intercepted rain at this timestep in this folder to 0


	// Decides whether to treat the incoming precipitation as rain or snow
	if (input->Tair[timeStep] > 32.0) {
		// heads to the treeInterceptionRain function defined below
		treeInterceptionRain(input, folder, timeStep);
	}
	else {
		// heads to the treeInterceptionRain function defined below
		treeInterceptionSnow(input, folder, timeStep);
	}

	//treeUnloadSnow function provides snow unloaded by Wind
	treeUnloadSnow(input, folder, timeStep);

    //Only allow for snowmelt by radiation if snow storage on tree
	if (folder->VarDict["treeStoredSWE"] > 0.0) {
		// Snow melted by Radiation
		treeSnowMeltbyRn(input, folder, timeStep);
	}

	// New function added to keep updating/storage in one place
	storeValues(input, folder, timeStep);
}


void TreeInterceptionCalc::storeValues(Inputs *input, DataFolder *folder, int timeStep)
{

	// Updating the combined throughfall to include the water throughfall and snow-water equivalent throughfall
	folder->VarDict["treeThruFlow"] = folder->VarDict["treeThruFlowW"] + folder->VarDict["treeThruFlowSWE"];

	// Only update the vectors if a semi-distributed run
	if (input->SimulationStringParams["Model"]=="StatisticalHydro")	{
		// Updating vector variables for extended outputs
		folder->VarVecDict["treeIntW"].push_back(folder->VarDict["treeIntW"]);
		folder->VarVecDict["treeIntSWE"].push_back(folder->VarDict["treeIntSWE"]);
		folder->VarVecDict["treeThroughW"].push_back(folder->VarDict["treeThruFlowW"]);
		folder->VarVecDict["treeThruFlowSWE"].push_back(folder->VarDict["treeThruFlowSWE"]);
	}

}

// treeUnloadSnow function 
// History: introduced by Yang Yang for use in iTree V5, adapted from Roesch et al. (2001)
// Reference: iTree Hydro: Snow Hydrology Update for the Impervious Forest Hydrology Model
// Authors: Yang Yang, Ted Endreny, Dave Nowak
void TreeInterceptionCalc::treeUnloadSnow(Inputs *input, DataFolder *folder, int timeStep)
{
	// Unloaded Snow = 0.0231 * Windspeed * currently stored snow
	double TreeUnloadSWE = 0.0231 * input->WindSp_mps[timeStep] * folder->VarDict["treeStoredSWE"];
	// If the calculated unloaded SWE is > available SWE to unload, the unloaded SWE is set to available snow.
	if (TreeUnloadSWE > folder->VarDict["treeStoredSWE"])
	{
		TreeUnloadSWE = folder->VarDict["treeStoredSWE"];
	}
	// Update the TreeStoredSWE.  TreeStoredSWE = (TreeStoredSWE - TreeUnloadSWE)
	folder->VarDict["treeStoredSWE"] = folder->VarDict["treeStoredSWE"] - TreeUnloadSWE;
	// Update the TreeThroughSWE.  TreeThroughSWE = (TreeThroughSWE + TreeUnloadSWE)
	folder->VarDict["treeThruFlowSWE"] = folder->VarDict["treeThruFlowSWE"] + TreeUnloadSWE;

}

// treeSnowMeltbyRn function 
// History: introduced by Yang Yang for use in iTree V5, adapted from Roesch et al. (2001)
// Reference: iTree Hydro: Snow Hydrology Update for the Impervious Forest Hydrology Model
// Authors: Yang Yang, Ted Endreny, Dave Nowak
void TreeInterceptionCalc::treeSnowMeltbyRn(Inputs *input, DataFolder *folder, int timeStep)
{
	//cte 20210517 check the coefficients on these equations
	// Mn = SWE melted by shortwave radiation - depends on cloudiness of sky and albedo (0.85)
	double Mn=0.000011*input->RadiationNet_Wpm2[timeStep]*(1-0.85); // 0.85 is the snow surface albedo which can be changed
	// Ml = SWE melted by longwave radiation - if it is negative, it is set equal to 0
	double Ml=0.0212*0.0010583*(input->Tair[timeStep]-32.0)-0.84*0.0010583;
	if (Ml<0)
		Ml=0;

	double RnMeltedSWE=Mn+Ml; // Radiation-melted SWE = SWE melted by shortwave rad +  SWE melted by longwave rad

	// If the calculated radiation metled SWE is greater than the stored SWE, set Rad melted SWE equal to stored SWE
	if (RnMeltedSWE > folder->VarDict["treeStoredSWE"])	{
		RnMeltedSWE = folder->VarDict["treeStoredSWE"];
	}
	
	//snowMeltTreeCanopy_m (m) is snow melted in tree canopy due to radiation, added to that melted by rain
	folder->VarDict["snowMeltTreeCanopy_m"] = folder->VarDict["snowMeltTreeCanopy_m"] + RnMeltedSWE;

	// Update TreeStoredSWE by subtracting rad melted SWE (RnMeltedSWE)
	folder->VarDict["treeStoredSWE"] = folder->VarDict["treeStoredSWE"] - RnMeltedSWE;

	// Since the snow has been melted into water, added the SWE to the TreeStoredW
	folder->VarDict["treeStoredW"] = folder->VarDict["treeStoredW"] + RnMeltedSWE;

	// Since the RadiationNet_Wpm2 (radiation) melted snow-water equivalent (SWE) has been added to the leaf as Water (W), 
	// we need to make sure the leaf storage isn't over its max 
	// If the TreeStoredW is > Smaxtree (the storage max of a leaf - the max storage depth)
	if (folder->VarDict["treeStoredW"] > folder->VarDict["storMaxTree"])  
	{   
		//treeThruFlowSWE (m) increased by treeStoredSWE (m) due to overflowing water pushing snow off of canopy
		folder->VarDict["treeThruFlowSWE"] = folder->VarDict["treeThruFlowSWE"] + folder->VarDict["treeStoredSWE"];
		folder->VarDict["treeStoredSWE"] = 0.0;
		// Adjust the current (for this folder, this timestep) TreeThroughW to equal that calculated during the interception step + the excess (over the maximum) stored water that was just updated with the metled snow
		folder->VarDict["treeThruFlowW"] = folder->VarDict["treeThruFlowW"] + (folder->VarDict["treeStoredW"] - folder->VarDict["storMaxTree"]);
		// The TreeStoredW at this timestep is set equal to the max storage.  
		folder->VarDict["treeStoredW"] = folder->VarDict["storMaxTree"];
	}
	//conditional operator "?" evaluates an expression, returning one value if that expression evaluates to true, and a different one if evaluates to false
	//Example: condition ? result1 : result2
	folder->VarDict["treeStoredSWE"] = (folder->VarDict["treeStoredSWE"] >= 0.0) ? folder->VarDict["treeStoredSWE"] : 0.0;
	folder->VarDict["treeStoredW"] = (folder->VarDict["treeStoredW"] >= 0.0) ? folder->VarDict["treeStoredW"] : 0.0;
}


// treeInterceptionRain function 
// History: introduced by Jun Wang for use in iTree V4, adapted from van Dijk and Bruijnzeel (2001)
// Reference: 1. Wang, Jun, Theodore A. Endreny, and David J. Nowak, 2008.
//                Mechanistic Simulation of Tree Effects in an Impervious Water Balance Modell 
//                Journal of the American Water Resources Association (JAWRA) 44(1): 75-85
//                DOI: 10.1111/j.1752-1688.2007.00139.x
// Authors: Jun Wang, Ted Endreny, Dave Nowak
// Modified by: Yang Yang, Thomas Taggart, Shannon Conley

void TreeInterceptionCalc::treeInterceptionRain(Inputs *input, DataFolder *folder, int timeStep)
{
	double temp = input->Precip_mpdt[timeStep]; 
	double tempTreeLAI = input->TreeLAI[timeStep];
	if (folder->ParamStringDict["Type"]  !="BulkArea")
	{
		//cte 20210517 address this hard coded 2.2 value, presuming a maximum LAI
		double factor = folder->ParamDict["MaxTreeLAI"] / 5.0;
		tempTreeLAI *= factor;
	}

	// The current Storage Max for a tree leaf (SmaxTree_m) is TreeLeafStorage_mm * 0.001 m/mm * LAI for this timestep
	// The storage max is therefore a volume per area measurement (m^3/m^2)
	folder->VarDict["storMaxTree"] = folder->ParamDict["TreeLeafStorage_mm"]*0.001*tempTreeLAI;


	//Eq 2a & Eq 2b from Wang, Endreny, Nowak (2008, JAWRA) based on Dijk and Bruijnzeel (2001) for free throughfall
	//Wang et al. (2008) after Eq 2b assign extinction coefficient k = 0.7 to trees
	// fTreeTQ (the tree throughfall coefficiant) =  e^(-0.7 * LAI)
	double fTreeTQ = (1-(1-exp(-0.7 *input->TreeLAI[timeStep])));

	// Current throughfall  = fTreeTQ * current precip
	folder->VarDict["treeThruFlowW"] = fTreeTQ * input->Precip_mpdt[timeStep];
	// Current interception = (1-fTreeTQ)*current precip
	folder->VarDict["treeIntW"] = (1-fTreeTQ) * input->Precip_mpdt[timeStep]; 

	//cte 20210517 check coefficient against original documentation
	// The amount of snow melted by rain (for this timestep) = 0.0129 * intercepted rain * current temp(F) - 32(F)
	double MeltedSnowbyRain = 0.0129 * folder->VarDict["treeIntW"] * (input->Tair[timeStep]-32);

	// Setting the melted snow = to the stored snow (within a folder) if it is larger
	if (MeltedSnowbyRain > folder->VarDict["treeStoredSWE"]) {
		// melted snow cannot be larger than the stored snow
		MeltedSnowbyRain = folder->VarDict["treeStoredSWE"];
	}

	//snowMeltTreeCanopy_m (m) is snow melted in tree canopy due to rain, later added to that melted by radiation
	folder->VarDict["snowMeltTreeCanopy_m"] = MeltedSnowbyRain;

	// Update the stored snow-water-equivalent (within the folder) to reflect the melted snow
	folder->VarDict["treeStoredSWE"] = folder->VarDict["treeStoredSWE"] - MeltedSnowbyRain;

	// sets the temporary tree stored water (within a folder) to reflect the previous amount of stored water (within a folder) + current timestep's intercepted  (within a folder) + current timestep's metled snow by rain (within a folder)
	double tempTreeStoredW = folder->VarDict["treeStoredW"] + folder->VarDict["treeIntW"] + MeltedSnowbyRain;

	// if the stored water is > storage max for the tree
	if (tempTreeStoredW > folder->VarDict["storMaxTree"]) {
		// reset the current intercepted rain to be (Smaxtree - current tree stored W (coming into the timestep) - melted snow) - so the intercepted rain is the amount available to fill up Smaxtree
		folder->VarDict["treeIntW"] = folder->VarDict["storMaxTree"] - folder->VarDict["treeStoredW"] - MeltedSnowbyRain;
		// Throughfall  = calculated throughfall + (temp stored W - smaxtree) (the amount of water above smaxtree, gets added to throughfall)
		folder->VarDict["treeThruFlowW"] = folder->VarDict["treeThruFlowW"] + (tempTreeStoredW - folder->VarDict["storMaxTree"]);
		
		folder->VarDict["treeThruFlowSWE"] = folder->VarDict["treeThruFlowSWE"] + folder->VarDict["treeStoredSWE"];
		// Reset the stored SWE (within a folder) to be zero
		folder->VarDict["treeStoredSWE"] = 0.0;      // Q(Tom): remaining intercepted snow is just lost?
		// set tempTreeStoredW = Smaxtree
		tempTreeStoredW = folder->VarDict["storMaxTree"];
	}

	// Updates/modifies TreeStoredW (within a folder) =  tempTreeStoredW (which is either <=  smaxtree thanks to the previous if-clause) 
	folder->VarDict["treeStoredW"] = tempTreeStoredW;

	// Make sure current treeInterceptedRain isn't less than 0
	if (folder->VarDict["treeIntW"] < 0)
		folder->VarDict["treeIntW"] = 0.0;

	// Possibly unnecessary now - check!  Tom . T 9/4/15
	// Updates/modifies TreeInterceptedRain (within a folder)
	//folder->VarDict[treeIntW = folder->VarDict[treeIntW;

}



// treeInterceptionSnow function 
// History: introduced by Yang Yang for use in iTree V5, adapted from Roesch et al. (2001)
// Reference: iTree Hydro: Snow Hydrology Update for the Impervious Forest Hydrology Model
// Authors: Yang Yang, Ted Endreny, Dave Nowak
void TreeInterceptionCalc::treeInterceptionSnow(Inputs *input, DataFolder *folder, int timeStep)
{
	double tempTreeLAI = input->TreeLAI[timeStep];
	if (folder->ParamStringDict["Type"]  !="BulkArea")
	{
		double factor = folder->ParamDict["MaxTreeLAI"] / 5.0;
		tempTreeLAI *= factor;
	}
	// Calculates the current storage max (water) - CurrentSmaxtree
	folder->VarDict["storMaxTree"] = folder->ParamDict["TreeLeafStorage_mm"] * 0.001 * tempTreeLAI;
	//Arpit
	//folder->VarDict["storMaxTree"] = input->paramValues->getParameter("SL") * 0.001 * folder->treeLAI[timeStep];
	// Converts the temperature into celsius
	double Tair = 5.0/9.0 * (input->Tair[timeStep] - 32);
	// calculates the snow density based on curent temp
	double SnowDensity = 67.92 + 51.25 * exp(Tair/2.59);
	// calculates the vertical maximum snow depth
	double S = 6.0 * (0.27 + 46.0/SnowDensity)/SnowDensity;	  // the "6.0" can be changed (different species have different value)
	// calculates the current snow storage max - storMaxTreeSnow
	double storMaxTreeSnow  = S * tempTreeLAI;
	// calculates the current snow-water-equiv storage max - storMaxTreeSnow
	double storMaxTreeSWE = storMaxTreeSnow * (SnowDensity/1000);
	// fTreeTQ (the tree throughfall coefficiant) =  e^(-0.7 * LAI)
	double fTreeTQ = exp(-0.7 * tempTreeLAI);
	// transfers preip input (snow since temp < 32F)
	double SnowP = input->Precip_mpdt[timeStep];
	folder->VarDict["treeThruFlowSWE"] = fTreeTQ * SnowP;   //SnowP is the Snow water equivalent depth
	folder->VarDict["treeIntSWE"] = (1-fTreeTQ) * SnowP; 

	// updates the tempTreeStoredSWE = current stored SWE (coming into the timestep) + CurrentTreeInterceptedSWE
	double tempTreeStoredSWE = folder->VarDict["treeStoredSWE"] + folder->VarDict["treeIntSWE"];

	// if the tempTreeStoredSWE > storMaxTreeSWE
	if (tempTreeStoredSWE > storMaxTreeSWE)
	{	
		// CurrentTreeInterceptedSWE = storMaxTreeSWE - incoming TreeStoredSWE.  So the CurrentTreeInterceptedSWE = available SWE space
		folder->VarDict["treeIntSWE"] = storMaxTreeSWE - folder->VarDict["treeStoredSWE"];	
		// SWE Throughfall  = calculated SWE throughfall + (temp stored SWE - smaxtreeSWQ) (the amount of water over smaxtreeSWE)
		folder->VarDict["treeThruFlowSWE"] = folder->VarDict["treeThruFlowSWE"] + (tempTreeStoredSWE - storMaxTreeSWE);
		// Reset the tempTreeStoredSWE to be storMaxTreeSWE
		tempTreeStoredSWE = storMaxTreeSWE;
	}  
	
	// Reset the TreeStoredSWE to be tempTreeStoreSWE
	folder->VarDict["treeStoredSWE"] = tempTreeStoredSWE;

	// if CurrentTreeInterceptedSWE < 0, set it = 0
	if (folder->VarDict["treeIntSWE"] < 0.0)
		folder->VarDict["treeIntSWE"] = 0.0;

	// No longer completely necessary - wil have to add in some storage of this vlaue into a vector for water balance purposes
	// Update TreeInterceptedSWE with CurrentTreeInterceptedSWE
	// folder->VarDict[treeIntSWE = folder->VarDict[treeIntSWE;
}
