#include "RootZoneDrainageCalc.h"

//Root zone storage capacity is only capillarity held water, and unsaturated zone storage capacity is only gravitational water
//	The root zone storage depth extends to the limit of evapotranspiration accessing capillarity held water, a constant for all TI bins
//	The unsaturated zone extends from the groundwater table to the ground, and accounts for all gravitational held water
//	The two zones are conceptualy situated alongside each other; if the unsaturated zone extends to greater depths it only represents gravitational water

void  RootZoneDrainageCalc::calculate(Inputs *input, DataFolder *folder, int timeStep)
{
	//initialize VarDict["iA"] as current index value of topographic index; used for passing index value to separate function while in loop
	int ia = folder->VarDict["iA"];


	//Variables Drainage_RootZone_MacroPore_frac + Drainage_RootZone_Infil_frac = 1, used to weight water balance updates due to saturation excess, when macropore drainage and infiltrated water are removed from root zone
	// Drainage_RootZone_MacroPore_frac and Drainage_RootZone_Infil_frac are not currently used to allocate fractions of removed soil water
	//Drainage_RootZone_MacroPore_frac (frac) reset to 0.5 for each index value, first presuming water is evenly contributed to root zone by macropore drainage and infiltrated water 
	folder->VarDict["Drainage_RootZone_MacroPore_frac"] = 0.5;
	//Drainage_RootZone_Infil_frac (frac) reset to 1 for each index value, first presuming water is evenly contributed to root zone by macropore drainage and infiltrated water 
	folder->VarDict["Drainage_RootZone_Infil_frac"] = 0.5;

	//Initialize vectors at start of simulation 
	if (timeStep == 0)
	{
		//cte 20210520 explore behavior of SD0_percent in regulating model performance and initial SMD

		//initRootZoneDef (m) is initial water deficit in root zone based on MSD_m, maximum soil deficit (m) and initial storage deficit (percent)
		//	Initialized by SD0_percent (0 = saturated, 100 = dry), initial soil moisture deficit parameter from HydroPlusConfig.xml
		double initRootZoneDef = ((folder->ParamDict["SD0_percent"] / 100)  *  folder->ParamDict["MSD_m"]);
		
		//Drainage_RootZone_TI_m (m) initialized to 0 for start of simulation for each TI bin
		folder->VarVecDict["Drainage_RootZone_TI_m"].push_back(0.0);

		//Storage_unUsed_RootZone_TI_m (m) initialized to initRootZoneDef (m) for start of simulaiton for each TI bin
		//	Note that deficit is capacity or available storage.
		folder->VarVecDict["Storage_unUsed_RootZone_TI_m"].push_back(initRootZoneDef);
	}
		
	//Storage routine to take in infiltration and determine gravitational water drainage; all root zone TI bins have same storage
	// Storage_unUsed_RootZone_TI_m (m) is a deficit, aka capacity, which is reduced by Infiltration_m (m) into root zone from infiltration
	folder->VarVecDict["Storage_unUsed_RootZone_TI_m"][ia] = folder->VarVecDict["Storage_unUsed_RootZone_TI_m"][ia] - folder->VarDict["Infiltration_m"];

	//Drainage routine, combining macropore flow with gravitational water within rooting zone 
	//Drainage_RootZone_TI_m (m) is initialized to Drainage_macroPore_m (m), and is gravitational drainage from root zone to unsaturated zone
	//Drainage_macroPore_m (m) is the water from Water_forSoilRecharge_m that bypasses infiltration and root zone, going directly to subsurface flow
	folder->VarVecDict["Drainage_RootZone_TI_m"][ia] = folder->VarDict["Drainage_macroPore_m"];

	//if Storage_unUsed_RootZone_TI_m (m) < 0 there is gravitational water beyond field capacity, and this drains to the unsaturated zone
	//	deficits < 0 exceed maximum storage capacity; deficits = 0 are at maximum storage capacity; 
	//	Storage_unUsed_RootZone_TI_m (m) capacity is only capillarity held water, and Storage_Used_UnsatZone_TI_m (m) capacity is only gravitational water
	if (folder->VarVecDict["Storage_unUsed_RootZone_TI_m"][ia] < 0.0 )         
	{
		// Drainage_RootZone_TI_m is increased by excess water in Storage_unUsed_RootZone_TI_m, where subtracting a negative is equivalent to adding Storage_unUsed_RootZone_TI_m to Drainage_RootZone_TI_m
		folder->VarVecDict["Drainage_RootZone_TI_m"][ia] = (folder->VarVecDict["Drainage_RootZone_TI_m"][ia] - folder->VarVecDict["Storage_unUsed_RootZone_TI_m"][ia]);

		//Drainage_RootZone_MacroPore_frac (frac) recomputed for situation with Storage_unUsed_RootZone_TI_m < 0.0
		//Drainage_RootZone_Infil_frac (frac) recomputed for situation with Storage_unUsed_RootZone_TI_m < 0.0
		folder->VarDict["Drainage_RootZone_Infil_frac"] = abs(folder->VarVecDict["Storage_unUsed_RootZone_TI_m"][ia]) / (abs(folder->VarVecDict["Storage_unUsed_RootZone_TI_m"][ia]) + folder->VarDict["Drainage_macroPore_m"]);
		folder->VarDict["Drainage_RootZone_MacroPore_frac"] = 1 - folder->VarDict["Drainage_RootZone_Infil_frac"];
		
		// Storage_unUsed_RootZone_TI_m is reset to 0
		folder->VarVecDict["Storage_unUsed_RootZone_TI_m"][ia] = 0.0;
	}

	//Drainage_RootZone_m = sum Drainage_RootZone_TI_m for each TI bin for time step, weighted by watershed fraction
	//	HydroPlus weights discrete TI parameter values, i.e, SoilParameter[ia], with TI_binArea_frac, catchment area bounding discrete value
	folder->VarDict["Drainage_RootZone_m"] = folder->VarDict["Drainage_RootZone_m"] + folder->VarVecDict["Drainage_RootZone_TI_m"][ia] * folder->VarDict["TI_binArea_frac"];
}