#include "RootZoneETCalc.h"

//Root zone evapotranspiration (ET) only removes water between field capacity and wilting point, referred to as capillarity held
//	The HydroPlus StatisticalHydro model uses the input data of potential ET to determine atmospheric demand
//	The SpatialTemperatureHydro or LCScalingTempSpatial model uses latent energy to determine atmospheric demand
//	Available water is defined as [maximum storage deficit] - [actual storage deficit]

void RootZoneETCalc::calculate(Inputs* input, DataFolder* folder, int timeStep)
{
	//initialize VarDict["iA"] as current index value of topographic index; used for passing index value to separate function while in loop
	int ia = folder->VarDict["iA"];

	//actualEvapoTranspiration (m) reset to zero at start of each TI value
	double actualEvapoTranspiration = 0;
	double rootZoneStorageAvailable = 0;
	//Latent heat of vaporization (J/kg)
	#define lamda 2257000.0

	//Initialize vectors at start of simulation 
	if (timeStep == 0)
	{
		//EvapoTranspiration_RootZone_TI_m (m) is soil evapotranspiration, initialized to 0
		folder->VarVecDict["EvapoTranspiration_RootZone_TI_m"].push_back(0.0);
	}
	
	//cte 20210618 Update ET function so actual ET is reducing potential ET by ratio of available soil moisture ET_act = ET_pot * (soil storage)/(soil storage max) based on Saxton (1986) Eq in Fisher et al. (2005)
	//Fisher et al (2005) show potential ET based on Penmam-Monteith is too high relative to observed actual ET. The Eqs of Fisher et al. (2005) are not numbered, the same LHS variable is used for multiple Eqs of potential ET
	//Fisher et al (2005) give the adjustment in text where Saxton (1986) is mentioned, as f(phi), and show the benefit by contrasting the fit in Fig 2 vs Fig 1. 
	//Fisher, J. B., DeBiase, T. A., Qi, Y., Xu, M., & Goldstein, A. H. (2005). Evapotranspiration models compared on a Sierra Nevada forest ecosystem. Environmental Modelling & Software, 20(6), 783-796. doi:https://doi.org/10.1016/j.envsoft.2004.04.009
	// 
	//EvapoTranspiration_RootZone_TI_m (m) reset to zero at start of each index value 
	folder->VarVecDict["EvapoTranspiration_RootZone_TI_m"][ia] = 0.0;

	if (input->SimulationStringParams["Model"] == "StatisticalHydro") {
		//actualEvapoTranspiration (m) = potential ET read in from Evaporation.dat input file
		actualEvapoTranspiration = input->PET[timeStep];
	}
	//if (input->SimulationStringParams["Model"] == "SpatialTemperatureHydro" || input->SimulationStringParams["Model"] == "LCScalingTempSpatial")
	else {
		//actualEvapoTranspiration (m) = sum of SoilLE_W_p_m2 / lamda + TreeLET_W_p_m2 / lamda + ShortVegLET_W_p_m2 / lamda
		actualEvapoTranspiration = (folder->VarDict["SoilLE_W_p_m2"] / lamda) + (folder->VarDict["TreeLET_W_p_m2"] / lamda) + (folder->VarDict["ShortVegLET_W_p_m2"] / lamda);
	}

	//if actualEvapoTranspiration > 0 then remove water from rootzone
	if (actualEvapoTranspiration > 0.0)
	{
		// Modify rootzone ET to equal = PET * (1 - current rootzone storage deficit / maximum rootzone storage deficit), for this index value
		// the more saturated the rootzone, the lower and closer to 0 the rootzone storage deficit is, and the great the actual ET
		// if rootzone storage deficit = 0, the actualt ET = PET
		if (folder->ParamDict["MSD_m"] > 0) {
			folder->VarVecDict["EvapoTranspiration_RootZone_TI_m"][ia] = actualEvapoTranspiration * (1 - (folder->VarVecDict["Storage_unUsed_RootZone_TI_m"][ia] / folder->ParamDict["MSD_m"]));
			rootZoneStorageAvailable = folder->ParamDict["MSD_m"] - folder->VarVecDict["Storage_unUsed_RootZone_TI_m"][ia];
		}
		else { 
			folder->VarVecDict["EvapoTranspiration_RootZone_TI_m"][ia] = 0; 
			rootZoneStorageAvailable = 0;
		}
		//total moisture available to evaporate  (maximum rootzone storage deficit - rootzone storage deficit), for this index value
		double t1 = folder->VarVecDict["Storage_unUsed_RootZone_TI_m"][0];
		// if the rootzone ET > is greater then the total moisture available to evaporate  
		if (folder->VarVecDict["EvapoTranspiration_RootZone_TI_m"][ia] > rootZoneStorageAvailable) {
			// Set rootzone ET to the difference maximum rootzone storage deficit and rootzone storage deficit - you can't evaporate water that's not there
			folder->VarVecDict["EvapoTranspiration_RootZone_TI_m"][ia] = rootZoneStorageAvailable;      
		}
	}

	//Storage_unUsed_RootZone_TI_m (m) is updated based on EvapoTranspiration_RootZone_TI_m removal, by folder and index value
	folder->VarVecDict["Storage_unUsed_RootZone_TI_m"][ia] += folder->VarVecDict["EvapoTranspiration_RootZone_TI_m"][ia]; 

	//EvapoTranspiration_RootZone_m (m) = sum EvapoTranspiration_RootZone_TI_m for each TI bin for time step, weighted by watershed fraction
	//	HydroPlus weights discrete TI parameter values, i.e, SoilParameter[ia], with TI_binArea_frac, catchment area bounding discrete value
	folder->VarDict["EvapoTranspiration_RootZone_m"] += folder->VarVecDict["EvapoTranspiration_RootZone_TI_m"][ia] * folder->VarDict["TI_binArea_frac"];

	//Storage_unUsed_RootZone_m (m) = sum Storage_unUsed_RootZone_TI_m for each TI bin for time step, weighted by watershed fraction
	//	HydroPlus weights discrete TI parameter values, i.e, SoilParameter[ia], with TI_binArea_frac, catchment area bounding discrete value
	folder->VarDict["Storage_unUsed_RootZone_m"] += folder->VarVecDict["Storage_unUsed_RootZone_TI_m"][ia] * folder->VarDict["TI_binArea_frac"];

}
