#include "SaturatedBaseFlowExpDecayCalc.h"

//Computes at every time step Runof_Subsurface_m (m), depth of subsurface runoff from pervious areas, using exponential decay approach
//Theory of computing Runof_Subsurface_m (m) with exponential decay based on Wolock (1993) and Beven and Kirby (1979)
//Wolock, D. M. (1993). Simulating the variable-source-area concept of watershed hydrology with TOPMODEL (USGS Water-Resources Investigation Report 93-4124). 
//Beven, K. J., & Kirkby, M. J. (1979). A physically based, variable contributing area model of basin hydrology. Hydrological Sciences Bulletin, 24(1), 43-69. doi:10.1080/02626667909491834

//Subsurface runoff is only from pervious area, which combines with drainage from unsaturated zone in pervious area to update average soil moisture deficit in SemiAveSMDCalc.cpp

void SaturatedBaseFlowExpDecayCalc::calculate(Inputs *input, DataFolder *folder, int timeStep)
{
	//SaturatedBaseFlowExpDecayCalc called by functors["calculateSaturatedBaseFlow"] for BulkArea, not called for GI DataFolder 

	//T0 transmissivity (m^2 / timestep) when AveSMD = 0 (watershed fully saturated) is adjusted to per time step based on input as per hour and TimeStep_sec
	double T0 = folder->ParamDict["T0_m2ph"] * folder->ParamDict["TimeStep_sec"] / 3600;
	//m (m) is scaling parameter describing exponential decay of saturated hydraulic conductivity with depth, due to decreasing macropores with depth
	//Beven, Lamb, et al. (1995) give a physical interpretation of the decay parameter m (m) is that it controls the effective depth, z (m) of the catchment soil profile. This it does interactively in Eq T = T0 * exp (-f*z), where f = (Theta_sat - Theta_fc)/m. See Figure 18.1 for values of m ranging from 0.02 to 0.05 creating effective depths z ranging from 1 to 2 m.
	//Beven, K., Lamb, R., Quinn, P., Romanowics, R., & Freer, J. (1995). TOPMODEL. In V. P. Singh (Ed.), Computer models of watershed hydrology (pp. 627-688). Colorado: Water Resources Publications.
	double mM = folder->ParamDict["m"];

	// Initializing baseflow to 0.0
	folder->VarDict["Runoff_Subsurface_m"] = 0.0;
	double aveSMD = input->AveSMD;
	double satZoneQ = 0.0;

	//TestSimulation for BulkArea DataFolder sets folder->ParamDict["BF_flag"] = 1, properly compumputing satZoneQ
	//If GI DataFolder were to enter this routine, it would have folder->ParamDict["BF_flag"] set in HydroPlusConfig.xml

	//satZoneQ (m) is maximum subsurface runoff from pervious watershed when soil fully saturated; multiply by watershed area to get volume (m3); runoff is per time step
	//Eq 32 from Wolock, D. M. (1993). Simulating the variable-source-area concept of watershed hydrology with TOPMODEL (USGS Water-Resources Investigation Report 93-4124). 
	satZoneQ = folder->ParamDict["BF_flag"] * T0 * exp(-input->AverageIndex);
	//TOPMODEL Code: SZQ = EXP(T0DT - TL), where TL is average topographic index, adn T0DT is natural log of transmissivity, ln(T0)

	//Runoff_Subsurface_m (m) is subsurface runoff from pervious area for actual level of saturation, using aveSMD and TOPMODEL relationship
	//Eq 28 from Wolock (1993), with 1st RHS term swapped out by Eq 31 (which replaces Eq 29 for spatial aggregation). Wolock, D. M. (1993). Simulating the variable-source-area concept of watershed hydrology with TOPMODEL (USGS Water-Resources Investigation Report 93-4124). 
	//Eq 3 from Beven and Kirby (1979) gives qb = q0 exp (S/m), where q0 is flow when S = 0 or storage deficit is 0
	folder->VarDict["Runoff_Subsurface_m"] = (satZoneQ * exp(-folder->VarDict["aveSMD"] / mM));
	//TOPMODEL Code: QB = SZQ * EXP(-SBAR / SZM), where SZQ is SatZoneQ (m), SBAR is aveSMD, SZM is m

}

