#include "SaturatedBaseFlowPowDecay.h"

//Computes at every time step Runof_Subsurface_m (m), depth of subsurface runoff from pervious areas, using exponential decay approach
//Theory of computing Runof_Subsurface_m (m) with power function decay emerged from exponential decay by Wolock (1993) and Beven and Kirby (1979)
//Wolock, D. M. (1993). Simulating the variable-source-area concept of watershed hydrology with TOPMODEL (USGS Water-Resources Investigation Report 93-4124). 
//Beven, K. J., & Kirkby, M. J. (1979). A physically based, variable contributing area model of basin hydrology. Hydrological Sciences Bulletin, 24(1), 43-69. doi:10.1080/02626667909491834

//Theory of power function decay requires m <= S (or aveSMD) based on constraint governing decay with depth of transmissivity, Tz = T0*(1-S/m)^n  
//S (m) is soil moisture deficit between top of soil and groundwater table, defined as Theta_sat-Theta_wp or Theta_sat-Theta_fc (based on Beven and Wood, 1983).
//Beven and Wood (1983). Catchment Geomorphology and the Dynamics of Runoff Contributing Areas, Journal of Hydrology. 65:139-158. In Special Issue: Scale Problems in Hydrology, Editor
//Theory explained after Eq 5 of Wang, J., Endreny, T. A., & Hassett, J. M. (2006). Power function decay of hydraulic conductivity for a TOPMODEL-based infiltration routine Hydrological Processes, 20(18), 3825-3834. 
//Wang et al (2006) after Eq 5. explains in power decay conditions the m parameter sets the upper limit for the soil moisture deficit S (m), due to Eq 7, T = T0(1-S/m)^n. If n = 2, then S > m. If n <=1, S>m will generate negative T profiles., if n >=2, S>m will generate non-monotonic T profiles.
//Figure 1 from Wang et al. (2006) illustrates the decay of transmissivity, Tz, with depth as a function of n, varying from 0.25 to 16

void SaturatedBaseFlowPowDecayCalc::calculate(Inputs* input, DataFolder* folder, int timeStep)
{
	//SaturatedBaseFlowPowDecayCalc called by functors["calculateSaturatedBaseFlow"] for BulkArea, not called for GI DataFolder 

	//T0 transmissivity (m^2 / timestep) when AveSMD = 0 (watershed fully saturated) is adjusted to per time step based on input as per hour and TimeStep_sec
	double T0 = folder->ParamDict["T0_m2ph"] * folder->ParamDict["TimeStep_sec"] / 3600;
	//m (m) is scaling parameter describing exponential decay of saturated hydraulic conductivity with depth, due to decreasing macropores with depth
	//Beven, Lamb, et al. (1995) give a physical interpretation of the decay parameter m (m) is that it controls the effective depth, z (m) of the catchment soil profile. This it does interactively in Eq T = T0 * exp (-f*z), where f = (Theta_sat - Theta_fc)/m. See Figure 18.1 for values of m ranging from 0.02 to 0.05 creating effective depths z ranging from 1 to 2 m.
	//Beven, K., Lamb, R., Quinn, P., Romanowics, R., & Freer, J. (1995). TOPMODEL. In V. P. Singh (Ed.), Computer models of watershed hydrology (pp. 627-688). Colorado: Water Resources Publications.
	//Wang et al (2006) after Eq 5. explains in power decay conditions the m parameter sets the upper limit for the soil moisture deficit S (m), due to Eq 7, T = T0(1-S/m)^n. If n = 2, then S > m. If n <=1, S>m will generate negative T profiles., if n >=2, S>m will generate non-monotonic T profiles.
	//Wang, J., Endreny, T. A., & Hassett, J. M. (2006). Power function decay of hydraulic conductivity for a TOPMODEL-based infiltration routine Hydrological Processes, 20(18), 3825-3834. 
	double mM = folder->ParamDict["m"];

	//n is coefficient for power function decay of saturated hydraulic conductivity with depth, due to decreasing macropores with depth
	//Figure 1 of Wang et al. (2006) shows how Transmissivity varies with n ranging from 0.25 to 16
	double nN = folder->ParamDict["n"];

	// Initializing baseflow to 0.0
	folder->VarDict["Runoff_Subsurface_m"] = 0.0;
	double satZoneQ = 0.0;

	//TestSimulation for BulkArea DataFolder sets folder->ParamDict["BF_flag"] = 1, properly compumputing satZoneQ
	//If GI DataFolder were to enter this routine, it would have folder->ParamDict["BF_flag"] set in HydroPlusConfig.xml

	//satZoneQ (m) is maximum subsurface runoff from pervious watershed when soil fully saturated; multiply by watershed area to get volume (m3); runoff is per time step
	//Eq 5, 1st RHS term from Wang, J., Endreny, T. A., & Hassett, J. M. (2005). Flexible Modeling Package for Topographically Based Watershed Hydrology. Journal of Hydrology, 314(1-4), 78-91. 
	satZoneQ = folder->ParamDict["BF_flag"] * T0 * pow(input->AverageIndex, -nN);

	//Theory of power function decay requires m <= S based on constraint governing decay with depth of transmissivity, Tz = T0*(1-S/m)^n  
	//S (m) is soil moisture deficit between top of soil and groundwater table, defined as Theta_sat-Theta_wp or Theta_sat-Theta_fc (based on Beven and Wood, 1983).
	//Theory explained after Eq 5 of Wang, J., Endreny, T. A., & Hassett, J. M. (2006). Power function decay of hydraulic conductivity for a TOPMODEL-based infiltration routine Hydrological Processes, 20(18), 3825-3834. 
	//Wang et al (2006) after Eq 5. explains in power decay conditions the m parameter sets the upper limit for the soil moisture deficit S (m), due to Eq 7, T = T0(1-S/m)^n. If n = 2, then S > m. If n <=1, S>m will generate negative T profiles., if n >=2, S>m will generate non-monotonic T profiles.
	//Figure 1 from Wang et al. (2006) illustrates the decay of transmissivity, Tz, with depth as a function of n, varying from 0.25 to 16
	if(folder->VarDict["aveSMD"] > mM)
		folder->VarDict["aveSMD"] = (mM - 0.000001);

	//Runoff_Subsurface_m (m) is subsurface runoff from pervious area for actual level of saturation, using aveSMD and TOPMODEL relationship
	folder->VarDict["Runoff_Subsurface_m"] = (satZoneQ * pow((1 - (folder->VarDict["aveSMD"]/mM)), nN));

}