#include "LocalSoilMoistureCalc.h"
#include "../RootZone/RootZoneETCalc.h"
#include "../RootZone/RootZoneDrainageCalc.h"
#include "../UnSatZone/UnsatZoneDrainageCalc.h"
#include "../UnSatZone/UnsatZoneSaturationExcessQ.h"

//Code uses either an exponential decay or power decay approach to compute local soil moisture deficit, determined by variable input->SimulationStringParams["Infiltration"] == "PowerDecay"
//Soil is represented as three components, with average soil moisture deficit representing total storage between soil and groundwater table: 
//	1) root zone with capillarity held water for ET, releasing gravitational water; it can saturate from below
//	2) unsaturated zone with gravitational water for saturation excess or drainage to saturated zone
//	3) saturated zone for subsurface runoff
//Soil moisture deficits are average for catchment and local for TI bin, computed for both approaches as:
//	ExponentialDecay (timeStep=0) aveSMD = -mM * log(Q0 / SZQ) and SZQ = T0 * exp(-avgIndex); or PowerDecay (timeStep=0) aveSMD = mM * (1.0 - SZQ) and SZQ = pow(Q0 / T0, 1 / nN)*(avgIndex);
//	ExponentialDecay Storage_Max_UnsatZone_TI_m = aveSMD_value + mM * (AverageIndex - Index[ia]) or PowerDecay Storage_Max_UnsatZone_TI_m = aveSMD_value + mM * pow(Q0/T0, 1/nN)*(AverageIndex - Index[ia]);

//Theory of exponential decay based on Wolock (1993) using Eq 31
	//Wolock, D. M. (1993). Simulating the variable-source-area concept of watershed hydrology with TOPMODEL (USGS Water-Resources Investigation Report 93-4124). 
//Theory of power function decay requires m <= S (AveSMD or Storage_Max_UnsatZone_TI_m) based on constraint governing decay with depth of transmissivity, Tz = T0*(1-S/m)^n  
	//Theory of power function decay explained between Eq 2 and 4 of Wang et al. (2005) and after Eq 5 of Wang et al. (2006)
	//Wang et al (2006) after Eq 5. explains in power decay conditions the m parameter sets the upper limit for the soil moisture deficit S (m), due to Eq 7, T = T0(1-S/m)^n. If n = 2, then S > m. If n <=1, S>m will generate negative T profiles., if n >=2, S>m will generate non-monotonic T profiles.
	//Figure 1 from Wang et al. (2006) illustrates the decay of transmissivity, Tz, with depth as a function of n, varying from 0.25 to 16
	//Wang, J., Hassett, J.M., & Endreny, T.A. (2005). An Object Oriented Approach to the Description& Simulation of Watershed Scale Hydrologic Processes.Computersand Geosciences, 31(4), 425 - 435.
	//Wang, J., Endreny, T. A., & Hassett, J. M. (2006). Power function decay of hydraulic conductivity for a TOPMODEL-based infiltration routine Hydrological Processes, 20(18), 3825-3834. 
//Note Z = SMD/ne, where soil moisture deficit, SMD (m), groundwater table depth, Z (m), effective porosity, ne, from Eq 16 in Wolock (1993)
//Note return flow is not simulated when Storage_Max_UnsatZone_TI_m < 0, given saturation may represent impounded water, not laterally redistributed groundwater breaching surface
//	Neither the Topmodel code of Beven nor of Wolock (1993) computes return flow, despite presentation of Eq 21 in Wolock (1993).

//calculate function computes local soil moisture deficit for each topographic index (TI) bin
void LocalSoilMoistureCalc::calculate(Inputs *input, DataFolder *folder, int timeStep)
{
	//n is coefficient for power function decay of saturated hydraulic conductivity with depth, due to decreasing macropores with depth
	//Figure 1 of Wang et al. (2006) shows how Transmissivity varies with n ranging from 0.25 to 16
	double nN = folder->ParamDict["n"];
	//Q0 (m/ts) is initial subsurface discharge from catchment, used to derive initial AveSMD, per time step based on input as per hour and TimeStep_sec
	double Q0 = folder->ParamDict["Q0_mph"] * folder->ParamDict["TimeStep_sec"] / 3600;
	//T0 transmissivity (m^2 / timestep) when AveSMD = 0 (watershed fully saturated) is adjusted to per time step based on input as per hour and TimeStep_sec
	double T0 = folder->ParamDict["T0_m2ph"] * folder->ParamDict["TimeStep_sec"] / 3600;
	//mM (m) is scaling parameter describing exponential decay of saturated hydraulic conductivity with depth, due to decreasing macropores with depth
	//Beven, Lamb, et al. (1995) give a physical interpretation of the decay parameter m (m) is that it controls the effective depth, z (m) of the catchment soil profile. This it does interactively in Eq T = T0 * exp (-f*z), where f = (Theta_sat - Theta_fc)/m. See Figure 18.1 for values of m ranging from 0.02 to 0.05 creating effective depths z ranging from 1 to 2 m.
	//Beven, K., Lamb, R., Quinn, P., Romanowics, R., & Freer, J. (1995). TOPMODEL. In V. P. Singh (Ed.), Computer models of watershed hydrology (pp. 627-688). Colorado: Water Resources Publications.
	double mM = folder->ParamDict["m"];
	//aveSMD (m) is average soil moisture deficit, used to update local SMD values
	double aveSMD_value = folder->VarDict["aveSMD"];

	//Groundwater_surficial_m (m) and Groundwater_surficial_frac (fraction) are used to note variable source area, where the groundwater is expressed above the ground surface
	//Groundwater_surficial_m (m) initialized for use in TI bin loop
	double Groundwater_surficial_m = 0.0;
	//variableSourceAreaExtent_m (m) initialized for use in TI bin loop
	double Groundwater_surficial_frac = 0.0;
	//Runoff_SatExcess_m(m) is the saturation excess runoff, reset to 0.0
	double Runoff_SatExcess_m= 0.0;

	//cte 20210715 Flag flag_GI_TI_IndexSize_single for testing GI simulation topographic index size; = 0 run with IndexSize same as BulkArea; = 1 run with IndexSize of 1
	//NB Set in LocalSoilMoistureCalc.cpp line 55 and GI_Outlets.cpp line 52
	int flag_GI_TI_IndexSize_single = 0;

	//Groundwater_surficial_m (m) reset to zero at start of index
	folder->VarDict["Groundwater_surficial_m"] = 0;
	//Groundwater_surficial_frac (m) reset to zero at start of index
	folder->VarDict["Groundwater_surficial_frac"] = 0;
	//Drainage_RootZone_m (m) reset to zero at start of index
	folder->VarDict["Drainage_RootZone_m"] = 0;
	//EvapoTranspiration_RootZone_m (m) reset to zero at start of index
	folder->VarDict["EvapoTranspiration_RootZone_m"] = 0.0;
	//Storage_unUsed_RootZone_m (m) reset to zero at start of index
	folder->VarDict["Storage_unUsed_RootZone_m"] = 0.0;
	//rootZoneStorageTemp_m (m) reset to zero at start of index
	folder->VarDict["Storage_Used_RootZone_m"] = 0.0;
	//Drainage_UnsatZone_m (m) initialized reset to zero at start of index
	folder->VarDict["Drainage_UnsatZone_m"] = 0.0;
	//Runoff_SatExcess_m (m) is total saturation excess runoff across all ia bins
	folder->VarDict["Runoff_SatExcess_m"] = 0.0;
	//Storage_Used_UnsatZone_m (m) reset to zero at start of index
	folder->VarDict["Storage_Used_UnsatZone_m"] = 0.0;

	//IndexLoopSize = input->IndexSize of topographic index (TI) for statistical model, which is 1 less than IndexAreaSize, which has all TI values in histogram 
	int IndexLoopSize = input->IndexSize;

	//if input->SimulationStringParams["Model"] == "SpatialTemperatureHydro" then IndexLoopSize = 1 the IndexLoopSize = 1, and input->IndexSize > input->IndexAreaSize
	if (input->SimulationStringParams["Model"] == "SpatialTemperatureHydro") { 
		//IndexLoopSize set for just one topographic index (TI) value in each folder
		IndexLoopSize = 1; 
	}

	//aveSMD_value for GI DataFolder is AveSMD_GI from GI at prior timestep (separate from BulkArea AveSMD); this aveSMD updated by GI is used below to compute local soil moisture storage
	//if Type != BulkArea (hence Green Infrastructure or GI Type) then redefine terms 
	if (folder->ParamStringDict["Type"] != "BulkArea") {
		//aveSMD_value (m) = GI based AveSMD_GI (m) term from prior time step, used below to compute soil moisture storage
		aveSMD_value = folder->VarDict["AveSMD_GI"];
		//cte 20210715 Condition to test running GI simulation with single index size
		if (flag_GI_TI_IndexSize_single != 0) {
			//IndexLoopSize redefined = 1 topographic index (TI) value, which will be the AverageIndex
			IndexLoopSize = 1;
		}
	}

	//Statistically distributed topographic index algorithm taken from TOPMODEL code developed by Keith Beven and used by USGS D.M. Wolock https://pubs.er.usgs.gov/publication/wri934124
	//Wolock, D. M. (1993). Simulating the variable-source-area concept of watershed hydrology with TOPMODEL (USGS Water-Resources Investigation Report 93-4124).
	//	The algorithm requires the topographic index (TI) bins (ia) are sorted from highest (wettest) to lowest TI values
	//	The algorithm requires when ia = 0, the TI bin Index[0] should be the upper limit and its IndexArea[0] = 0
	//	The algorithm presumes for each ia, the area of catchment between discrete TI values Index[ia] and Index[ia+1] is IndexArea[ia]
	//	The algorithm requires for each ia, the area of catchment bounding Index[ia] is = to (IndexArea[ia] + IndexArea[ia+1])/2 

	//Loop through all discrete topographic index (TI) values, sorted from largest to smallest, with fractional area for each TI = to average of bounding bin areas
	//	For spatially distributed model, the folder area has a single TI value
	for (int ia = 0; ia < IndexLoopSize; ia++) {

		//Initialize vectors at start of simulation 
		if (timeStep == 0)
		{
			folder->VarVecDict["Storage_Max_UnsatZone_TI_m"].push_back(0.0);
			folder->VarVecDict["Storage_Max_UnsatZone_TI_temp_m"].push_back(0.0);
		}

		//VarDict["iA"] = ia, current index value of topographic index; used for passing index value to separate function while in loop
		folder->VarDict["iA"] = ia;

		//ia is the TI bin set one place later, drier, in the sorted list than bin ia; on first pass iaa = 1
		int iaa = ia + 1;

		//cte 20210715 Confirm for SpatialTemperatureHydro the TI_binArea_frac = 1, with 1 assigned to both IndexArea[ia] and IndexArea[iaa]
		//TI_binArea_frac is catchment area (frac) bounding each discrete TI value
		folder->VarDict["TI_binArea_frac"] = 0.5 * (input->IndexArea[ia] + input->IndexArea[iaa]);
		//Original TOPMODEL weights discrete TI catchment area fractions of histogram with bounding TI parameter values
		//	HydroPlus weights discrete TI parameter values, i.e, SoilParameter[ia], with TI_binArea_frac, catchment area bounding discrete value

		//Compute local soil moisture deficit, which will determine which unsaturated zones are saturated
		//Power decay computation of Storage_Max_UnsatZone_TI_m
		if (input->SimulationStringParams["Infiltration"] == "PowerDecay") {

			//if Model = "StatisticalHydro" BulkArea, then IndexAreaSize > 1 and Storage_Max_UnsatZone_TI_m[ia] (m) function of histogram position TI value
			if (input->SimulationStringParams["Model"] == "StatisticalHydro") {

				//Storage_Max_UnsatZone_TI_m (m) computed as Power Decay from aveSMD, mM, Q0, T0, nN, and TI values based material between Eq 2 and 4 in Wang et al. (2005), using histogram bins
				//	Note the Eq below combines from Wang et al. (2005) S_bar and S_i, and S_i (S = SMD) is using topographic index, not soil TI, and power (1/n), not (1/-n), and (a/tanB)^(1/n) should not be power term
				folder->VarVecDict["Storage_Max_UnsatZone_TI_m"][ia] = aveSMD_value + mM * pow(Q0 / T0, 1 / nN) * (input->AverageIndex - input->Index[ia]);

				//cte 20210715 Condition to test running GI simulation with single index size
				if (flag_GI_TI_IndexSize_single != 0) {
					//cte 20210715 if Type != BulkArea (hence Green Infrastructure or GI Type) then redefine terms 
					if (folder->ParamStringDict["Type"] != "BulkArea") {
						//Storage_Max_UnsatZone_TI_m[ia] (m) is set to aveSMD_value, representing GI as AverageIndex from histogram values
						folder->VarVecDict["Storage_Max_UnsatZone_TI_m"][ia] = aveSMD_value;
						//TI_binArea_frac defined to be entire area of index size of 1; don't define actual vector to avoid messing with BulkArea calcs
						folder->VarDict["TI_binArea_frac"] = 1;

					}
				}
			}
			//if Model = "SpatialTemperatureHydro", then IndexAreaSize = 1 and Storage_Max_UnsatZone_TI_m[ia] (m) function of map position TI value
			if (input->SimulationStringParams["Model"] == "SpatialTemperatureHydro") {
				//Storage_Max_UnsatZone_TI_m (m) computed as Power Decay from aveSMD, mM, Q0, T0, nN, and TI values based material between Eq 2 and 4 in Wang et al. (2005), using map pixels
				//	Note the Eq below combines from Wang et al. (2005) S_bar and S_i, and S_i (S = SMD) is using topographic index, not soil TI, and power (1/n), not (1/-n), and (a/tanB)^(1/n) should not be power term
				folder->VarVecDict["Storage_Max_UnsatZone_TI_m"][ia] = aveSMD_value + mM * pow(Q0 / T0, 1 / nN) * (input->AverageIndex - input->Index[folder->VarDict["position"]]);
			}

			//Theory of power function decay requires m <= S (aveSMD or Storage_Max_UnsatZone_TI_m) based on constraint governing decay with depth of transmissivity, Tz = T0*(1-S/m)^n  
			//Storage_Max_UnsatZone_TI_m (m) is soil moisture deficit between top of soil and groundwater table, defined as Theta_sat-Theta_wp or Theta_sat-Theta_fc (based on Beven and Wood, 1983).
			//Beven and Wood (1983). Catchment Geomorphology and the Dynamics of Runoff Contributing Areas, Journal of Hydrology. 65:139-158. In Special Issue: Scale Problems in Hydrology, Editors: Rodriguez-Iturbe and V.K. Gupta.
			//Theory explained after Eq 5 of Wang, J., Endreny, T. A., & Hassett, J. M. (2006). Power function decay of hydraulic conductivity for a TOPMODEL-based infiltration routine Hydrological Processes, 20(18), 3825-3834. 
			//Wang et al (2006) after Eq 5. explains in power decay conditions the m parameter sets the upper limit for the soil moisture deficit S (m), due to Eq 7, T = T0(1-S/m)^n. If n = 2, then S > m. If n <=1, S>m will generate negative T profiles., if n >=2, S>m will generate non-monotonic T profiles.
			//Figure 1 from Wang et al. (2006) illustrates the decay of transmissivity, Tz, with depth as a function of n, varying from 0.25 to 16
			if (folder->VarVecDict["Storage_Max_UnsatZone_TI_m"][ia] > mM)
				folder->VarVecDict["Storage_Max_UnsatZone_TI_m"][ia] = mM;
		}
		//Exponential decay computation of Storage_Max_UnsatZone_TI_m
		else {

			//if Model = "StatisticalHydro" BulkArea, then IndexAreaSize > 1 and Storage_Max_UnsatZone_TI_m[ia] (m) function of histogram position TI value
			if (input->SimulationStringParams["Model"] == "StatisticalHydro") {

				//Storage_Max_UnsatZone_TI_m (m) computed as Exponential Decay from aveSMD, mM, and TI values based on Eq 31 from Wolock (1993), using histogram bins
				folder->VarVecDict["Storage_Max_UnsatZone_TI_m"][ia] = aveSMD_value + mM * (input->AverageIndex - input->Index[ia]); // real for coresponding index

				//cte 20210715 Condition to test running GI simulation with single index size
				if (flag_GI_TI_IndexSize_single != 0) {
					//cte 20210715 if Type != BulkArea (hence Green Infrastructure or GI Type) then redefine terms 
					if (folder->ParamStringDict["Type"] != "BulkArea") {
						//Storage_Max_UnsatZone_TI_m[ia] (m) is set to aveSMD_value, representing GI as AverageIndex from histogram values
						folder->VarVecDict["Storage_Max_UnsatZone_TI_m"][ia] = aveSMD_value;
						//TI_binArea_frac defined to be entire area of index size of 1; don't define actual vector to avoid messing with BulkArea calcs
						folder->VarDict["TI_binArea_frac"] = 1;
					}
				}
			}
			//if Model= "SpatialTemperatureHydro", then IndexAreaSize = 1 and Storage_Max_UnsatZone_TI_m[ia] (m) function of map position TI value
			if (input->SimulationStringParams["Model"] == "SpatialTemperatureHydro") {

				//Storage_Max_UnsatZone_TI_m (m) computed as Exponential Decay from aveSMD, mM, and TI values based on Eq 31 from Wolock (1993), using map pixels
				folder->VarVecDict["Storage_Max_UnsatZone_TI_m"][ia] = aveSMD_value + mM * (input->AverageIndex - input->Index[folder->VarDict["position"]]);
			}
		}

		//Storage_Max_UnsatZone_TI_temp_m set to save temporary record of Storage_Max_UnsatZone_TI_m for use in UnsatZoneSaturationExcessQ.cpp
		folder->VarVecDict["Storage_Max_UnsatZone_TI_temp_m"][ia] = folder->VarVecDict["Storage_Max_UnsatZone_TI_m"][ia];

		//if Storage_Max_UnsatZone_TI_m < 0, reset to 0 to avoid erroneous calculation in UnsatZoneStorageCalc.cpp
		if (folder->VarVecDict["Storage_Max_UnsatZone_TI_m"][ia] < 0.0) {
			//Storage_Max_UnsatZone_TI_m reset to 0 after generating return flow
			folder->VarVecDict["Storage_Max_UnsatZone_TI_m"][ia] = 0.0;
			//Topmodel code: IF(SD(IA).LT.0.)SD(IA) = 0.
		}

		//Compute soil water balance, separating capillarity water for ET and gravitational water for saturation excess or subsurface flow
		//call RootZoneDrainageCalc.cpp to add infiltrated water, and release gravitational and macropore water to unsaturated zone
		RootZoneDrainageCalc::calculate(input, folder, timeStep);
		//call RootZoneETCalc.cpp to remove water held by capillarity into evapotranspirational demand
		RootZoneETCalc::calculate(input, folder, timeStep);
		//call UnsatZoneDrainageCalc.cpp to update gravitational water storage, saturation excess runoff, and drainage to groundwater
		UnsatZoneDrainageCalc::calculate(input, folder, timeStep);
		//call UnsatZoneSaturationExcessQ.cpp to sum the saturation excess runoff
		UnsatZoneSaturationExcessQ::calculate(input, folder, timeStep);
	}

	//Infiltration_m (m) = Infiltration_viaSatEx_m + Infiltration_viaInfilEx_m, where RHS terms had saturation excess runoff removed	
	//	Infiltration_m updated here, at end of TI for loop, not within loop, to avoid interfering with RootZoneDrainageCalc routines
	folder->VarDict["Infiltration_m"] = folder->VarDict["Infiltration_viaSatEx_m"] + folder->VarDict["Infiltration_viaInfilEx_m"];
	
	//Storage_Used_RootZone_m (m) computed at end of TI loop; 
	//	Storage_Used_RootZone_m (m) is root zone storage (water available) = difference of maximum storage deficit and actual storage deficit
	folder->VarDict["Storage_Used_RootZone_m"] = folder->ParamDict["MSD_m"] - folder->VarDict["Storage_unUsed_RootZone_m"];

	//If ExtendedOutputsTimeStep, store vectors for GIOutputWriter.cpp, after removing any folder->VarDict["Runoff_SatExcess_m"]
	if (input->SimulationStringParams["ExtendedOutputsTimeStep"] != "NoExtendedOutputs")
	{
		folder->VarVecDict["Infiltration_viaSatEx_m"].emplace_back(folder->VarDict["Infiltration_viaSatEx_m"]);
		folder->VarVecDict["Infiltration_viaInfilEx_m"].emplace_back(folder->VarDict["Infiltration_viaInfilEx_m"]);
		folder->VarVecDict["Infiltration_m"].emplace_back(folder->VarDict["Infiltration_m"]);
		//Runoff_SatExcess_m (m), saturated excess runoff
		folder->VarVecDict["Runoff_SatExcess_m"].emplace_back(folder->VarDict["Runoff_SatExcess_m"]);
		//Runoff_PervInfilEx_m (m), infiltration excess runoff 
		folder->VarVecDict["Runoff_PervInfilEx_m"].emplace_back(folder->VarDict["Runoff_PervInfilEx_m"]);
		//EvapoTranspiration_RootZone_m (m), soil ET
		folder->VarVecDict["EvapoTranspiration_RootZone_m"].emplace_back(folder->VarDict["EvapoTranspiration_RootZone_m"]);
	}

	//If GI is operational in HydroPlusConfig.xml then implement the transfer of water between BulkArea and GI
	//Determine the saturation excess runoff (m) for the BulkArea, for use as runon in the GI area
	if (input->RepoDict["GI_FolderOperational"] = 1 && folder->ParamStringDict["Type"] == "BulkArea") {

		//RepoDict is used for transfer between BulkArea and GI area folders
		//BulkArea area (m2) transferred to GI
		input->RepoDict["BulkArea_Area_m2"] = folder->ParamDict["Area"];

		//The subsequent sequence of steps will 1) adjust depths to extract runoff going from Bulk Area to GI and 2) adjust depths to move between areas and conserve water
		//1. BulkAreaPerviousDrainingToGI_frac used to extract fraction of pervious area depth going from Bulk Area to GI
		double BulkAreaPerviousDrainingToGI_frac = input->RepoDict["BulkAreaPerArea_DrainingToGI_frac"];

		//BulkArea_SatExRunoffDrainingToGI_m (m) is pervious area depth leaving Bulk Area and going to GI via saturation excess ruonff
		input->RepoDict["BulkArea_SatExRunoffDrainingToGI_m"] = folder->VarDict["Runoff_SatExcess_m"] * BulkAreaPerviousDrainingToGI_frac;
		//BulkArea_InfilExRunoffDrainingToGI_m (m) is pervious area depth leaving Bulk Area and going to GI via infiltration excess runoff
		input->RepoDict["BulkArea_InfilExRunoffDrainingToGI_m"] = folder->VarDict["Runoff_PervInfilEx_m"] * BulkAreaPerviousDrainingToGI_frac;

		//Runoff_SatExcess_m (m) is pervious area depth remaining in Bulk Area after removing BulkArea_SatExRunoffDrainingToGI_m (m) going to GI 
		folder->VarDict["Runoff_SatExcess_m"] = folder->VarDict["Runoff_SatExcess_m"] - input->RepoDict["BulkArea_SatExRunoffDrainingToGI_m"];
		//Runoff_PervInfilEx_m (m) is pervious area depth remaining in Bulk Area after removing BulkArea_InfilExRunoffDrainingToGI_m (m) going to GI 
		folder->VarDict["Runoff_PervInfilEx_m"] = folder->VarDict["Runoff_PervInfilEx_m"] - input->RepoDict["BulkArea_InfilExRunoffDrainingToGI_m"];

		//2. BulkArea_SatExRunoffDrainingToGI_m (m) is adjusted to folder area depth by multiplication with TotalPerviousCover_frac, with fraction comparable to TotalPerviousCover_m2/TotalArea_m2
		input->RepoDict["BulkArea_SatExRunoffDrainingToGI_m"] = input->RepoDict["BulkArea_SatExRunoffDrainingToGI_m"] * folder->ParamDict["TotalPerviousCover_frac"];
		//2. BulkArea_InfilExRunoffDrainingToGI_m (m) is adjusted to folder area depth by multiplication with TotalPerviousCover_frac, with fraction comparable to TotalPerviousCover_m2/TotalArea_m2
		input->RepoDict["BulkArea_InfilExRunoffDrainingToGI_m"] = input->RepoDict["BulkArea_InfilExRunoffDrainingToGI_m"] * folder->ParamDict["TotalPerviousCover_frac"];

		//cte 20210520 All GI transfers of water to Bulk Area likely need to be represented as depths for catchment area (GI+BulkArea), not just Bulk Area
		//cte 20210520 GI transfers to Bulk Area subsurface should be optionally disconnected for Bioretention etc. devices designed to be contained separately from regional groundwater
		//	This could use HydroPlusConfig.xml parameter BF_flag or new parameter to turn on and off this transfer of GI water to Bulk Area saturation 
		
		//Transfer of GI pervious area drainage water to Bulk Area pervious saturated zone; using a volume to uphold conservation of mass in transfer
		//Drainage_UnsatZone_m contains GIstorageExfil (m3) volume, generated in GI pervious area, after it is dispersed across Bulk Area pervious area
		//	GIstorageExfil_m3 is from prior timestep, distributed across Bulk Area pervious area but should likely be distributed across Catchment pervious area
		folder->VarDict["Drainage_UnsatZone_m"] += input->RepoDict["GIstorageExfil_m3"] / (folder->ParamDict["Area"] * folder->ParamDict["TotalPerviousCover_frac"]);
	}
	if (folder->ParamStringDict["Type"] != "BulkArea") {

		//RepoDict is used for transfer between BulkArea and GI area folders
		//ContArea_SatExRunoffDraining_m3 (m^3) is volume of runon to GI, which is conserved by continuity; derived from folder depth BulkArea_SatExRunoffDrainingToGI_m and folder area BulkArea_Area_m2
		input->RepoDict["ContArea_SatExRunoffDraining_m3"] = input->RepoDict["BulkArea_SatExRunoffDrainingToGI_m"] * input->RepoDict["BulkArea_Area_m2"];
		//ContArea_InfilExRunoffDraining_m3 (m^3) is volume of runon to GI, which is conserved by continuity; derived from folder depth BulkArea_InfilExRunoffDrainingToGI_m and folder area BulkArea_Area_m2
		input->RepoDict["ContArea_InfilExRunoffDraining_m3"] = input->RepoDict["BulkArea_InfilExRunoffDrainingToGI_m"] * input->RepoDict["BulkArea_Area_m2"];

		//ContArea_SatExRunoffDrainingToGI_m (m) is depth of runon over GI area; derived from volume of runon divided by GI area (m2), where Area (m^2) is for GI adn giNumber is number of GI devices
		input->RepoDict["ContArea_SatExRunoffDrainingToGI_m"] = (input->RepoDict["ContArea_SatExRunoffDraining_m3"] /
			(folder->ParamDict["Area"] * folder->ParamDict["giNumber"]));
		//ContArea_InfilExRunoffDrainingToGI_m (m) is depth of runon over GI area; derived from volume of runon divided by GI area (m2), where Area (m^2) is for GI adn giNumber is number of GI devices
		input->RepoDict["ContArea_InfilExRunoffDrainingToGI_m"] = (input->RepoDict["ContArea_InfilExRunoffDraining_m3"]) /
			(folder->ParamDict["Area"] * folder->ParamDict["giNumber"]);

		//Forcing Runoff_SatExcess_m and Runoff_PervInfilEx_m to 0 for GI, given all upwelling water remains onsite in the GI as storage depth
		//Option to create new term that tracks all internal runoff that is contained as storage
		folder->VarDict["Runoff_SatExcess_m"] = 0;
		folder->VarDict["Runoff_PervInfilEx_m"] = 0;

		//cte 20210520 GI routine needs updated topographic index (TI) approach to conserve water and uphold physical constraints on space. 
		//cte 20210520 All GI transfers of water to Bulk Area likely need to be represented as depths for catchment area (GI+BulkArea), not just Bulk Area
		//	Remove binNumber from HydroPlusCong.xml, 
		//	Determine Bulk Area fractional area treated by GI using HydroPlusConfig.xml GI terms ImpAreaTreated_frac and PerAreaTreated_frac along with Bulk Area terms Area and DCIA_frac
		//	Determine GI fractional area using HydroPlusCongig.xml GI terms Area and giNumber (we already have GI term, AreaOfCatchment_frac)
		//	Record the number of the TI histogram bin that does not contain any Bulk Area fraction treated by GI, summing TI fractional area from bottom of TI array, which is the driest and highest elevation
		//	Record the number of TI histogram bins that contain the GI fractional area, noting these must be further up in the TI array than the Bulk Area, which are wetter and lower elevation 
		//	Check that the GI fractional area and Bulk Area do not exceed the catchment area, otherwise exit the program
		//	Create a vector of TI bins that will be used to allow for updating Storage_Max_UnsatZone_TI_m in GI from saturated water from Bulk Area

		//Green Infrastructure needs the Bulk Area depth to groundwater to determine if the GI device is saturated
		//	Uses GI Folder in the HydroPlusConfig.xml for binNumber and SoilPorosity_m3pm3 keys and values
		//	Note: topographic index (TI) values are sorted from largest TI in top row of histogram, which is wettest
		//Zi = [Storage_Max_UnsatZone_TI_m[binNumber] - 1] / SoilPorosity_m3pm3; based on Eq 16 in Wolock, D. M. (1993); [binNumber - 1] adjusts the input value for a C++ vector starting at 0 
		folder->VarDict["Zi"] = folder->VarVecDict["Storage_Max_UnsatZone_TI_m"][folder->ParamDict["binNumber"] - 1] / folder->ParamDict["SoilPorosity_m3pm3"];

		//cte 20210715 Condition to test running GI simulation with single index size
		if (flag_GI_TI_IndexSize_single != 0) {
			//cte 20210715 GI has 1 TI bin, so Storage_Max_UnsatZone_TI_m =  aveSMD_value;
			folder->VarDict["Zi"] = aveSMD_value / folder->ParamDict["SoilPorosity_m3pm3"];
		}

		//cte 20210520 Testing output to determine correct TI bins to receive the GI area
		/*if (timeStep % int(0.25 * input->SimulationNumericalParams["TotalTimeSteps"]) == 0)
		{
			cout << "folder->VarDict[Zi] " << folder->VarDict["Zi"] << " at binNumber " << folder->ParamDict["binNumber"] << endl;
		}*/
	}

	//Storing values from the soil zone processes at this timestep for this folder
	storeValues(input, folder, timeStep);
}	

//Storing values from the soil zone processes at this timestep for this folder
void  LocalSoilMoistureCalc::storeValues(Inputs* input, DataFolder* folder, int timeStep)
{
	//Drainage_RootZone_prior_m (m) is used to potentially update cumulative infiltration in the next time step; it is combined with EvapoTranspiration_RootZone_prior_m (m). 
	folder->VarDict["Drainage_RootZone_prior_m"] = folder->VarDict["Drainage_RootZone_m"];
	//cte 20210502 testing impacts on infiltration folder->VarDict["Drainage_RootZone_prior_m"] = folder->VarVecDict["Drainage_RootZone_TI_m"][0];
	//EvapoTranspiration_RootZone_prior_m (m) is used to potentially update cumulative infiltration in the next time step; it is combined with Drainage_RootZone_prior_m (m). 
	folder->VarDict["EvapoTranspiration_RootZone_prior_m"] = folder->VarDict["EvapoTranspiration_RootZone_m"];
}
