﻿#include "SnowMeltOpenAreaCalc.h"

/**************************************************************************** 
Function Name: SnowMeltOpenAreaCalc::calculate(Inputs *input, DataFolder *folder, int timeStep) 
Purpose: To calculate:
			- the amount of melted snow in an open area (no tree or shortveg cover)
			- the total amount of water in an open area (o tree or shortveg cover)
		 Updates:
			- the amount of snow on the open area (leftover snow + new snow)
			- the amount of water on the open area (rain + metled snow)

Requires:		int timeStep - the timestep value
				DataFolder *folder - the key number for the folder being processed
Returns:		Void

Reference:   1. Yang, Yang, Theodore A. Endreny, and David J. Nowak, 2011. 
                  iTree-Hydro: Snow Hydrology Update for the Impervious Forest Hydrology Model. 
                  Journal of the American Water Resources Association (JAWRA) 47(6):1211–1218.
                  DOI: 10.1111/j.1752-1688.2011.00564.x

Comments:	Introduced by Yang Yang in iTree V5, to include snow processes, adapts and uses equations from
            Roesch et al. (2001) and USACE (1958, 1998)
			The differences in in the snow melt equations (between open area, under tree, under shortveg) 
			are explained in Yang's paper.

Author(s):		Jun Wang, jwang06@mailbox.syr.edu, V4/V4.5
				Yang Yang, yyang31@syr.edu, V5
				 	
				Thomas Taggart, traggatmot@gmail.com
						
*****************************************************************************/


void SnowMeltOpenAreaCalc::calculate(Inputs *input, DataFolder *folder, int timeStep)
{
	meltsnowOpen(input, folder, timeStep);
	storeValues(input, folder, timeStep);
}

void SnowMeltOpenAreaCalc::meltsnowOpen(Inputs *input, DataFolder *folder, int timeStep)
{
	double Tair = input->Tair[timeStep]; 
	double P = input->Precip_mpdt[timeStep];   
	double RadiationNet_Wpm2 = input->RadiationNet_Wpm2[timeStep];		
	double Tdew = input->Tdew[timeStep];  
	double UV = input->WindSp_mps[timeStep];  
	double Kp = 1.0;
	double Za = 2.0;
	double Zb = 2.0;

	// The following Snow equations are applied by Yang, and originate from:
	// USACE, 1956. Snow Hydrology: Summary Report of the Snow Investigations. 
	// US Army Corps of Engineers North Pacific Division Report NTIS PB-151660, Portland, Oregon.
	// USACE, 1998. Engineering and Design: Runoff From Snowmelt. 
	// US Army Corps of Engineers Report Engineer Manual 1110-2-1406, Washington, D.C.
	
	// initializing the snow melt by rain to 0.0
	double Mp = 0.0;

	// Kp, Za, Zb are initialized in the SnowMeltUnderTreeRepo constructor - Kp(0.8), Za(2.0), Zb(2.0)
	// kp is the ratio of atmospheric pressure at the location to the atmospheric pressure at the sea level
	// za (m) is the height at which the temperature and the air vapor pressure are measured
	// zb (m) is the height of wind velocity measurement
	// rho_ice = 992 kg/m^3 for USACE 1998 report
	// rho_ice = 1000 kg/m^3 

	// Eq 2-2 (USACE, 1998) converts between melt energy and melt depth, and can work for melt power and melt rate
	//	M = Q / (B * F * rho), where M is melt depth (mm), Q is melt energy (kJ/m^2), B is snow thermal quality or ratio of energy to melt snow vs ice,
	//			F is latent heat of fusion (kJ/kg), and rho is density of water (kg/m^3)
	//		Common Values: B = 0.97, F = 334.9 kJ/kg, rho = 1000 kg/m^3
	//		Ex. If ground heat flux Q = 2.5 kJ/m^2, M = 7.696E-6 mm OR Q = 2.5 kJ/m^2/s, M = 2.77E-5 m/hr

	// Mh (m/hr) = the snow melt by sensible heat = 0.0000044*Kp*(Za*Zb)^(-1/6) * (Tair - 32) * UV
	//	Eq 5-10 (USACE, 1998) gave LHS term Mc in units in/day, w RHS coefficient 0.00629, product of heights in ft^2, temp in F, winds in miles/hr
	//	Eq 7 (Yang et al., 2011) is Eq 5-10 for units m/hr, and RHS product of heights in m^2, temp in C, winds in m/s; published coefficient incorrect
	//	Eq 7 (derivation) RHS coefficient represents unit conversions, which includes steps below and coefficients shown further down: 
	//		1. conversion (1/12)*0.3048*(1/24) to move between in/day to m/hr
	//		2. conversion (1/(5280/1*0.3048/1*1/3600)) to move between mile/hr to m/s
	//		3. conversion (1/0.3048^2)^(-1/6) to move between (ft^2)^(-1/6) to (m^2)^(-1/6)
	//		4. converstion (9/5) to move between F and C
	//	Eq 7 (derivation) with LHS term Mh in units m/hr, w RHS coefficient 0.0000066569, product of heights in ft^2, temp in F, winds in miles/hr
	//	Eq 7 (derivation) with LHS term Mh in units m/hr, w RHS coefficient 0.000014891, product of heights in ft^2, temp in F, winds in m/s
	//	Eq 7 (derivation) with LHS term Mh in units m/hr, w RHS coefficient 0.000010021, product of heights in m^2, temp in F, winds in m/s
	//	Eq 7 (derivation) with LHS term Mh in units m/hr, w RHS coefficient 0.0000180387, product of heights in m^2, temp in C, winds in m/s
	//	Eq 7 (derivation) with LHS term Mh in units m/hr, w RHS coefficient 0.0000180387, product of heights in m^2, temp in C, winds in m/s

	// Me (m/hr) = snowmelt by latent heat.  0.0000124 has been arrived by applying (5.0/9.0) to 0.00000223
	//	Eq 5-12 (USACE, 1998) gave LHS term Me in units in/day, w RHS coefficient 0.054, product of heights in ft^2, vapor pressure in mb, winds in miles/hr
	//	Eq 8 (Yang et al., 2011) is Eq 5-12 for units m/hr, and RHS product of heights in m^2, temp in C, winds in m/s, but coefficient was incorrect
	//	Eq 5-13 (USACE, 1998) computes vapor pressure (mb) from air temperature (F) using a coefficient 0.339, 
	//	Eq 5-14 (USACE, 1998) created by substituting a fixed height into Eq 5-12, and combining Eq 5-12 with Eq 5-13; we don't want to use fixed heights
	//	Eq 8 (Yang et al., 2011) replaced RHS vapor pressure with air temperature, by using coefficient 0.339 from Eq 5-13 
	//	Eq 8 (derivation) RHS coefficient represents unit conversions, which includes: 
	//		1. conversion (1/12)*0.3048*(1/24) to move between in/day to m/hr
	//		2. conversion (1/(5280/1*0.3048/1*1/3600)) to move between mile/hr to m/s
	//		3. conversion (1/0.3048^2)^(-1/6) to move between (ft^2)^(-1/6) to (m^2)^(-1/6)
	//		4. converstion (9/5) to move between F and C
	//		5. conversion 0.339 to move between mb and F
	//	Eq 8 (derivation) with LHS term Me in units m/hr, w RHS coefficient 0.00005715, product of heights in ft^2, vapor in mb, winds in miles/hr
	//	Eq 8 (derivation) with LHS term Me in units m/hr, w RHS coefficient 0.000127841, product of heights in ft^2, vapor in mb, winds in m/s
	//	Eq 8 (derivation) with LHS term Me in units m/hr, w RHS coefficient 0.000086035, product of heights in m^2, vapor in mb, winds in m/s
	//	Eq 8 (derivation) with LHS term Me in units m/hr, w RHS coefficient 0.000029166, product of heights in m^2, temp in F, winds in m/s
	//	Eq 8 (derivation) with LHS term Me in units m/hr, w RHS coefficient 0.0000524985, product of heights in m^2, temp in C, winds in m/s

	//cte 20210506 testing 1st RHS coefficient
	// Eq 6 (Yang et al, 2011) or Eq 5-1 to 5-9 (USACE, 1998)
	// Mn (m⁄hr) =  the snowmelt by net radiation of both shortwave radiation and longwave radiation 
	//	The Eq 5-1 to 5-9 (USACE, 1998) account for back radiation as function of air temperature, rain or snow, and cloud cover effects
	//	Figure 5-1 (USACE, 1998) shows air temperature needs to be 70D for melt due to long wave radition in clear sky conditions
	//		Given for clear skies Fig 5-1 shows 0.22 inches/day melts when temperature is 70 F, Eq 5-8 should have 2nd coefficient of 0.8, not 0.84
	//		Given so little snow melts under clear skies, we should create overcast conditions based on vapor pressure or cloud data, not just P > 0
	//0.85 is the snow surface albedo
	double Mn = 0.000011 * RadiationNet_Wpm2 * (1 - 0.85);   
	// Ml (m/hr) = the snow melt 
	double Ml = 0.0;
	if (P > 0)  
		// Eq 5-9 (USACE, 1998) for cloudy sky - Having input precipitation assumes cloudy sky
		//	cte 20210507 Note the Error below
		// Eq 6 (Yang et al., 2011) for cloudy sky used Ml = 0.000054 * T (C), which is Ml = 0.00003069 * T (F); the below equation uses 0.00002244 * Tair (F)
		Ml = 0.0212 * 0.0010583 * (Tair - 32.0);   // Melt by long wave radiation on cloudy day
	else   
		// Eq 5-8 (USACE, 1998) for clear sky - Having no precip to input assumes clear sky
		Ml = 0.0212 * 0.0010583 * (Tair - 32.0) - 0.84 * 0.0010583;   // Melt by longwave radiation on clear day
	// Completing the math, we get the original numners: Ml = 0.00002243596*(Tair-32)(5.0/9.0) - 0.000888972     
	// If melt by long wave radiation < 0, set it to 0; This happens when clear sky and temperature below 70F
	if (Ml < 0)
		Ml = 0;
	// The total melt by radiaton  = the shortwave melt plus the long wave melt
	Mn = Mn + Ml;

	//cte 20210506 Eq 5-10 below has apparent error with 1st RHS coefficient; see my notes above
	// Eq 7 (Yang et al, 2011) or Eq 5-11 (USACE, 1998)
	// Mh (m/hr) = the snow melt by sensible heat = 0.0000044*Kp*(Za*Zb)^(-1/6) * (Tair - 32) * UV
	// The original equation used this constant: 0.00000792 m/hr/C.  This equation applies the fraction (5.0/9.0), from conversion of F to C, to get 0.0000044 m/hr/F
	double Mh = 0.0000044 * Kp * pow(Za * Zb, -(1.0/6.0)) * (Tair - 32) * UV;

	//cte 20210506 Eq 5-12 combined with Eq 5-13 below has apparent error with 1st RHS coefficient; see my notes above
	// Eq 8 (Yang et al, 2011) or Eq 5-12 (USACE, 1998)
	// Me (m/hr) = snowmelt by latent heat.  0.0000124 has been arrived by applying (5.0/9.0) to 0.00000223
	double Me = 0.0000124 * pow(Za * Zb, -(1.0/6.0)) * (Tdew - 32) * UV;


	// Mg (m⁄hr) is the snowmelt by the ground heat
	// USACE (1998) page 5-6, section 5-2.i. recommends use of a constant value between Mg = 0.0254 to 0.0762 cm/day, or 0.00001058 to 0.00003175 m/hr
	// USACE (1998) page 2-5, section 2-2.d. recommends use of a constant value between Qg = 0 to 5 kJ/m^2/s, converted in Eq 2-2, M = 0.0000277 m/hr
	double Mg = 0.0000277;    
	
	// Mp (m/hr) = snowmelt by precipitation.  
	//	Eq 2-9 (USACE, 1998) gave Qp = Cp_r * rho_w * P * (Tr - Ts)/1000, but RHS division by 1000 appears erroneous based on Eq 5-17 coefficient
	//	Eq 8 (Yang et al., 2011) is Eq 5-12 for units m/hr, and RHS product of heights in m^2, temp in C, winds in m/s, but coefficient was incorrect
	// Initializing the rainfall (Precuip) to 0.0
	double RainOpen = 0.0;   

	// If the temperature is above 32F, we are receiving rain
	//cte 20210506 Eq 5-17 below has apparent error with 1st RHS coefficient; see my notes in Excel worksheet
	if (Tair > 32.0)
	{
		RainOpen = P;
		Mp = 0.0129 * RainOpen * (Tair - 32);    // Update the snow melted by rain
	}
	
	// The temp is less than 32F, we are receiving snow
	if(Tair <= 32.0)
	{
		RainOpen = 0.0;
		// Update the snowOpen with the additional snow fall
		double snow = P + folder->VarDict["snowOpen"];
		folder->VarDict["snowOpen"] = snow;
	}


	// Modify the melted OpenSnow amount = melted snow from all the different melt pathways
	folder->VarDict["snowMeltOpen"] = Mn + Mp + Mg + Mh + Me;

	// If the metled snow is somehow less than 0 (meaning snow was gained through the melt processes) set it to 0
	if (folder->VarDict["snowMeltOpen"] < 0.0)
		folder->VarDict["snowMeltOpen"] = 0.0;

	// If the melted snow is > snowOpen, set it equal to snowOpen
	if(folder->VarDict["snowMeltOpen"] > folder->VarDict["snowOpen"])
		folder->VarDict["snowMeltOpen"] = folder->VarDict["snowOpen"];

	// Update the snowOpen to reflect the loss of the melted snow
	folder->VarDict["snowOpen"] = folder->VarDict["snowOpen"] - folder->VarDict["snowMeltOpen"];
			
	// Read in the potential Evap for Snow		
	double SnowEP = input->PESnow[timeStep];
	
	// The amount of snow that could sublimate is > snow available, set equal to snowOpen (snow available)
	if (SnowEP > folder->VarDict["snowOpen"])
		SnowEP = folder->VarDict["snowOpen"];
	
	// Update snowOpen area to reflect the loss of the sublimated snow
	double snowLeft = folder->VarDict["snowOpen"] - SnowEP;
	folder->VarDict["snowOpen"] = snowLeft;

	// Update the TotalWaterFromOpenArea (this folder, this timestep) to be = (RainOpen (Precip_mpdt falling on the open area) + MeltedSOpen)
	folder->VarDict["waterOpenArea"] = folder->VarDict["snowMeltOpen"] + RainOpen;

	// updating the snow evap under tree
	folder->VarDict["snowEvapOpen"] = SnowEP;

}


void SnowMeltOpenAreaCalc::storeValues(Inputs *input, DataFolder *folder, int timeStep)
{
	folder->VarDict["totalSnowEvapOpen_mm"] += folder->VarDict["snowEvapOpen"];
	if (input->SimulationStringParams["Model"]=="StatisticalHydro")
	{
		// Updating vectors
		folder->VarVecDict["snowEvapOpen"].emplace_back(folder->VarDict["snowEvapOpen"]);
		folder->VarVecDict["snowMeltOpen"].emplace_back(folder->VarDict["snowMeltOpen"]);
		folder->VarVecDict["snowOpen"].emplace_back(folder->VarDict["snowOpen"]);
		//cte 20210517 debugging snow balance
		//cout << "SnowMeltOpenAreaCalc snowMeltOpen: " << folder->VarDict["snowMeltOpen"] << endl;

	}
}
