#include "TestSimulation.h"
#include "Interception/TreeInterceptionCalc.h"
#include "Interception/ShortVegInterceptionCalc.h"
#include "Evaporation/TreeEvaporationCalc.h"
#include "Evaporation/ShortVegEvaporationCalc.h"
#include "ThroughFlow/PerviousAreaThroughFlowCalc.h"
#include "SnowMelt/SnowMeltOpenAreaCalc.h"
#include "SnowMelt/SnowMeltUnderShortVegCalc.h"
#include "SnowMelt/SnowMeltUnderTreeCalc.h"
#include "WaterOnImpArea/WaterOnImperviousAreaCalc.h"
#include "ImperviousDepressionStorage/ImperviousDepressionStorageCalc.h"
#include "WaterOnPerArea/WaterOnPerviousAreaCalc.h"
#include "PerviousDepressionStorage/PerviousDepressionStorageCalc.h"
#include "InfiltrationPowerDecay/InfiltrationPowDecayCalc.h"
#include "InfiltrationExpDecay/InfiltrationExpDecayCalc.h"
#include "SemiAveSMD/SemiAveSMDCalc.h"
#include "SemiAveSMD/LocalSoilMoistureCalc.h"
#include "SatZone/SaturatedBaseFlowExpDecayCalc.h"
#include "SatZone/SaturatedBaseFlowPowDecay.h"
#include "Flow/FlowRouteCalc.h"
#include "Flow/FlowSumCalc.h"
#include "StoreTimeStepData/StoreTimeStepTotalsCalc.h"
#include "CoolAir/HeatFluxCal.h"
#include "CoolAir/TemperatureOutputWriter.h"
#include "CoolAir/BlockGroupHandler.h"
#include "CoolAir/ICTCInterpolater.h"
#include "GI/GI_Outlets.h"
#include "CoolRiver/SolarCalculation.h"
#include "CoolRiver/StreamTemperature.h"
#include "Buffer/DynamicFunctions.h"
#include "Outputs/CoolRiverOutputWriter.h"
#include "Outputs/WriteOutputs.h"
#include <algorithm>
#include <limits>
#include <cstddef>
#include <string>
#include <initializer_list>

void TestSimulation::Go(DataOrganizer *organizer, Inputs *input, std::string inputDirectory)
{
	std::cout << "Model: " << input->SimulationStringParams["Model"] << std::endl;
	std::cout << "Infiltration: " << input->SimulationStringParams["Infiltration"] << std::endl;
	std::cout << "Routing: " << input->SimulationStringParams["Routing"] << std::endl;
	std::cout << "Running Simulation..." << std::endl;
	LogMessage::log("Model: " + input->SimulationStringParams["Model"]);
	LogMessage::log("Infiltration: " + input->SimulationStringParams["Infiltration"]);
	LogMessage::log("Routing: " + input->SimulationStringParams["Routing"]);
	LogMessage::log("Running Simulation...");
	if (input->SimulationStringParams["Model"] == "StatisticalHydro")
	{
		RunStatisticalModel(organizer, input, inputDirectory);
	}
	else if (input->SimulationStringParams["Model"] == "SpatialTemperatureHydro" || input->SimulationStringParams["Model"] == "LCScalingTempSpatial")
	{
		RunTemperatureSpatialModel(organizer, input, inputDirectory);
	}
	else if (input->SimulationStringParams["Model"] == "CoolRiver")
	{
		RunCoolRiverModel(organizer, input, inputDirectory);
	}
	if (input->SimulationStringParams["Model"] == "ECDynamic")
	{
		DynamicFunctions::YearAverage(input->DischargeRecord, input->year, input);
	}
}

void TestSimulation::RunTemperatureSpatialModel(DataOrganizer *organizer, Inputs *input, std::string inputDirectory)
{
	BlockGroupHandler handleBlockGroups(input);
	std::vector<double> waterOnValues;
	HeatFluxCal heatC(input);
	//SimulationNumericalParams["TotalTimeSteps"] initially defined in HydroPlusConfig.xml, and redefined in Inputs::readHydroPlusConfigFile() if value in HydroPlusConfig.xml = 0 or empty, then TotalTimeSteps = daysDiff * 24.0
	bool needDailyBGFiles = (input->SimulationNumericalParams["TotalTimeSteps"] >= 24);
	bool needHourlyBGFiles = (input->TemperatureExecutionParams["PrintBlockGroupDailyAndHourly"] == "true" || input->TemperatureExecutionParams["PrintBlockGroupDailyAndHourly"] == "True");
	TemperatureOutputWriter::generateBlockFiles(handleBlockGroups.BlockGroupIDToItsFolder, input->SimulationStringParams["OutputDirectory"], needHourlyBGFiles, needDailyBGFiles);
	TemperatureOutputWriter::generateRowColFiles(organizer, input);
	int refLocation = input->TemperatureCalculationParams["RefWeatherLocation"];

	DataFolder *refLocationFolder = organizer->DataDrawers[refLocation][0];
	double waterOnValuesIndex = 0;

	//SimulationNumericalParams["TotalTimeSteps"] initially defined in HydroPlusConfig.xml, and redefined in Inputs::readHydroPlusConfigFile() if value in HydroPlusConfig.xml = 0 or empty, then TotalTimeSteps = daysDiff * 24.0
	for (int timeStep = 0; timeStep < input->SimulationNumericalParams["TotalTimeSteps"]; ++timeStep)
	{
		if (timeStep > 0)
		{
			heatC.VariableForAllfolders(timeStep, input);
			heatC.assignRefTaTd(input, timeStep);
			heatC.assignUrbanParam(input->NLCDLandCoverData[refLocation]);
			heatC.CalaveHeight(refLocationFolder->ParamDict["TreeP"], refLocationFolder->ParamDict["ShortVegP"], refLocationFolder->ParamDict["SoilP"], refLocationFolder->ParamDict["WaterP"], refLocationFolder->ParamDict["ImperP"], input->NLCDLandCoverData[refLocation]); // rcc 4/22/2020 modified to fix aveHeight calc as per 4/13/2020 email with Ted et al
			heatC.StateCal(refLocationFolder, timeStep, input, refLocation); // calculate meso-scale temperature and humidity
			heatC.IterationCalTb(refLocationFolder, timeStep);
			heatC.savefolderValue(refLocationFolder, refLocationFolder, timeStep, refLocation);
		}

		for (int organizerLocationIndex = 0; organizerLocationIndex < organizer->DataDrawers.size(); ++organizerLocationIndex)
		{
			for (int folderIndex = 0; folderIndex < organizer->DataDrawers[organizerLocationIndex].size(); ++folderIndex)
			{
				if (organizer->DataDrawers[organizerLocationIndex][folderIndex]->ParamStringDict["Type"] == "NODATA")
				{
					//std::cout<<"continue";
				}
				else
				{
					if (timeStep > 0 && organizerLocationIndex != refLocation)
					{
						heatC.assignRefTaTd(input, timeStep);
						heatC.assignUrbanParam(input->NLCDLandCoverData[organizerLocationIndex]);
						heatC.CalaveHeight(organizer->DataDrawers[organizerLocationIndex][folderIndex]->ParamDict["TreeP"], organizer->DataDrawers[organizerLocationIndex][folderIndex]->ParamDict["ShortVegP"], organizer->DataDrawers[organizerLocationIndex][folderIndex]->ParamDict["SoilP"], organizer->DataDrawers[organizerLocationIndex][folderIndex]->ParamDict["WaterP"], organizer->DataDrawers[organizerLocationIndex][folderIndex]->ParamDict["ImperP"], input->NLCDLandCoverData[organizerLocationIndex]); // rcc 4 / 22 / 2020 modified to fix aveHeight calc as per 4 / 13 / 2020 email with Ted et al
						heatC.StateCal(organizer->DataDrawers[organizerLocationIndex][folderIndex], timeStep, input, organizerLocationIndex); // calculate meso-scale temperature and humidity
						heatC.LocalTCal(organizer->DataDrawers[organizerLocationIndex][folderIndex], refLocationFolder, timeStep, input->TemperatureCalculationParams["IterationCount"]);
						heatC.savefolderValue(organizer->DataDrawers[organizerLocationIndex][folderIndex], refLocationFolder, timeStep, organizerLocationIndex);
					}
					runCalculations(organizer->DataDrawers[organizerLocationIndex][folderIndex], input, timeStep, organizer->DataDrawers[organizerLocationIndex]);
					//cte 20201111 Program never calls FlowSumCalc::calculateSpatial; is this mistakenly undeveloped?
					FlowSumCalc::addfolderFlowValues(input, organizer->DataDrawers[organizerLocationIndex][folderIndex], timeStep);
					TemperatureOutputWriter::rowColFileBatchProcessor(timeStep, organizerLocationIndex, organizer->DataDrawers[organizerLocationIndex][folderIndex], input->SimulationStringParams["OutputDirectory"], input);
					if (timeStep > 0)
						handleBlockGroups.AddToBlockGroup(organizerLocationIndex, input, organizer->DataDrawers[organizerLocationIndex][folderIndex]);
					if (timeStep == 1 && input->SimulationStringParams["Model"] == "LCScalingTempSpatial")
						TemperatureOutputWriter::outputLandCoverDetails(organizerLocationIndex, input->SimulationStringParams["OutputDirectory"], organizer->DataDrawers[organizerLocationIndex][folderIndex]);
				}
			}
		}
		//cte 20201111 Why does RunTemperatureSpatialModel call FlowRouteCalc::calculateStatistical rather than Spatial? Can they use a shared routine?
		FlowRouteCalc::calculateStatistical(input, timeStep);
		if (timeStep > 0)
			handleBlockGroups.TimeStepProcessing(input, timeStep);
		TemperatureOutputWriter::timeStepResultsFileProcessor(timeStep, organizer, input->SimulationStringParams["OutputDirectory"], input);
	}
	if (input->TemperatureExecutionParams["LCScalingOutput"] == "Yes")
	{
		ICTCInterpolator interpolator;
		std::cout << "===" << std::endl;
		std::cout << "Running LCScalingOutput interpolation" << std::endl;
		LogMessage::log("Running LCScalingOutput interpolation");
		std::cout << "===" << std::endl;
		interpolator.interpolateData(input);
	}
	std::cout << "===" << std::endl;
	std::cout << "Writing output to " << std::endl;
	LogMessage::log("Writing output to " + input->SimulationStringParams["OutputDirectory"]);
	std::cout << input->SimulationStringParams["OutputDirectory"] << std::endl;
	std::cout << "===" << std::endl;
}

void TestSimulation::RunCoolRiverModel(DataOrganizer *organizer, Inputs *input, std::string inputDirectory)
{
	std::vector<double> solarAzimuth;
	std::vector<double> solarAltitude;

	SolarCalculation *sc = new SolarCalculation(input);
	int timeSteps = input->WindSp_mps.size();
	//check that SolarRadiation.dat & Weather.dat have enough rows for totTime param in config
	if (input->WindSp_mps.size() != timeSteps)
	{
		std::ofstream logfile("log.txt", ios::app);
		logfile << "Weather.dat data entries should equal totTime param in config";
		logfile << "\n";
		logfile.close();
		return;
	}
	int x = input->DirectShortWaveRadiation.size();
	if (input->DirectShortWaveRadiation.size() != timeSteps)
	{
		std::ofstream logfile("log.txt", ios::app);
		logfile << "SolarRadiation.dat data entries/rows should be equal totTime param in config";
		logfile << "\n";
		logfile.close();
		return;
	}
	for (int ts = 0; ts < timeSteps; ++ts)
	{
		sc->SolarHour(ts);
		solarAltitude.push_back(sc->getSolarAltitude());
		solarAzimuth.push_back(sc->getSolarAzimuth());
	}
	StreamTemperature *st = new StreamTemperature(input, sc); 
	st->avgTemperature();
	//TotalTimeMod (hr) is total time steps for model simulation in time step of hour, later converted to minute; this is same time step for input RiverTimeSeries.txt, RiverSWTimeSeries.txt and RiverGWTimeSeries.txt
	//SimulationNumericalParams["TotalTimeSteps"] initially defined in HydroPlusConfig.xml, and redefined in Inputs::readHydroPlusConfigFile() if value in HydroPlusConfig.xml = 0 or empty, then TotalTimeSteps = daysDiff * 24.0
	int TotalTimeMod = input->SimulationNumericalParams["TotalTimeSteps"];

	int TotalDistMod = input->CoolRiverVariables["CR_totDist"];
	int dx_Uns = input->CoolRiverVariables["CR_unsteady_dx"];
	int dx_HR = input->CoolRiverVariables["CR_interval_dx"];
	double dtime_IntervalHR = input->CoolRiverVariables["CR_unsteady_dt"]; // in second
	int dt_HR = input->CoolRiverVariables["CR_interval_dt"];
	int calcMethod = input->CoolRiverVariables["CR_calcMethod"];
	int TotalTimeModHR = TotalTimeMod;
	int TotalDist_Uns;
	if (calcMethod == 3) {
		TotalDist_Uns = (TotalDistMod / dx_Uns) + 1;
	}
	else {
		TotalDist_Uns = (TotalDistMod / dx_Uns);
	}
	int TotalDist_Uns_HR = (TotalDistMod / dx_HR);

	// Crank-Nicolson method
	int TotalDistMod_CN = 1;
	int TotalTimeMod_CN = 1;
	if (calcMethod == 1)
	{
		TotalDistMod_CN = TotalDistMod;
		TotalTimeMod_CN = TotalTimeMod;
	}
	//All Calculations for calculationMethod1 
	CoolRiverOutputWriter::calculationMethod1(input, inputDirectory, st, calcMethod, TotalDistMod_CN, TotalTimeMod_CN);

	int TotalDistMod_FD = 1;
	int TotalTimeUns = 1;
	if (calcMethod == 2)
	{
		TotalDistMod_FD = TotalDistMod;
		TotalTimeUns = TotalTimeMod * 60;
	}
	//Creating flag to redirect program around block that will be moved to new .cpp file
	//Creating CoolRiverOutputWriter.cpp in Outputs folder
	//iflag_coolRiverOutput = 1 will enter this section; = 0 will bypass to new .cpp file
	int iflag_coolRiverOutput = 1;
	if (iflag_coolRiverOutput == 1) {

	}
	//All Calculations for calculationMethod2 
	CoolRiverOutputWriter::calculationMethod2(input, inputDirectory, st, calcMethod, TotalDistMod_FD, TotalTimeUns, TotalDist_Uns, TotalTimeMod);

	int TotalDistMod_HR = 1;
	int TotalTime_HR = 1;
	if (calcMethod == 3)
	{
		TotalDistMod_FD = TotalDistMod;
		TotalTime_HR = (TotalTimeModHR - 1) * 60;
	}
	//as 20210115 All Calculations for calculationMethod3 
	CoolRiverOutputWriter::calculationMethod3(input, inputDirectory, st, calcMethod, TotalDistMod_FD, TotalTime_HR, TotalDist_Uns);

	switch (calcMethod)
	{
	case 1:
		
		if (input->CoolRiverVariables["CR_Minute_output"] == 0) {
			//When CR_Minute_output is 0, then we are setting values in output for calcMethod1
			CoolRiverOutputWriter::swtichCalMethod1True(input, st, TotalDistMod, TotalTimeMod);
		}
		else {
			//When CR_Minute_output is other than 0, then we are setting values in output for calcMethod1
			CoolRiverOutputWriter::swtichCalMethod1False(input, st, TotalDist_Uns, TotalTimeMod_CN);
		}
		break;

	case 3:
		if (input->CoolRiverVariables["CR_Minute_output"] == 0) {
			if (input->CoolRiverVariables["CR_Extended"] == 1) {
				//When CR_Minute_output is 0 and CR_Extended is 1, then we are setting values in output for calcMethod3
				CoolRiverOutputWriter::swtichCalMethod3ExtTrue(input, st, TotalDist_Uns_HR, TotalTimeModHR);

			}
			//When CR_Minute_output is 0, then we are setting values in output for calcMethod3
			CoolRiverOutputWriter::swtichCalMethod3True(input, st, TotalDist_Uns_HR, TotalTimeModHR);
		}
		else {
			if (input->CoolRiverVariables["CR_Extended"] == 1) {
				//When CR_Minute_output is other than 0 and CR_Extended is 1, then we are setting values in output for calcMethod3
				CoolRiverOutputWriter::swtichCalMethod3FalseExtTrue(input, st, TotalDist_Uns_HR, TotalTime_HR);

			}
			//When CR_Minute_output is other than 0, then we are setting values in output for calcMethod3
			CoolRiverOutputWriter::swtichCalMethod3False(input, st, TotalDist_Uns_HR, TotalTime_HR);
		}
		break;
	case 2:
		if (input->CoolRiverVariables["CR_Minute_output"] == 0) {
			if (input->CoolRiverVariables["CR_Extended"] == 1) {
				//When CR_Minute_output is 0 and CR_Extended is 1, then we are setting values in output for calcMethod2
				CoolRiverOutputWriter::swtichCalMethod2ExtTrue(input, st, TotalDist_Uns, TotalTimeMod);
			}
			//When CR_Minute_output is other than 0 , then we are setting values in output for calcMethod2
			CoolRiverOutputWriter::swtichCalMethod2True(input, st, TotalDist_Uns, TotalTimeMod);
		}
		else {
			if (input->CoolRiverVariables["CR_Extended"] == 1) {
				//When CR_Minute_output other than 0 and CR_Extended is 1, then we are setting values in output for calcMethod2
				CoolRiverOutputWriter::swtichCalMethod2FalseExtTrue(input, st, TotalDist_Uns, TotalTimeMod);

			}
			//When CR_Minute_output is other than 0, then we are setting values in output for calcMethod2
			CoolRiverOutputWriter::swtichCalMethod2False(input, st, TotalDist_Uns, TotalTimeMod);

		}
		break;

	default:
		std::ofstream logfile3("log.txt", ios::app);
		logfile3 << "Wrong modeling methodology code! Check the methodology code and try again.";
		logfile3 << "\n";
		logfile3.close();
		cout << "Wrong modeling methodology code! Check the methodology code and try again." << endl;
		break;
	}

	delete sc;
	std::ofstream logfile4("log.txt", ios::app);
	logfile4 << "End of Simulation...";
	logfile4 << "\n";
	logfile4.close();
	std::cout << "End of Simulation..." << std::endl;
}

void TestSimulation::RunStatisticalModel(DataOrganizer *organizer, Inputs *input, std::string inputDirectory)
{
	//GI_FolderOperational flag set = 0 at start of simulation, set = 1 in TestSimulation::runCalculations if HydroPlusConfig.xml has key <Type> != BulkArea
	//GI_FolderOperational regulates GI to BulkArea water transfer. Perhaps can set this in BuildDataOrganizer.cpp 
	input->RepoDict["GI_FolderOperational"] = 0;

	//SimulationNumericalParams["TotalTimeSteps"] initially defined in HydroPlusConfig.xml, and redefined in Inputs::readHydroPlusConfigFile() if value in HydroPlusConfig.xml = 0 or empty, then TotalTimeSteps = daysDiff * 24.0
	for (int timeStep = 0; timeStep < input->SimulationNumericalParams["TotalTimeSteps"]; ++timeStep)
	{
		//cte 20210917 Can we structure loop of DataDrawers on simpler functions than Tree.cpp t.WalkTree 
		for (int organizerLocationIndex = 0; organizerLocationIndex < organizer->DataDrawers.size(); ++organizerLocationIndex)
		{
			for (int folderIndex = 0; folderIndex < organizer->DataDrawers[organizerLocationIndex].size(); ++folderIndex)
			{
				runProcesses(organizer->DataDrawers[organizerLocationIndex][folderIndex], input, timeStep, organizer->DataDrawers[organizerLocationIndex]);
			}
		}
		//FlowSumCalc will loop through the DataDrawers and DataFolders, only summing BulkArea flows for the BulkArea DataDrawer
		FlowSumCalc::calculateStatistical(input, timeStep, organizer->DataDrawers);
		//FlowRouteCalc will route summed flows from BulkArea
		FlowRouteCalc::calculateStatistical(input, timeStep);
		//StoreTimeStepTotalsCalc will loop through DataDrawers and DataFolders 
		StoreTimeStepTotalsCalc::calculate(input, timeStep, organizer->DataDrawers);
	}
	std::cout << "===" << std::endl;
	std::cout << "Writing output to " << std::endl;
	std::cout << input->SimulationStringParams["OutputDirectory"] << std::endl;
	std::cout << "===" << std::endl;
	LogMessage::log("Writing output to " + input->SimulationStringParams["OutputDirectory"]);
	WriteOutputs::writeStatisticalOutput(organizer, input);
}

void TestSimulation::runCalculations(DataFolder *folder, Inputs *input, int timeStep, std::vector<DataFolder *> &drawer)
{
	std::map<std::string, void (*)(Inputs * input, DataFolder * folder, int timeStep)> functors;
	if (input->SimulationStringParams["Model"] == "StatisticalHydro")
	{
		functors.emplace("calculateTreeEvaporation", TreeEvaporationCalc::calculateTreeEvaporationStatistical);
		functors.emplace("calculateShortVegEvaporation", ShortVegEvaporationCalc::calculateShortVegEvaporationStatistical);
		functors.emplace("calculatePerDepEvap", PerviousDepressionStorageCalc::calculatePerDepEvapStatistical);
		functors.emplace("calculateImpEvap", ImperviousDepressionStorageCalc::calculateImpEvapStatistical);
	}
	if (input->SimulationStringParams["Model"] == "SpatialTemperatureHydro" || input->SimulationStringParams["Model"] == "LCScalingTempSpatial")
	{
		functors.emplace("calculateTreeEvaporation", TreeEvaporationCalc::calculateTreeEvaporationTemperature);
		functors.emplace("calculateShortVegEvaporation", ShortVegEvaporationCalc::calculateShortVegEvaporationTemperature);
		functors.emplace("calculatePerDepEvap", PerviousDepressionStorageCalc::calculatePerDepEvapTemperature);
		functors.emplace("calculateImpEvap", ImperviousDepressionStorageCalc::calculateImpEvapTemperature);
	}

	if (input->SimulationStringParams["Infiltration"] == "PowerDecay")
	{
		functors.emplace("calculateInfiltration", SoilInfiltrationPowDecayCalc::calculate);
		functors.emplace("calculateSaturatedBaseFlow", SaturatedBaseFlowPowDecayCalc::calculate);
	}
	if (input->SimulationStringParams["Infiltration"] == "ExponentialDecay")
	{
		functors.emplace("calculateInfiltration", SoilInfiltrationExpDecayCalc::calculate);
		functors.emplace("calculateSaturatedBaseFlow", SaturatedBaseFlowExpDecayCalc::calculate);
	}

	//BulkArea DataFolder processes
	if (folder->ParamStringDict["Type"] == "BulkArea")
	{
		//BF_flag = 1.0 for BulkArea DataFolder is used as coefficient in basflow computation within functor calculateSaturatedBaseFlow (SaturatedBaseFlowExpDecayCalc or SaturatedBaseFlowPowDecay)
		//	GI DataFolder gets BF_flag from HydroPlusConfig.xml, and uses default = 0 to turn baseflow off.
		folder->ParamDict["BF_flag"] = 1.0;

		TreeInterceptionCalc::calculate(input, folder, timeStep);
		ShortVegInterceptionCalc::calculate(input, folder, timeStep);
		functors["calculateTreeEvaporation"](input, folder, timeStep);
		functors["calculateShortVegEvaporation"](input, folder, timeStep);
		PerviousAreaThroughFlowCalc::calculate(input, folder, timeStep);
		SnowMeltOpenAreaCalc::calculate(input, folder, timeStep);
		SnowMeltUnderTreeCalc::calculate(input, folder, timeStep);
		SnowMeltUnderShortVegCalc::calculate(input, folder, timeStep);
		WaterOnImperviousAreaCalc::calculate(input, folder, timeStep);
		ImperviousDepressionStorageCalc::calculate(input, folder, timeStep, functors);
		WaterOnPerviousAreaCalc::calculate(input, folder, timeStep, drawer);
		PerviousDepressionStorageCalc::calculate(input, folder, timeStep, functors);
		functors["calculateInfiltration"](input, folder, timeStep);
		SemiAveSMDCalc::calculate(input, folder, timeStep);
		LocalSoilMoistureCalc::calculate(input, folder, timeStep);
		functors["calculateSaturatedBaseFlow"](input, folder, timeStep);
	}

	//GI DataFolder processes
	if (folder->ParamStringDict["Type"] != "BulkArea")
	{
		//cte 20210408 Determine method to read in these values earlier (perhaps BuildDataOrganizer), so they can be used in BulkArea during 1st time step
		//GI_FolderOperational flag set = 1, used to regulate GI to BulkArea water transfer
		input->RepoDict["GI_FolderOperational"] = 1;
		input->RepoDict["GI_upSlopeArea"] = folder->ParamDict["upSlopeArea"];
		//PerAreaTreated_frac (frac) is fraction of bulk area upslope pervious area that drains to GI
		input->RepoDict["BulkAreaPerArea_DrainingToGI_frac"] = folder->ParamDict["PerAreaTreated_frac"];
		//ImpAreaTreated_frac (frac) is fraction of bulk area upslope directly connected impervious area (DCIA) that drains to GI
		//	Bulk Area DCIA_frac should equal 1 if the goal is to drain all bulk area imperviousness to GI
		input->RepoDict["BulkAreaDCIA_DrainingToGI_frac"] = folder->ParamDict["ImpAreaTreated_frac"];

		TreeInterceptionCalc::calculate(input, folder, timeStep);
		ShortVegInterceptionCalc::calculate(input, folder, timeStep);
		functors["calculateTreeEvaporation"](input, folder, timeStep);
		functors["calculateShortVegEvaporation"](input, folder, timeStep);
		PerviousAreaThroughFlowCalc::calculate(input, folder, timeStep);
		SnowMeltOpenAreaCalc::calculate(input, folder, timeStep);
		SnowMeltUnderTreeCalc::calculate(input, folder, timeStep);
		SnowMeltUnderShortVegCalc::calculate(input, folder, timeStep);
		WaterOnImperviousAreaCalc::calculate(input, folder, timeStep);
		ImperviousDepressionStorageCalc::calculate(input, folder, timeStep, functors);
		WaterOnPerviousAreaCalc::calculate(input, folder, timeStep, drawer);
		PerviousDepressionStorageCalc::calculate(input, folder, timeStep, functors);

		//GI_WaterBalanceInitial prepares GI variables, setting many to zero to initialize water balance
		GI_Outlets::GI_WaterBalanceInitial(folder);
		//GI_SurfaceInflowSurfaceOutflow transfers BulkArea runoff into GI; runon into GI generated by BulkArea Runoff_PervInfilEx_m, Runoff_SatExcess_m, Runoff_ImperviousToOutlet_m
		GI_Outlets::GI_SurfaceInflowSurfaceOutflow(input, folder, timeStep);

		//Update aveSMD, subtracting Drainage_UnsatZone_m and adding baseflow, then combine with BulkArea aveSMD
		functors["calculateInfiltration"](input, folder, timeStep);
		SemiAveSMDCalc::calculate(input, folder, timeStep);
		LocalSoilMoistureCalc::calculate(input, folder, timeStep);
		//cte 20210715 cout << "TestSimulation.cpp AveSMD_GI: " << folder->VarDict["AveSMD_GI"] << " at time step: " << timeStep << endl;
		//Calling GI subroutines, which proceed as: GI_WaterBalancePrepareUpdate > GI_WaterBalanceUpdate > GI_FluxRatesAllTypesInitialize > GIFluxRates (e.g., GI_FluxRatesSwale) which updates terms in GI_WaterBalanceUpdate
		GI_Outlets::GI_WaterBalancePrepareUpdate(input, folder, timeStep);

		//Subsurface exfiltration to native soil from GI enters subsurface flow from BulkArea w/ 1 timestep delay
		//		transfer of water from GI to BulkArea uses input->RepoDict["storageExfil_m3"] into folder->VarDict["Drainage_UnsatZone_m"] 

		//Surface runoff from GI enters BulkArea directly connected impervious area runoff w/ i timestep delay; must enter FlowSumCalc.cpp to avoid recirculating as flow into GI
		//		transfer of water from GI to BulkArea uses input->RepoDict["surfaceOutflow_m3"] andinput->RepoDict["storageDrain_m3"] into double folderDCIAQ
	}
}

void TestSimulation::runProcesses(DataFolder *folder, Inputs *input, int timeStep, std::vector<DataFolder *> &stafolders)
{

	if ((folder->ParamStringDict["Type"] == "BulkArea") || (folder->ParamStringDict["Type"] == "RainGarden") ||
		(folder->ParamStringDict["Type"] == "BioRetention") || (folder->ParamStringDict["Type"] == "GreenRoof") ||
		(folder->ParamStringDict["Type"] == "RainBarrel") || (folder->ParamStringDict["Type"] == "PermeablePavement") ||
		(folder->ParamStringDict["Type"] == "Swale") || (folder->ParamStringDict["Type"] == "InfilTrench") ||
		(folder->ParamStringDict["Type"] == "RoofDiscon"))
	{
		runCalculations(folder, input, timeStep, stafolders);
	}
	else
	{
		LogMessage::log("Cell type not yet supported.");
		std::cout << "Cell type not yet supported." << std::endl;
	}
}