#include "UnsatZoneDrainageCalc.h"

//Unsaturated zone storage capacity is only gravitational water, and root zone storage capacity is only capillarity held water
//	The unsaturated zone extends from the groundwater table to the ground, and accounts for all gravitational held water
//	The root zone storage depth extends to the limit of evapotranspiration accessing capillarity held water, a constant for all TI bins
//	The two zones are conceptualy situated alongside each other; if the unsaturated zone extends to greater depths it only represents gravitational water

void UnsatZoneDrainageCalc::calculate(Inputs *input, DataFolder *folder, int timeStep)
{
	//initialize VarDict["iA"] as current index value of topographic index; used for passing index value to separate function while in loop
	int ia = folder->VarDict["iA"];

	//if start of simulation then initialize variables
	if (timeStep == 0)
	{
		//Storage_Used_UnsatZone_TI_m (m) is unsaturated zone storage, initialized to 0
		folder->VarVecDict["Storage_Used_UnsatZone_TI_m"].push_back(0.0);
		//Runoff_SatExcess_TI_m (m) is saturation excess runoff, initialized to 0
		folder->VarVecDict["Runoff_SatExcess_TI_m"].push_back(0.0);
		//Drainage_UnsatZone_TI_m (m) is unsaturated zone runoff, initialized to 0
		folder->VarVecDict["Drainage_UnsatZone_TI_m"].push_back(0.0);
	}

	//Storage routine to determine saturation excess runoff and drainage
	//Runoff_SatExcess_TI_m (m) is reset to 0 at each time step
	folder->VarVecDict["Runoff_SatExcess_TI_m"][ia] = 0.0;

	//Storage_Used_UnsatZone_TI_m (m) is increased by Drainage_RootZone_TI_m (m) flowing out of root zone, inluding Drainage_macroPore_m (m), created in RootZoneDrainageCalc.cpp
	// TOPMODEL code uses: IF(SRZ(IA).LT.0.)THEN SUZ(IA) = SUZ(IA) - SRZ(IA) and SRZ(IA) = 0; note SRZ(IA) is a deficit, negative deficits are flow into SUZ 
	folder->VarVecDict["Storage_Used_UnsatZone_TI_m"][ia] += folder->VarVecDict["Drainage_RootZone_TI_m"][ia];

	//if Storage_Used_UnsatZone_TI_m (m) > Storage_Max_UnsatZone_TI_m (m), saturation excess runoff occurs; note Storage_Used_UnsatZone_TI_m is a storage and Storage_Max_UnsatZone_TI_m is a deficit (or capacity)
	// TOPMODEL code uses: IF(SUZ(IA).GT.SD(IA)) THEN EX(IA) = SUZ(IA) - SD(IA) and SUZ(IA)=SD(IA)
	if (folder->VarVecDict["Storage_Used_UnsatZone_TI_m"][ia] > folder->VarVecDict["Storage_Max_UnsatZone_TI_m"][ia])
	{
		//Runoff_SatExcess_TI_m (m) equals Storage_Used_UnsatZone_TI_m (m) - Storage_Max_UnsatZone_TI_m (m); note Storage_Used_UnsatZone_TI_m is a storage and Storage_Max_UnsatZone_TI_m is a deficit (or capacity)
		folder->VarVecDict["Runoff_SatExcess_TI_m"][ia] = folder->VarVecDict["Storage_Used_UnsatZone_TI_m"][ia] - folder->VarVecDict["Storage_Max_UnsatZone_TI_m"][ia];

		//Storage_Used_UnsatZone_TI_m (m) reset to equal the Storage_Max_UnsatZone_TI_m (m) deficit (or capacity)
		folder->VarVecDict["Storage_Used_UnsatZone_TI_m"][ia] = folder->VarVecDict["Storage_Max_UnsatZone_TI_m"][ia];
	}

	//Update soil moisture terms to remove infiltrated water that was rejected due to saturation excess runoff 
	//	terms included: Drainage_RootZone_m (m), Infiltration_viaInfilEx_m (m), Infiltration_viaSatEx_m (m), and maybe Drainage_macroPore_m (m) if PMacro = 1.
	//	terms not included: Storage_Used_UnsatZone_TI_m (m) was updated when it is reset to Storage_Max_UnsatZone_TI_m; Infiltration_m (m) is updated at end of TI for loop
	//	extra weighting by Drainage_RootZone_Infil_frac (frac) or Drainage_RootZone_MacroPore_frac (frac), and InfiltExcessGovernedArea_frac
	//	HydroPlus weights discrete TI parameter values, i.e, SoilParameter[ia], with TI_binArea_frac, catchment area bounding discrete value
	folder->VarDict["Drainage_RootZone_m"] = folder->VarDict["Drainage_RootZone_m"] - folder->VarVecDict["Runoff_SatExcess_TI_m"][ia] * folder->VarDict["TI_binArea_frac"];
	folder->VarDict["Infiltration_viaSatEx_m"] = folder->VarDict["Infiltration_viaSatEx_m"] - folder->VarVecDict["Runoff_SatExcess_TI_m"][ia] * (1 - folder->ParamDict["InfiltExcessGovernedArea_frac"]) * folder->VarDict["TI_binArea_frac"];
	folder->VarDict["Infiltration_viaInfilEx_m"] = folder->VarDict["Infiltration_viaInfilEx_m"] - folder->VarVecDict["Runoff_SatExcess_TI_m"][ia] * folder->ParamDict["InfiltExcessGovernedArea_frac"] * folder->VarDict["TI_binArea_frac"];

	//cte 20210504 These are adjusted by Drainage_RootZone_MacroPore_frac and Drainage_RootZone_Infil_frac; should be needed when PMacro = 1
	//folder->VarDict["Drainage_macroPore_m"] = folder->VarDict["Drainage_macroPore_m"] - folder->VarVecDict["Runoff_SatExcess_TI_m"][ia] * input->IndexArea[ia] * folder->VarDict["Drainage_RootZone_MacroPore_frac"];
	//folder->VarDict["Infiltration_viaSatEx_m"] = folder->VarDict["Infiltration_viaSatEx_m"] - folder->VarVecDict["Runoff_SatExcess_TI_m"][ia] * input->IndexArea[ia] * (1 - folder->ParamDict["InfiltExcessGovernedArea_frac"]) * folder->VarDict["Drainage_RootZone_Infil_frac"];
	//folder->VarDict["Infiltration_viaInfilEx_m"] = folder->VarDict["Infiltration_viaInfilEx_m"] - folder->VarVecDict["Runoff_SatExcess_TI_m"][ia] * input->IndexArea[ia] * folder->ParamDict["InfiltExcessGovernedArea_frac"] * folder->VarDict["Drainage_RootZone_Infil_frac"];


	//Drainage routine, treating all water as gravitational other than water in root zone water held by capillarity 
	//Drainage_UnsatZone_TI_m (m) reset to zero at start of each TI value
	folder->VarVecDict["Drainage_UnsatZone_TI_m"][ia] = 0.0;
	
	//if Storage_Max_UnsatZone_TI_m (m), the local soil mositure deficit, > 0 then there is an unsaturated zone, and it can drain to lower level groundwater
	if (folder->VarVecDict["Storage_Max_UnsatZone_TI_m"][ia] > 0.0 )
	{
		//Drainage_UnsatZone_TI_m (m), unsaturated zone flow to groundwater = (unsaturated zone storage / (local soil moisture deficit * Unsaturated zone time delay (Td_dt, adjusted to time step) constant)
		//Eq 16 of Beven and Wood (1983) for time delay of water release from unsaturated zone, for water above field capacity
		//Beven and Wood (1983). Catchment Geomorphology and the Dynamics of Runoff Contributing Areas, Journal of Hydrology. 65:139-158. In Special Issue: Scale Problems in Hydrology, Editors: Rodriguez-Iturbe and V.K. Gupta.
		folder->VarVecDict["Drainage_UnsatZone_TI_m"][ia] = folder->VarVecDict["Storage_Used_UnsatZone_TI_m"][ia] / 
													(folder->VarVecDict["Storage_Max_UnsatZone_TI_m"][ia] * folder->ParamDict["Td_h"] * 3600 / folder->ParamDict["TimeStep_sec"]);

		//if (unsaturated zone flow > unsaturated zone storage), set it equal to the unsaturated zone storage
		if (folder->VarVecDict["Drainage_UnsatZone_TI_m"][ia] > folder->VarVecDict["Storage_Used_UnsatZone_TI_m"][ia]) {
			folder->VarVecDict["Drainage_UnsatZone_TI_m"][ia] = folder->VarVecDict["Storage_Used_UnsatZone_TI_m"][ia];
		}
			
		//Storage_Used_UnsatZone_TI_m (m) storage decreased by flow out of the unsaturated zone and into groundwater 
		folder->VarVecDict["Storage_Used_UnsatZone_TI_m"][ia] = folder->VarVecDict["Storage_Used_UnsatZone_TI_m"][ia] - folder->VarVecDict["Drainage_UnsatZone_TI_m"][ia];
			
		//cte 20210427 Change multiplication by area to multiplication by watershed fraction
		//Drainage_UnsatZone_TI_m (m) scaled to the IndexArea, or fraction of DataFolder area in this TI bin outside of this if statement
		//folder->VarVecDict["Drainage_UnsatZone_TI_m"][ia] = folder->VarVecDict["Drainage_UnsatZone_TI_m"][ia]; 
			
	}
	//Drainage_UnsatZone_m (m) = sum Drainage_UnsatZone_TI_m for each TI bin for time step, weighted by watershed fraction
	//	HydroPlus weights discrete TI parameter values, i.e, SoilParameter[ia], with TI_binArea_frac, catchment area bounding discrete value
	folder->VarDict["Drainage_UnsatZone_m"] += folder->VarVecDict["Drainage_UnsatZone_TI_m"][ia] * folder->VarDict["TI_binArea_frac"];
	//Storage_Used_UnsatZone_m = sum Storage_Used_UnsatZone_TI_m for each TI bin for time step, weighted by watershed fraction
	//	HydroPlus weights discrete TI parameter values, i.e, SoilParameter[ia], with TI_binArea_frac, catchment area bounding discrete value
	folder->VarDict["Storage_Used_UnsatZone_m"] += folder->VarVecDict["Storage_Used_UnsatZone_TI_m"][ia] * folder->VarDict["TI_binArea_frac"];

}