#include "WaterOnPerviousAreaCalc.h"
#include <iostream>


void WaterOnPerviousAreaCalc::calculate(Inputs *input, DataFolder *folder, int timeStep, std::vector<DataFolder*> &stafolders)
{
	// TotalPerviousCover_frac = fraction of watershed with tree cover on pervious area + fraction of watershed with shortveg cover + bare soil area
	double TotalPerviousCover_frac;
	double TotalImperviousCover_frac;
	if (folder->ParamStringDict["Type"] == "BulkArea") {
		TotalPerviousCover_frac = folder->ParamDict["TotalPerviousCover_frac"];
		TotalImperviousCover_frac = folder->ParamDict["TotalImperviousCover_frac"];
	}
	else {
		TotalPerviousCover_frac = folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"] + 
			folder->ParamDict["ShortVegetationCover_frac"] +
			folder->ParamDict["SoilCover_frac"];
		TotalImperviousCover_frac = folder->ParamDict["SurfaceWaterIA_frac"] + folder->ParamDict["ImperviousCoverUnderTreeCanopy_frac"] + folder->ParamDict["ImperviousCover_frac"];
	}

	// Computing weighted average of through flow ppt on pervious area results in actual depth of ppt, not fractional depth of ppt
	// 1st: summing 3 fractional area contributions gets fractional depth
	// 2nd: dividing by total fractional area gets actual depth

	//cte 20210520 Likely replace this section below "if (TotalPerviousCover_frac > 0 )" with condition below to simply avoid divsion by 0
	//If TotalPerviousCover_frac is > 0, gather water from various catchment features
	if (TotalPerviousCover_frac > 0)     
	{
		//Note, multiplication of variables by fractional area below is to adjust depths from one analysis area to entire catchment area
		//	Rationale:	1. variables arrive as depths for an analysis area (e.g., pervious area), which are not conserved when transfered to the catchment area
		//				2. multiplying these depths by their analysis area gives a volume, which are conserved when transfered between different areas
		//				3. multiplying these depths by their fractional area is equivalent to multiplying by their analysis area and dividing by the catchment area, getting depth for catchment area
		//Note, division of variables by fractional area below is to adjust depths from entire catchment area to one analysis area
		//	Rationale:	1. variables arrive as depths for the entire catchment area
		//				2. multiplying these depths by the catchment area gives a volume
		//				3. dividing these depths by the analysis area is equivalent to multiplying by the catchment area and dividing by the analysis area, getting depth for analysis area

		//cte 20210715 Rename waterOn terms to PrecipAndRunonOnPervious_m or similar; rename waterUnder terms to PrecipOnGrndUnderTree_m etc.
		// 1st: Summing 3 fractional area contributions, each adjusted to depth over folder area (e.g., bulk area) by multiplying by folder area fraction in analysis area
		// ThroughFlow of Pervious Area (value) = water under trees * fraction of watershed with trees on pervious area
		double waterOnPerviousAreaVal_m = folder->VarDict["waterUnderTree"] * folder->ParamDict["PerviousCoverUnderTreeCanopy_frac"];
		// + water under shortveg * fraction of watershed with shortveg on pervious area
		waterOnPerviousAreaVal_m += folder->VarDict["waterUnderShortVeg"] * folder->ParamDict["ShortVegetationCover_frac"];
		// + water on open pervious area * fraction of waterhsed with bare soil (open pervious area)
		waterOnPerviousAreaVal_m += folder->VarDict["waterOpenArea"] * folder->ParamDict["SoilCover_frac"];

		// 2nd: Dividing the catchment area depth by the analysis area gets depth in analysis area; 
		//	Equivalent to multiplying by the catchment area and dividing by the analysis area, getting depth for analysis area
		waterOnPerviousAreaVal_m = waterOnPerviousAreaVal_m / TotalPerviousCover_frac;

		// Storing waterOnPerviousAreaVal_m as VarDict["waterOnPerviousArea"], without impervious area runon
		//		Later in this function, will store waterOnPerviousAreaVal_m as a variable with impervious runon included
		// Storing folder->VarDict["waterOnPerviousArea"] without Runoff_ImperviousToPervious_m, so it only represents water from pervious areas
		folder->VarDict["waterOnPerviousArea"] = waterOnPerviousAreaVal_m;

		//cte 20210520 consider keeping Runoff_ImperviousToPervious_m as depth for impervious area, not folder area, until StoreTimeStepData or StatisticalOutputWriter
		//cte 20210520 consider creating Runon_PerviousFromImpervious_m as depth for pervious area
		// Adding Runoff_ImperviousToPervious_m, previously adjusted for catchment area with multiplication by ImpAreaNotDirectlyConnected2Outlet_frac in ImperviousDepressionStorageCalc.cpp
		// Adjusting Runoff_ImperviousToPervious_m to depth (m) on pervious area with division by TotalPerviousCover_frac
		// Later storing folder->VarDict["waterToPerDep"] with Runoff_ImperviousToPervious_m, so it represents both water from pervious and impervious areas
		//cte 20210520 Likely use "if (TotalPerviousCover_frac <= 0 )" condition here to simply avoid divsion by 0
		waterOnPerviousAreaVal_m += (folder->VarDict["Runoff_ImperviousToPervious_m"] / TotalPerviousCover_frac);

		// Storing waterOnPerviousAreaVal_m as VarDict["waterToPerArea"], with impervious area runon included
		if (folder->ParamStringDict["Type"] == "BulkArea") {
			
			// Storing folder->VarDict["waterToPerDep"] with Runoff_ImperviousToPervious_m, so it represents both water from pervious and impervious areas
			folder->VarDict["waterToPerDep"] = waterOnPerviousAreaVal_m;
		}
		if (folder->ParamStringDict["Type"] != "BulkArea") {
			//Combine waterOnPerviousAreaVal_m with runon from Bulk Area (BA) contributing area draining to GI 
			//cte 20210715 Perhaps rename waterToPerDep_pre (and waterToPerDep) to contain unit and suggest it is water on GI and transferred to GI
			//cte 20210715 Perhaps rename PrecipOnPerviousArea_InGI_m to explain it contains Runoff_ImperviousToPervious_m and waterOnPerviousAreaVal_m; confirm it is adjusted to proper depth
			input->RepoDict["PrecipOnPerviousArea_InGI_m"] = waterOnPerviousAreaVal_m;
			//input->RepoDict["BulkAreaImpRunoffDrainingToGI_m"] * input->RepoDict["BulkArea_Area_m2"]
			//cte 20210715 Rename waterToPerDep_pre into two terms, PrecipAndRunonOnPervious_m and Runoff_BA_to_GI_m, combining three last terms below into one term
			//cte 20210715 Remove RepoDict from RepoDict["PrecipOnPerviousArea_InGI_m"]
			folder->VarDict["waterToPerDep_pre"] = input->RepoDict["PrecipOnPerviousArea_InGI_m"] +
				input->RepoDict["impRunoffFromContAreaToGI_m"] +
				input->RepoDict["ContArea_InfilExRunoffDrainingToGI_m"] +
				input->RepoDict["ContArea_SatExRunoffDrainingToGI_m"];	
		}
	}

	//cte 20210520 Likely replace this section below "if (TotalPerviousCover_frac <= 0 )" with condition above to simply avoid divsion by 0
	//If TotalPerviousCover_frac = 0, do not gather water from various catchment features
	if (TotalPerviousCover_frac <= 0 ) {
		folder->VarDict["waterOnPerviousArea"] = 0.0;

		//BulkArea waterToPerDep (m) = 0 for use in depression storage
		if (folder->ParamStringDict["Type"] == "BulkArea") {

			folder->VarDict["waterToPerDep"] = folder->VarDict["waterOnPerviousArea"];
		}
		//GI waterToPerDep_pre (m) = 0 from GI + runon from BulkArea for use GI water budget
		if (folder->ParamStringDict["Type"] != "BulkArea") {
			//Combine waterOnPerviousAreaVal_m with runon from Bulk Area (BA) contributing area draining to GI 
			folder->VarDict["waterToPerDep_pre"] = folder->VarDict["waterOnPerviousArea"] +
				input->RepoDict["impRunoffFromContAreaToGI_m"] +
				input->RepoDict["ContArea_InfilExRunoffDrainingToGI_m"] +
				input->RepoDict["ContArea_SatExRunoffDrainingToGI_m"];
		}
	}

	//storeValues for output
    storeValues(input, folder, timeStep);

}

void WaterOnPerviousAreaCalc::storeValues(Inputs *input, DataFolder *folder, int timeStep)
{
	if (input->SimulationStringParams["Model"]=="StatisticalHydro"){
		folder->VarVecDict["waterToPerDep"].emplace_back(folder->VarDict["waterToPerDep"]);
		folder->VarVecDict["waterOnPerArea"].emplace_back(folder->VarDict["waterOnPerviousArea"]);
		//Impervious runoff to soil (m) as depth over pervious area, by dividing Runoff_ImperviousToPervious_m (depth over impervious area) by pervious area fraction
		folder->VarVecDict["impRunoffToSoil_adj"].emplace_back(folder->VarDict["Runoff_ImperviousToPervious_m"] / folder->ParamDict["TotalPerviousCover_frac"]);
	}
}


